@@ -35,11 +35,11 @@ use BackRestDoc::Markdown::DocMarkdown;
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
-use pgBackRest::Storage::Local;
-use pgBackRest::Storage::Posix::Driver;
 use pgBackRest::Version;
 
 use pgBackRestTest::Common::ExecuteTest;
+use pgBackRestTest::Common::Storage;
+use pgBackRestTest::Common::StoragePosix;
 
 ####################################################################################################################################
 # Usage
@@ -64,9 +64,10 @@ doc.pl [options]
    --cache-only     Only use the execution cache - don't attempt to generate it
    --pre            Pre-build containers for execute elements marked pre
    --var            Override defined variable
-   --var-key        Override defined variable and use in cache key
+   --key-var        Override defined variable and use in cache key
    --doc-path       Document path to render (manifest.xml should be located here)
    --out            Output types (html, pdf, markdown)
+   --out-preserve   Don't clean output directory
    --require        Require only certain sections of the document (to speed testing)
    --include        Include source in generation (links will reference website)
    --exclude        Exclude source from generation (links will reference website)
@@ -90,6 +91,7 @@ my $rhVariableOverride = {};
 my $rhKeyVariableOverride = {};
 my $strDocPath;
 my @stryOutput;
+my $bOutPreserve = false;
 my @stryRequire;
 my @stryInclude;
 my @stryExclude;
@@ -103,6 +105,7 @@ GetOptions ('help' => \$bHelp,
             'quiet' => \$bQuiet,
             'log-level=s' => \$strLogLevel,
             'out=s@' => \@stryOutput,
+            'out-preserve' => \$bOutPreserve,
             'require=s@' => \@stryRequire,
             'include=s@' => \@stryInclude,
             'exclude=s@' => \@stryExclude,
@@ -190,8 +193,8 @@ eval
     # Get the base path
     my $strBasePath = abs_path(dirname($0));
 
-    my $oStorageDoc = new pgBackRest::Storage::Local(
-        $strBasePath, new pgBackRest::Storage::Posix::Driver({bFileSync => false, bPathSync => false}));
+    my $oStorageDoc = new pgBackRestTest::Common::Storage(
+        $strBasePath, new pgBackRestTest::Common::StoragePosix({bFileSync => false, bPathSync => false}));
 
     if (!defined($strDocPath))
     {
@@ -278,6 +281,24 @@ eval
 
         &log(INFO, "render ${strOutput} output");
 
+        # Clean contents of out directory
+        if (!$bOutPreserve)
+        {
+            my $strOutputPath = $strOutput eq 'pdf' ? "${strOutputPath}/latex" : "${strOutputPath}/$strOutput";
+
+            # Clean the current out path if it exists
+            if (-e $strOutputPath)
+            {
+                executeTest("rm -rf ${strOutputPath}/*");
+            }
+            # Else create the html path
+            else
+            {
+                mkdir($strOutputPath)
+                    or confess &log(ERROR, "unable to create path ${strOutputPath}");
+            }
+        }
+
         if ($strOutput eq 'markdown')
         {
             my $oMarkdown =
