@@ -8,6 +8,7 @@ use warnings FATAL => qw(all);
 use Carp qw(confess);
 use English '-no_match_vars';
 
+use Cwd qw(abs_path);
 use File::Basename qw(dirname);
 use Scalar::Util qw(blessed);
 use XML::Checker::Parser;
@@ -45,7 +46,8 @@ sub new
     if (defined($self->{strFileName}))
     {
         my $oParser = XML::Checker::Parser->new(ErrorContext => 2, Style => 'Tree');
-        $oParser->set_sgml_search_path(defined($strSgmlSearchPath) ? $strSgmlSearchPath : dirname(dirname($0)) . '/doc/xml/dtd');
+        $oParser->set_sgml_search_path(
+            defined($strSgmlSearchPath) ? $strSgmlSearchPath : dirname(dirname(abs_path($0))) . '/doc/xml/dtd');
 
         my $oTree;
 
@@ -115,7 +117,8 @@ sub parse
     my %oOut;
     my $iIndex = 0;
     my $bText = $strName eq 'text' || $strName eq 'li' || $strName eq 'p' || $strName eq 'title' ||
-                $strName eq 'summary' || $strName eq 'table-cell' || $strName eq 'table-column' || $strName eq 'list-item';
+                $strName eq 'summary' || $strName eq 'table-cell' || $strName eq 'table-column' || $strName eq 'list-item' ||
+                $strName eq 'admonition';
 
     # Store the node name
     $oOut{name} = $strName;
@@ -230,7 +233,8 @@ sub build
     }
 
     if ($$oDoc{name} eq 'p' || $$oDoc{name} eq 'title' || $$oDoc{name} eq 'summary' ||
-        $$oDoc{name} eq 'table-cell' || $$oDoc{name} eq 'table-column' || $$oDoc{name} eq 'list-item')
+        $$oDoc{name} eq 'table-cell' || $$oDoc{name} eq 'table-column' || $$oDoc{name} eq 'list-item' ||
+        $$oDoc{name} eq 'admonition')
     {
         $$oOut{field}{text} = $oDoc;
     }
