@@ -195,7 +195,8 @@ sub process
 
     foreach my $strCommand (cfgDefineCommandList())
     {
-        if ($strCommand eq CFGCMD_REMOTE || $strCommand eq CFGCMD_LOCAL)
+        if ($strCommand eq CFGCMD_REMOTE || $strCommand eq CFGCMD_LOCAL || $strCommand eq CFGCMD_ARCHIVE_GET_ASYNC ||
+            $strCommand eq CFGCMD_ARCHIVE_PUSH_ASYNC)
         {
             next;
         }
@@ -336,6 +337,12 @@ sub process
 
                 if (@stryNameAlt > 0)
                 {
+                    if (@stryNameAlt != 1)
+                    {
+                        confess &log(
+                            ERROR, "multiple alt names are not supported for option '${strOption}':  " . join(', ', @stryNameAlt));
+                    }
+
                     $oCommandOption->{&CONFIG_HELP_NAME_ALT} = \@stryNameAlt;
                 }
             }
