@@ -167,8 +167,8 @@ sub new
         (
             __PACKAGE__ . '->new', \@_,
             {name => 'strType'},
-            {name => 'oManifest'},
-            {name => 'bExe'},
+            {name => 'oManifest', required => false},
+            {name => 'bExe', required => false},
             {name => 'strRenderOutKey', required => false}
         );
 
@@ -275,6 +275,20 @@ sub new
     );
 }
 
+####################################################################################################################################
+# Set begin and end values for a tag
+####################################################################################################################################
+sub tagSet
+{
+    my $self = shift;
+    my $strTag = shift;
+    my $strBegin = shift;
+    my $strEnd = shift;
+
+    $oRenderTag->{$self->{strType}}{$strTag}[0] = defined($strBegin) ? $strBegin : '';
+    $oRenderTag->{$self->{strType}}{$strTag}[1] = defined($strEnd) ? $strEnd : '';
+}
+
 ####################################################################################################################################
 # variableReplace
 #
@@ -284,7 +298,7 @@ sub variableReplace
 {
     my $self = shift;
 
-    return $self->{oManifest}->variableReplace(shift, $self->{strType});
+    return defined($self->{oManifest}) ? $self->{oManifest}->variableReplace(shift, $self->{strType}) : shift;
 }
 
 ####################################################################################################################################
@@ -527,7 +541,7 @@ sub build
 
         my $strNodeJSON = $self->{oJSON}->encode($self->{oyBlockDefine}{$strBlockId});
 
-        foreach my $oVariable ($oNode->nodeList('block-variable-replace'))
+        foreach my $oVariable ($oNode->nodeList('block-variable-replace', false))
         {
             my $strVariableKey = $oVariable->paramGet('key');
             my $strVariableReplace = $oVariable->valueGet();
@@ -1022,6 +1036,7 @@ sub processText
     {
         $strBuffer =~ s/\&mdash\;/--/g;
         $strBuffer =~ s/\&lt\;/\</g;
+        $strBuffer =~ s/\&ge\;/\>\=/g;
     }
 
     $strBuffer = $self->variableReplace($strBuffer);
