@@ -35,11 +35,11 @@ use BackRestDoc::Markdown::DocMarkdown;
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
-use pgBackRest::Storage::Local;
-use pgBackRest::Storage::Posix::Driver;
 use pgBackRest::Version;
 
 use pgBackRestTest::Common::ExecuteTest;
+use pgBackRestTest::Common::Storage;
+use pgBackRestTest::Common::StoragePosix;
 
 ####################################################################################################################################
 # Usage
@@ -62,6 +62,7 @@ release.pl [options]
  Release Options:
    --build          Build the cache before release (should be included in the release commit)
    --deploy         Deploy documentation to website (can be done as docs are updated)
+   --no-gen         Don't auto-generate
 =cut
 
 ####################################################################################################################################
@@ -73,13 +74,15 @@ my $bQuiet = false;
 my $strLogLevel = 'info';
 my $bBuild = false;
 my $bDeploy = false;
+my $bNoGen = false;
 
 GetOptions ('help' => \$bHelp,
             'version' => \$bVersion,
             'quiet' => \$bQuiet,
             'log-level=s' => \$strLogLevel,
             'build' => \$bBuild,
-            'deploy' => \$bDeploy)
+            'deploy' => \$bDeploy,
+            'no-gen' => \$bNoGen)
     or pod2usage(2);
 
 ####################################################################################################################################
@@ -119,9 +122,10 @@ eval
     my $strDocPath = dirname(abs_path($0));
     my $strDocHtml = "${strDocPath}/output/html";
     my $strDocExe = "${strDocPath}/doc.pl";
+    my $strTestExe = dirname($strDocPath) . "/test/test.pl";
 
-    my $oStorageDoc = new pgBackRest::Storage::Local(
-        $strDocPath, new pgBackRest::Storage::Posix::Driver({bFileSync => false, bPathSync => false}));
+    my $oStorageDoc = new pgBackRestTest::Common::Storage(
+        $strDocPath, new pgBackRestTest::Common::StoragePosix({bFileSync => false, bPathSync => false}));
 
     # Determine if this is a dev release
     my $bDev = PROJECT_VERSION =~ /dev$/;
@@ -129,6 +133,88 @@ eval
 
     if ($bBuild)
     {
+        if (!$bNoGen)
+        {
+            # Update git history
+            my $strGitCommand =
+                'git -C ' . $strDocPath .
+                ' log --pretty=format:\'{^^^^commit^^^^:^^^^%H^^^^,^^^^date^^^^:^^^^%ci^^^^,^^^^subject^^^^:^^^^%s^^^^,^^^^body^^^^:^^^^%b^^^^},\'';
+            my $strGitLog = qx($strGitCommand);
+            $strGitLog =~ s/\^\^\^\^\}\,\n/\#\#\#\#/mg;
+            $strGitLog =~ s/\\/\\\\/g;
+            $strGitLog =~ s/\n/\\n/mg;
+            $strGitLog =~ s/\r/\\r/mg;
+            $strGitLog =~ s/\t/\\t/mg;
+            $strGitLog =~ s/\"/\\\"/g;
+            $strGitLog =~ s/\^\^\^\^/\"/g;
+            $strGitLog =~ s/\#\#\#\#/\"\}\,\n/mg;
+            $strGitLog = '[' . substr($strGitLog, 0, length($strGitLog) - 1) . ']';
+            my @hyGitLog = @{(JSON::PP->new()->allow_nonref())->decode($strGitLog)};
+
+            # Load prior history
+            my @hyGitLogPrior = @{(JSON::PP->new()->allow_nonref())->decode(
+                ${$oStorageDoc->get("${strDocPath}/resource/git-history.cache")})};
+
+            # Add new commits
+            for (my $iGitLogIdx = @hyGitLog - 1; $iGitLogIdx >= 0; $iGitLogIdx--)
+            {
+                my $rhGitLog = $hyGitLog[$iGitLogIdx];
+                my $bFound = false;
+
+                foreach my $rhGitLogPrior (@hyGitLogPrior)
+                {
+                    if ($rhGitLog->{commit} eq $rhGitLogPrior->{commit})
+                    {
+                        $bFound = true;
+                    }
+                }
+
+                next if $bFound;
+
+                $rhGitLog->{body} = trim($rhGitLog->{body});
+
+                if ($rhGitLog->{body} eq '')
+                {
+                    delete($rhGitLog->{body});
+                }
+
+                unshift(@hyGitLogPrior, $rhGitLog);
+            }
+
+            # Write git log
+            $strGitLog = undef;
+
+            foreach my $rhGitLog (@hyGitLogPrior)
+            {
+                $strGitLog .=
+                    (defined($strGitLog) ? ",\n" : '') .
+                    "    {\n" .
+                    '        "commit": ' . trim((JSON::PP->new()->allow_nonref()->pretty())->encode($rhGitLog->{commit})) . ",\n" .
+                    '        "date": ' . trim((JSON::PP->new()->allow_nonref()->pretty())->encode($rhGitLog->{date})) . ",\n" .
+                    '        "subject": ' . trim((JSON::PP->new()->allow_nonref()->pretty())->encode($rhGitLog->{subject}));
+
+                # Skip the body if it is empty or a release (since we already have the release note content)
+                if ($rhGitLog->{subject} !~ /^v[0-9]{1,2}\.[0-9]{1,2}\: /g && defined($rhGitLog->{body}))
+                {
+                    $strGitLog .=
+                        ",\n" .
+                        '        "body": ' . trim((JSON::PP->new()->allow_nonref()->pretty())->encode($rhGitLog->{body}));
+                }
+
+                $strGitLog .=
+                    "\n" .
+                    "    }";
+            }
+
+            $oStorageDoc->put("${strDocPath}/resource/git-history.cache", "[\n${strGitLog}\n]\n");
+
+            # Generate coverage summmary
+            &log(INFO, "Generate Coverage Summary");
+            executeTest(
+                "${strTestExe} --no-package --no-valgrind --no-optimize --vm-max=3 --coverage-summary",
+                {bShowOutputAsync => true});
+        }
+
         # Remove permanent cache file
         $oStorageDoc->remove("${strDocPath}/resource/exe.cache", {bIgnoreMissing => true});
 
@@ -151,7 +237,18 @@ eval
         &log(INFO, "Generate Debian/Ubuntu documentation");
 
         executeTest("${strDocExe} --deploy", {bShowOutputAsync => true});
-        executeTest("${strDocExe} --deploy --cache-only --out=man --out=html --var=project-url-root=index.html");
+
+        # Generate a full copy of the docs for review
+        &log(INFO, "Generate full documentation for review");
+
+        executeTest("${strDocExe} --deploy --cache-only --key-var=os-type=centos7 --out=html --var=project-url-root=index.html");
+        $oStorageDoc->move("$strDocHtml/user-guide.html", "$strDocHtml/user-guide-centos7.html");
+        executeTest(
+            "${strDocExe} --deploy --out-preserve --cache-only --key-var=os-type=centos6 --out=html" .
+                " --var=project-url-root=index.html");
+        $oStorageDoc->move("$strDocHtml/user-guide.html", "$strDocHtml/user-guide-centos6.html");
+
+        executeTest("${strDocExe} --deploy --out-preserve --cache-only --out=man --out=html --var=project-url-root=index.html");
     }
 
     if ($bDeploy)
@@ -161,9 +258,16 @@ eval
         # Generate docs for the website history
         &log(INFO, 'Generate website ' . ($bDev ? 'dev' : 'history') . ' documentation');
 
-        executeTest(
-            $strDocExe . ($bDev ? '' : ' --deploy --cache-only') . ' --out=html --var=project-url-root=index.html' .
-            ($bDev ? ' --dev --no-exe' :  ' --exclude=release'));
+        my $strDocExeVersion =
+            ${strDocExe} . ($bDev ? ' --dev' : ' --deploy --cache-only') . ' --var=project-url-root=index.html --out=html';
+
+        executeTest("${strDocExeVersion} --key-var=os-type=centos7");
+        $oStorageDoc->move("$strDocHtml/user-guide.html", "$strDocHtml/user-guide-centos7.html");
+        executeTest("${strDocExeVersion} --out-preserve --key-var=os-type=centos6");
+        $oStorageDoc->move("$strDocHtml/user-guide.html", "$strDocHtml/user-guide-centos6.html");
+
+        $oStorageDoc->remove("$strDocHtml/release.html");
+        executeTest("${strDocExeVersion} --out-preserve --exclude=release");
 
         # Deploy to repository
         &log(INFO, '...Deploy to repository');
@@ -176,7 +280,11 @@ eval
         {
             &log(INFO, "Generate website documentation");
 
-            executeTest("${strDocExe} --deploy --cache-only --out=html");
+            executeTest("${strDocExe} --deploy --cache-only --key-var=os-type=centos7 --out=html");
+            $oStorageDoc->move("$strDocHtml/user-guide.html", "$strDocHtml/user-guide-centos7.html");
+            executeTest("${strDocExe} --deploy --out-preserve --cache-only --key-var=os-type=centos6 --out=html");
+            $oStorageDoc->move("$strDocHtml/user-guide.html", "$strDocHtml/user-guide-centos6.html");
+            executeTest("${strDocExe} --deploy --out-preserve --cache-only --out=html");
 
             # Deploy to repository
             &log(INFO, '...Deploy to repository');
