@@ -12,7 +12,6 @@ use Exporter qw(import);
 use File::Basename;
 
 use pgBackRest::Archive::Common;
-use pgBackRest::Archive::Get::Get;
 use pgBackRest::Backup::Common;
 use pgBackRest::Backup::File;
 use pgBackRest::Backup::Info;
@@ -31,8 +30,6 @@ use pgBackRest::Protocol::Helper;
 use pgBackRest::Protocol::Storage::Helper;
 use pgBackRest::Common::Io::Handle;
 use pgBackRest::Storage::Base;
-use pgBackRest::Storage::Filter::Gzip;
-use pgBackRest::Storage::Filter::Sha;
 use pgBackRest::Storage::Helper;
 use pgBackRest::Version;
 
@@ -230,7 +227,7 @@ sub resumeClean
         if ($cType eq 'd')
         {
             logDebugMisc($strOperation, "remove path ${strName}");
-            $oStorageRepo->remove(STORAGE_REPO_BACKUP . "/${strBackupLabel}/${strName}", {bRecurse => true});
+            $oStorageRepo->pathRemove(STORAGE_REPO_BACKUP . "/${strBackupLabel}/${strName}", {bRecurse => true});
         }
         # Else add the file/link to be deleted later
         else
@@ -324,7 +321,7 @@ sub processManifest
             storageRepo()->pathCreate(STORAGE_REPO_BACKUP . "/${strBackupLabel}/${strPath}", {bIgnoreExists => true});
         }
 
-        if (storageRepo()->driver()->capability(STORAGE_CAPABILITY_LINK))
+        if (storageRepo()->capability(STORAGE_CAPABILITY_LINK))
         {
             for my $strTarget ($oBackupManifest->keys(MANIFEST_SECTION_BACKUP_TARGET))
             {
@@ -339,13 +336,6 @@ sub processManifest
         }
     }
 
-    # Build the lsn start parameter to pass to the extra function
-    my $hStartLsnParam =
-    {
-        iWalId => defined($strLsnStart) ? hex((split('/', $strLsnStart))[0]) : 0xFFFF,
-        iWalOffset => defined($strLsnStart) ? hex((split('/', $strLsnStart))[1]) : 0xFFFF,
-    };
-
     # Iterate all files in the manifest
     foreach my $strRepoFile (
         sort {sprintf("%016d-%s", $oBackupManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $b, MANIFEST_SUBKEY_SIZE), $b) cmp
@@ -404,13 +394,13 @@ sub processManifest
         # Queue for parallel backup
         $oBackupProcess->queueJob(
             $iHostConfigIdx, $strQueueKey, $strRepoFile, OP_BACKUP_FILE,
-            [$strDbFile, $strRepoFile, $lSize,
+            [$strDbFile, $bIgnoreMissing, $lSize,
                 $oBackupManifest->get(MANIFEST_SECTION_TARGET_FILE, $strRepoFile, MANIFEST_SUBKEY_CHECKSUM, false),
-                cfgOption(CFGOPT_CHECKSUM_PAGE) ? isChecksumPage($strRepoFile) : false, $strBackupLabel, $bCompress,
-                cfgOption(CFGOPT_COMPRESS_LEVEL), $oBackupManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $strRepoFile,
-                MANIFEST_SUBKEY_TIMESTAMP, false), $bIgnoreMissing,
-                cfgOption(CFGOPT_CHECKSUM_PAGE) && isChecksumPage($strRepoFile) ? $hStartLsnParam : undef,
-                cfgOption(CFGOPT_DELTA), defined($strReference) ? true : false],
+                cfgOption(CFGOPT_CHECKSUM_PAGE) ? isChecksumPage($strRepoFile) : false,
+                defined($strLsnStart) ? hex((split('/', $strLsnStart))[0]) : 0xFFFFFFFF,
+                defined($strLsnStart) ? hex((split('/', $strLsnStart))[1]) : 0xFFFFFFFF,
+                $strRepoFile, defined($strReference) ? true : false, $bCompress, cfgOption(CFGOPT_COMPRESS_LEVEL),
+                $strBackupLabel, cfgOption(CFGOPT_DELTA)],
             {rParamSecure => $oBackupManifest->cipherPassSub() ? [$oBackupManifest->cipherPassSub()] : undef});
 
         # Size and checksum will be removed and then verified later as a sanity check
@@ -452,7 +442,8 @@ sub processManifest
         {
             ($lSizeCurrent, $lManifestSaveCurrent) = backupManifestUpdate(
                 $oBackupManifest, cfgOption(cfgOptionIdFromIndex(CFGOPT_PG_HOST, $hJob->{iHostConfigIdx}), false),
-                $hJob->{iProcessId}, @{$hJob->{rParam}}[0..4], @{$hJob->{rResult}}, $lSizeTotal, $lSizeCurrent, $lManifestSaveSize,
+                $hJob->{iProcessId}, @{$hJob->{rParam}}[0], @{$hJob->{rParam}}[7], @{$hJob->{rParam}}[2], @{$hJob->{rParam}}[3],
+                @{$hJob->{rParam}}[4], @{$hJob->{rResult}}, $lSizeTotal, $lSizeCurrent, $lManifestSaveSize,
                 $lManifestSaveCurrent);
         }
 
@@ -746,7 +737,7 @@ sub process
                 &log(WARN, "aborted backup ${strAbortedBackup} cannot be resumed: ${strReason}");
                 &log(TEST, TEST_BACKUP_NORESUME);
 
-                $oStorageRepo->remove(STORAGE_REPO_BACKUP . "/${strAbortedBackup}", {bRecurse => true});
+                $oStorageRepo->pathRemove(STORAGE_REPO_BACKUP . "/${strAbortedBackup}", {bRecurse => true});
                 undef($oAbortedManifest);
             }
 
@@ -779,7 +770,12 @@ sub process
     $oBackupManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TYPE, undef, $strType);
     $oBackupManifest->numericSet(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_TIMESTAMP_START, undef, $lTimestampStart);
     $oBackupManifest->boolSet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_BACKUP_STANDBY, undef, cfgOption(CFGOPT_BACKUP_STANDBY));
+    $oBackupManifest->numericSet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_BUFFER_SIZE, undef, cfgOption(CFGOPT_BUFFER_SIZE));
     $oBackupManifest->boolSet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_COMPRESS, undef, $bCompress);
+    $oBackupManifest->numericSet(
+        MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_COMPRESS_LEVEL, undef, cfgOption(CFGOPT_COMPRESS_LEVEL));
+    $oBackupManifest->numericSet(
+        MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_COMPRESS_LEVEL_NETWORK, undef, cfgOption(CFGOPT_COMPRESS_LEVEL_NETWORK));
     $oBackupManifest->boolSet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_HARDLINK, undef, $bHardLink);
     $oBackupManifest->boolSet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_ONLINE, undef, cfgOption(CFGOPT_ONLINE));
     $oBackupManifest->boolSet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_ARCHIVE_COPY, undef,
@@ -787,6 +783,7 @@ sub process
                               (cfgOption(CFGOPT_ARCHIVE_CHECK) && cfgOption(CFGOPT_ARCHIVE_COPY)));
     $oBackupManifest->boolSet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_ARCHIVE_CHECK, undef,
                               !cfgOption(CFGOPT_ONLINE) || cfgOption(CFGOPT_ARCHIVE_CHECK));
+    $oBackupManifest->numericSet(MANIFEST_SECTION_BACKUP_OPTION, MANIFEST_KEY_PROCESS_MAX, undef, cfgOption(CFGOPT_PROCESS_MAX));
 
     # Database settings
     $oBackupManifest->numericSet(MANIFEST_SECTION_BACKUP_DB, MANIFEST_KEY_DB_ID, undef, $iDbHistoryId);
@@ -974,7 +971,9 @@ sub process
                 # Add compression filter
                 if ($bCompress)
                 {
-                    push(@{$rhyFilter}, {strClass => STORAGE_FILTER_GZIP});
+                    push(
+                        @{$rhyFilter},
+                        {strClass => STORAGE_FILTER_GZIP, rxyParam => [STORAGE_COMPRESS, false, cfgOption(CFGOPT_COMPRESS_LEVEL)]});
                 }
 
                 # If the backups are encrypted, then the passphrase for the backup set from the manifest file is required to access
@@ -1059,7 +1058,7 @@ sub process
     $oBackupManifest->set(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_LABEL, undef, $strBackupLabel);
 
     # Sync backup path if supported
-    if ($oStorageRepo->driver()->capability(STORAGE_CAPABILITY_PATH_SYNC))
+    if ($oStorageRepo->capability(STORAGE_CAPABILITY_PATH_SYNC))
     {
         # Sync all paths in the backup
         $oStorageRepo->pathSync(STORAGE_REPO_BACKUP . "/${strBackupLabel}");
@@ -1091,12 +1090,12 @@ sub process
             {'strCipherPass' => $strCipherPassManifest}),
         $oStorageRepo->openWrite(
             "${strHistoryPath}/${strBackupLabel}.manifest." . COMPRESS_EXT,
-            {rhyFilter => [{strClass => STORAGE_FILTER_GZIP}],
+            {rhyFilter => [{strClass => STORAGE_FILTER_GZIP, rxyParam => [STORAGE_COMPRESS, false, 9]}],
                 bPathCreate => true, bAtomic => true,
                 strCipherPass => defined($strCipherPassManifest) ? $strCipherPassManifest : undef}));
 
     # Sync history path if supported
-    if ($oStorageRepo->driver()->capability(STORAGE_CAPABILITY_PATH_SYNC))
+    if ($oStorageRepo->capability(STORAGE_CAPABILITY_PATH_SYNC))
     {
         $oStorageRepo->pathSync(STORAGE_REPO_BACKUP . qw{/} . PATH_BACKUP_HISTORY);
         $oStorageRepo->pathSync($strHistoryPath);
@@ -1105,7 +1104,7 @@ sub process
     # Create a link to the most recent backup
     $oStorageRepo->remove(STORAGE_REPO_BACKUP . qw(/) . LINK_LATEST);
 
-    if (storageRepo()->driver()->capability(STORAGE_CAPABILITY_LINK))
+    if (storageRepo()->capability(STORAGE_CAPABILITY_LINK))
     {
         $oStorageRepo->linkCreate(
             STORAGE_REPO_BACKUP . "/${strBackupLabel}", STORAGE_REPO_BACKUP . qw{/} . LINK_LATEST, {bRelative => true});
@@ -1115,7 +1114,7 @@ sub process
     $oBackupInfo->add($oBackupManifest);
 
     # Sync backup root path if supported
-    if ($oStorageRepo->driver()->capability(STORAGE_CAPABILITY_PATH_SYNC))
+    if ($oStorageRepo->capability(STORAGE_CAPABILITY_PATH_SYNC))
     {
         $oStorageRepo->pathSync(STORAGE_REPO_BACKUP);
     }
