@@ -98,8 +98,8 @@ sub new
             {name => 'bLoad', optional => true, default => true, trace => true},
             {name => 'strContent', optional => true, trace => true},
             {name => 'oStorage', optional => true, default => storageLocal(), trace => true},
-            {name => 'iInitFormat', optional => true, default => BACKREST_FORMAT, trace => true},
-            {name => 'strInitVersion', optional => true, default => BACKREST_VERSION, trace => true},
+            {name => 'iInitFormat', optional => true, default => REPOSITORY_FORMAT, trace => true},
+            {name => 'strInitVersion', optional => true, default => PROJECT_VERSION, trace => true},
             {name => 'bIgnoreMissing', optional => true, default => false, trace => true},
             {name => 'strCipherPass', optional => true, trace => true},
             {name => 'strCipherPassSub', optional => true, trace => true},
@@ -107,7 +107,7 @@ sub new
 
     if (defined($self->{oStorage}->cipherPassUser()) && !defined($self->{strCipherPass}))
     {
-        confess &log(ERROR, 'passphrase is required when storage is encrypted', ERROR_CIPHER);
+        confess &log(ERROR, 'passphrase is required when storage is encrypted', ERROR_CRYPTO);
     }
 
     # Set changed to false
@@ -188,7 +188,7 @@ sub loadVersion
     else
     {
         confess &log(ERROR, "unable to parse '$self->{strFileName}" . ($bCopy ? INI_COPY_EXT : '') . "'" .
-            "\nHINT: Is or was the repo encrypted?", ERROR_CIPHER);
+            "\nHINT: Is or was the repo encrypted?", ERROR_CRYPTO);
     }
 
     return defined($self->{oContent});
