@@ -65,7 +65,8 @@ sub configLogging
             cfgOptionValid(CFGOPT_LOG_LEVEL_FILE) ? cfgOption(CFGOPT_LOG_LEVEL_FILE) : OFF,
             cfgOptionValid(CFGOPT_LOG_LEVEL_CONSOLE) ? cfgOption(CFGOPT_LOG_LEVEL_CONSOLE) : OFF,
             cfgOptionValid(CFGOPT_LOG_LEVEL_STDERR) ? cfgOption(CFGOPT_LOG_LEVEL_STDERR) : OFF,
-            cfgOptionValid(CFGOPT_LOG_TIMESTAMP) ? cfgOption(CFGOPT_LOG_TIMESTAMP) : undef);
+            cfgOptionValid(CFGOPT_LOG_TIMESTAMP) ? cfgOption(CFGOPT_LOG_TIMESTAMP) : undef,
+            cfgOptionValid(CFGOPT_PROCESS_MAX) ? cfgOption(CFGOPT_PROCESS_MAX) : undef);
 
         $bInitLog = true;
     }
@@ -93,9 +94,6 @@ sub configLoad
 
     eval
     {
-        # Hacky fix for backslashes that need to be escaped
-        $$rstrConfigJson =~ s/\\/\\\\/g;
-
         %oOption = %{(JSON::PP->new()->allow_nonref())->decode($$rstrConfigJson)};
         return true;
     }
@@ -358,7 +356,7 @@ sub cfgCommandWrite
         my $bSecure = cfgDefOptionSecure($iOptionId);
 
         # Skip option if it is secure and should not be output in logs or the command line
-        next if ($bSecure && !$bDisplayOnly);
+        next if (($bSecure || $iOptionId == CFGOPT_REPO_CIPHER_TYPE) && !$bDisplayOnly);
 
         # Process any option id overrides first
         if (defined($oOptionOverride->{$iOptionId}))
