@@ -103,14 +103,22 @@ use constant MANIFEST_KEY_ARCHIVE_CHECK                             => 'option-'
     push @EXPORT, qw(MANIFEST_KEY_ARCHIVE_CHECK);
 use constant MANIFEST_KEY_ARCHIVE_COPY                              => 'option-' .cfgOptionName(CFGOPT_ARCHIVE_COPY);
     push @EXPORT, qw(MANIFEST_KEY_ARCHIVE_COPY);
+use constant MANIFEST_KEY_BUFFER_SIZE                               => 'option-' . cfgOptionName(CFGOPT_BUFFER_SIZE);
+    push @EXPORT, qw(MANIFEST_KEY_BUFFER_SIZE);
 use constant MANIFEST_KEY_CHECKSUM_PAGE                             => 'option-' . cfgOptionName(CFGOPT_CHECKSUM_PAGE);
     push @EXPORT, qw(MANIFEST_KEY_CHECKSUM_PAGE);
 use constant MANIFEST_KEY_COMPRESS                                  => 'option-' . cfgOptionName(CFGOPT_COMPRESS);
     push @EXPORT, qw(MANIFEST_KEY_COMPRESS);
+use constant MANIFEST_KEY_COMPRESS_LEVEL                            => 'option-' . cfgOptionName(CFGOPT_COMPRESS_LEVEL);
+    push @EXPORT, qw(MANIFEST_KEY_COMPRESS_LEVEL);
+use constant MANIFEST_KEY_COMPRESS_LEVEL_NETWORK                    => 'option-' . cfgOptionName(CFGOPT_COMPRESS_LEVEL_NETWORK);
+    push @EXPORT, qw(MANIFEST_KEY_COMPRESS_LEVEL_NETWORK);
 use constant MANIFEST_KEY_ONLINE                                    => 'option-' . cfgOptionName(CFGOPT_ONLINE);
     push @EXPORT, qw(MANIFEST_KEY_ONLINE);
 use constant MANIFEST_KEY_DELTA                                     => 'option-' . cfgOptionName(CFGOPT_DELTA);
     push @EXPORT, qw(MANIFEST_KEY_DELTA);
+use constant MANIFEST_KEY_PROCESS_MAX                               => 'option-' . cfgOptionName(CFGOPT_PROCESS_MAX);
+    push @EXPORT, qw(MANIFEST_KEY_PROCESS_MAX);
 
 # Information about the database that was backed up
 use constant MANIFEST_KEY_DB_ID                                     => 'db-id';
@@ -314,7 +322,8 @@ sub new
             confess &log(ASSERT, 'strDbVersion and iDbCatalogVersion must be provided with bLoad = false');
         }
 
-        $self->set(MANIFEST_SECTION_BACKUP_DB, MANIFEST_KEY_DB_VERSION, undef, $strDbVersion);
+        # Force the version to a string since newer versions of JSON::PP lose track of the fact that it is one
+        $self->set(MANIFEST_SECTION_BACKUP_DB, MANIFEST_KEY_DB_VERSION, undef, $strDbVersion . '');
         $self->numericSet(MANIFEST_SECTION_BACKUP_DB, MANIFEST_KEY_CATALOG, undef, $iDbCatalogVersion);
     }
 
