@@ -25,14 +25,6 @@ use constant OP_BACKUP_FILE                                          => 'backupF
 # Archive Module
 use constant OP_ARCHIVE_GET_CHECK                                   => 'archiveCheck';
     push @EXPORT, qw(OP_ARCHIVE_GET_CHECK);
-use constant OP_ARCHIVE_PUSH_CHECK                                  => 'archivePushCheck';
-    push @EXPORT, qw(OP_ARCHIVE_PUSH_CHECK);
-
-# Archive File Module
-use constant OP_ARCHIVE_GET_FILE                                    => 'archiveGetFile';
-    push @EXPORT, qw(OP_ARCHIVE_GET_FILE);
-use constant OP_ARCHIVE_PUSH_FILE                                   => 'archivePushFile';
-    push @EXPORT, qw(OP_ARCHIVE_PUSH_FILE);
 
 # Check Module
 use constant OP_CHECK_BACKUP_INFO_CHECK                             => 'backupInfoCheck';
@@ -82,7 +74,7 @@ my $hProtocol = {};         # Global remote hash that is created on first reques
 ####################################################################################################################################
 # isRepoLocal
 #
-# Is the backup/archive repository local?  This does not take into account the spool path.
+# Is the backup/archive repository local?
 ####################################################################################################################################
 sub isRepoLocal
 {
