@@ -15,7 +15,6 @@ use pgBackRest::Common::Log;
 use pgBackRest::Common::Io::Buffered;
 use pgBackRest::Common::Wait;
 use pgBackRest::Archive::Get::File;
-use pgBackRest::Archive::Push::File;
 use pgBackRest::Check::Check;
 use pgBackRest::Config::Config;
 use pgBackRest::Db;
@@ -78,9 +77,6 @@ sub init
         # ArchiveGet commands
         &OP_ARCHIVE_GET_CHECK => sub {archiveGetCheck(@{shift()})},
 
-        # ArchivePush commands
-        &OP_ARCHIVE_PUSH_CHECK => sub {archivePushCheck(@{shift()})},
-
         # Check commands
         &OP_CHECK_BACKUP_INFO_CHECK => sub {$oCheck->backupInfoCheck(@{shift()})},
 
@@ -95,11 +91,12 @@ sub init
             my $oSourceFileIo = $oStorage->openRead(@{shift()});
 
             # If the source file exists
-            if (defined($oSourceFileIo))
+            if (defined($oSourceFileIo) && (!defined($oSourceFileIo->{oStorageCRead}) || $oSourceFileIo->open()))
             {
                 $self->outputWrite(true);
 
-                $oStorage->copy($oSourceFileIo, new pgBackRest::Protocol::Storage::File($self, $oSourceFileIo));
+                $oStorage->copy(
+                    $oSourceFileIo, new pgBackRest::Protocol::Storage::File($self, $oSourceFileIo), {bSourceOpen => true});
 
                 return true;
             }
