@@ -123,7 +123,7 @@ Simplifies creation of the memory context in contructors and includes error hand
     }                                                                                                                              \
 }
 
-#define MEM_COMTEXT_XS()                                                                                                           \
+#define MEM_CONTEXT_XS()                                                                                                           \
     MEM_CONTEXT_XS_memContext
 
 /***********************************************************************************************************************************
@@ -181,3 +181,15 @@ Free memory context in destructors
 ***********************************************************************************************************************************/
 #define MEM_CONTEXT_XS_DESTROY(memContext)                                                                                         \
     memContextFree(memContext)
+
+/***********************************************************************************************************************************
+Create new string from an SV
+***********************************************************************************************************************************/
+#define STR_NEW_SV(param)                                                                                                          \
+    (SvOK(param) ? strNewN(SvPV_nolen(param), SvCUR(param)) : NULL)
+
+/***********************************************************************************************************************************
+Create const buffer from an SV
+***********************************************************************************************************************************/
+#define BUF_CONST_SV(param)                                                                                                        \
+    (SvOK(param) ? BUF(SvPV_nolen(param), SvCUR(param)) : NULL)
