@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
-Archive Push Command
+Archive Common
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -11,6 +13,7 @@ Archive Push Command
 #include "common/memContext.h"
 #include "common/regExp.h"
 #include "common/wait.h"
+#include "config/config.h"
 #include "postgres/version.h"
 #include "storage/helper.h"
 #include "storage/helper.h"
@@ -23,6 +26,27 @@ STRING_EXTERN(WAL_SEGMENT_PARTIAL_REGEXP_STR,                       WAL_SEGMENT_
 STRING_EXTERN(WAL_SEGMENT_DIR_REGEXP_STR,                           WAL_SEGMENT_DIR_REGEXP);
 STRING_EXTERN(WAL_SEGMENT_FILE_REGEXP_STR,                          WAL_SEGMENT_FILE_REGEXP);
 
+/***********************************************************************************************************************************
+Global error file constant
+***********************************************************************************************************************************/
+#define STATUS_FILE_GLOBAL                                          "global"
+    STRING_STATIC(STATUS_FILE_GLOBAL_STR,                           STATUS_FILE_GLOBAL);
+
+STRING_STATIC(STATUS_FILE_GLOBAL_ERROR_STR,                         STATUS_FILE_GLOBAL STATUS_EXT_ERROR);
+
+/***********************************************************************************************************************************
+Get the correct spool queue based on the archive mode
+***********************************************************************************************************************************/
+static const String *
+archiveAsyncSpoolQueue(ArchiveMode archiveMode)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(ENUM, archiveMode);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN((archiveMode == archiveModeGet ? STORAGE_SPOOL_ARCHIVE_IN_STR : STORAGE_SPOOL_ARCHIVE_OUT_STR));
+}
+
 /***********************************************************************************************************************************
 Check for ok/error status files in the spool in/out directory
 ***********************************************************************************************************************************/
@@ -39,27 +63,27 @@ archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool confe
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        const String *spoolQueue = (archiveMode == archiveModeGet ? STORAGE_SPOOL_ARCHIVE_IN_STR : STORAGE_SPOOL_ARCHIVE_OUT_STR);
-        String *okFile = strNewFmt("%s.ok", strPtr(walSegment));
-        String *errorFile = strNewFmt("%s.error", strPtr(walSegment));
+        const String *errorFile = NULL;
+        bool errorFileExists = false;
 
+        const String *spoolQueue = archiveAsyncSpoolQueue(archiveMode);
+
+        String *okFile = strNewFmt("%s" STATUS_EXT_OK, strPtr(walSegment));
         bool okFileExists = storageExistsNP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(okFile)));
-        bool errorFileExists = storageExistsNP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(errorFile)));
 
-        // If both status files are found then warn, remove the files, and return false so the segment will be retried.  This may be
-        // a bug in the async process but it may also be a failed fsync or other filesystem issue.  In any case, a hard failure here
-        // would mean that archiving is completely stuck so it is better to attempt a retry.
-        if (okFileExists && errorFileExists)
+        // If the ok file does not exist then check to see if a file-specific or global error exists
+        if (!okFileExists)
         {
-            LOG_WARN(
-                "multiple status files found in '%s' for WAL segment '%s' will be removed and the command retried",
-                strPtr(storagePath(storageSpool(), spoolQueue)), strPtr(walSegment));
+            // Check for a file-specific error first
+            errorFile = strNewFmt("%s" STATUS_EXT_ERROR, strPtr(walSegment));
+            errorFileExists = storageExistsNP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(errorFile)));
 
-            storageRemoveNP(storageSpoolWrite(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(okFile)));
-            okFileExists = false;
-
-            storageRemoveNP(storageSpoolWrite(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(errorFile)));
-            errorFileExists = false;
+            // If that doesn't exist then check for a global error
+            if (!errorFileExists)
+            {
+                errorFile = STATUS_FILE_GLOBAL_ERROR_STR;
+                errorFileExists = storageExistsNP(storageSpool(), strNewFmt("%s/%s", strPtr(spoolQueue), strPtr(errorFile)));
+            }
         }
 
         // If either of them exists then check what happened and report back
@@ -89,7 +113,7 @@ archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool confe
                     THROW_FMT(FormatError, "%s message must be > 0", strPtr(statusFile));
 
                 // Get contents
-                code = varIntForce(varNewStr(strNewN(strPtr(content), (size_t)(linefeedPtr - strPtr(content)))));
+                code = varIntForce(VARSTR(strNewN(strPtr(content), (size_t)(linefeedPtr - strPtr(content)))));
                 message = strTrim(strNew(linefeedPtr + 1));
             }
 
@@ -99,7 +123,7 @@ archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool confe
                 // If there is content in the status file it is a warning
                 if (strSize(content) != 0)
                 {
-                    // If error code is not success, then this was a renamed .error file
+                    // If error code is not success, then this was a renamed error file
                     if (code != 0)
                     {
                         message = strNewFmt(
@@ -128,6 +152,65 @@ archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool confe
     FUNCTION_LOG_RETURN(BOOL, result);
 }
 
+/***********************************************************************************************************************************
+Write an error status file
+***********************************************************************************************************************************/
+void
+archiveAsyncStatusErrorWrite(ArchiveMode archiveMode, const String *walSegment, int code, const String *message)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(ENUM, archiveMode);
+        FUNCTION_LOG_PARAM(STRING, walSegment);
+        FUNCTION_LOG_PARAM(INT, code);
+        FUNCTION_LOG_PARAM(STRING, message);
+    FUNCTION_LOG_END();
+
+    ASSERT(code != 0);
+    ASSERT(message != NULL);
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        const String *errorFile = walSegment == NULL ? STATUS_FILE_GLOBAL_STR : walSegment;
+
+        storagePutNP(
+            storageNewWriteNP(
+                storageSpoolWrite(),
+                strNewFmt("%s/%s" STATUS_EXT_ERROR, strPtr(archiveAsyncSpoolQueue(archiveMode)), strPtr(errorFile))),
+            BUFSTR(strNewFmt("%d\n%s", code, strPtr(message))));
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+/***********************************************************************************************************************************
+Write an ok status file
+***********************************************************************************************************************************/
+void
+archiveAsyncStatusOkWrite(ArchiveMode archiveMode, const String *walSegment, const String *warning)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(ENUM, archiveMode);
+        FUNCTION_LOG_PARAM(STRING, walSegment);
+        FUNCTION_LOG_PARAM(STRING, warning);
+    FUNCTION_LOG_END();
+
+    ASSERT(walSegment != NULL);
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        // Write file
+        storagePutNP(
+            storageNewWriteNP(
+                storageSpoolWrite(),
+                strNewFmt("%s/%s" STATUS_EXT_OK, strPtr(archiveAsyncSpoolQueue(archiveMode)), strPtr(walSegment))),
+            warning == NULL ? NULL : BUFSTR(strNewFmt("0\n%s", strPtr(warning))));
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
 /***********************************************************************************************************************************
 Is the segment partial?
 ***********************************************************************************************************************************/
@@ -144,6 +227,43 @@ walIsPartial(const String *walSegment)
     FUNCTION_LOG_RETURN(BOOL, strEndsWithZ(walSegment, WAL_SEGMENT_PARTIAL_EXT));
 }
 
+/***********************************************************************************************************************************
+Generates the location of the wal directory using a relative wal path and the supplied pg path
+***********************************************************************************************************************************/
+String *
+walPath(const String *walFile, const String *pgPath, const String *command)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(STRING, walFile);
+        FUNCTION_LOG_PARAM(STRING, pgPath);
+        FUNCTION_LOG_PARAM(STRING, command);
+    FUNCTION_LOG_END();
+
+    ASSERT(walFile != NULL);
+    ASSERT(command != NULL);
+
+    String *result = NULL;
+
+    if (!strBeginsWithZ(walFile, "/"))
+    {
+        if (pgPath == NULL)
+        {
+            THROW_FMT(
+                OptionRequiredError,
+                "option '" CFGOPT_PG1_PATH "' must be specified when relative wal paths are used\n"
+                    "HINT: Is %%f passed to %s instead of %%p?\n"
+                    "HINT: PostgreSQL may pass relative paths even with %%p depending on the environment.",
+                strPtr(command));
+        }
+
+        result = strNewFmt("%s/%s", strPtr(pgPath), strPtr(walFile));
+    }
+    else
+        result = strDup(walFile);
+
+    FUNCTION_LOG_RETURN(STRING, result);
+}
+
 /***********************************************************************************************************************************
 Is the file a segment or some other file (e.g. .history, .backup, etc)
 ***********************************************************************************************************************************/
@@ -178,12 +298,13 @@ The file name can have several things appended such as a hash, compression exten
 have multiple files that match the segment, though more than one match is not a good thing.
 ***********************************************************************************************************************************/
 String *
-walSegmentFind(const Storage *storage, const String *archiveId, const String *walSegment)
+walSegmentFind(const Storage *storage, const String *archiveId, const String *walSegment, TimeMSec timeout)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE, storage);
         FUNCTION_LOG_PARAM(STRING, archiveId);
         FUNCTION_LOG_PARAM(STRING, walSegment);
+        FUNCTION_LOG_PARAM(TIME_MSEC, timeout);
     FUNCTION_LOG_END();
 
     ASSERT(storage != NULL);
@@ -195,29 +316,36 @@ walSegmentFind(const Storage *storage, const String *archiveId, const String *wa
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        // Get a list of all WAL segments that match
-        StringList *list = storageListP(
-            storage, strNewFmt(STORAGE_REPO_ARCHIVE "/%s/%s", strPtr(archiveId), strPtr(strSubN(walSegment, 0, 16))),
-            .expression = strNewFmt("^%s%s-[0-f]{40}(\\.gz){0,1}$", strPtr(strSubN(walSegment, 0, 24)),
-                walIsPartial(walSegment) ? WAL_SEGMENT_PARTIAL_EXT : ""));
+        Wait *wait = timeout > 0 ? waitNew(timeout) : NULL;
 
-        // If there are results
-        if (list != NULL && strLstSize(list) > 0)
+        do
         {
-            // Error if there is more than one match
-            if (strLstSize(list) > 1)
-            {
-                THROW_FMT(
-                    ArchiveDuplicateError,
-                    "duplicates found in archive for WAL segment %s: %s\n"
-                        "HINT: are multiple primaries archiving to this stanza?",
-                    strPtr(walSegment), strPtr(strLstJoin(strLstSort(list, sortOrderAsc), ", ")));
-            }
+            // Get a list of all WAL segments that match
+            StringList *list = storageListP(
+                storage, strNewFmt(STORAGE_REPO_ARCHIVE "/%s/%s", strPtr(archiveId), strPtr(strSubN(walSegment, 0, 16))),
+                .expression = strNewFmt("^%s%s-[0-f]{40}(\\.gz){0,1}$", strPtr(strSubN(walSegment, 0, 24)),
+                    walIsPartial(walSegment) ? WAL_SEGMENT_PARTIAL_EXT : ""), .nullOnMissing = true);
 
-            // Copy file name of WAL segment found into the calling context
-            memContextSwitch(MEM_CONTEXT_OLD());
-            result = strDup(strLstGet(list, 0));
+            // If there are results
+            if (list != NULL && strLstSize(list) > 0)
+            {
+                // Error if there is more than one match
+                if (strLstSize(list) > 1)
+                {
+                    THROW_FMT(
+                        ArchiveDuplicateError,
+                        "duplicates found in archive for WAL segment %s: %s\n"
+                            "HINT: are multiple primaries archiving to this stanza?",
+                        strPtr(walSegment), strPtr(strLstJoin(strLstSort(list, sortOrderAsc), ", ")));
+                }
+
+                // Copy file name of WAL segment found into the calling context
+                memContextSwitch(MEM_CONTEXT_OLD());
+                result = strDup(strLstGet(list, 0));
+                memContextSwitch(MEM_CONTEXT_TEMP());
+            }
         }
+        while (result == NULL && wait != NULL && waitMore(wait));
     }
     MEM_CONTEXT_TEMP_END();
 
