@@ -20,6 +20,15 @@ typedef enum
 #include "common/type/stringList.h"
 #include "storage/storage.h"
 
+/***********************************************************************************************************************************
+Status file extension constants
+***********************************************************************************************************************************/
+#define STATUS_EXT_ERROR                                            ".error"
+#define STATUS_EXT_ERROR_SIZE                                       (sizeof(STATUS_EXT_ERROR) - 1)
+
+#define STATUS_EXT_OK                                               ".ok"
+#define STATUS_EXT_OK_SIZE                                          (sizeof(STATUS_EXT_OK) - 1)
+
 /***********************************************************************************************************************************
 WAL segment constants
 ***********************************************************************************************************************************/
@@ -50,10 +59,13 @@ WAL segment constants
 Functions
 ***********************************************************************************************************************************/
 bool archiveAsyncStatus(ArchiveMode archiveMode, const String *walSegment, bool confessOnError);
+void archiveAsyncStatusOkWrite(ArchiveMode archiveMode, const String *walSegment, const String *warning);
+void archiveAsyncStatusErrorWrite(ArchiveMode archiveMode, const String *walSegment, int code, const String *message);
 
 bool walIsPartial(const String *walSegment);
 bool walIsSegment(const String *walSegment);
-String *walSegmentFind(const Storage *storage, const String *archiveId, const String *walSegment);
+String *walPath(const String *walFile, const String *pgPath, const String *command);
+String *walSegmentFind(const Storage *storage, const String *archiveId, const String *walSegment, TimeMSec timeout);
 String *walSegmentNext(const String *walSegment, size_t walSegmentSize, unsigned int pgVersion);
 StringList *walSegmentRange(const String *walSegmentBegin, size_t walSegmentSize, unsigned int pgVersion, unsigned int range);
 
