@@ -1,27 +1,28 @@
 /***********************************************************************************************************************************
 Archive Get File
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include "command/archive/get/file.h"
 #include "command/archive/common.h"
-#include "command/control/control.h"
+#include "command/control/common.h"
+#include "common/compress/gzip/common.h"
+#include "common/compress/gzip/decompress.h"
+#include "common/crypto/cipherBlock.h"
 #include "common/debug.h"
 #include "common/io/filter/group.h"
 #include "common/log.h"
-#include "compress/gzip.h"
-#include "compress/gzipDecompress.h"
 #include "config/config.h"
-#include "crypto/cipherBlock.h"
 #include "info/infoArchive.h"
 #include "postgres/interface.h"
 #include "storage/helper.h"
-#include "storage/helper.h"
 
 /***********************************************************************************************************************************
 Check if a WAL file exists in the repository
 ***********************************************************************************************************************************/
-#define FUNCTION_LOG_ARCHIVE_GET_CHECK_RESULT_TYPE                                                                               \
+#define FUNCTION_LOG_ARCHIVE_GET_CHECK_RESULT_TYPE                                                                                 \
     ArchiveGetCheckResult
-#define FUNCTION_LOG_ARCHIVE_GET_CHECK_RESULT_FORMAT(value, buffer, bufferSize)                                                  \
+#define FUNCTION_LOG_ARCHIVE_GET_CHECK_RESULT_FORMAT(value, buffer, bufferSize)                                                    \
     objToLog(&value, "ArchiveGetCheckResult", buffer, bufferSize)
 
 typedef struct ArchiveGetCheckResult
@@ -30,13 +31,13 @@ typedef struct ArchiveGetCheckResult
     String *cipherPass;
 } ArchiveGetCheckResult;
 
-ArchiveGetCheckResult
+static ArchiveGetCheckResult
 archiveGetCheck(const String *archiveFile, CipherType cipherType, const String *cipherPass)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING, archiveFile);
         FUNCTION_LOG_PARAM(ENUM, cipherType);
-        // cipherPass omitted for security
+        FUNCTION_TEST_PARAM(STRING, cipherPass);
     FUNCTION_LOG_END();
 
     ASSERT(archiveFile != NULL);
@@ -49,8 +50,7 @@ archiveGetCheck(const String *archiveFile, CipherType cipherType, const String *
         PgControl controlInfo = pgControlFromFile(cfgOptionStr(cfgOptPgPath));
 
         // Attempt to load the archive info file
-        InfoArchive *info = infoArchiveNew(
-            storageRepo(), STRING_CONST(STORAGE_REPO_ARCHIVE "/" INFO_ARCHIVE_FILE), false, cipherType, cipherPass);
+        InfoArchive *info = infoArchiveNewLoad(storageRepo(), INFO_ARCHIVE_PATH_FILE_STR, cipherType, cipherPass);
 
         // Loop through the pg history in case the WAL we need is not in the most recent archive id
         String *archiveId = NULL;
@@ -68,7 +68,7 @@ archiveGetCheck(const String *archiveFile, CipherType cipherType, const String *
                 // If a WAL segment search among the possible file names
                 if (walIsSegment(archiveFile))
                 {
-                    String *walSegmentFile = walSegmentFind(storageRepo(), archiveId, archiveFile);
+                    String *walSegmentFile = walSegmentFind(storageRepo(), archiveId, archiveFile, 0);
 
                     if (walSegmentFile != NULL)
                     {
@@ -112,13 +112,17 @@ archiveGetCheck(const String *archiveFile, CipherType cipherType, const String *
 Copy a file from the archive to the specified destination
 ***********************************************************************************************************************************/
 int
-archiveGetFile(const String *archiveFile, const String *walDestination, CipherType cipherType, const String *cipherPass)
+archiveGetFile(
+    const Storage *storage, const String *archiveFile, const String *walDestination, bool durable, CipherType cipherType,
+    const String *cipherPass)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(STORAGE, storage);
         FUNCTION_LOG_PARAM(STRING, archiveFile);
         FUNCTION_LOG_PARAM(STRING, walDestination);
+        FUNCTION_LOG_PARAM(BOOL, durable);
         FUNCTION_LOG_PARAM(ENUM, cipherType);
-        // cipherPass omitted for security
+        FUNCTION_TEST_PARAM(STRING, cipherPass);
     FUNCTION_LOG_END();
 
     ASSERT(archiveFile != NULL);
@@ -127,6 +131,9 @@ archiveGetFile(const String *archiveFile, const String *walDestination, CipherTy
     // By default result indicates WAL segment not found
     int result = 1;
 
+    // Is the file compressible during the copy?
+    bool compressible = true;
+
     // Test for stop file
     lockStopTest();
 
@@ -137,35 +144,32 @@ archiveGetFile(const String *archiveFile, const String *walDestination, CipherTy
 
         if (archiveGetCheckResult.archiveFileActual != NULL)
         {
-            StorageFileWrite *destination = storageNewWriteP(
-                storageLocalWrite(), walDestination, .noCreatePath = true,  .noSyncFile = true, .noSyncPath = true,
-                .noAtomic = true);
-
-            // Add filters
-            IoFilterGroup *filterGroup = ioFilterGroupNew();
+            StorageWrite *destination = storageNewWriteP(
+                storage, walDestination, .noCreatePath = true, .noSyncFile = !durable, .noSyncPath = !durable,
+                .noAtomic = !durable);
 
             // If there is a cipher then add the decrypt filter
             if (cipherType != cipherTypeNone)
             {
                 ioFilterGroupAdd(
-                    filterGroup,
-                    cipherBlockFilter(
-                        cipherBlockNew(cipherModeDecrypt, cipherType, bufNewStr(archiveGetCheckResult.cipherPass), NULL)));
+                    ioWriteFilterGroup(storageWriteIo(destination)), cipherBlockNew(cipherModeDecrypt, cipherType,
+                        BUFSTR(archiveGetCheckResult.cipherPass), NULL));
+                compressible = false;
             }
 
-            // If file is gzipped then add the decompression filter
+            // If file is compressed then add the decompression filter
             if (strEndsWithZ(archiveGetCheckResult.archiveFileActual, "." GZIP_EXT))
             {
-                ioFilterGroupAdd(filterGroup, gzipDecompressFilter(gzipDecompressNew(false)));
+                ioFilterGroupAdd(ioWriteFilterGroup(storageWriteIo(destination)), gzipDecompressNew(false));
+                compressible = false;
             }
 
-            ioWriteFilterGroupSet(storageFileWriteIo(destination), filterGroup);
-
             // Copy the file
             storageCopyNP(
-                storageNewReadNP(
-                    storageRepo(), strNewFmt("%s/%s", STORAGE_REPO_ARCHIVE, strPtr(archiveGetCheckResult.archiveFileActual))),
-                    destination);
+                storageNewReadP(
+                    storageRepo(), strNewFmt("%s/%s", STORAGE_REPO_ARCHIVE, strPtr(archiveGetCheckResult.archiveFileActual)),
+                    .compressible = compressible),
+                destination);
 
             // The WAL file was found
             result = 0;
