@@ -1,11 +1,17 @@
 /***********************************************************************************************************************************
 Archive Get Command
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
 
 #include "command/archive/common.h"
 #include "command/archive/get/file.h"
+#include "command/archive/get/protocol.h"
 #include "command/command.h"
 #include "common/debug.h"
 #include "common/fork.h"
@@ -14,22 +20,23 @@ Archive Get Command
 #include "common/regExp.h"
 #include "common/wait.h"
 #include "config/config.h"
-#include "config/load.h"
+#include "config/exec.h"
 #include "perl/exec.h"
 #include "postgres/interface.h"
-#include "storage/helper.h"
+#include "protocol/helper.h"
+#include "protocol/parallel.h"
 #include "storage/helper.h"
 
 /***********************************************************************************************************************************
 Clean the queue and prepare a list of WAL segments that the async process should get
 ***********************************************************************************************************************************/
 static StringList *
-queueNeed(const String *walSegment, bool found, size_t queueSize, size_t walSegmentSize, unsigned int pgVersion)
+queueNeed(const String *walSegment, bool found, uint64_t queueSize, size_t walSegmentSize, unsigned int pgVersion)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING, walSegment);
         FUNCTION_LOG_PARAM(BOOL, found);
-        FUNCTION_LOG_PARAM(SIZE, queueSize);
+        FUNCTION_LOG_PARAM(UINT64, queueSize);
         FUNCTION_LOG_PARAM(SIZE, walSegmentSize);
         FUNCTION_LOG_PARAM(UINT, pgVersion);
     FUNCTION_LOG_END();
@@ -60,7 +67,7 @@ queueNeed(const String *walSegment, bool found, size_t queueSize, size_t walSegm
         StringList *actualQueue = strLstSort(
             storageListP(storageSpool(), STORAGE_SPOOL_ARCHIVE_IN_STR, .errorOnMissing = true), sortOrderAsc);
 
-        // Only preserve files that match the ideal queue. '.error'/'.ok' files are deleted so the async process can try again.
+        // Only preserve files that match the ideal queue. error/ok files are deleted so the async process can try again.
         RegExp *regExpPreserve = regExpNew(strNewFmt("^(%s)$", strPtr(strLstJoin(idealQueue, "|"))));
 
         // Build a list of WAL segments that are being kept so we can later make a list of what is needed
@@ -93,7 +100,7 @@ queueNeed(const String *walSegment, bool found, size_t queueSize, size_t walSegm
 }
 
 /***********************************************************************************************************************************
-Push a WAL segment to the repository
+Get an archive file from the repository (WAL segment, history file, etc.)
 ***********************************************************************************************************************************/
 int
 cmdArchiveGet(void)
@@ -122,16 +129,11 @@ cmdArchiveGet(void)
         // Get the segment name
         String *walSegment = strBase(strLstGet(commandParam, 0));
 
-        // Destination is wherever we were told to move the WAL segment.  In some cases the path that PostgreSQL passes will not be
-        // absolute so prefix pg-path.
-        const String *walDestination = strLstGet(commandParam, 1);
-
-        if (!strBeginsWithZ(walDestination, "/"))
-            walDestination = strNewFmt("%s/%s", strPtr(cfgOptionStr(cfgOptPgPath)), strPtr(walDestination));
+        // Destination is wherever we were told to move the WAL segment
+        const String *walDestination =
+            walPath(strLstGet(commandParam, 1), cfgOptionStr(cfgOptPgPath), STR(cfgCommandName(cfgCommand())));
 
         // Async get can only be performed on WAL segments, history or other files must use synchronous mode
-        bool asyncServer = false;
-
         if (cfgOptionBool(cfgOptArchiveAsync) && walIsSegment(walSegment))
         {
             bool found = false;                                         // Has the WAL segment been found yet?
@@ -144,13 +146,13 @@ cmdArchiveGet(void)
 
             do
             {
-                // Check for errors or missing files.  For archive-get '.ok' indicates that the process succeeded but there is no
-                // WAL file to download.
+                // Check for errors or missing files.  For archive-get ok indicates that the process succeeded but there is no WAL
+                // file to download.
                 if (archiveAsyncStatus(archiveModeGet, walSegment, confessOnError))
                 {
                     storageRemoveP(
                         storageSpoolWrite(),
-                        strNewFmt(STORAGE_SPOOL_ARCHIVE_IN "/%s.ok", strPtr(walSegment)), .errorOnMissing = true);
+                        strNewFmt(STORAGE_SPOOL_ARCHIVE_IN "/%s" STATUS_EXT_OK, strPtr(walSegment)), .errorOnMissing = true);
                     break;
                 }
 
@@ -161,7 +163,7 @@ cmdArchiveGet(void)
                 if (found)
                 {
                     // Source is the WAL segment in the spool queue
-                    StorageFileRead *source = storageNewReadNP(
+                    StorageRead *source = storageNewReadNP(
                         storageSpool(), strNewFmt(STORAGE_SPOOL_ARCHIVE_IN "/%s", strPtr(walSegment)));
 
                     // A move will be attempted but if the spool queue and the WAL path are on different file systems then a copy
@@ -171,7 +173,7 @@ cmdArchiveGet(void)
                     // is safe because if the system crashes Postgres will not try to reuse a restored WAL segment but will instead
                     // request it again using the restore_command. In the case of a move this hardly matters since path syncs are
                     // cheap but if a copy is required we could save a lot of writes.
-                    StorageFileWrite *destination = storageNewWriteP(
+                    StorageWrite *destination = storageNewWriteP(
                         storageLocalWrite(), walDestination, .noCreatePath = true, .noSyncFile = true, .noSyncPath = true,
                         .noAtomic = true);
 
@@ -183,16 +185,15 @@ cmdArchiveGet(void)
 
                     // Get a list of WAL segments left in the queue
                     StringList *queue = storageListP(
-                        storageSpool(), STORAGE_SPOOL_ARCHIVE_IN_STR, .expression = WAL_SEGMENT_REGEXP_STR);
+                        storageSpool(), STORAGE_SPOOL_ARCHIVE_IN_STR, .expression = WAL_SEGMENT_REGEXP_STR, .errorOnMissing = true);
 
                     if (strLstSize(queue) > 0)
                     {
                         // Get size of the WAL segment
-                        size_t walSegmentSize = storageInfoNP(storageLocal(), walDestination).size;
+                        uint64_t walSegmentSize = storageInfoNP(storageLocal(), walDestination).size;
 
                         // Use WAL segment size to estimate queue size and determine if the async process should be launched
-                        queueFull =
-                            strLstSize(queue) * walSegmentSize > (size_t)cfgOptionInt64(cfgOptArchiveGetQueueMax) / 2;
+                        queueFull = strLstSize(queue) * walSegmentSize > cfgOptionUInt64(cfgOptArchiveGetQueueMax) / 2;
                     }
                 }
 
@@ -202,57 +203,52 @@ cmdArchiveGet(void)
                 if (!forked && (!found || !queueFull)  &&
                     lockAcquire(cfgOptionStr(cfgOptLockPath), cfgOptionStr(cfgOptStanza), cfgLockType(), 0, false))
                 {
-                    // Release the lock and mark the async process as forked
+                    // Get control info
+                    PgControl pgControl = pgControlFromFile(cfgOptionStr(cfgOptPgPath));
+
+                    // Create the queue
+                    storagePathCreateNP(storageSpoolWrite(), STORAGE_SPOOL_ARCHIVE_IN_STR);
+
+                    // The async process should not output on the console at all
+                    KeyValue *optionReplace = kvNew();
+
+                    kvPut(optionReplace, VARSTR(CFGOPT_LOG_LEVEL_CONSOLE_STR), VARSTRDEF("off"));
+                    kvPut(optionReplace, VARSTR(CFGOPT_LOG_LEVEL_STDERR_STR), VARSTRDEF("off"));
+
+                    // Generate command options
+                    StringList *commandExec = cfgExecParam(cfgCmdArchiveGetAsync, optionReplace);
+                    strLstInsert(commandExec, 0, cfgExe());
+
+                    // Clean the current queue using the list of WAL that we ideally want in the queue.  queueNeed()
+                    // will return the list of WAL needed to fill the queue and this will be passed to the async process.
+                    const StringList *queue = queueNeed(
+                        walSegment, found, cfgOptionUInt64(cfgOptArchiveGetQueueMax), pgControl.walSegmentSize,
+                        pgControl.version);
+
+                    for (unsigned int queueIdx = 0; queueIdx < strLstSize(queue); queueIdx++)
+                        strLstAdd(commandExec, strLstGet(queue, queueIdx));
+
+                    // Release the lock so the child process can acquire it
                     lockRelease(true);
-                    forked = true;
 
                     // Fork off the async process
-                    if (fork() == 0)
+                    if (forkSafe() == 0)
                     {
-                        // In the async server
-                        asyncServer = true;
-                        result = 0;
+                        // Disable logging and close log file
+                        logClose();
 
-                        // Only run async if the lock can be reacquired.  We just held it so this should not be an issue unless
-                        // another process sneaks in.  In general there should be only one archive-get process running but in
-                        // theory there could be more than one.
-                        if (lockAcquire(                                // {uncoverable - almost impossible to make this lock fail}
-                                cfgOptionStr(cfgOptLockPath), cfgOptionStr(cfgOptStanza), cfgLockType(), 0, false))
-                        {
-                            // Get control info
-                            PgControl pgControl = pgControlFromFile(cfgOptionStr(cfgOptPgPath));
+                        // Detach from parent process
+                        forkDetach();
 
-                            // Create the queue
-                            storagePathCreateNP(storageSpoolWrite(), STORAGE_SPOOL_ARCHIVE_IN_STR);
-
-                            // Clean the current queue using the list of WAL that we ideally want in the queue.  queueNeed()
-                            // will return the list of WAL needed to fill the queue and this will be passed to the async process.
-                            cfgCommandParamSet(
-                                queueNeed(
-                                    walSegment, found, (size_t)cfgOptionInt64(cfgOptArchiveGetQueueMax), pgControl.walSegmentSize,
-                                    pgControl.version));
-
-                            // The async process should not output on the console at all
-                            cfgOptionSet(cfgOptLogLevelConsole, cfgSourceParam, varNewStrZ("off"));
-                            cfgOptionSet(cfgOptLogLevelStderr, cfgSourceParam, varNewStrZ("off"));
-                            cfgLoadLogSetting();
-
-                            // Open the log file
-                            cfgLoadLogFile(
-                                strNewFmt("%s/%s-%s-async.log", strPtr(cfgOptionStr(cfgOptLogPath)),
-                                strPtr(cfgOptionStr(cfgOptStanza)), cfgCommandName(cfgCommand())));
-
-                            // Log command info since we are starting a new log
-                            cmdBegin(true);
-
-                            // Detach from parent process
-                            forkDetach();
-
-                            perlExec();
-                        }
-
-                        break;                                          // {uncovered - async calls always return errors for now}
+                        // Execute the binary.  This statement will not return if it is successful.
+                        THROW_ON_SYS_ERROR(
+                            execvp(strPtr(cfgExe()), (char ** const)strLstPtr(commandExec)) == -1,
+                            ExecuteError, "unable to execute '" CFGCMD_ARCHIVE_GET_ASYNC "'");
                     }
+
+                    // Mark the async process as forked so it doesn't get forked again.  A single run of the async process should be
+                    // enough to do the job, running it again won't help anything.
+                    forked = true;
                 }
 
                 // Exit loop if WAL was found
@@ -267,30 +263,118 @@ cmdArchiveGet(void)
         // Else perform synchronous get
         else
         {
-            // Disable async if it was enabled
-            cfgOptionSet(cfgOptArchiveAsync, cfgOptionSource(cfgOptArchiveAsync), varNewBool(false));
+            // Get the repo storage in case it is remote and encryption settings need to be pulled down
+            storageRepo();
 
-            // If repo server is not remote then this can be done entirely in C
-            if (!cfgOptionTest(cfgOptRepoHost))                                 // {uncovered - Perl code is covered in unit tests}
-            {
-                result = archiveGetFile(
-                    walSegment, walDestination, cipherType(cfgOptionStr(cfgOptRepoCipherType)), cfgOptionStr(cfgOptRepoCipherPass));
-            }
-            // Else do it in Perl
-            else
-                result = perlExec();                                            // {+uncovered}
+            // Get the archive file
+            result = archiveGetFile(
+                storageLocalWrite(), walSegment, walDestination, false, cipherType(cfgOptionStr(cfgOptRepoCipherType)),
+                cfgOptionStr(cfgOptRepoCipherPass));
         }
 
         // Log whether or not the file was found
-        if (!asyncServer)                                               // {uncovered - async calls always return errors for now}
-        {
-            if (result == 0)
-                LOG_INFO("found %s in the archive", strPtr(walSegment));
-            else
-                LOG_INFO("unable to find %s in the archive", strPtr(walSegment));
-        }
+        if (result == 0)
+            LOG_INFO("found %s in the archive", strPtr(walSegment));
+        else
+            LOG_INFO("unable to find %s in the archive", strPtr(walSegment));
     }
     MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN(INT, result);
 }
+
+/***********************************************************************************************************************************
+Async version of archive get that runs in parallel for performance
+***********************************************************************************************************************************/
+void
+cmdArchiveGetAsync(void)
+{
+    FUNCTION_LOG_VOID(logLevelDebug);
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        TRY_BEGIN()
+        {
+            // Check the parameters
+            const StringList *walSegmentList = cfgCommandParam();
+
+            if (strLstSize(walSegmentList) < 1)
+                THROW(ParamInvalidError, "at least one wal segment is required");
+
+            LOG_INFO(
+                "get %u WAL file(s) from archive: %s%s", strLstSize(walSegmentList), strPtr(strLstGet(walSegmentList, 0)),
+                strLstSize(walSegmentList) == 1 ?
+                    "" : strPtr(strNewFmt("...%s", strPtr(strLstGet(walSegmentList, strLstSize(walSegmentList) - 1)))));
+
+            // Create the parallel executor
+            ProtocolParallel *parallelExec = protocolParallelNew(
+                (TimeMSec)(cfgOptionDbl(cfgOptProtocolTimeout) * MSEC_PER_SEC) / 2);
+
+            for (unsigned int processIdx = 1; processIdx <= cfgOptionUInt(cfgOptProcessMax); processIdx++)
+                protocolParallelClientAdd(parallelExec, protocolLocalGet(protocolStorageTypeRepo, processIdx));
+
+            // Queue jobs in executor
+            for (unsigned int walSegmentIdx = 0; walSegmentIdx < strLstSize(walSegmentList); walSegmentIdx++)
+            {
+                const String *walSegment = strLstGet(walSegmentList, walSegmentIdx);
+
+                ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_ARCHIVE_GET_STR);
+                protocolCommandParamAdd(command, VARSTR(walSegment));
+
+                protocolParallelJobAdd(parallelExec, protocolParallelJobNew(VARSTR(walSegment), command));
+            }
+
+            // Process jobs
+            do
+            {
+                unsigned int completed = protocolParallelProcess(parallelExec);
+
+                for (unsigned int jobIdx = 0; jobIdx < completed; jobIdx++)
+                {
+                    // Get the job and job key
+                    ProtocolParallelJob *job = protocolParallelResult(parallelExec);
+                    unsigned int processId = protocolParallelJobProcessId(job);
+                    const String *walSegment = varStr(protocolParallelJobKey(job));
+
+                    // The job was successful
+                    if (protocolParallelJobErrorCode(job) == 0)
+                    {
+                        // Get the archive file
+                        if (varIntForce(protocolParallelJobResult(job)) == 0)
+                        {
+                            LOG_DETAIL_PID(processId, "found %s in the archive", strPtr(walSegment));
+                        }
+                        // If it does not exist write an ok file to indicate that it was checked
+                        else
+                        {
+                            LOG_DETAIL_PID(processId, "unable to find %s in the archive", strPtr(walSegment));
+                            archiveAsyncStatusOkWrite(archiveModeGet, walSegment, NULL);
+                        }
+                    }
+                    // Else the job errored
+                    else
+                    {
+                        LOG_WARN_PID(
+                            processId,
+                            "could not get %s from the archive (will be retried): [%d] %s", strPtr(walSegment),
+                            protocolParallelJobErrorCode(job), strPtr(protocolParallelJobErrorMessage(job)));
+
+                        archiveAsyncStatusErrorWrite(
+                            archiveModeGet, walSegment, protocolParallelJobErrorCode(job), protocolParallelJobErrorMessage(job));
+                    }
+                }
+            }
+            while (!protocolParallelDone(parallelExec));
+        }
+        // On any global error write a single error file to cover all unprocessed files
+        CATCH_ANY()
+        {
+            archiveAsyncStatusErrorWrite(archiveModeGet, NULL, errorCode(), STR(errorMessage()));
+            RETHROW();
+        }
+        TRY_END();
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN_VOID();
+}
