@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Help Command
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -144,7 +146,7 @@ helpRender(void)
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Message for more help when it is available
-        String *more = NULL;
+        const String *more = NULL;
 
         // Display general help
         if (cfgCommand() == cfgCmdHelp || cfgCommand() == cfgCmdNone)
@@ -163,7 +165,7 @@ helpRender(void)
 
             for (ConfigCommand commandId = 0; commandId < CFG_COMMAND_TOTAL; commandId++)
             {
-                if (commandId == cfgCmdNone)
+                if (commandId == cfgCmdNone || cfgCommandInternal(commandId))
                     continue;
 
                 if (strlen(cfgCommandName(commandId)) > commandSizeMax)
@@ -173,18 +175,16 @@ helpRender(void)
             // Output help for each command
             for (ConfigCommand commandId = 0; commandId < CFG_COMMAND_TOTAL; commandId++)
             {
-                if (commandId == cfgCmdNone)
+                if (commandId == cfgCmdNone || cfgCommandInternal(commandId))
                     continue;
 
-                const char *helpSummary = cfgDefCommandHelpSummary(cfgCommandDefIdFromId(commandId));
-
-                if (helpSummary != NULL)
-                {
-                    strCatFmt(
-                        result, "    %s%*s%s\n", cfgCommandName(commandId),
-                        (int)(commandSizeMax - strlen(cfgCommandName(commandId)) + 2), "",
-                        strPtr(helpRenderText(strNew(helpSummary), commandSizeMax + 6, false, CONSOLE_WIDTH)));
-                }
+                strCatFmt(
+                    result, "    %s%*s%s\n", cfgCommandName(commandId),
+                    (int)(commandSizeMax - strlen(cfgCommandName(commandId)) + 2), "",
+                    strPtr(
+                        helpRenderText(
+                            STR(cfgDefCommandHelpSummary(cfgCommandDefIdFromId(commandId))), commandSizeMax + 6, false,
+                            CONSOLE_WIDTH)));
             }
 
             // Construct message for more help
@@ -210,8 +210,8 @@ helpRender(void)
                     "%s\n"
                     "\n"
                     "%s\n",
-                    strPtr(helpRenderText(strNew(cfgDefCommandHelpSummary(commandDefId)), 0, true, CONSOLE_WIDTH)),
-                    strPtr(helpRenderText(strNew(cfgDefCommandHelpDescription(commandDefId)), 0, true, CONSOLE_WIDTH)));
+                    strPtr(helpRenderText(STR(cfgDefCommandHelpSummary(commandDefId)), 0, true, CONSOLE_WIDTH)),
+                    strPtr(helpRenderText(STR(cfgDefCommandHelpDescription(commandDefId)), 0, true, CONSOLE_WIDTH)));
 
                 // Construct key/value of sections and options
                 KeyValue *optionKv = kvNew();
@@ -221,7 +221,7 @@ helpRender(void)
                 {
                     if (cfgDefOptionValid(commandDefId, optionDefId) && !cfgDefOptionInternal(commandDefId, optionDefId))
                     {
-                        String *section = NULL;
+                        const String *section = NULL;
 
                         if (cfgDefOptionHelpSection(optionDefId) != NULL)
                             section = strNew(cfgDefOptionHelpSection(optionDefId));
@@ -233,7 +233,7 @@ helpRender(void)
                             section = strNew("command");
                         }
 
-                        kvAdd(optionKv, varNewStr(section), varNewInt((int)optionDefId));
+                        kvAdd(optionKv, VARSTR(section), VARINT((int)optionDefId));
 
                         if (strlen(cfgDefOptionName(optionDefId)) > optionSizeMax)
                             optionSizeMax = strlen(cfgDefOptionName(optionDefId));
@@ -250,7 +250,7 @@ helpRender(void)
                     strCatFmt(result, "\n%s Options:\n\n", strPtr(strFirstUpper(strDup(section))));
 
                     // Output options
-                    VariantList *optionList = kvGetList(optionKv, varNewStr(section));
+                    VariantList *optionList = kvGetList(optionKv, VARSTR(section));
 
                     for (unsigned int optionIdx = 0; optionIdx < varLstSize(optionList); optionIdx++)
                     {
@@ -274,7 +274,7 @@ helpRender(void)
                             strCat(summary, " [");
 
                             if (value != NULL)
-                                strCatFmt(summary, "current=%s", strPtr(value));
+                                strCatFmt(summary, "current=%s", cfgDefOptionSecure(optionDefId) ? "<redacted>" : strPtr(value));
 
                             if (defaultValue != NULL)
                             {
@@ -329,8 +329,8 @@ helpRender(void)
                     "\n"
                     "%s\n",
                     optionName,
-                    strPtr(helpRenderText(strNew(cfgDefOptionHelpSummary(commandDefId, optionDefId)), 0, true, CONSOLE_WIDTH)),
-                    strPtr(helpRenderText(strNew(cfgDefOptionHelpDescription(commandDefId, optionDefId)), 0, true, CONSOLE_WIDTH)));
+                    strPtr(helpRenderText(STR(cfgDefOptionHelpSummary(commandDefId, optionDefId)), 0, true, CONSOLE_WIDTH)),
+                    strPtr(helpRenderText(STR(cfgDefOptionHelpDescription(commandDefId, optionDefId)), 0, true, CONSOLE_WIDTH)));
 
                 // Ouput current and default values if they exist
                 const String *defaultValue = helpRenderValue(cfgOptionDefault(optionId));
@@ -344,32 +344,15 @@ helpRender(void)
                     strCat(result, "\n");
 
                     if (value != NULL)
-                        strCatFmt(result, "current: %s\n", strPtr(value));
+                        strCatFmt(result, "current: %s\n", cfgDefOptionSecure(optionDefId) ? "<redacted>" : strPtr(value));
 
                     if (defaultValue != NULL)
                         strCatFmt(result, "default: %s\n", strPtr(defaultValue));
                 }
 
-                // Output alternate names (call them deprecated so the user will know not to use them)
+                // Output alternate name (call it deprecated so the user will know not to use it)
                 if (cfgDefOptionHelpNameAlt(optionDefId))
-                {
-                    strCat(result, "\ndeprecated name");
-
-                    if (cfgDefOptionHelpNameAltValueTotal(optionDefId) > 1) // {uncovered - no option has more than one alt name}
-                        strCat(result, "s");                                // {+uncovered}
-
-                    strCat(result, ": ");
-
-                    for (unsigned int nameAltIdx = 0; nameAltIdx < cfgDefOptionHelpNameAltValueTotal(optionDefId); nameAltIdx++)
-                    {
-                        if (nameAltIdx != 0)                                // {uncovered - no option has more than one alt name}
-                            strCat(result, ", ");                           // {+uncovered}
-
-                        strCat(result, cfgDefOptionHelpNameAltValue(optionDefId, nameAltIdx));
-                    }
-
-                    strCat(result, "\n");
-                }
+                    strCatFmt(result, "\ndeprecated name: %s\n", cfgDefOptionHelpNameAltValue(optionDefId, 0));
             }
         }
 
