@@ -25,14 +25,6 @@ Extern variables that are needed for unit testing
         static
 #endif
 
-/***********************************************************************************************************************************
-Convert a macro to a string
-***********************************************************************************************************************************/
-#define MACRO_TO_STR_INNER(macro)                                                                                                  \
-    #macro
-#define MACRO_TO_STR(macro)                                                                                                        \
-    MACRO_TO_STR_INNER(macro)
-
 /***********************************************************************************************************************************
 Base function debugging macros
 
@@ -42,36 +34,29 @@ level is set to debug or trace.
 #define FUNCTION_LOG_LEVEL()                                                                                                       \
     FUNCTION_LOG_logLevel
 
-#ifdef DEBUG
+#ifdef DEBUG_TEST_TRACE
     #define FUNCTION_LOG_BEGIN_BASE(logLevel)                                                                                      \
         LogLevel FUNCTION_LOG_LEVEL() = STACK_TRACE_PUSH(logLevel);                                                                \
                                                                                                                                    \
         {                                                                                                                          \
-            stackTraceParamLog();
+            stackTraceParamLog();                                                                                                  \
+            stackTraceTestStop();
 
     #define FUNCTION_LOG_END_BASE()                                                                                                \
-            LOG_WILL(FUNCTION_LOG_LEVEL(), 0, "(%s)", stackTraceParam());                                                          \
+            stackTraceTestStart();                                                                                                 \
+            LOG(FUNCTION_LOG_LEVEL(), 0, "(%s)", stackTraceParam());                                                               \
         }
-
-    #define FUNCTION_LOG_PARAM_BASE_BEGIN()                                                                                        \
-        stackTraceTestStop()                                                                                                       \
-
-    #define FUNCTION_LOG_PARAM_BASE_END()                                                                                          \
-        stackTraceTestStart()
 #else
     #define FUNCTION_LOG_BEGIN_BASE(logLevel)                                                                                      \
         LogLevel FUNCTION_LOG_LEVEL() = STACK_TRACE_PUSH(logLevel);                                                                \
                                                                                                                                    \
-        if (logWill(FUNCTION_LOG_LEVEL()))                                                                                         \
+        if (logAny(FUNCTION_LOG_LEVEL()))                                                                                          \
         {                                                                                                                          \
-            stackTraceParamLog()
+            stackTraceParamLog();
 
     #define FUNCTION_LOG_END_BASE()                                                                                                \
             LOG(FUNCTION_LOG_LEVEL(), 0, "(%s)", stackTraceParam());                                                               \
         }
-
-    #define FUNCTION_LOG_PARAM_BASE_BEGIN()
-    #define FUNCTION_LOG_PARAM_BASE_END()
 #endif
 
 /***********************************************************************************************************************************
@@ -90,15 +75,11 @@ FUNCTION_LOG_VOID() is provided as a shortcut for functions that have no paramet
     FUNCTION_LOG_END_BASE()
 
 #define FUNCTION_LOG_PARAM(typeMacroPrefix, param)                                                                                 \
-    FUNCTION_LOG_PARAM_BASE_BEGIN();                                                                                               \
-    stackTraceParamAdd(FUNCTION_LOG_##typeMacroPrefix##_FORMAT(param, stackTraceParamBuffer(#param), STACK_TRACE_PARAM_MAX));      \
-    FUNCTION_LOG_PARAM_BASE_END()
+    stackTraceParamAdd(FUNCTION_LOG_##typeMacroPrefix##_FORMAT(param, stackTraceParamBuffer(#param), STACK_TRACE_PARAM_MAX))
 
 #define FUNCTION_LOG_PARAM_P(typeMacroPrefix, param)                                                                               \
     do                                                                                                                             \
     {                                                                                                                              \
-        FUNCTION_LOG_PARAM_BASE_BEGIN();                                                                                           \
-                                                                                                                                   \
         char *buffer = stackTraceParamBuffer(#param);                                                                              \
                                                                                                                                    \
         if (param == NULL)                                                                                                         \
@@ -108,16 +89,12 @@ FUNCTION_LOG_VOID() is provided as a shortcut for functions that have no paramet
             buffer[0] = '*';                                                                                                       \
             stackTraceParamAdd(FUNCTION_LOG_##typeMacroPrefix##_FORMAT(*param, buffer + 1, STACK_TRACE_PARAM_MAX - 1) + 1);        \
         }                                                                                                                          \
-                                                                                                                                   \
-        FUNCTION_LOG_PARAM_BASE_END();                                                                                             \
     }                                                                                                                              \
     while(0)
 
 #define FUNCTION_LOG_PARAM_PP(typeMacroPrefix, param)                                                                              \
     do                                                                                                                             \
     {                                                                                                                              \
-        FUNCTION_LOG_PARAM_BASE_BEGIN();                                                                                           \
-                                                                                                                                   \
         char *buffer = stackTraceParamBuffer(#param);                                                                              \
                                                                                                                                    \
         if (param == NULL)                                                                                                         \
@@ -130,8 +107,6 @@ FUNCTION_LOG_VOID() is provided as a shortcut for functions that have no paramet
             buffer[1] = '*';                                                                                                       \
             stackTraceParamAdd(FUNCTION_LOG_##typeMacroPrefix##_FORMAT(**param, buffer + 2, STACK_TRACE_PARAM_MAX - 2) + 2);       \
         }                                                                                                                          \
-                                                                                                                                   \
-        FUNCTION_LOG_PARAM_BASE_END();                                                                                             \
     }                                                                                                                              \
     while(0)
 
@@ -206,6 +181,11 @@ size_t typeToLog(const char *typeName, char *buffer, size_t bufferSize);
 #define FUNCTION_LOG_SIZE_FORMAT(value, buffer, bufferSize)                                                                        \
     cvtSizeToZ(value, buffer, bufferSize)
 
+#define FUNCTION_LOG_SSIZE_TYPE                                                                                                    \
+    ssize_t
+#define FUNCTION_LOG_SSIZE_FORMAT(value, buffer, bufferSize)                                                                       \
+    cvtSSizeToZ(value, buffer, bufferSize)
+
 #define FUNCTION_LOG_UINT_TYPE                                                                                                     \
     unsigned int
 #define FUNCTION_LOG_UINT_FORMAT(value, buffer, bufferSize)                                                                        \
@@ -244,9 +224,9 @@ Macros to return function results (or void)
     {                                                                                                                              \
         typePre FUNCTION_LOG_##typeMacroPrefix##_TYPE typePost FUNCTION_LOG_RETURN_result = result;                                \
                                                                                                                                    \
-        STACK_TRACE_POP();                                                                                                         \
+        STACK_TRACE_POP(false);                                                                                                    \
                                                                                                                                    \
-        if (logWill(FUNCTION_LOG_LEVEL()))                                                                                         \
+        IF_LOG_ANY(FUNCTION_LOG_LEVEL())                                                                                           \
         {                                                                                                                          \
             char buffer[STACK_TRACE_PARAM_MAX];                                                                                    \
                                                                                                                                    \
@@ -279,9 +259,9 @@ Macros to return function results (or void)
 #define FUNCTION_LOG_RETURN_VOID()                                                                                                 \
     do                                                                                                                             \
     {                                                                                                                              \
-        STACK_TRACE_POP();                                                                                                         \
+        STACK_TRACE_POP(false);                                                                                                    \
                                                                                                                                    \
-        LOG_WILL(FUNCTION_LOG_LEVEL(), 0, "=> void");                                                                              \
+        LOG(FUNCTION_LOG_LEVEL(), 0, "=> void");                                                                                   \
     }                                                                                                                              \
     while(0)
 
@@ -289,14 +269,15 @@ Macros to return function results (or void)
 Function Test Macros
 
 In debug builds these macros will update the stack trace with function names and parameters but not log.  In production builds all
-test macros are compiled out.
+test macros are compiled out (except for return statements).
 ***********************************************************************************************************************************/
-#ifdef DEBUG
+#ifdef DEBUG_TEST_TRACE
     #define FUNCTION_TEST_BEGIN()                                                                                                  \
         if (stackTraceTest())                                                                                                      \
         {                                                                                                                          \
             STACK_TRACE_PUSH(logLevelDebug);                                                                                       \
-            stackTraceParamLog()
+            stackTraceParamLog();                                                                                                  \
+            stackTraceTestStop()
 
     #define FUNCTION_TEST_PARAM(typeMacroPrefix, param)                                                                            \
         FUNCTION_LOG_PARAM(typeMacroPrefix, param)
@@ -308,6 +289,7 @@ test macros are compiled out.
         FUNCTION_LOG_PARAM_PP(typeName, param)
 
     #define FUNCTION_TEST_END()                                                                                                    \
+            stackTraceTestStart();                                                                                                 \
         }
 
     #define FUNCTION_TEST_VOID()                                                                                                   \
@@ -317,20 +299,13 @@ test macros are compiled out.
     #define FUNCTION_TEST_RETURN(result)                                                                                           \
         do                                                                                                                         \
         {                                                                                                                          \
-            if (stackTraceTest())                                                                                                  \
-                STACK_TRACE_POP();                                                                                                 \
-                                                                                                                                   \
+            STACK_TRACE_POP(true);                                                                                                 \
             return result;                                                                                                         \
         }                                                                                                                          \
         while(0);
 
     #define FUNCTION_TEST_RETURN_VOID()                                                                                            \
-        do                                                                                                                         \
-        {                                                                                                                          \
-            if (stackTraceTest())                                                                                                  \
-                STACK_TRACE_POP();                                                                                                 \
-        }                                                                                                                          \
-        while(0);
+        STACK_TRACE_POP(true);
 #else
     #define FUNCTION_TEST_BEGIN()
     #define FUNCTION_TEST_PARAM(typeMacroPrefix, param)
