@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Error Handler
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,7 +31,7 @@ struct ErrorType
 /***********************************************************************************************************************************
 Maximum allowed number of nested try blocks
 ***********************************************************************************************************************************/
-#define ERROR_TRY_MAX                                             32
+#define ERROR_TRY_MAX                                             64
 
 /***********************************************************************************************************************************
 States for each try
@@ -423,18 +425,50 @@ Throw a system error
 ***********************************************************************************************************************************/
 void
 errorInternalThrowSys(
-    int errNo, const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *message)
+#ifdef DEBUG_COVERAGE
+    bool error,
+#else
+    int errNo,
+#endif
+    const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *message)
 {
+#ifdef DEBUG_COVERAGE
+    if (error)
+    {
+        int errNo = errno;
+#endif
+
     // Format message with system message appended
-    snprintf(messageBufferTemp, ERROR_MESSAGE_BUFFER_SIZE - 1, "%s: [%d] %s", message, errNo, strerror(errNo));
+    if (errNo == 0)
+    {
+        strncpy(messageBufferTemp, message, ERROR_MESSAGE_BUFFER_SIZE - 1);
+        messageBufferTemp[sizeof(messageBuffer) - 1] = 0;
+    }
+    else
+        snprintf(messageBufferTemp, ERROR_MESSAGE_BUFFER_SIZE - 1, "%s: [%d] %s", message, errNo, strerror(errNo));
 
     errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp);
+
+#ifdef DEBUG_COVERAGE
+    }
+#endif
 }
 
 void
 errorInternalThrowSysFmt(
-    int errNo, const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *format, ...)
+#ifdef DEBUG_COVERAGE
+    bool error,
+#else
+    int errNo,
+#endif
+    const ErrorType *errorType, const char *fileName, const char *functionName, int fileLine, const char *format, ...)
 {
+#ifdef DEBUG_COVERAGE
+    if (error)
+    {
+        int errNo = errno;
+#endif
+
     // Format message
     va_list argument;
     va_start(argument, format);
@@ -442,7 +476,12 @@ errorInternalThrowSysFmt(
     va_end(argument);
 
     // Append the system message
-    snprintf(messageBufferTemp + messageSize, ERROR_MESSAGE_BUFFER_SIZE - 1 - messageSize, ": [%d] %s", errNo, strerror(errNo));
+    if (errNo != 0)
+        snprintf(messageBufferTemp + messageSize, ERROR_MESSAGE_BUFFER_SIZE - 1 - messageSize, ": [%d] %s", errNo, strerror(errNo));
 
     errorInternalThrow(errorType, fileName, functionName, fileLine, messageBufferTemp);
+
+#ifdef DEBUG_COVERAGE
+    }
+#endif
 }
