@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Exit Routines
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -13,7 +15,7 @@ Exit Routines
 #include "config/config.h"
 #include "protocol/helper.h"
 
-#ifdef WITH_PERL
+#ifdef HAVE_LIBPERL
     #include "perl/exec.h"
 #endif
 
@@ -105,7 +107,7 @@ exitSafe(int result, bool error, SignalType signalType)
     if (error)
     {
         // Don't log the error if it has already been logged by Perl
-#ifdef WITH_PERL
+#ifdef HAVE_LIBPERL
         if (strcmp(errorMessage(), PERL_EMBED_ERROR) != 0)
         {
 #endif
@@ -117,30 +119,23 @@ exitSafe(int result, bool error, SignalType signalType)
             else
             {
                 // Log just the error to non-debug levels
-                LOG_INTERNAL(logLevel, LOG_LEVEL_MIN, logLevelDetail, errorCode(), errorMessage());
+                LOG_INTERNAL(logLevel, LOG_LEVEL_MIN, logLevelDetail, 0, errorCode(), errorMessage());
 
                 // Log the stack trace debug levels
-                if (logWill(logLevelDebug))
+                if (logAny(logLevelDebug))
                 {
                     LOG_INTERNAL(
-                        logLevel, logLevelDebug, LOG_LEVEL_MAX, errorCode(), "%s\nSTACK TRACE:\n%s", errorMessage(),
+                        logLevel, logLevelDebug, LOG_LEVEL_MAX, 0, errorCode(), "%s\nSTACK TRACE:\n%s", errorMessage(),
                         errorStackTrace());
                 }
             }
-#ifdef WITH_PERL
+#ifdef HAVE_LIBPERL
         }
 #endif
 
         result = errorCode();
     }
 
-    // Release any locks but ignore errors
-    TRY_BEGIN()
-    {
-        lockRelease(false);
-    }
-    TRY_END();
-
     // Free protocol objects but ignore errors
     TRY_BEGIN()
     {
@@ -149,7 +144,7 @@ exitSafe(int result, bool error, SignalType signalType)
     TRY_END();
 
     // Free Perl but ignore errors
-#ifdef WITH_PERL
+#ifdef HAVE_LIBPERL
     TRY_BEGIN()
     {
         perlFree(result);
@@ -185,6 +180,13 @@ exitSafe(int result, bool error, SignalType signalType)
         cmdEnd(result, errorMessage);
     }
 
+    // Release any locks but ignore errors
+    TRY_BEGIN()
+    {
+        lockRelease(false);
+    }
+    TRY_END();
+
     // Return result - caller should immediate pass this result to exit()
     FUNCTION_LOG_RETURN(INT, result);
 }
