@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Ini Handler
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -9,9 +11,10 @@ Ini Handler
 #include "common/memContext.h"
 #include "common/ini.h"
 #include "common/type/keyValue.h"
+#include "common/object.h"
 
 /***********************************************************************************************************************************
-Contains information about the ini
+Object type
 ***********************************************************************************************************************************/
 struct Ini
 {
@@ -19,6 +22,8 @@ struct Ini
     KeyValue *store;                                                // Key value store that contains the ini data
 };
 
+OBJECT_DEFINE_FREE(INI);
+
 /***********************************************************************************************************************************
 Create a new Ini object
 ***********************************************************************************************************************************/
@@ -46,12 +51,13 @@ iniNew(void)
 Internal function to get an ini value
 ***********************************************************************************************************************************/
 static const Variant *
-iniGetInternal(const Ini *this, const String *section, const String *key)
+iniGetInternal(const Ini *this, const String *section, const String *key, bool required)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(INI, this);
         FUNCTION_TEST_PARAM(STRING, section);
         FUNCTION_TEST_PARAM(STRING, key);
+        FUNCTION_TEST_PARAM(BOOL, required);
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
@@ -60,16 +66,16 @@ iniGetInternal(const Ini *this, const String *section, const String *key)
 
     const Variant *result = NULL;
 
-    MEM_CONTEXT_TEMP_BEGIN()
-    {
-        // Get the section
-        KeyValue *sectionKv = varKv(kvGet(this->store, varNewStr(section)));
+    // Get the section
+    KeyValue *sectionKv = varKv(kvGet(this->store, VARSTR(section)));
 
-        // Section must exist to get the value
-        if (sectionKv != NULL)
-            result = kvGet(sectionKv, varNewStr(key));
-    }
-    MEM_CONTEXT_TEMP_END();
+    // Section must exist to get the value
+    if (sectionKv != NULL)
+        result = kvGet(sectionKv, VARSTR(key));
+
+    // If value is null and required then error
+    if (result == NULL && required)
+        THROW_FMT(FormatError, "section '%s', key '%s' does not exist", strPtr(section), strPtr(key));
 
     FUNCTION_TEST_RETURN(result);
 }
@@ -77,7 +83,7 @@ iniGetInternal(const Ini *this, const String *section, const String *key)
 /***********************************************************************************************************************************
 Get an ini value -- error if it does not exist
 ***********************************************************************************************************************************/
-const Variant *
+const String *
 iniGet(const Ini *this, const String *section, const String *key)
 {
     FUNCTION_TEST_BEGIN();
@@ -91,26 +97,22 @@ iniGet(const Ini *this, const String *section, const String *key)
     ASSERT(key != NULL);
 
     // Get the value
-    const Variant *result = iniGetInternal(this, section, key);
+    const Variant *result = iniGetInternal(this, section, key, true);
 
-    // If value is null replace it with default
-    if (result == NULL)
-        THROW_FMT(FormatError, "section '%s', key '%s' does not exist", strPtr(section), strPtr(key));
-
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(varStr(result));
 }
 
 /***********************************************************************************************************************************
 Get an ini value -- if it does not exist then return specified default
 ***********************************************************************************************************************************/
-const Variant *
-iniGetDefault(const Ini *this, const String *section, const String *key, Variant *defaultValue)
+const String *
+iniGetDefault(const Ini *this, const String *section, const String *key, const String *defaultValue)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(INI, this);
         FUNCTION_TEST_PARAM(STRING, section);
         FUNCTION_TEST_PARAM(STRING, key);
-        FUNCTION_TEST_PARAM(VARIANT, defaultValue);
+        FUNCTION_TEST_PARAM(STRING, defaultValue);
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
@@ -118,13 +120,53 @@ iniGetDefault(const Ini *this, const String *section, const String *key, Variant
     ASSERT(key != NULL);
 
     // Get the value
-    const Variant *result = iniGetInternal(this, section, key);
+    const Variant *result = iniGetInternal(this, section, key, false);
 
-    // If value is null replace it with default
-    if (result == NULL)
-        result = defaultValue;
+    FUNCTION_TEST_RETURN(result == NULL ? defaultValue : varStr(result));
+}
 
-    FUNCTION_TEST_RETURN(result);
+/***********************************************************************************************************************************
+Internal function to get an ini value list
+***********************************************************************************************************************************/
+StringList *
+iniGetList(const Ini *this, const String *section, const String *key)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(INI, this);
+        FUNCTION_TEST_PARAM(STRING, section);
+        FUNCTION_TEST_PARAM(STRING, key);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+    ASSERT(section != NULL);
+    ASSERT(key != NULL);
+
+    // Get the value
+    const Variant *result = iniGetInternal(this, section, key, false);
+
+    FUNCTION_TEST_RETURN(result == NULL ? NULL : strLstNewVarLst(varVarLst(result)));
+}
+
+/***********************************************************************************************************************************
+Internal function to get an ini value list
+***********************************************************************************************************************************/
+bool
+iniSectionKeyIsList(const Ini *this, const String *section, const String *key)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(INI, this);
+        FUNCTION_TEST_PARAM(STRING, section);
+        FUNCTION_TEST_PARAM(STRING, key);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+    ASSERT(section != NULL);
+    ASSERT(key != NULL);
+
+    // Get the value
+    const Variant *result = iniGetInternal(this, section, key, true);
+
+    FUNCTION_TEST_RETURN(varType(result) == varTypeVariantList);
 }
 
 /***********************************************************************************************************************************
@@ -146,7 +188,7 @@ iniSectionKeyList(const Ini *this, const String *section)
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Get the section
-        KeyValue *sectionKv = varKv(kvGet(this->store, varNewStr(section)));
+        KeyValue *sectionKv = varKv(kvGet(this->store, VARSTR(section)));
 
         // Return key list if the section exists
         if (sectionKv != NULL)
@@ -253,11 +295,8 @@ iniParse(Ini *this, const String *content)
                             if (strSize(key) == 0)
                                 THROW_FMT(FormatError, "key is zero-length at line %u: %s", lineIdx++, linePtr);
 
-                            // Extract the value
-                            Variant *value = varNewStr(strTrim(strNew(lineEqual + 1)));
-
                             // Store the section/key/value
-                            iniSet(this, section, key, value);
+                            iniSet(this, section, key, strTrim(strNew(lineEqual + 1)));
                         }
                     }
                 }
@@ -274,13 +313,13 @@ iniParse(Ini *this, const String *content)
 Set an ini value
 ***********************************************************************************************************************************/
 void
-iniSet(Ini *this, const String *section, const String *key, const Variant *value)
+iniSet(Ini *this, const String *section, const String *key, const String *value)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(INI, this);
         FUNCTION_TEST_PARAM(STRING, section);
         FUNCTION_TEST_PARAM(STRING, key);
-        FUNCTION_TEST_PARAM(VARIANT, value);
+        FUNCTION_TEST_PARAM(STRING, value);
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
@@ -290,13 +329,13 @@ iniSet(Ini *this, const String *section, const String *key, const Variant *value
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        Variant *sectionKey = varNewStr(section);
+        const Variant *sectionKey = VARSTR(section);
         KeyValue *sectionKv = varKv(kvGet(this->store, sectionKey));
 
         if (sectionKv == NULL)
             sectionKv = kvPutKv(this->store, sectionKey);
 
-        kvAdd(sectionKv, varNewStr(key), value);
+        kvAdd(sectionKv, VARSTR(key), VARSTR(value));
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -304,17 +343,70 @@ iniSet(Ini *this, const String *section, const String *key, const Variant *value
 }
 
 /***********************************************************************************************************************************
-Free the ini
+Save the ini file
 ***********************************************************************************************************************************/
 void
-iniFree(Ini *this)
+iniSave(Ini *this, IoWrite *write)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(INI, this);
+        FUNCTION_TEST_PARAM(IO_WRITE, write);
     FUNCTION_TEST_END();
 
-    if (this != NULL)
-        memContextFree(this->memContext);
+    ASSERT(this != NULL);
+    ASSERT(write != NULL);
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        ioWriteOpen(write);
+
+        StringList *sectionList = strLstSort(iniSectionList(this), sortOrderAsc);
+
+        for (unsigned int sectionIdx = 0; sectionIdx < strLstSize(sectionList); sectionIdx++)
+        {
+            const String *section = strLstGet(sectionList, sectionIdx);
+
+            if (sectionIdx != 0)
+                ioWrite(write, LF_BUF);
+
+            ioWrite(write, BRACKETL_BUF);
+            ioWriteStr(write, section);
+            ioWriteLine(write, BRACKETR_BUF);
+
+            StringList *keyList = strLstSort(iniSectionKeyList(this, section), sortOrderAsc);
+
+            for (unsigned int keyIdx = 0; keyIdx < strLstSize(keyList); keyIdx++)
+            {
+                const String *key = strLstGet(keyList, keyIdx);
+
+                ioWriteStr(write, key);
+                ioWrite(write, EQ_BUF);
+                ioWriteStrLine(write, iniGet(this, section, key));
+            }
+        }
+
+        ioWriteClose(write);
+    }
+    MEM_CONTEXT_TEMP_END();
 
     FUNCTION_TEST_RETURN_VOID();
 }
+
+/***********************************************************************************************************************************
+Move to a new mem context
+***********************************************************************************************************************************/
+Ini *
+iniMove(Ini *this, MemContext *parentNew)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(INI, this);
+        FUNCTION_TEST_PARAM(MEM_CONTEXT, parentNew);
+    FUNCTION_TEST_END();
+
+    ASSERT(parentNew != NULL);
+
+    if (this != NULL)
+        memContextMove(this->memContext, parentNew);
+
+    FUNCTION_TEST_RETURN(this);
+}
