@@ -7,24 +7,44 @@ Ini Handler
 /***********************************************************************************************************************************
 Ini object
 ***********************************************************************************************************************************/
+#define INI_TYPE                                                    Ini
+#define INI_PREFIX                                                  ini
+
 typedef struct Ini Ini;
 
+#include "common/io/write.h"
 #include "common/type/variant.h"
 
+/***********************************************************************************************************************************
+Constructor
+***********************************************************************************************************************************/
+Ini *iniNew(void);
+
 /***********************************************************************************************************************************
 Functions
 ***********************************************************************************************************************************/
-Ini *iniNew(void);
-const Variant *iniGet(const Ini *this, const String *section, const String *key);
-const Variant *iniGetDefault(const Ini *this, const String *section, const String *key, Variant *defaultValue);
+Ini *iniMove(Ini *this, MemContext *parentNew);
+void iniParse(Ini *this, const String *content);
+void iniSave(Ini *this, IoWrite *write);
+void iniSet(Ini *this, const String *section, const String *key, const String *value);
+
+/***********************************************************************************************************************************
+Getters
+***********************************************************************************************************************************/
+const String *iniGet(const Ini *this, const String *section, const String *key);
+const String *iniGetDefault(const Ini *this, const String *section, const String *key, const String *defaultValue);
+StringList *iniGetList(const Ini *this, const String *section, const String *key);
+bool iniSectionKeyIsList(const Ini *this, const String *section, const String *key);
 StringList *iniSectionKeyList(const Ini *this, const String *section);
 StringList *iniSectionList(const Ini *this);
-void iniParse(Ini *this, const String *content);
-void iniSet(Ini *this, const String *section, const String *key, const Variant *value);
-void iniFree(Ini *this);
 String *iniFileName(const Ini *this);
 bool iniFileExists(const Ini *this);
 
+/***********************************************************************************************************************************
+Destructor
+***********************************************************************************************************************************/
+void iniFree(Ini *this);
+
 /***********************************************************************************************************************************
 Macros for function logging
 ***********************************************************************************************************************************/
