@@ -1,57 +1,43 @@
 /***********************************************************************************************************************************
 Buffer IO Read
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include "common/debug.h"
 #include "common/io/bufferRead.h"
 #include "common/io/read.intern.h"
 #include "common/log.h"
 #include "common/memContext.h"
+#include "common/object.h"
 
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
-struct IoBufferRead
+typedef struct IoBufferRead
 {
     MemContext *memContext;                                         // Object memory context
-    IoRead *io;                                                     // IoRead interface
     const Buffer *read;                                             // Buffer to read data from
 
     size_t readPos;                                                 // Current position in the read buffer
     bool eof;                                                       // Has the end of the buffer been reached?
-};
+} IoBufferRead;
 
 /***********************************************************************************************************************************
-New object
+Macros for function logging
 ***********************************************************************************************************************************/
-IoBufferRead *
-ioBufferReadNew(const Buffer *buffer)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(BUFFER, buffer);
-    FUNCTION_LOG_END();
-
-    ASSERT(buffer != NULL);
-
-    IoBufferRead *this = NULL;
-
-    MEM_CONTEXT_NEW_BEGIN("IoBufferRead")
-    {
-        this = memNew(sizeof(IoBufferRead));
-        this->memContext = memContextCurrent();
-        this->io = ioReadNewP(this, .eof = (IoReadInterfaceEof)ioBufferReadEof, .read = (IoReadInterfaceRead)ioBufferRead);
-        this->read = buffer;
-    }
-    MEM_CONTEXT_NEW_END();
-
-    FUNCTION_LOG_RETURN(IO_BUFFER_READ, this);
-}
+#define FUNCTION_LOG_IO_BUFFER_READ_TYPE                                                                                           \
+    IoBufferRead *
+#define FUNCTION_LOG_IO_BUFFER_READ_FORMAT(value, buffer, bufferSize)                                                              \
+    objToLog(value, "IoBufferRead", buffer, bufferSize)
 
 /***********************************************************************************************************************************
 Read data from the buffer
 ***********************************************************************************************************************************/
-size_t
-ioBufferRead(IoBufferRead *this, Buffer *buffer, bool block)
+static size_t
+ioBufferRead(THIS_VOID, Buffer *buffer, bool block)
 {
+    THIS(IoBufferRead);
+
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(IO_BUFFER_READ, this);
         FUNCTION_LOG_PARAM(BUFFER, buffer);
@@ -81,31 +67,14 @@ ioBufferRead(IoBufferRead *this, Buffer *buffer, bool block)
     FUNCTION_LOG_RETURN(SIZE, actualBytes);
 }
 
-/***********************************************************************************************************************************
-Move the object to a new context
-***********************************************************************************************************************************/
-IoBufferRead *
-ioBufferReadMove(IoBufferRead *this, MemContext *parentNew)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(IO_BUFFER_READ, this);
-        FUNCTION_TEST_PARAM(MEM_CONTEXT, parentNew);
-    FUNCTION_TEST_END();
-
-    ASSERT(parentNew != NULL);
-
-    if (this != NULL)
-        memContextMove(this->memContext, parentNew);
-
-    FUNCTION_TEST_RETURN(this);
-}
-
 /***********************************************************************************************************************************
 Have all bytes been read from the buffer?
 ***********************************************************************************************************************************/
-bool
-ioBufferReadEof(const IoBufferRead *this)
+static bool
+ioBufferReadEof(THIS_VOID)
 {
+    THIS(IoBufferRead);
+
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(IO_BUFFER_READ, this);
     FUNCTION_LOG_END();
@@ -116,32 +85,28 @@ ioBufferReadEof(const IoBufferRead *this)
 }
 
 /***********************************************************************************************************************************
-Get io interface
+New object
 ***********************************************************************************************************************************/
 IoRead *
-ioBufferReadIo(const IoBufferRead *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(IO_BUFFER_READ, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->io);
-}
-
-/***********************************************************************************************************************************
-Free the object
-***********************************************************************************************************************************/
-void
-ioBufferReadFree(IoBufferRead *this)
+ioBufferReadNew(const Buffer *buffer)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(IO_BUFFER_READ, this);
+        FUNCTION_LOG_PARAM(BUFFER, buffer);
     FUNCTION_LOG_END();
 
-    if (this != NULL)
-        memContextFree(this->memContext);
+    ASSERT(buffer != NULL);
 
-    FUNCTION_LOG_RETURN_VOID();
+    IoRead *this = NULL;
+
+    MEM_CONTEXT_NEW_BEGIN("IoBufferRead")
+    {
+        IoBufferRead *driver = memNew(sizeof(IoBufferRead));
+        driver->memContext = memContextCurrent();
+        driver->read = buffer;
+
+        this = ioReadNewP(driver, .eof = ioBufferReadEof, .read = ioBufferRead);
+    }
+    MEM_CONTEXT_NEW_END();
+
+    FUNCTION_LOG_RETURN(IO_READ, this);
 }
