@@ -1,10 +1,13 @@
 /***********************************************************************************************************************************
 IO Filter Interface
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include "common/debug.h"
 #include "common/io/filter/filter.intern.h"
 #include "common/log.h"
 #include "common/memContext.h"
+#include "common/object.h"
 
 /***********************************************************************************************************************************
 Object type
@@ -14,20 +17,26 @@ struct IoFilter
     MemContext *memContext;                                         // Mem context of filter
     const String *type;                                             // Filter type
     void *driver;                                                   // Filter driver
+    const VariantList *paramList;                                   // Filter parameters
     IoFilterInterface interface;                                    // Filter interface
+
+    bool flushing;                                                  // Has the filter started flushing?
 };
 
+OBJECT_DEFINE_FREE(IO_FILTER);
+
 /***********************************************************************************************************************************
 New object
 
 Allocations will be in the memory context of the caller.
 ***********************************************************************************************************************************/
 IoFilter *
-ioFilterNew(const String *type, void *driver, IoFilterInterface interface)
+ioFilterNew(const String *type, void *driver, VariantList *paramList, IoFilterInterface interface)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(STRING, type);
         FUNCTION_LOG_PARAM_P(VOID, driver);
+        FUNCTION_LOG_PARAM(VARIANT_LIST, paramList);
         FUNCTION_LOG_PARAM(IO_FILTER_INTERFACE, interface);
     FUNCTION_LOG_END();
 
@@ -39,13 +48,12 @@ ioFilterNew(const String *type, void *driver, IoFilterInterface interface)
     ASSERT(!(interface.in != NULL && interface.inOut != NULL));
     // If the filter does not produce output then it should produce a result
     ASSERT(interface.in == NULL || (interface.result != NULL && interface.done == NULL && interface.inputSame == NULL));
-    // Filters that produce output will not always be able to dump all their output and will need to get the same input again
-    ASSERT(interface.inOut == NULL || interface.inputSame != NULL);
 
     IoFilter *this = memNew(sizeof(IoFilter));
     this->memContext = memContextCurrent();
     this->type = type;
     this->driver = driver;
+    this->paramList = paramList;
     this->interface = interface;
 
     FUNCTION_LOG_RETURN(IO_FILTER, this);
@@ -64,8 +72,13 @@ ioFilterProcessIn(IoFilter *this, const Buffer *input)
 
     ASSERT(this != NULL);
     ASSERT(this->interface.in != NULL);
+    CHECK(input == NULL || bufUsed(input) > 0);
+    CHECK(!this->flushing || input == NULL);
 
-    this->interface.in(this->driver, input);
+    if (input == NULL)
+        this->flushing = true;
+    else
+        this->interface.in(this->driver, input);
 
     FUNCTION_TEST_RETURN_VOID();
 }
@@ -85,9 +98,16 @@ ioFilterProcessInOut(IoFilter *this, const Buffer *input, Buffer *output)
     ASSERT(this != NULL);
     ASSERT(output != NULL);
     ASSERT(this->interface.inOut != NULL);
+    CHECK(input == NULL || bufUsed(input) > 0);
+    CHECK(!this->flushing || input == NULL);
 
-    this->interface.inOut(this->driver, input, output);
+    if (input == NULL && !this->flushing)
+        this->flushing = true;
 
+    if (!ioFilterDone(this))
+        this->interface.inOut(this->driver, input, output);
+
+    CHECK(!ioFilterInputSame(this) || bufUsed(output) > 0);
     FUNCTION_TEST_RETURN_VOID();
 }
 
@@ -113,7 +133,8 @@ ioFilterMove(IoFilter *this, MemContext *parentNew)
 /***********************************************************************************************************************************
 Is the filter done?
 
-If done is not defined by the filter then check inputSame.  If inputSame is true then the filter is not done.
+If done is not defined by the filter then check inputSame.  If inputSame is true then the filter is not done.  Even if the filter
+is done the interface will not report done until the interface is flushing.
 ***********************************************************************************************************************************/
 bool
 ioFilterDone(const IoFilter *this)
@@ -124,7 +145,27 @@ ioFilterDone(const IoFilter *this)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->interface.done != NULL ? this->interface.done(this->driver) : !ioFilterInputSame(this));
+    bool result = false;
+
+    if (this->flushing)
+        result = this->interface.done != NULL ? this->interface.done(this->driver) : !ioFilterInputSame(this);
+
+    FUNCTION_TEST_RETURN(result);
+}
+
+/***********************************************************************************************************************************
+Driver for the filter
+***********************************************************************************************************************************/
+void *
+ioFilterDriver(IoFilter *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(IO_FILTER, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    FUNCTION_TEST_RETURN(this->driver);
 }
 
 /***********************************************************************************************************************************
@@ -144,10 +185,25 @@ ioFilterInputSame(const IoFilter *this)
     FUNCTION_TEST_RETURN(this->interface.inputSame != NULL ? this->interface.inputSame(this->driver) : false);
 }
 
+/***********************************************************************************************************************************
+Interface for the filter
+***********************************************************************************************************************************/
+const IoFilterInterface *
+ioFilterInterface(const IoFilter *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(IO_FILTER, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    FUNCTION_TEST_RETURN(&this->interface);
+}
+
 /***********************************************************************************************************************************
 Does filter produce output?
 
-All InOut filters produce output.
+All In filters produce output.
 ***********************************************************************************************************************************/
 bool
 ioFilterOutput(const IoFilter *this)
@@ -161,10 +217,25 @@ ioFilterOutput(const IoFilter *this)
     FUNCTION_TEST_RETURN(this->interface.inOut != NULL);
 }
 
+/***********************************************************************************************************************************
+List of filter parameters
+***********************************************************************************************************************************/
+const VariantList *
+ioFilterParamList(const IoFilter *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(IO_FILTER, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    FUNCTION_TEST_RETURN(this->paramList);
+}
+
 /***********************************************************************************************************************************
 Get filter result
 ***********************************************************************************************************************************/
-const Variant *
+Variant *
 ioFilterResult(const IoFilter *this)
 {
     FUNCTION_TEST_BEGIN();
@@ -192,3 +263,12 @@ ioFilterType(const IoFilter *this)
 
     FUNCTION_TEST_RETURN(this->type);
 }
+
+/***********************************************************************************************************************************
+Render as string for logging
+***********************************************************************************************************************************/
+String *
+ioFilterToLog(const IoFilter *this)
+{
+    return strNewFmt("{type: %s}", strPtr(this->type));
+}
