@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 IO Filter Group
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdio.h>
 
 #include "common/debug.h"
@@ -10,6 +12,7 @@ IO Filter Group
 #include "common/io/io.h"
 #include "common/log.h"
 #include "common/memContext.h"
+#include "common/object.h"
 #include "common/type/list.h"
 
 /***********************************************************************************************************************************
@@ -19,7 +22,7 @@ Contains the filter object and inout/output buffers.
 ***********************************************************************************************************************************/
 typedef struct IoFilterData
 {
-    const Buffer *input;                                            // Input buffer for filter
+    const Buffer **input;                                           // Input buffer for filter
     Buffer *inputLocal;                                             // Non-null if a locally created buffer that can be cleared
     IoFilter *filter;                                               // Filter to apply
     Buffer *output;                                                 // Output buffer for filter
@@ -38,7 +41,7 @@ struct IoFilterGroup
 {
     MemContext *memContext;                                         // Mem context
     List *filterList;                                               // List of filters to apply
-    unsigned int firstOutputFilter;                                 // Index of the first output filter
+    const Buffer *input;                                            // Input buffer passed in for processing
     KeyValue *filterResult;                                         // Filter results (if any)
     bool inputSame;                                                 // Same input required again?
     bool done;                                                      // Is processing done?
@@ -50,6 +53,8 @@ struct IoFilterGroup
 #endif
 };
 
+OBJECT_DEFINE_FREE(IO_FILTER_GROUP);
+
 /***********************************************************************************************************************************
 New Object
 ***********************************************************************************************************************************/
@@ -64,7 +69,7 @@ ioFilterGroupNew(void)
     {
         this = memNew(sizeof(IoFilterGroup));
         this->memContext = memContextCurrent();
-        this->done = true;
+        this->done = false;
         this->filterList = lstNew(sizeof(IoFilterData));
     }
     MEM_CONTEXT_NEW_END();
@@ -97,11 +102,36 @@ ioFilterGroupAdd(IoFilterGroup *this, IoFilter *filter)
     FUNCTION_LOG_RETURN(IO_FILTER_GROUP, this);
 }
 
+/***********************************************************************************************************************************
+Insert a filter before an index
+***********************************************************************************************************************************/
+IoFilterGroup *
+ioFilterGroupInsert(IoFilterGroup *this, unsigned int listIdx, IoFilter *filter)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(IO_FILTER_GROUP, this);
+        FUNCTION_LOG_PARAM(IO_FILTER, filter);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(!this->opened && !this->closed);
+    ASSERT(filter != NULL);
+
+    // Move the filter to this object's mem context
+    ioFilterMove(filter, this->memContext);
+
+    // Add the filter
+    IoFilterData filterData = {.filter = filter};
+    lstInsert(this->filterList, listIdx, &filterData);
+
+    FUNCTION_LOG_RETURN(IO_FILTER_GROUP, this);
+}
+
 /***********************************************************************************************************************************
 Get a filter
 ***********************************************************************************************************************************/
 static IoFilterData *
-ioFilterGroupGet(IoFilterGroup *this, unsigned int filterIdx)
+ioFilterGroupGet(const IoFilterGroup *this, unsigned int filterIdx)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(IO_FILTER_GROUP, this);
@@ -113,6 +143,27 @@ ioFilterGroupGet(IoFilterGroup *this, unsigned int filterIdx)
     FUNCTION_TEST_RETURN((IoFilterData *)lstGet(this->filterList, filterIdx));
 }
 
+/***********************************************************************************************************************************
+Clear filters
+***********************************************************************************************************************************/
+IoFilterGroup *
+ioFilterGroupClear(IoFilterGroup *this)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(IO_FILTER_GROUP, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(!this->opened);
+
+    for (unsigned int filterIdx = 0; filterIdx < ioFilterGroupSize(this); filterIdx++)
+        ioFilterFree(ioFilterGroupGet(this, filterIdx)->filter);
+
+    lstClear(this->filterList);
+
+    FUNCTION_LOG_RETURN(IO_FILTER_GROUP, this);
+}
+
 /***********************************************************************************************************************************
 Open filter group
 
@@ -131,38 +182,40 @@ ioFilterGroupOpen(IoFilterGroup *this)
     {
         // If the last filter is not an output filter then add a filter to buffer/copy data.  Input filters won't copy to an output
         // buffer so we need some way to get the data to the output buffer.
-        if (lstSize(this->filterList) == 0 || !ioFilterOutput((ioFilterGroupGet(this, lstSize(this->filterList) - 1))->filter))
-            ioFilterGroupAdd(this, ioBufferFilter(ioBufferNew()));
+        if (ioFilterGroupSize(this) == 0 ||
+            !ioFilterOutput((ioFilterGroupGet(this, ioFilterGroupSize(this) - 1))->filter))
+        {
+            ioFilterGroupAdd(this, ioBufferNew());
+        }
 
         // Create filter input/output buffers.  Input filters do not get an output buffer since they don't produce output.
-        Buffer *lastOutputBuffer = NULL;
+        Buffer **lastOutputBuffer = NULL;
 
-        for (unsigned int filterIdx = 0; filterIdx < lstSize(this->filterList); filterIdx++)
+        for (unsigned int filterIdx = 0; filterIdx < ioFilterGroupSize(this); filterIdx++)
         {
             IoFilterData *filterData = ioFilterGroupGet(this, filterIdx);
 
-            // Assign the last output buffer to the input.  At first there won't be an input filter because it will be passed into
-            // the process function as an input.
-            if (lastOutputBuffer != NULL)
+            // If there is no last output buffer yet, then use the input buffer that will be provided by the caller
+            if (lastOutputBuffer == NULL)
             {
-                filterData->input = lastOutputBuffer;
-                filterData->inputLocal = lastOutputBuffer;
+                filterData->input = &this->input;
+            }
+            // Else assign the last output buffer to the input
+            else
+            {
+                // This cast is required because the compiler can't guarantee the const-ness of this object, i.e. it could be
+                // modified in other parts of the code.  This is actually expected and the only reason we need this const is to
+                // match the const-ness of the input buffer provided by the caller.
+                filterData->input = (const Buffer **)lastOutputBuffer;
+                filterData->inputLocal = *lastOutputBuffer;
             }
 
-            // Is this an output filter?
-            if (ioFilterOutput(filterData->filter))
+            // If this is not the last output filter then create a new output buffer for it.  The output buffer for the last filter
+            // will be provided to the process function.
+            if (ioFilterOutput(filterData->filter) && filterIdx < ioFilterGroupSize(this) - 1)
             {
-                // If this is the first output buffer found, store the index so it can be easily found during processing
-                if (lastOutputBuffer == NULL)
-                    this->firstOutputFilter = filterIdx;
-
-                // If this is not the last output filter then create a new output buffer for it.  The output buffer for the last
-                // filter will be provided to the process function.
-                if (filterIdx < lstSize(this->filterList) - 1)
-                {
-                    lastOutputBuffer = bufNew(ioBufferSize());
-                    filterData->output = lastOutputBuffer;
-                }
+                filterData->output = bufNew(ioBufferSize());
+                lastOutputBuffer = &filterData->output;
             }
         }
     }
@@ -190,6 +243,7 @@ ioFilterGroupProcess(IoFilterGroup *this, const Buffer *input, Buffer *output)
 
     ASSERT(this != NULL);
     ASSERT(this->opened && !this->closed);
+    ASSERT(input == NULL || bufUsed(input) > 0);
     ASSERT(!this->flushing || input == NULL);
     ASSERT(output != NULL);
     ASSERT(bufRemains(output) > 0);
@@ -200,15 +254,9 @@ ioFilterGroupProcess(IoFilterGroup *this, const Buffer *input, Buffer *output)
         this->flushing = true;
 #endif
 
-    // Assign the input buffer up to the first output filter.  After this point the input buffers were locally created during open.
-    if (!this->inputSame)
-    {
-        for (unsigned int filterIdx = 0; filterIdx <= this->firstOutputFilter; filterIdx++)
-            (ioFilterGroupGet(this, filterIdx))->input = input;
-    }
-
-    // Assign the output buffer
-    (ioFilterGroupGet(this, lstSize(this->filterList) - 1))->output = output;
+    // Assign input and output buffers
+    this->input = input;
+    (ioFilterGroupGet(this, ioFilterGroupSize(this) - 1))->output = output;
 
     //
     do
@@ -221,7 +269,7 @@ ioFilterGroupProcess(IoFilterGroup *this, const Buffer *input, Buffer *output)
         if (this->inputSame)
         {
             this->inputSame = false;
-            filterIdx = lstSize(this->filterList);
+            filterIdx = ioFilterGroupSize(this);
 
             do
             {
@@ -233,7 +281,7 @@ ioFilterGroupProcess(IoFilterGroup *this, const Buffer *input, Buffer *output)
                     break;
                 }
             }
-            while (filterIdx != this->firstOutputFilter);
+            while (filterIdx != 0);
 
             // If no filter is found that needs the same input that means we are done with the current input.  So end the loop and
             // get some more input.
@@ -243,46 +291,42 @@ ioFilterGroupProcess(IoFilterGroup *this, const Buffer *input, Buffer *output)
 
         // Process forward from the filter that has input to process.  This may be a filter that needs the same input or it may be
         // new input for the first filter.
-        for (; filterIdx < lstSize(this->filterList); filterIdx++)
+        for (; filterIdx < ioFilterGroupSize(this); filterIdx++)
         {
             IoFilterData *filterData = ioFilterGroupGet(this, filterIdx);
 
-            // If the filter produces output
-            if (ioFilterOutput(filterData->filter))
+            // Process the filter if it is not done
+            if (!ioFilterDone(filterData->filter))
             {
-                // Keep processing while the filter is not done or there is input
-                if (!ioFilterDone(filterData->filter) || filterData->input != NULL)
+                // If the filter produces output
+                if (ioFilterOutput(filterData->filter))
                 {
-                    // If we are flushing and the prior filter is done and is not producing any more output then this filter should
-                    // be flushing as well.  Set filterData->input = NULL so it knows there is no more input coming.
-                    //
-                    // If the filter is already done then there is no need to set input to NULL because it has already flushed and
-                    // the filter shouldn't need to hand NULL input if it doesn't need it to know when to flush.
-                    //
-                    // Checking filterIdx - 1 is safe because the first filter's filterData->input is always set to NULL when input
-                    // is NULL.
-                    if (input == NULL && filterData->input != NULL && !ioFilterDone(filterData->filter) &&
-                        ioFilterDone(ioFilterGroupGet(this, filterIdx - 1)->filter) && bufUsed(filterData->input) == 0)
-                    {
-                        filterData->input = NULL;
-                    }
+                    ioFilterProcessInOut(filterData->filter, *filterData->input, filterData->output);
 
-                    ioFilterProcessInOut(filterData->filter, filterData->input, filterData->output);
-
-                    // If inputSame is set then the output buffer for this filter is full and it will need to be pre-processed with
-                    // the same input once the output buffer is cleared.
+                    // If inputSame is set then the output buffer for this filter is full and it will need to be re-processed with
+                    // the same input once the output buffer is cleared
                     if (ioFilterInputSame(filterData->filter))
+                    {
                         this->inputSame = true;
-
-                    // Else clear the buffer if it was locally allocated.  If this is an input buffer that was passed in then the
-                    // caller is responsible for clearing it.
+                    }
+                    // Else clear the buffer if it was locally allocated.  If the input buffer was passed in then the caller is
+                    // responsible for clearing it.
                     else if (filterData->inputLocal != NULL)
                         bufUsedZero(filterData->inputLocal);
+
+                    // If the output buffer is not full and the filter is not done then more data is required
+                    if (!bufFull(filterData->output) && !ioFilterDone(filterData->filter))
+                        break;
                 }
+                // Else the filter does not produce output
+                else
+                    ioFilterProcessIn(filterData->filter, *filterData->input);
             }
-            // Else the filter does not produce output.  No need to flush these filters because they don't buffer data.
-            else if (filterData->input != NULL)
-                ioFilterProcessIn(filterData->filter, filterData->input);
+
+            // If the filter is done and has no more output then null the output buffer.  Downstream filters have a pointer to this
+            // buffer so their inputs will also change to null and they'll flush.
+            if (filterData->output != NULL && ioFilterDone(filterData->filter) && bufUsed(filterData->output) == 0)
+                filterData->output = NULL;
         }
     }
     while (!bufFull(output) && this->inputSame);
@@ -293,7 +337,7 @@ ioFilterGroupProcess(IoFilterGroup *this, const Buffer *input, Buffer *output)
     this->done = true;
     this->inputSame = false;
 
-    for (unsigned int filterIdx = 0; filterIdx < lstSize(this->filterList); filterIdx++)
+    for (unsigned int filterIdx = 0; filterIdx < ioFilterGroupSize(this); filterIdx++)
     {
         IoFilterData *filterData = ioFilterGroupGet(this, filterIdx);
 
@@ -326,7 +370,7 @@ ioFilterGroupClose(IoFilterGroup *this)
     ASSERT(this != NULL);
     ASSERT(this->opened && !this->closed);
 
-    for (unsigned int filterIdx = 0; filterIdx < lstSize(this->filterList); filterIdx++)
+    for (unsigned int filterIdx = 0; filterIdx < ioFilterGroupSize(this); filterIdx++)
     {
         IoFilterData *filterData = ioFilterGroupGet(this, filterIdx);
         const Variant *filterResult = ioFilterResult(filterData->filter);
@@ -342,7 +386,7 @@ ioFilterGroupClose(IoFilterGroup *this)
 
         MEM_CONTEXT_TEMP_BEGIN()
         {
-            kvAdd(this->filterResult, varNewStr(ioFilterType(filterData->filter)), filterResult);
+            kvAdd(this->filterResult, VARSTR(ioFilterType(filterData->filter)), filterResult);
         }
         MEM_CONTEXT_TEMP_END();
     }
@@ -389,6 +433,36 @@ ioFilterGroupInputSame(const IoFilterGroup *this)
     FUNCTION_TEST_RETURN(this->inputSame);
 }
 
+/***********************************************************************************************************************************
+Get all filters and parameters so they can be passed to a remote
+***********************************************************************************************************************************/
+Variant *
+ioFilterGroupParamAll(const IoFilterGroup *this)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(IO_FILTER_GROUP, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(!this->opened);
+    ASSERT(this->filterList != NULL);
+
+    VariantList *result = varLstNew();
+
+    for (unsigned int filterIdx = 0; filterIdx < ioFilterGroupSize(this); filterIdx++)
+    {
+        IoFilter *filter = ioFilterGroupGet(this, filterIdx)->filter;
+        const VariantList *paramList = ioFilterParamList(filter);
+
+        KeyValue *filterParam = kvNew();
+        kvAdd(filterParam, VARSTR(ioFilterType(filter)), paramList ? varNewVarLst(paramList) : NULL);
+
+        varLstAdd(result, varNewKv(filterParam));
+    }
+
+    FUNCTION_LOG_RETURN(VARIANT, varNewVarLst(result));
+}
+
 /***********************************************************************************************************************************
 Get filter results
 ***********************************************************************************************************************************/
@@ -400,42 +474,88 @@ ioFilterGroupResult(const IoFilterGroup *this, const String *filterType)
         FUNCTION_LOG_PARAM(STRING, filterType);
     FUNCTION_LOG_END();
 
-    ASSERT(this != NULL);
-    ASSERT(this->opened && this->closed);
+    ASSERT(this->opened);
     ASSERT(filterType != NULL);
 
     const Variant *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        result = kvGet(this->filterResult, varNewStr(filterType));
+        result = kvGet(this->filterResult, VARSTR(filterType));
     }
     MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN_CONST(VARIANT, result);
 }
 
+/***********************************************************************************************************************************
+Get all filter results
+***********************************************************************************************************************************/
+const Variant *
+ioFilterGroupResultAll(const IoFilterGroup *this)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(IO_FILTER_GROUP, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(this->closed);
+
+    FUNCTION_LOG_RETURN_CONST(VARIANT, varNewKv(this->filterResult));
+}
+
+/***********************************************************************************************************************************
+Set all filter results
+***********************************************************************************************************************************/
+void
+ioFilterGroupResultAllSet(IoFilterGroup *this, const Variant *filterResult)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(IO_FILTER_GROUP, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+
+    if (filterResult != NULL)
+    {
+        MEM_CONTEXT_BEGIN(this->memContext)
+        {
+            this->filterResult = kvDup(varKv(filterResult));
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+/***********************************************************************************************************************************
+Return total number of filters
+***********************************************************************************************************************************/
+unsigned int
+ioFilterGroupSize(const IoFilterGroup *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(IO_FILTER_GROUP, this);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(lstSize(this->filterList));
+}
+
 /***********************************************************************************************************************************
 Render as string for logging
 ***********************************************************************************************************************************/
 String *
 ioFilterGroupToLog(const IoFilterGroup *this)
 {
-    return strNewFmt("{inputSame: %s, done: %s}", cvtBoolToConstZ(this->inputSame), cvtBoolToConstZ(this->done));
-}
-
-/***********************************************************************************************************************************
-Free the filter group
-***********************************************************************************************************************************/
-void
-ioFilterGroupFree(IoFilterGroup *this)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(IO_FILTER_GROUP, this);
-    FUNCTION_LOG_END();
-
-    if (this != NULL)
-        memContextFree(this->memContext);
-
-    FUNCTION_LOG_RETURN_VOID();
+    return strNewFmt(
+        "{inputSame: %s, done: %s"
+#ifdef DEBUG
+            ", opened %s, flushing %s, closed %s"
+#endif
+            "}",
+        cvtBoolToConstZ(this->inputSame), cvtBoolToConstZ(this->done)
+#ifdef DEBUG
+        , cvtBoolToConstZ(this->opened), cvtBoolToConstZ(this->flushing), cvtBoolToConstZ(this->closed)
+#endif
+    );
 }
