@@ -1,9 +1,12 @@
 /***********************************************************************************************************************************
 Http Header
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include "common/debug.h"
 #include "common/io/http/header.h"
 #include "common/memContext.h"
+#include "common/object.h"
 #include "common/type/keyValue.h"
 
 /***********************************************************************************************************************************
@@ -16,6 +19,8 @@ struct HttpHeader
     KeyValue *kv;                                                   // KeyValue store
 };
 
+OBJECT_DEFINE_FREE(HTTP_HEADER);
+
 /***********************************************************************************************************************************
 New object
 ***********************************************************************************************************************************/
@@ -32,7 +37,7 @@ httpHeaderNew(const StringList *redactList)
         this = memNew(sizeof(HttpHeader));
         this->memContext = MEM_CONTEXT_NEW();
 
-        this->redactList = redactList;
+        this->redactList = strLstDup(redactList);
         this->kv = kvNew();
     }
     MEM_CONTEXT_NEW_END();
@@ -40,6 +45,37 @@ httpHeaderNew(const StringList *redactList)
     FUNCTION_TEST_RETURN(this);
 }
 
+/***********************************************************************************************************************************
+Duplicate object
+***********************************************************************************************************************************/
+HttpHeader *
+httpHeaderDup(const HttpHeader *header, const StringList *redactList)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(HTTP_HEADER, header);
+        FUNCTION_TEST_PARAM(STRING_LIST, redactList);
+    FUNCTION_TEST_END();
+
+    HttpHeader *this = NULL;
+
+    if (header != NULL)
+    {
+
+        MEM_CONTEXT_NEW_BEGIN("HttpHeader")
+        {
+            // Allocate state and set context
+            this = memNew(sizeof(HttpHeader));
+            this->memContext = MEM_CONTEXT_NEW();
+
+            this->redactList = redactList == NULL ? strLstDup(header->redactList) : strLstDup(redactList);
+            this->kv = kvDup(header->kv);
+        }
+        MEM_CONTEXT_NEW_END();
+    }
+
+    FUNCTION_TEST_RETURN(this);
+}
+
 /***********************************************************************************************************************************
 Add a header
 ***********************************************************************************************************************************/
@@ -56,18 +92,14 @@ httpHeaderAdd(HttpHeader *this, const String *key, const String *value)
     ASSERT(key != NULL);
     ASSERT(value != NULL);
 
-    MEM_CONTEXT_BEGIN(this->memContext)
-    {
-        // Make sure the key does not already exist
-        Variant *keyVar = varNewStr(key);
+    // Make sure the key does not already exist
+    const Variant *keyVar = VARSTR(key);
 
-        if (kvGet(this->kv, keyVar) != NULL)
-            THROW_FMT(AssertError, "key '%s' already exists", strPtr(key));
+    if (kvGet(this->kv, keyVar) != NULL)
+        THROW_FMT(AssertError, "key '%s' already exists", strPtr(key));
 
-        // Store the key
-        kvPut(this->kv, keyVar, varNewStr(value));
-    }
-    MEM_CONTEXT_END();
+    // Store the key
+    kvPut(this->kv, keyVar, VARSTR(value));
 
     FUNCTION_TEST_RETURN(this);
 }
@@ -86,15 +118,7 @@ httpHeaderGet(const HttpHeader *this, const String *key)
     ASSERT(this != NULL);
     ASSERT(key != NULL);
 
-    String *result = NULL;
-
-    MEM_CONTEXT_BEGIN(this->memContext)
-    {
-        result = varStr(kvGet(this->kv, varNewStr(key)));
-    }
-    MEM_CONTEXT_END();
-
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(varStr(kvGet(this->kv, VARSTR(key))));
 }
 
 /***********************************************************************************************************************************
@@ -147,12 +171,8 @@ httpHeaderPut(HttpHeader *this, const String *key, const String *value)
     ASSERT(key != NULL);
     ASSERT(value != NULL);
 
-    MEM_CONTEXT_BEGIN(this->memContext)
-    {
-        // Store the key
-        kvPut(this->kv, varNewStr(key), varNewStr(value));
-    }
-    MEM_CONTEXT_END();
+    // Store the key
+    kvPut(this->kv, VARSTR(key), VARSTR(value));
 
     FUNCTION_TEST_RETURN(this);
 }
@@ -200,19 +220,3 @@ httpHeaderToLog(const HttpHeader *this)
 
     return result;
 }
-
-/***********************************************************************************************************************************
-Free the object
-***********************************************************************************************************************************/
-void
-httpHeaderFree(HttpHeader *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(HTTP_HEADER, this);
-    FUNCTION_TEST_END();
-
-    if (this != NULL)
-        memContextFree(this->memContext);
-
-    FUNCTION_TEST_RETURN_VOID();
-}
