@@ -1,10 +1,13 @@
 /***********************************************************************************************************************************
 Http Query
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include "common/debug.h"
 #include "common/io/http/common.h"
 #include "common/io/http/query.h"
 #include "common/memContext.h"
+#include "common/object.h"
 #include "common/type/keyValue.h"
 
 /***********************************************************************************************************************************
@@ -16,6 +19,8 @@ struct HttpQuery
     KeyValue *kv;                                                   // KeyValue store
 };
 
+OBJECT_DEFINE_FREE(HTTP_QUERY);
+
 /***********************************************************************************************************************************
 New object
 ***********************************************************************************************************************************/
@@ -54,18 +59,14 @@ httpQueryAdd(HttpQuery *this, const String *key, const String *value)
     ASSERT(key != NULL);
     ASSERT(value != NULL);
 
-    MEM_CONTEXT_BEGIN(this->memContext)
-    {
-        // Make sure the key does not already exist
-        Variant *keyVar = varNewStr(key);
+    // Make sure the key does not already exist
+    const Variant *keyVar = VARSTR(key);
 
-        if (kvGet(this->kv, keyVar) != NULL)
-            THROW_FMT(AssertError, "key '%s' already exists", strPtr(key));
+    if (kvGet(this->kv, keyVar) != NULL)
+        THROW_FMT(AssertError, "key '%s' already exists", strPtr(key));
 
-        // Store the key
-        kvPut(this->kv, keyVar, varNewStr(value));
-    }
-    MEM_CONTEXT_END();
+    // Store the key
+    kvPut(this->kv, keyVar, VARSTR(value));
 
     FUNCTION_TEST_RETURN(this);
 }
@@ -84,15 +85,7 @@ httpQueryGet(const HttpQuery *this, const String *key)
     ASSERT(this != NULL);
     ASSERT(key != NULL);
 
-    String *result = NULL;
-
-    MEM_CONTEXT_BEGIN(this->memContext)
-    {
-        result = varStr(kvGet(this->kv, varNewStr(key)));
-    }
-    MEM_CONTEXT_END();
-
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(varStr(kvGet(this->kv, VARSTR(key))));
 }
 
 /***********************************************************************************************************************************
@@ -145,12 +138,8 @@ httpQueryPut(HttpQuery *this, const String *key, const String *value)
     ASSERT(key != NULL);
     ASSERT(value != NULL);
 
-    MEM_CONTEXT_BEGIN(this->memContext)
-    {
-        // Store the key
-        kvPut(this->kv, varNewStr(key), varNewStr(value));
-    }
-    MEM_CONTEXT_END();
+    // Store the key
+    kvPut(this->kv, VARSTR(key), VARSTR(value));
 
     FUNCTION_TEST_RETURN(this);
 }
@@ -217,19 +206,3 @@ httpQueryToLog(const HttpQuery *this)
 
     return result;
 }
-
-/***********************************************************************************************************************************
-Free the object
-***********************************************************************************************************************************/
-void
-httpQueryFree(HttpQuery *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(HTTP_QUERY, this);
-    FUNCTION_TEST_END();
-
-    if (this != NULL)
-        memContextFree(this->memContext);
-
-    FUNCTION_TEST_RETURN_VOID();
-}
