@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 TLS Client
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <stdio.h>
@@ -14,6 +16,7 @@ TLS Client
 #include <openssl/ssl.h>
 #include <openssl/x509v3.h>
 
+#include "common/crypto/common.h"
 #include "common/debug.h"
 #include "common/log.h"
 #include "common/io/tls/client.h"
@@ -21,10 +24,15 @@ TLS Client
 #include "common/io/read.intern.h"
 #include "common/io/write.intern.h"
 #include "common/memContext.h"
+#include "common/object.h"
 #include "common/time.h"
 #include "common/type/keyValue.h"
 #include "common/wait.h"
-#include "crypto/crypto.h"
+
+/***********************************************************************************************************************************
+Statistics
+***********************************************************************************************************************************/
+static TlsClientStat tlsClientStatLocal;
 
 /***********************************************************************************************************************************
 Object type
@@ -45,6 +53,69 @@ struct TlsClient
     IoWrite *write;                                                 // Write interface
 };
 
+OBJECT_DEFINE_FREE(TLS_CLIENT);
+
+/***********************************************************************************************************************************
+Free connection
+***********************************************************************************************************************************/
+OBJECT_DEFINE_FREE_RESOURCE_BEGIN(TLS_CLIENT, LOG, logLevelTrace)
+{
+    tlsClientClose(this);
+    SSL_CTX_free(this->context);
+}
+OBJECT_DEFINE_FREE_RESOURCE_END(LOG);
+
+/***********************************************************************************************************************************
+Report TLS errors.  Returns true if the command should continue and false if it should exit.
+***********************************************************************************************************************************/
+static bool
+tlsError(TlsClient *this, int code)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
+        FUNCTION_LOG_PARAM(INT, code);
+    FUNCTION_LOG_END();
+
+    bool result = false;
+
+    switch (code)
+    {
+        // The connection was closed
+        case SSL_ERROR_ZERO_RETURN:
+        {
+            tlsClientClose(this);
+            break;
+        }
+
+        // Try the read/write again
+        case SSL_ERROR_WANT_READ:
+        case SSL_ERROR_WANT_WRITE:
+        {
+            result = true;
+            break;
+        }
+
+        // A syscall failed (this usually indicates eof)
+        case SSL_ERROR_SYSCALL:
+        {
+            // Get the error before closing so it is not cleared
+            int errNo = errno;
+            tlsClientClose(this);
+
+            // Throw the sys error if there is one
+            THROW_ON_SYS_ERROR(errNo, KernelError, "tls failed syscall");
+
+            break;
+        }
+
+        // Some other tls error that cannot be handled
+        default:
+            THROW_FMT(ServiceError, "tls error [%d]", code);
+    }
+
+    FUNCTION_LOG_RETURN(BOOL, result);
+}
+
 /***********************************************************************************************************************************
 New object
 ***********************************************************************************************************************************/
@@ -91,11 +162,14 @@ tlsClientNew(
         this->context = SSL_CTX_new(method);
         cryptoError(this->context == NULL, "unable to create TLS context");
 
-        memContextCallback(this->memContext, (MemContextCallback)tlsClientFree, this);
+        memContextCallbackSet(this->memContext, tlsClientFreeResource, this);
 
         // Exclude SSL versions to only allow TLS and also disable compression
         SSL_CTX_set_options(this->context, (long)(SSL_OP_ALL | SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_COMPRESSION));
 
+        // Disable auto-retry to prevent SSL_read() from hanging
+        SSL_CTX_clear_mode(this->context, SSL_MODE_AUTO_RETRY);
+
         // Set location of CA certificates if the server certificate will be verified
         // -------------------------------------------------------------------------------------------------------------------------
         if (this->verifyPeer)
@@ -111,6 +185,8 @@ tlsClientNew(
             else
                 cryptoError(SSL_CTX_set_default_verify_paths(this->context) != 1, "unable to set default CA certificate location");
         }
+
+        tlsClientStatLocal.object++;
     }
     MEM_CONTEXT_NEW_END();
 
@@ -238,17 +314,13 @@ tlsClientHostVerify(const String *host, X509 *certificate)
         if (!altNameFound)
         {
             X509_NAME *subjectName = X509_get_subject_name(certificate);
+            CHECK(subjectName != NULL);
 
-            if (subjectName != NULL)                            // {uncovered - not sure how to create cert with null common name}
-            {
-                int commonNameIndex = X509_NAME_get_index_by_NID(subjectName, NID_commonName, -1);
+            int commonNameIndex = X509_NAME_get_index_by_NID(subjectName, NID_commonName, -1);
+            CHECK(commonNameIndex >= 0);
 
-                if (commonNameIndex >= 0)                       // {uncovered - it seems this must be >= 0 if CN is not null}
-                {
-                    result = tlsClientHostVerifyName(
-                        host, asn1ToStr(X509_NAME_ENTRY_get_data(X509_NAME_get_entry(subjectName, commonNameIndex))));
-                }
-            }
+            result = tlsClientHostVerifyName(
+                host, asn1ToStr(X509_NAME_ENTRY_get_data(X509_NAME_get_entry(subjectName, commonNameIndex))));
         }
     }
     MEM_CONTEXT_TEMP_END();
@@ -257,9 +329,214 @@ tlsClientHostVerify(const String *host, X509 *certificate)
 }
 
 /***********************************************************************************************************************************
-Open connection if this is a new client or if the connection was closed by the server
+Wait for the socket to be readable
+***********************************************************************************************************************************/
+static void
+tlsClientReadWait(TlsClient *this)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(this->session != NULL);
+
+    // Initialize the file descriptor set used for select
+    fd_set selectSet;
+    FD_ZERO(&selectSet);
+
+    // We know the socket is not negative because it passed error handling, so it is safe to cast to unsigned
+    FD_SET((unsigned int)this->socket, &selectSet);
+
+    // Initialize timeout struct used for select.  Recreate this structure each time since Linux (at least) will modify it.
+    struct timeval timeoutSelect;
+    timeoutSelect.tv_sec = (time_t)(this->timeout / MSEC_PER_SEC);
+    timeoutSelect.tv_usec = (time_t)(this->timeout % MSEC_PER_SEC * 1000);
+
+    // Determine if there is data to be read
+    int result = select(this->socket + 1, &selectSet, NULL, NULL, &timeoutSelect);
+    THROW_ON_SYS_ERROR_FMT(result == -1, AssertError, "unable to select from '%s:%u'", strPtr(this->host), this->port);
+
+    // If no data read after time allotted then error
+    if (!result)
+    {
+        THROW_FMT(
+            FileReadError, "timeout after %" PRIu64 "ms waiting for read from '%s:%u'", this->timeout, strPtr(this->host),
+            this->port);
+    }
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+/***********************************************************************************************************************************
+Read from the TLS session
+***********************************************************************************************************************************/
+size_t
+tlsClientRead(THIS_VOID, Buffer *buffer, bool block)
+{
+    THIS(TlsClient);
+
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
+        FUNCTION_LOG_PARAM(BUFFER, buffer);
+        FUNCTION_LOG_PARAM(BOOL, block);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(this->session != NULL);
+    ASSERT(buffer != NULL);
+    ASSERT(!bufFull(buffer));
+
+    ssize_t result = 0;
+
+    // If blocking read keep reading until buffer is full
+    do
+    {
+        // If no tls data pending then check the socket
+        if (!SSL_pending(this->session))
+            tlsClientReadWait(this);
+
+        // Read and handle errors
+        size_t expectedBytes = bufRemains(buffer);
+        result = SSL_read(this->session, bufRemainsPtr(buffer), (int)expectedBytes);
+
+        if (result <= 0)
+        {
+            // Break if the error indicates that we should not continue trying
+            if (!tlsError(this, SSL_get_error(this->session, (int)result)))
+                break;
+        }
+        // Update amount of buffer used
+        else
+            bufUsedInc(buffer, (size_t)result);
+    }
+    while (block && bufRemains(buffer) > 0);
+
+    FUNCTION_LOG_RETURN(SIZE, (size_t)result);
+}
+
+/***********************************************************************************************************************************
+Write to the tls session
+***********************************************************************************************************************************/
+static bool
+tlsWriteContinue(TlsClient *this, int writeResult, int writeError, size_t writeSize)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
+        FUNCTION_LOG_PARAM(INT, writeResult);
+        FUNCTION_LOG_PARAM(INT, writeError);
+        FUNCTION_LOG_PARAM(SIZE, writeSize);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(writeSize > 0);
+
+    bool result = true;
+
+    // Handle errors
+    if (writeResult <= 0)
+    {
+        // If error = SSL_ERROR_NONE then this is the first write attempt so continue
+        if (writeError != SSL_ERROR_NONE)
+        {
+            // Error if the error indicates that we should not continue trying
+            if (!tlsError(this, writeError))
+                THROW_FMT(FileWriteError, "unable to write to tls [%d]", writeError);
+
+            // Wait for the socket to be readable for tls renegotiation
+            tlsClientReadWait(this);
+        }
+    }
+    else
+    {
+        if ((size_t)writeResult != writeSize)
+        {
+            THROW_FMT(
+                FileWriteError, "unable to write to tls, write size %d does not match expected size %zu", writeResult, writeSize);
+        }
+
+        result = false;
+    }
+
+    FUNCTION_LOG_RETURN(BOOL, result);
+}
+
+void
+tlsClientWrite(THIS_VOID, const Buffer *buffer)
+{
+    THIS(TlsClient);
+
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
+        FUNCTION_LOG_PARAM(BUFFER, buffer);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(this->session != NULL);
+    ASSERT(buffer != NULL);
+
+    int result = 0;
+    int error = SSL_ERROR_NONE;
+
+    while (tlsWriteContinue(this, result, error, bufUsed(buffer)))
+    {
+        result = SSL_write(this->session, bufPtr(buffer), (int)bufUsed(buffer));
+        error = SSL_get_error(this->session, result);
+    }
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+/***********************************************************************************************************************************
+Close the connection
 ***********************************************************************************************************************************/
 void
+tlsClientClose(TlsClient *this)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+
+    // Close the socket
+    if (this->socket != -1)
+    {
+        close(this->socket);
+        this->socket = -1;
+    }
+
+    // Free the TLS session
+    if (this->session != NULL)
+    {
+        SSL_free(this->session);
+        this->session = NULL;
+    }
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+/***********************************************************************************************************************************
+Has session been closed by the server?
+***********************************************************************************************************************************/
+bool
+tlsClientEof(THIS_VOID)
+{
+    THIS(TlsClient);
+
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+
+    FUNCTION_LOG_RETURN(BOOL, this->session == NULL);
+}
+
+/***********************************************************************************************************************************
+Open connection if this is a new client or if the connection was closed by the server
+***********************************************************************************************************************************/
+bool
 tlsClientOpen(TlsClient *this)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace)
@@ -268,6 +545,8 @@ tlsClientOpen(TlsClient *this)
 
     ASSERT(this != NULL);
 
+    bool result = false;
+
     if (this->session == NULL)
     {
         // Free the read/write interfaces
@@ -292,13 +571,13 @@ tlsClientOpen(TlsClient *this)
                     // Set hits that narrow the type of address we are looking for -- we'll take ipv4 or ipv6
                     struct addrinfo hints;
 
-                    memset(&hints, 0, sizeof hints);
+                    memset(&hints, 0, sizeof(hints));
                     hints.ai_family = AF_UNSPEC;
                     hints.ai_socktype = SOCK_STREAM;
                     hints.ai_protocol = IPPROTO_TCP;
 
                     // Convert the port to a zero-terminated string for use with getaddrinfo()
-                    char port[32];
+                    char port[CVT_BASE10_BUFFER_SIZE];
                     cvtUIntToZ(this->port, port, sizeof(port));
 
                     // Get an address for the host.  We are only going to try the first address returned.
@@ -370,6 +649,8 @@ tlsClientOpen(TlsClient *this)
                     {
                         LOG_DEBUG("retry %s: %s", errorTypeName(errorType()), errorMessage());
                         retry = true;
+
+                        tlsClientStatLocal.retry++;
                     }
 
                     tlsClientClose(this);
@@ -402,156 +683,50 @@ tlsClientOpen(TlsClient *this)
             X509_free(certificate);
 
             if (!nameResult)
-                THROW(CryptoError, "unable to find matching hostname in certificate");
+            {
+                THROW_FMT(
+                    CryptoError,
+                    "unable to find hostname '%s' in certificate common name or subject alternative names", strPtr(this->host));
+            }
         }
 
         MEM_CONTEXT_BEGIN(this->memContext)
         {
             // Create read and write interfaces
-            this->write = ioWriteNewP(this, .write = (IoWriteInterfaceWrite)tlsClientWrite);
+            this->write = ioWriteNewP(this, .write = tlsClientWrite);
             ioWriteOpen(this->write);
-            this->read = ioReadNewP(this, .eof = (IoReadInterfaceEof)tlsClientEof, .read = (IoReadInterfaceRead)tlsClientRead);
+            this->read = ioReadNewP(this, .block = true, .eof = tlsClientEof, .read = tlsClientRead);
             ioReadOpen(this->read);
         }
         MEM_CONTEXT_END();
+
+        tlsClientStatLocal.session++;
+        result = true;
     }
 
-    FUNCTION_LOG_RETURN_VOID();
+    tlsClientStatLocal.request++;
+
+    FUNCTION_LOG_RETURN(BOOL, result);
 }
 
 /***********************************************************************************************************************************
-Read from the TLS session
+Format statistics to a string
 ***********************************************************************************************************************************/
-size_t
-tlsClientRead(TlsClient *this, Buffer *buffer, bool block)
+String *
+tlsClientStatStr(void)
 {
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
-        FUNCTION_LOG_PARAM(BUFFER, buffer);
-        FUNCTION_LOG_PARAM(BOOL, block);
-    FUNCTION_LOG_END();
+    FUNCTION_TEST_VOID();
 
-    ASSERT(this != NULL);
-    ASSERT(this->session != NULL);
-    ASSERT(buffer != NULL);
-    ASSERT(!bufFull(buffer));
+    String *result = NULL;
 
-    ssize_t actualBytes = 0;
-
-    // If blocking read keep reading until buffer is full
-    do
+    if (tlsClientStatLocal.object > 0)
     {
-        // If no tls data pending then check the socket
-        if (!SSL_pending(this->session))
-        {
-            // Initialize the file descriptor set used for select
-            fd_set selectSet;
-            FD_ZERO(&selectSet);
-
-            // We know the socket is not negative because it passed error handling, so it is safe to cast to unsigned
-            FD_SET((unsigned int)this->socket, &selectSet);
-
-            // Initialize timeout struct used for select.  Recreate this structure each time since Linux (at least) will modify it.
-            struct timeval timeoutSelect;
-            timeoutSelect.tv_sec = (time_t)(this->timeout / MSEC_PER_SEC);
-            timeoutSelect.tv_usec = (time_t)(this->timeout % MSEC_PER_SEC * 1000);
-
-            // Determine if there is data to be read
-            int result = select(this->socket + 1, &selectSet, NULL, NULL, &timeoutSelect);
-            THROW_ON_SYS_ERROR_FMT(result == -1, AssertError, "unable to select from '%s:%u'", strPtr(this->host), this->port);
-
-            // If no data read after time allotted then error
-            if (!result)
-            {
-                THROW_FMT(
-                    FileReadError, "unable to read data from '%s:%u' after %" PRIu64 "ms",
-                    strPtr(this->host), this->port, this->timeout);
-            }
-        }
-
-        // Read and handle errors
-        size_t expectedBytes = bufRemains(buffer);
-        actualBytes = SSL_read(this->session, bufRemainsPtr(buffer), (int)expectedBytes);
-
-        cryptoError(actualBytes < 0, "unable to read from TLS");
-
-        // Update amount of buffer used
-        bufUsedInc(buffer, (size_t)actualBytes);
-
-        // If zero bytes were returned then the connection was closed
-        if (actualBytes == 0)
-        {
-            tlsClientClose(this);
-            break;
-        }
-    }
-    while (block && bufRemains(buffer) > 0);
-
-    FUNCTION_LOG_RETURN(SIZE, (size_t)actualBytes);
-}
-
-/***********************************************************************************************************************************
-Write to the tls session
-***********************************************************************************************************************************/
-void
-tlsClientWrite(TlsClient *this, const Buffer *buffer)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
-        FUNCTION_LOG_PARAM(BUFFER, buffer);
-    FUNCTION_LOG_END();
-
-    ASSERT(this != NULL);
-    ASSERT(this->session != NULL);
-    ASSERT(buffer != NULL);
-
-    cryptoError(SSL_write(this->session, bufPtr(buffer), (int)bufUsed(buffer)) != (int)bufUsed(buffer), "unable to write");
-
-    FUNCTION_LOG_RETURN_VOID();
-}
-
-/***********************************************************************************************************************************
-Close the connection
-***********************************************************************************************************************************/
-void
-tlsClientClose(TlsClient *this)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
-    FUNCTION_LOG_END();
-
-    ASSERT(this != NULL);
-
-    // Close the socket
-    if (this->socket != -1)
-    {
-        close(this->socket);
-        this->socket = -1;
+        result = strNewFmt(
+            "tls statistics: objects %" PRIu64 ", sessions %" PRIu64 ", requests %" PRIu64 ", retries %" PRIu64,
+            tlsClientStatLocal.object, tlsClientStatLocal.session, tlsClientStatLocal.request, tlsClientStatLocal.retry);
     }
 
-    // Free the TLS session
-    if (this->session != NULL)
-    {
-        SSL_free(this->session);
-        this->session = NULL;
-    }
-
-    FUNCTION_LOG_RETURN_VOID();
-}
-
-/***********************************************************************************************************************************
-Has session been closed by the server?
-***********************************************************************************************************************************/
-bool
-tlsClientEof(const TlsClient *this)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(TLS_CLIENT, this);
-    FUNCTION_LOG_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_LOG_RETURN(BOOL, this->session == NULL);
+    FUNCTION_TEST_RETURN(result);
 }
 
 /***********************************************************************************************************************************
@@ -583,25 +758,3 @@ tlsClientIoWrite(const TlsClient *this)
 
     FUNCTION_TEST_RETURN(this->write);
 }
-
-/***********************************************************************************************************************************
-Free the object
-***********************************************************************************************************************************/
-void
-tlsClientFree(TlsClient *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(TLS_CLIENT, this);
-    FUNCTION_TEST_END();
-
-    if (this != NULL)
-    {
-        tlsClientClose(this);
-        SSL_CTX_free(this->context);
-
-        memContextCallbackClear(this->memContext);
-        memContextFree(this->memContext);
-    }
-
-    FUNCTION_TEST_RETURN_VOID();
-}
