@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 IO Write Interface
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <string.h>
 
 #include "common/debug.h"
@@ -8,24 +10,28 @@ IO Write Interface
 #include "common/io/write.intern.h"
 #include "common/log.h"
 #include "common/memContext.h"
+#include "common/object.h"
 
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
 struct IoWrite
 {
-    MemContext *memContext;                                         // Mem context of driver
+    MemContext *memContext;                                         // Mem context
     void *driver;                                                   // Driver object
     IoWriteInterface interface;                                     // Driver interface
     IoFilterGroup *filterGroup;                                     // IO filters
     Buffer *output;                                                 // Output buffer
 
 #ifdef DEBUG
+    bool filterGroupSet;                                            // Were filters set?
     bool opened;                                                    // Has the io been opened?
     bool closed;                                                    // Has the io been closed?
 #endif
 };
 
+OBJECT_DEFINE_FREE(IO_WRITE);
+
 /***********************************************************************************************************************************
 New object
 ***********************************************************************************************************************************/
@@ -48,6 +54,7 @@ ioWriteNew(void *driver, IoWriteInterface interface)
         this->memContext = memContextCurrent();
         this->driver = driver;
         this->interface = interface;
+        this->filterGroup = ioFilterGroupNew();
         this->output = bufNew(ioBufferSize());
     }
     MEM_CONTEXT_NEW_END();
@@ -71,10 +78,12 @@ ioWriteOpen(IoWrite *this)
     if (this->interface.open != NULL)
         this->interface.open(this->driver);
 
-    // If no filter group exists create one to do buffering
-    if (this->filterGroup == NULL)
-        this->filterGroup = ioFilterGroupNew();
+    // Track whether filters were added to prevent flush() from being called later since flush() won't work with most filters
+#ifdef DEBUG
+    this->filterGroupSet = ioFilterGroupSize(this->filterGroup) > 0;
+#endif
 
+    // Open the filter group
     ioFilterGroupOpen(this->filterGroup);
 
 #ifdef DEBUG
@@ -122,7 +131,27 @@ ioWrite(IoWrite *this, const Buffer *buffer)
 Write linefeed-terminated string
 ***********************************************************************************************************************************/
 void
-ioWriteLine(IoWrite *this, const String *string)
+ioWriteLine(IoWrite *this, const Buffer *buffer)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(IO_WRITE, this);
+        FUNCTION_LOG_PARAM(BUFFER, buffer);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(buffer != NULL);
+
+    ioWrite(this, buffer);
+    ioWrite(this, LF_BUF);
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+/***********************************************************************************************************************************
+Write string
+***********************************************************************************************************************************/
+void
+ioWriteStr(IoWrite *this, const String *string)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(IO_WRITE, this);
@@ -131,20 +160,28 @@ ioWriteLine(IoWrite *this, const String *string)
 
     ASSERT(this != NULL);
     ASSERT(string != NULL);
-    ASSERT(this->opened && !this->closed);
 
-    MEM_CONTEXT_TEMP_BEGIN()
-    {
-        // Load a buffer with the linefeed-terminated string
-        Buffer *buffer = bufNew(strSize(string) + 1);
-        memcpy(bufPtr(buffer), strPtr(string), strSize(string));
-        bufPtr(buffer)[strSize(string)] = '\n';
-        bufUsedSet(buffer, bufSize(buffer));
+    ioWrite(this, BUFSTR(string));
 
-        // Write the string
-        ioWrite(this, buffer);
-    }
-    MEM_CONTEXT_TEMP_END()
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+/***********************************************************************************************************************************
+Write linefeed-terminated string
+***********************************************************************************************************************************/
+void
+ioWriteStrLine(IoWrite *this, const String *string)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(IO_WRITE, this);
+        FUNCTION_LOG_PARAM(STRING, string);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(string != NULL);
+
+    ioWrite(this, BUFSTR(string));
+    ioWrite(this, LF_BUF);
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -163,6 +200,7 @@ ioWriteFlush(IoWrite *this)
 
     ASSERT(this != NULL);
     ASSERT(this->opened && !this->closed);
+    ASSERT(!this->filterGroupSet);
 
     if (bufUsed(this->output) > 0)
     {
@@ -214,12 +252,27 @@ ioWriteClose(IoWrite *this)
     FUNCTION_LOG_RETURN_VOID();
 }
 
+/***********************************************************************************************************************************
+Interface for the write object
+***********************************************************************************************************************************/
+void *
+ioWriteDriver(IoWrite *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(IO_WRITE, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    FUNCTION_TEST_RETURN(this->driver);
+}
+
 /***********************************************************************************************************************************
 Get/set filters
 
 Filters must be set before open and cannot be reset.
 ***********************************************************************************************************************************/
-const IoFilterGroup *
+IoFilterGroup *
 ioWriteFilterGroup(const IoWrite *this)
 {
     FUNCTION_TEST_BEGIN();
@@ -227,41 +280,38 @@ ioWriteFilterGroup(const IoWrite *this)
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
-    ASSERT(this->opened && this->closed);
 
     FUNCTION_TEST_RETURN(this->filterGroup);
 }
 
-void
-ioWriteFilterGroupSet(IoWrite *this, IoFilterGroup *filterGroup)
+/***********************************************************************************************************************************
+Handle (file descriptor) for the write object
+
+No all write objects have a handle and -1 will be returned in that case.
+***********************************************************************************************************************************/
+int
+ioWriteHandle(const IoWrite *this)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(IO_WRITE, this);
-        FUNCTION_LOG_PARAM(IO_FILTER_GROUP, filterGroup);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
-    ASSERT(filterGroup != NULL);
-    ASSERT(this->filterGroup == NULL);
-    ASSERT(!this->opened && !this->closed);
 
-    this->filterGroup = filterGroup;
-
-    FUNCTION_LOG_RETURN_VOID();
+    FUNCTION_LOG_RETURN(INT, this->interface.handle == NULL ? -1 : this->interface.handle(this->driver));
 }
 
 /***********************************************************************************************************************************
-Free the object
+Interface for the write object
 ***********************************************************************************************************************************/
-void
-ioWriteFree(IoWrite *this)
+const IoWriteInterface *
+ioWriteInterface(const IoWrite *this)
 {
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(IO_WRITE, this);
-    FUNCTION_LOG_END();
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(IO_WRITE, this);
+    FUNCTION_TEST_END();
 
-    if (this != NULL)
-        memContextFree(this->memContext);
+    ASSERT(this != NULL);
 
-    FUNCTION_LOG_RETURN_VOID();
+    FUNCTION_TEST_RETURN(&this->interface);
 }
