@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Lock Handler
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
@@ -96,7 +98,7 @@ lockAcquireFile(const String *lockFile, TimeMSec lockTimeout, bool failOnNoLock)
                 const String *errorHint = NULL;
 
                 if (errNo == EWOULDBLOCK)
-                    errorHint = STRING_CONST("\nHINT: is another " PROJECT_NAME " process running?");
+                    errorHint = strNew("\nHINT: is another " PROJECT_NAME " process running?");
                 else if (errNo == EACCES)
                 {
                     errorHint = strNewFmt(
