@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Memory Context Manager
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -29,17 +31,20 @@ Contains information about the memory context
 struct MemContext
 {
     MemContextState state;                                          // Current state of the context
-    const char name[MEM_CONTEXT_NAME_SIZE + 1];                     // Indicates what the context is being used for
+    const char *name;                                               // Indicates what the context is being used for
 
     MemContext *contextParent;                                      // All contexts have a parent except top
+    unsigned int contextParentIdx;                                  // Index in the parent context list
 
     MemContext **contextChildList;                                  // List of contexts created in this context
     unsigned int contextChildListSize;                              // Size of child context list (not the actual count of contexts)
+    unsigned int contextChildFreeIdx;                               // Index of first free space in the context list
 
     MemContextAlloc *allocList;                                     // List of memory allocations created in this context
     unsigned int allocListSize;                                     // Size of alloc list (not the actual count of allocations)
+    unsigned int allocFreeIdx;                                      // Index of first free space in the alloc list
 
-    MemContextCallback callbackFunction;                            // Function to call before the context is freed
+    void (*callbackFunction)(void *);                               // Function to call before the context is freed
     void *callbackArgument;                                         // Argument to pass to callback function
 };
 
@@ -145,19 +150,17 @@ memContextNewIndex(MemContext *memContext, bool allowFree)
     ASSERT(memContext != NULL);
 
     // Try to find space for the new context
-    unsigned int contextIdx;
-
-    for (contextIdx = 0; contextIdx < memContext->contextChildListSize; contextIdx++)
+    for (; memContext->contextChildFreeIdx < memContext->contextChildListSize; memContext->contextChildFreeIdx++)
     {
-        if (!memContext->contextChildList[contextIdx] ||
-            (allowFree && memContext->contextChildList[contextIdx]->state == memContextStateFree))
+        if (!memContext->contextChildList[memContext->contextChildFreeIdx] ||
+            (allowFree && memContext->contextChildList[memContext->contextChildFreeIdx]->state == memContextStateFree))
         {
             break;
         }
     }
 
     // If no space was found then allocate more
-    if (contextIdx == memContext->contextChildListSize)
+    if (memContext->contextChildFreeIdx == memContext->contextChildListSize)
     {
         // If no space has been allocated to the list
         if (memContext->contextChildListSize == 0)
@@ -184,7 +187,7 @@ memContextNewIndex(MemContext *memContext, bool allowFree)
         }
     }
 
-    FUNCTION_TEST_RETURN(contextIdx);
+    FUNCTION_TEST_RETURN(memContext->contextChildFreeIdx);
 }
 
 /***********************************************************************************************************************************
@@ -200,31 +203,34 @@ memContextNew(const char *name)
     ASSERT(name != NULL);
 
     // Check context name length
-    if (strlen(name) == 0 || strlen(name) > MEM_CONTEXT_NAME_SIZE)
-        THROW_FMT(AssertError, "context name length must be > 0 and <= %d", MEM_CONTEXT_NAME_SIZE);
+    ASSERT(name[0] != '\0');
 
     // Find space for the new context
-    unsigned int contextIdx = memContextNewIndex(memContextCurrent(), true);
+    unsigned int contextIdx = memContextNewIndex(contextCurrent, true);
 
     // If the context has not been allocated yet
-    if (!memContextCurrent()->contextChildList[contextIdx])
-        memContextCurrent()->contextChildList[contextIdx] = memAllocInternal(sizeof(MemContext), true);
+    if (!contextCurrent->contextChildList[contextIdx])
+        contextCurrent->contextChildList[contextIdx] = memAllocInternal(sizeof(MemContext), true);
 
     // Get the context
-    MemContext *this = memContextCurrent()->contextChildList[contextIdx];
+    MemContext *this = contextCurrent->contextChildList[contextIdx];
 
     // Create initial space for allocations
     this->allocList = memAllocInternal(sizeof(MemContextAlloc) * MEM_CONTEXT_ALLOC_INITIAL_SIZE, true);
     this->allocListSize = MEM_CONTEXT_ALLOC_INITIAL_SIZE;
 
     // Set the context name
-    strcpy((char *)this->name, name);
+    this->name = name;
 
     // Set new context active
     this->state = memContextStateActive;
 
     // Set current context as the parent
-    this->contextParent = memContextCurrent();
+    this->contextParent = contextCurrent;
+    this->contextParentIdx = contextIdx;
+
+    // Possible free context must be in the next position
+    contextCurrent->contextChildFreeIdx++;
 
     // Return context
     FUNCTION_TEST_RETURN(this);
@@ -234,7 +240,7 @@ memContextNew(const char *name)
 Register a callback to be called just before the context is freed
 ***********************************************************************************************************************************/
 void
-memContextCallback(MemContext *this, void (*callbackFunction)(void *), void *callbackArgument)
+memContextCallbackSet(MemContext *this, void (*callbackFunction)(void *), void *callbackArgument)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(MEM_CONTEXT, this);
@@ -250,7 +256,7 @@ memContextCallback(MemContext *this, void (*callbackFunction)(void *), void *cal
         THROW(AssertError, "cannot assign callback to inactive context");
 
     // Top context cannot have a callback
-    if (this == memContextTop())
+    if (this == &contextTop)
         THROW(AssertError, "top context may not have a callback");
 
     // Error if callback has already been set - there may be valid use cases for this but error until one is found
@@ -281,7 +287,7 @@ memContextCallbackClear(MemContext *this)
     ASSERT(this->state == memContextStateActive || this->state == memContextStateFreeing);
 
     // Top context cannot have a callback
-    ASSERT(this != memContextTop());
+    ASSERT(this != &contextTop);
 
     // Clear callback function and argument
     this->callbackFunction = NULL;
@@ -302,47 +308,46 @@ memContextAlloc(size_t size, bool zero)
     FUNCTION_TEST_END();
 
     // Find space for the new allocation
-    unsigned int allocIdx;
-
-    for (allocIdx = 0; allocIdx < memContextCurrent()->allocListSize; allocIdx++)
-        if (!memContextCurrent()->allocList[allocIdx].active)
+    for (; contextCurrent->allocFreeIdx < contextCurrent->allocListSize; contextCurrent->allocFreeIdx++)
+        if (!contextCurrent->allocList[contextCurrent->allocFreeIdx].active)
             break;
 
     // If no space was found then allocate more
-    if (allocIdx == memContextCurrent()->allocListSize)
+    if (contextCurrent->allocFreeIdx == contextCurrent->allocListSize)
     {
         // Only the top context will not have initial space for allocations
-        if (memContextCurrent()->allocListSize == 0)
+        if (contextCurrent->allocListSize == 0)
         {
             // Allocate memory before modifying anything else in case there is an error
-            memContextCurrent()->allocList = memAllocInternal(sizeof(MemContextAlloc) * MEM_CONTEXT_ALLOC_INITIAL_SIZE, true);
+            contextCurrent->allocList = memAllocInternal(sizeof(MemContextAlloc) * MEM_CONTEXT_ALLOC_INITIAL_SIZE, true);
 
             // Set new size
-            memContextCurrent()->allocListSize = MEM_CONTEXT_ALLOC_INITIAL_SIZE;
+            contextCurrent->allocListSize = MEM_CONTEXT_ALLOC_INITIAL_SIZE;
         }
         // Else grow the list
         else
         {
             // Calculate new list size
-            unsigned int allocListSizeNew = memContextCurrent()->allocListSize * 2;
+            unsigned int allocListSizeNew = contextCurrent->allocListSize * 2;
 
             // ReAllocate memory before modifying anything else in case there is an error
-            memContextCurrent()->allocList = memReAllocInternal(
-                memContextCurrent()->allocList, sizeof(MemContextAlloc) * memContextCurrent()->allocListSize,
+            contextCurrent->allocList = memReAllocInternal(
+                contextCurrent->allocList, sizeof(MemContextAlloc) * contextCurrent->allocListSize,
                 sizeof(MemContextAlloc) * allocListSizeNew, true);
 
             // Set new size
-            memContextCurrent()->allocListSize = allocListSizeNew;
+            contextCurrent->allocListSize = allocListSizeNew;
         }
     }
 
     // Allocate the memory
-    memContextCurrent()->allocList[allocIdx].active = true;
-    memContextCurrent()->allocList[allocIdx].size = (unsigned int)size;
-    memContextCurrent()->allocList[allocIdx].buffer = memAllocInternal(size, zero);
+    contextCurrent->allocList[contextCurrent->allocFreeIdx].active = true;
+    contextCurrent->allocList[contextCurrent->allocFreeIdx].size = (unsigned int)size;
+    contextCurrent->allocList[contextCurrent->allocFreeIdx].buffer = memAllocInternal(size, zero);
+    contextCurrent->allocFreeIdx++;
 
     // Return buffer
-    FUNCTION_TEST_RETURN(memContextCurrent()->allocList[allocIdx].buffer);
+    FUNCTION_TEST_RETURN(contextCurrent->allocList[contextCurrent->allocFreeIdx - 1].buffer);
 }
 
 /***********************************************************************************************************************************
@@ -360,12 +365,12 @@ memFind(const void *buffer)
     // Find memory allocation
     unsigned int allocIdx;
 
-    for (allocIdx = 0; allocIdx < memContextCurrent()->allocListSize; allocIdx++)
-        if (memContextCurrent()->allocList[allocIdx].buffer == buffer && memContextCurrent()->allocList[allocIdx].active)
+    for (allocIdx = 0; allocIdx < contextCurrent->allocListSize; allocIdx++)
+        if (contextCurrent->allocList[allocIdx].buffer == buffer && contextCurrent->allocList[allocIdx].active)
             break;
 
     // Error if the buffer was not found
-    if (allocIdx == memContextCurrent()->allocListSize)
+    if (allocIdx == contextCurrent->allocListSize)
         THROW(AssertError, "unable to find allocation");
 
     FUNCTION_TEST_RETURN(allocIdx);
@@ -398,7 +403,7 @@ memGrowRaw(const void *buffer, size_t size)
     ASSERT(buffer != NULL);
 
     // Find the allocation
-    MemContextAlloc *alloc = &(memContextCurrent()->allocList[memFind(buffer)]);
+    MemContextAlloc *alloc = &(contextCurrent->allocList[memFind(buffer)]);
 
     // Grow the buffer
     alloc->buffer = memReAllocInternal(alloc->buffer, alloc->size, size, false);
@@ -433,12 +438,17 @@ memFree(void *buffer)
     ASSERT(buffer != NULL);
 
     // Find the allocation
-    MemContextAlloc *alloc = &(memContextCurrent()->allocList[memFind(buffer)]);
+    unsigned int allocIdx = memFind(buffer);
+    MemContextAlloc *alloc = &(contextCurrent->allocList[allocIdx]);
 
     // Free the buffer
     memFreeInternal(alloc->buffer);
     alloc->active = false;
 
+    // If this allocation is before the current free allocation then make it the current free allocation
+    if (allocIdx < contextCurrent->allocFreeIdx)
+        contextCurrent->allocFreeIdx = allocIdx;
+
     FUNCTION_TEST_RETURN_VOID();
 }
 
@@ -506,7 +516,7 @@ memContextSwitch(MemContext *this)
     if (this->state != memContextStateActive)
         THROW(AssertError, "cannot switch to inactive context");
 
-    MemContext *memContextOld = memContextCurrent();
+    MemContext *memContextOld = contextCurrent;
     contextCurrent = this;
 
     FUNCTION_TEST_RETURN(memContextOld);
@@ -567,7 +577,7 @@ memContextFree(MemContext *this)
     if (this->state != memContextStateFreeing)
     {
         // Current context cannot be freed unless it is top (top is never really freed, just the stuff under it)
-        if (this == memContextCurrent() && this != memContextTop())
+        if (this == contextCurrent && this != &contextTop)
             THROW_FMT(AssertError, "cannot free current context '%s'", this->name);
 
         // Error if context is not active
@@ -585,8 +595,20 @@ memContextFree(MemContext *this)
         this->state = memContextStateFreeing;
 
         // Execute callback if defined
+        bool rethrow = false;
+
         if (this->callbackFunction)
-            this->callbackFunction(this->callbackArgument);
+        {
+            TRY_BEGIN()
+            {
+                this->callbackFunction(this->callbackArgument);
+            }
+            CATCH_ANY()
+            {
+                rethrow = true;
+            }
+            TRY_END();
+        }
 
         // Free child context allocations
         if (this->contextChildListSize > 0)
@@ -614,12 +636,20 @@ memContextFree(MemContext *this)
             this->allocListSize = 0;
         }
 
+        // If the context index is lower than the current free index in the parent then replace it
+        if (this->contextParent != NULL && this->contextParentIdx < this->contextParent->contextChildFreeIdx)
+            this->contextParent->contextChildFreeIdx = this->contextParentIdx;
+
         // Make top context active again
-        if (this == memContextTop())
+        if (this == &contextTop)
             this->state = memContextStateActive;
         // Else reset the memory context so it can be reused
         else
             memset(this, 0, sizeof(MemContext));
+
+        // Rethrow the error that was caught in the callback
+        if (rethrow)
+            RETHROW();
     }
 
     FUNCTION_TEST_RETURN_VOID();
