@@ -13,11 +13,6 @@ typedef struct MemContext MemContext;
 
 #include "common/error.h"
 
-/***********************************************************************************************************************************
-Memory context names cannot be larger than this size (excluding terminator) or an error will be thrown
-***********************************************************************************************************************************/
-#define MEM_CONTEXT_NAME_SIZE                                       64
-
 /***********************************************************************************************************************************
 Define initial number of memory contexts
 
@@ -33,11 +28,6 @@ Space is reserved for this many allocations when a context is created.  When mor
 ***********************************************************************************************************************************/
 #define MEM_CONTEXT_ALLOC_INITIAL_SIZE                              4
 
-/***********************************************************************************************************************************
-Memory context callback function type, useful for casts in memContextCallback()
-***********************************************************************************************************************************/
-typedef void (*MemContextCallback)(void *callbackArgument);
-
 /***********************************************************************************************************************************
 Memory context management functions
 
@@ -65,7 +55,7 @@ Use the MEM_CONTEXT*() macros when possible rather than implement error-handling
 ***********************************************************************************************************************************/
 MemContext *memContextNew(const char *name);
 void memContextMove(MemContext *this, MemContext *parentNew);
-void memContextCallback(MemContext *this, void (*callbackFunction)(void *), void *callbackArgument);
+void memContextCallbackSet(MemContext *this, void (*callbackFunction)(void *), void *);
 void memContextCallbackClear(MemContext *this);
 MemContext *memContextSwitch(MemContext *this);
 void memContextFree(MemContext *this);
@@ -135,6 +125,8 @@ MEM_CONTEXT_NEW_BEGIN(memContextName)
 MEM_CONTEXT_NEW_END();
 
 <Old memory context is restored>
+
+Note that memory context names are expected to live for the lifetime of the context -- no copy is made.
 ***********************************************************************************************************************************/
 #define MEM_CONTEXT_NEW()                                                                                                          \
     MEM_CONTEXT_NEW_memContext
