@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Convert Base Data Types
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <ctype.h>
 #include <inttypes.h>
 #include <limits.h>
@@ -344,6 +346,28 @@ cvtSizeToZ(size_t value, char *buffer, size_t bufferSize)
     FUNCTION_TEST_RETURN(result);
 }
 
+/***********************************************************************************************************************************
+Convert ssize to zero-terminated string
+***********************************************************************************************************************************/
+size_t
+cvtSSizeToZ(ssize_t value, char *buffer, size_t bufferSize)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(SSIZE, value);
+        FUNCTION_TEST_PARAM_P(CHARDATA, buffer);
+        FUNCTION_TEST_PARAM(SIZE, bufferSize);
+    FUNCTION_TEST_END();
+
+    ASSERT(buffer != NULL);
+
+    size_t result = (size_t)snprintf(buffer, bufferSize, "%zd", value);
+
+    if (result >= bufferSize)
+        THROW(AssertError, "buffer overflow");
+
+    FUNCTION_TEST_RETURN(result);
+}
+
 /***********************************************************************************************************************************
 Convert uint to zero-terminated string
 ***********************************************************************************************************************************/
