@@ -7,6 +7,12 @@ Convert Base Data Types
 #include <inttypes.h>
 #include <sys/types.h>
 
+/***********************************************************************************************************************************
+Required buffer sizes
+***********************************************************************************************************************************/
+#define CVT_BOOL_BUFFER_SIZE                                        6
+#define CVT_BASE10_BUFFER_SIZE                                      64
+
 /***********************************************************************************************************************************
 Functions
 ***********************************************************************************************************************************/
@@ -26,6 +32,7 @@ int64_t cvtZToInt64Base(const char *value, int base);
 size_t cvtModeToZ(mode_t value, char *buffer, size_t bufferSize);
 
 size_t cvtSizeToZ(size_t value, char *buffer, size_t bufferSize);
+size_t cvtSSizeToZ(ssize_t value, char *buffer, size_t bufferSize);
 
 size_t cvtUIntToZ(unsigned int value, char *buffer, size_t bufferSize);
 unsigned int cvtZToUInt(const char *value);
