@@ -1,10 +1,13 @@
 /***********************************************************************************************************************************
 Key Value Handler
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <limits.h>
 
 #include "common/debug.h"
 #include "common/memContext.h"
+#include "common/object.h"
 #include "common/type/keyValue.h"
 #include "common/type/list.h"
 #include "common/type/variantList.h"
@@ -24,6 +27,8 @@ struct KeyValue
     VariantList *keyList;                                           // List of keys
 };
 
+OBJECT_DEFINE_FREE(KEY_VALUE);
+
 /***********************************************************************************************************************************
 Contains information about an individual key/value pair
 ***********************************************************************************************************************************/
@@ -123,6 +128,20 @@ kvGetIdx(const KeyValue *this, const Variant *key)
     FUNCTION_TEST_RETURN(result);
 }
 
+/***********************************************************************************************************************************
+Does the key exist (even if the value is NULL)
+***********************************************************************************************************************************/
+bool
+kvKeyExists(const KeyValue *this, const Variant *key)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(KEY_VALUE, this);
+        FUNCTION_TEST_PARAM(VARIANT, key);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(kvGetIdx(this, key) != KEY_NOT_FOUND);
+}
+
 /***********************************************************************************************************************************
 Get list of keys
 ***********************************************************************************************************************************/
@@ -279,10 +298,8 @@ kvPutKv(KeyValue *this, const Variant *key)
 
     MEM_CONTEXT_BEGIN(this->memContext)
     {
-        Variant *keyValue = varNewKv();
-        result = varKv(keyValue);
-
-        kvPutInternal(this, key, keyValue);
+        result = kvNew();
+        kvPutInternal(this, key, varNewKv(result));
     }
     MEM_CONTEXT_END();
 
@@ -360,19 +377,3 @@ kvMove(KeyValue *this, MemContext *parentNew)
 
     FUNCTION_TEST_RETURN(this);
 }
-
-/***********************************************************************************************************************************
-Free the string
-***********************************************************************************************************************************/
-void
-kvFree(KeyValue *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(KEY_VALUE, this);
-    FUNCTION_TEST_END();
-
-    if (this != NULL)
-        memContextFree(this->memContext);
-
-    FUNCTION_TEST_RETURN_VOID();
-}
