@@ -7,6 +7,9 @@ Key Value Handler
 /***********************************************************************************************************************************
 KeyValue object
 ***********************************************************************************************************************************/
+#define KEY_VALUE_TYPE                                              KeyValue
+#define KEY_VALUE_PREFIX                                            kv
+
 typedef struct KeyValue KeyValue;
 
 #include "common/type/variantList.h"
@@ -22,6 +25,7 @@ KeyValue *kvMove(KeyValue *this, MemContext *parentNew);
 KeyValue *kvPut(KeyValue *this, const Variant *key, const Variant *value);
 KeyValue *kvPutKv(KeyValue *this, const Variant *key);
 const Variant *kvGet(const KeyValue *this, const Variant *key);
+bool kvKeyExists(const KeyValue *this, const Variant *key);
 VariantList *kvGetList(const KeyValue *this, const Variant *key);
 void kvFree(KeyValue *this);
 
