@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 List Handler
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -8,9 +10,10 @@ List Handler
 
 #include "common/debug.h"
 #include "common/type/list.h"
+#include "common/object.h"
 
 /***********************************************************************************************************************************
-Contains information about the list
+Object type
 ***********************************************************************************************************************************/
 struct List
 {
@@ -21,6 +24,8 @@ struct List
     unsigned char *list;
 };
 
+OBJECT_DEFINE_FREE(LIST);
+
 /***********************************************************************************************************************************
 Create a new list
 ***********************************************************************************************************************************/
@@ -62,6 +67,33 @@ lstAdd(List *this, const void *item)
     FUNCTION_TEST_RETURN(this);
 }
 
+/***********************************************************************************************************************************
+Clear items from a list
+***********************************************************************************************************************************/
+List *
+lstClear(List *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(LIST, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    if (this->list != NULL)
+    {
+        MEM_CONTEXT_BEGIN(this->memContext)
+        {
+            memFree(this->list);
+        }
+        MEM_CONTEXT_END();
+
+        this->listSize = 0;
+        this->listSizeMax = 0;
+    }
+
+    FUNCTION_TEST_RETURN(this);
+}
+
 /***********************************************************************************************************************************
 Get an item from the list
 ***********************************************************************************************************************************/
@@ -134,6 +166,29 @@ lstInsert(List *this, unsigned int listIdx, const void *item)
     FUNCTION_TEST_RETURN(this);
 }
 
+/***********************************************************************************************************************************
+Remove an item from the list
+***********************************************************************************************************************************/
+List *
+lstRemove(List *this, unsigned int listIdx)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(LIST, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+    ASSERT(listIdx <= lstSize(this));
+
+    // Remove the item by moving the items after it down
+    this->listSize--;
+
+    memmove(
+        this->list + (listIdx * this->itemSize), this->list + ((listIdx + 1) * this->itemSize),
+        (lstSize(this) - listIdx) * this->itemSize);
+
+    FUNCTION_TEST_RETURN(this);
+}
+
 /***********************************************************************************************************************************
 Return the memory context for this list
 ***********************************************************************************************************************************/
@@ -210,19 +265,3 @@ lstToLog(const List *this)
 {
     return strNewFmt("{size: %u}", this->listSize);
 }
-
-/***********************************************************************************************************************************
-Free the string
-***********************************************************************************************************************************/
-void
-lstFree(List *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(LIST, this);
-    FUNCTION_TEST_END();
-
-    if (this != NULL)
-        memContextFree(this->memContext);
-
-    FUNCTION_TEST_RETURN_VOID();
-}
