@@ -7,6 +7,9 @@ List Handler
 /***********************************************************************************************************************************
 List object
 ***********************************************************************************************************************************/
+#define LIST_TYPE                                                   List
+#define LIST_PREFIX                                                 lst
+
 typedef struct List List;
 
 #include "common/memContext.h"
@@ -22,8 +25,10 @@ Functions
 ***********************************************************************************************************************************/
 List *lstNew(size_t itemSize);
 List *lstAdd(List *this, const void *item);
+List *lstClear(List *this);
 void *lstGet(const List *this, unsigned int listIdx);
 List *lstInsert(List *this, unsigned int listIdx, const void *item);
+List *lstRemove(List *this, unsigned int listIdx);
 MemContext *lstMemContext(const List *this);
 List *lstMove(List *this, MemContext *parentNew);
 unsigned int lstSize(const List *this);
