@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Variant Data Type
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <errno.h>
 #include <inttypes.h>
 #include <limits.h>
@@ -14,15 +16,87 @@ Variant Data Type
 #include "common/type/convert.h"
 #include "common/type/variant.h"
 
+/***********************************************************************************************************************************
+Constant variants that are generally useful
+***********************************************************************************************************************************/
+// Used to declare Bool Variant constants that will be externed using VARIANT_DECLARE().  Must be used in a .c file.
+#define VARIANT_BOOL_EXTERN(name, dataParam)                                                                                       \
+    const Variant *name = ((const Variant *)&(const VariantBoolConst){.type = varTypeBool, .data = dataParam})
+
+VARIANT_BOOL_EXTERN(BOOL_FALSE_VAR,                                 false);
+VARIANT_BOOL_EXTERN(BOOL_TRUE_VAR,                                  true);
+
 /***********************************************************************************************************************************
 Information about the variant
 ***********************************************************************************************************************************/
 struct Variant
 {
-    MemContext *memContext;                                         // Mem context
-    unsigned int type:3;                                            // Variant Type
+    VARIANT_COMMON
 };
 
+typedef struct VariantBool
+{
+    VARIANT_COMMON
+    VARIANT_BOOL_COMMON
+    MemContext *memContext;
+} VariantBool;
+
+typedef struct VariantDouble
+{
+    VARIANT_COMMON
+    VARIANT_DOUBLE_COMMON
+    MemContext *memContext;
+} VariantDouble;
+
+typedef struct VariantInt
+{
+    VARIANT_COMMON
+    VARIANT_INT_COMMON
+    MemContext *memContext;
+} VariantInt;
+
+typedef struct VariantInt64
+{
+    VARIANT_COMMON
+    VARIANT_INT64_COMMON
+    MemContext *memContext;
+} VariantInt64;
+
+typedef struct VariantKeyValue
+{
+    VARIANT_COMMON
+    KeyValue *data;                                                 /* KeyValue data */
+    MemContext *memContext;
+} VariantKeyValue;
+
+typedef struct VariantString
+{
+    VARIANT_COMMON
+    VARIANT_STRING_COMMON
+    MemContext *memContext;
+} VariantString;
+
+typedef struct VariantUInt
+{
+    VARIANT_COMMON
+    VARIANT_UINT_COMMON
+    MemContext *memContext;
+} VariantUInt;
+
+typedef struct VariantUInt64
+{
+    VARIANT_COMMON
+    VARIANT_UINT64_COMMON
+    MemContext *memContext;
+} VariantUInt64;
+
+typedef struct VariantVariantList
+{
+    VARIANT_COMMON
+    VariantList *data;                                              /* VariantList data */
+    MemContext *memContext;
+} VariantVariantList;
+
 /***********************************************************************************************************************************
 Variant type names
 ***********************************************************************************************************************************/
@@ -34,51 +108,11 @@ static const char *variantTypeName[] =
     "int64",                                                        // varTypeInt64
     "KeyValue",                                                     // varTypeKeyValue
     "String",                                                       // varTypeString
-    "VariantList",                                                  // varTypeVariantList
+    "unsigned int",                                                 // varTypeUInt
     "uint64",                                                       // varTypeUInt64
+    "VariantList",                                                  // varTypeVariantList
 };
 
-/***********************************************************************************************************************************
-New variant of any supported type
-***********************************************************************************************************************************/
-static Variant *
-varNewInternal(VariantType type, void *data, size_t dataSize)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(ENUM, type);
-        FUNCTION_TEST_PARAM_P(VOID, data);
-        FUNCTION_TEST_PARAM(SIZE, dataSize);
-    FUNCTION_TEST_END();
-
-    ASSERT(data != NULL);
-    ASSERT(dataSize > 0);
-
-    // Allocate memory for the variant and set the type
-    Variant *this = memNew(sizeof(Variant) + dataSize);
-    this->memContext = memContextCurrent();
-    this->type = type;
-
-    // Copy data
-    memcpy((unsigned char *)this + sizeof(Variant), data, dataSize);
-
-    FUNCTION_TEST_RETURN(this);
-}
-
-/***********************************************************************************************************************************
-Get a pointer to the data stored in the variant.  This hides the complicated pointer arithmetic.
-***********************************************************************************************************************************/
-static void *
-varData(const Variant *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(VARIANT, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN((void *)((unsigned char *)this + sizeof(Variant)));
-}
-
 /***********************************************************************************************************************************
 Duplicate a variant
 ***********************************************************************************************************************************/
@@ -119,16 +153,14 @@ varDup(const Variant *this)
                 break;
             }
 
-            case varTypeUInt64:
-            {
-                result = varNewUInt64(varUInt64(this));
-                break;
-            }
-
             case varTypeKeyValue:
             {
-                KeyValue *data = kvDup(varKv(this));
-                result = varNewInternal(varTypeKeyValue, (void *)&data, sizeof(data));
+                VariantKeyValue *keyValue = memNew(sizeof(VariantKeyValue));
+                keyValue->memContext = memContextCurrent();
+                keyValue->type = varTypeKeyValue;
+                keyValue->data = kvDup(varKv(this));
+
+                result = (Variant *)keyValue;
                 break;
             }
 
@@ -138,6 +170,18 @@ varDup(const Variant *this)
                 break;
             }
 
+            case varTypeUInt:
+            {
+                result = varNewUInt(varUInt(this));
+                break;
+            }
+
+            case varTypeUInt64:
+            {
+                result = varNewUInt64(varUInt64(this));
+                break;
+            }
+
             case varTypeVariantList:
             {
                 result = varNewVarLst(varVarLst(this));
@@ -194,20 +238,25 @@ varEq(const Variant *this1, const Variant *this2)
                     break;
                 }
 
-                case varTypeUInt64:
-                {
-                    result = varUInt64(this1) == varUInt64(this2);
-                    break;
-                }
-
                 case varTypeString:
                 {
                     result = strEq(varStr(this1), varStr(this2));
                     break;
                 }
 
-                case varTypeKeyValue:
-                case varTypeVariantList:
+                case varTypeUInt:
+                {
+                    result = varUInt(this1) == varUInt(this2);
+                    break;
+                }
+
+                case varTypeUInt64:
+                {
+                    result = varUInt64(this1) == varUInt64(this2);
+                    break;
+                }
+
+                default:
                     THROW_FMT(AssertError, "unable to test equality for %s", variantTypeName[this1->type]);
             }
         }
@@ -244,7 +293,13 @@ varNewBool(bool data)
         FUNCTION_TEST_PARAM(BOOL, data);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(varNewInternal(varTypeBool, (void *)&data, sizeof(data)));
+    // Allocate memory for the variant and set the type and data
+    VariantBool *this = memNew(sizeof(VariantBool));
+    this->memContext = memContextCurrent();
+    this->type = varTypeBool;
+    this->data = data;
+
+    FUNCTION_TEST_RETURN((Variant *)this);
 }
 
 /***********************************************************************************************************************************
@@ -258,10 +313,9 @@ varBool(const Variant *this)
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
-
     ASSERT(this->type == varTypeBool);
 
-    FUNCTION_TEST_RETURN(*((bool *)varData(this)));
+    FUNCTION_TEST_RETURN(((VariantBool *)this)->data);
 }
 
 /***********************************************************************************************************************************
@@ -292,10 +346,6 @@ varBoolForce(const Variant *this)
             result = varInt64(this) != 0;
             break;
 
-        case varTypeUInt64:
-            result = varUInt64(this) != 0;
-            break;
-
         case varTypeString:
         {
             // List of false/true boolean string values.  Note that false/true values must be equal.
@@ -323,6 +373,14 @@ varBoolForce(const Variant *this)
             break;
         }
 
+        case varTypeUInt:
+            result = varUInt(this) != 0;
+            break;
+
+        case varTypeUInt64:
+            result = varUInt64(this) != 0;
+            break;
+
         default:
             THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[this->type], variantTypeName[varTypeBool]);
     }
@@ -340,7 +398,13 @@ varNewDbl(double data)
         FUNCTION_TEST_PARAM(DOUBLE, data);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(varNewInternal(varTypeDouble, (unsigned char *)&data, sizeof(data)));
+    // Allocate memory for the variant and set the type and data
+    VariantDouble *this = memNew(sizeof(VariantDouble));
+    this->memContext = memContextCurrent();
+    this->type = varTypeDouble;
+    this->data = data;
+
+    FUNCTION_TEST_RETURN((Variant *)this);
 }
 
 /***********************************************************************************************************************************
@@ -354,10 +418,9 @@ varDbl(const Variant *this)
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
-
     ASSERT(this->type == varTypeDouble);
 
-    FUNCTION_TEST_RETURN(*((double *)varData(this)));
+    FUNCTION_TEST_RETURN(((VariantDouble *)this)->data);
 }
 
 /***********************************************************************************************************************************
@@ -400,18 +463,24 @@ varDblForce(const Variant *this)
             break;
         }
 
-        case varTypeUInt64:
-        {
-            result = (double)varUInt64(this);
-            break;
-        }
-
         case varTypeString:
         {
             result = cvtZToDouble(strPtr(varStr(this)));
             break;
         }
 
+        case varTypeUInt:
+        {
+            result = (double)varUInt(this);
+            break;
+        }
+
+        case varTypeUInt64:
+        {
+            result = (double)varUInt64(this);
+            break;
+        }
+
         default:
             THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[this->type], variantTypeName[varTypeDouble]);
     }
@@ -429,7 +498,13 @@ varNewInt(int data)
         FUNCTION_TEST_PARAM(INT, data);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(varNewInternal(varTypeInt, (void *)&data, sizeof(data)));
+    // Allocate memory for the variant and set the type and data
+    VariantInt *this = memNew(sizeof(VariantInt));
+    this->memContext = memContextCurrent();
+    this->type = varTypeInt;
+    this->data = data;
+
+    FUNCTION_TEST_RETURN((Variant *)this);
 }
 
 /***********************************************************************************************************************************
@@ -443,10 +518,9 @@ varInt(const Variant *this)
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
-
     ASSERT(this->type == varTypeInt);
 
-    FUNCTION_TEST_RETURN(*((int *)varData(this)));
+    FUNCTION_TEST_RETURN(((VariantInt *)this)->data);
 }
 
 /***********************************************************************************************************************************
@@ -491,6 +565,26 @@ varIntForce(const Variant *this)
             break;
         }
 
+        case varTypeString:
+        {
+            result = cvtZToInt(strPtr(varStr(this)));
+            break;
+        }
+
+        case varTypeUInt:
+        {
+            unsigned int resultTest = varUInt(this);
+
+            // Make sure the value fits into a normal 32-bit int range
+            if (resultTest > INT32_MAX)
+                THROW_FMT(
+                    FormatError, "unable to convert %s %u to %s", variantTypeName[this->type], resultTest,
+                    variantTypeName[varTypeInt]);
+
+            result = (int)resultTest;
+            break;
+        }
+
         case varTypeUInt64:
         {
             uint64_t resultTest = varUInt64(this);
@@ -505,12 +599,6 @@ varIntForce(const Variant *this)
             break;
         }
 
-        case varTypeString:
-        {
-            result = cvtZToInt(strPtr(varStr(this)));
-            break;
-        }
-
         default:
             THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[this->type], variantTypeName[varTypeInt]);
     }
@@ -528,7 +616,13 @@ varNewInt64(int64_t data)
         FUNCTION_TEST_PARAM(INT64, data);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(varNewInternal(varTypeInt64, (void *)&data, sizeof(data)));
+    // Allocate memory for the variant and set the type and data
+    VariantInt64 *this = memNew(sizeof(VariantInt64));
+    this->memContext = memContextCurrent();
+    this->type = varTypeInt64;
+    this->data = data;
+
+    FUNCTION_TEST_RETURN((Variant *)this);
 }
 
 /***********************************************************************************************************************************
@@ -542,10 +636,9 @@ varInt64(const Variant *this)
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
-
     ASSERT(this->type == varTypeInt64);
 
-    FUNCTION_TEST_RETURN(*((int64_t *)varData(this)));
+    FUNCTION_TEST_RETURN(((VariantInt64 *)this)->data);
 }
 
 /***********************************************************************************************************************************
@@ -582,6 +675,19 @@ varInt64Force(const Variant *this)
             break;
         }
 
+        case varTypeString:
+        {
+            result = cvtZToInt64(strPtr(varStr(this)));
+            break;
+        }
+
+        case varTypeUInt:
+        {
+            result = varUInt(this);
+
+            break;
+        }
+
         case varTypeUInt64:
         {
             uint64_t resultTest = varUInt64(this);
@@ -599,14 +705,135 @@ varInt64Force(const Variant *this)
             break;
         }
 
+        default:
+            THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[this->type], variantTypeName[varTypeInt64]);
+    }
+
+    FUNCTION_TEST_RETURN(result);
+}
+
+/***********************************************************************************************************************************
+New unsigned int variant
+***********************************************************************************************************************************/
+Variant *
+varNewUInt(unsigned int data)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(UINT, data);
+    FUNCTION_TEST_END();
+
+    // Allocate memory for the variant and set the type and data
+    VariantUInt *this = memNew(sizeof(VariantUInt));
+    this->memContext = memContextCurrent();
+    this->type = varTypeUInt;
+    this->data = data;
+
+    FUNCTION_TEST_RETURN((Variant *)this);
+}
+
+/***********************************************************************************************************************************
+Return unsigned int
+***********************************************************************************************************************************/
+unsigned int
+varUInt(const Variant *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(VARIANT, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+    ASSERT(this->type == varTypeUInt);
+
+    FUNCTION_TEST_RETURN(((VariantUInt *)this)->data);
+}
+
+/***********************************************************************************************************************************
+Return unsigned int regardless of variant type
+***********************************************************************************************************************************/
+unsigned int
+varUIntForce(const Variant *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(VARIANT, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    unsigned int result = 0;
+
+    switch (this->type)
+    {
+        case varTypeBool:
+        {
+            result = varBool(this);
+            break;
+        }
+
+        case varTypeInt:
+        {
+            int resultTest = varInt(this);
+
+            // If integer is a negative number, throw an error since the resulting conversion would be a different number
+            if (resultTest >= 0)
+                result = (unsigned int)resultTest;
+            else
+            {
+                THROW_FMT(
+                    FormatError, "unable to convert %s %d to %s", variantTypeName[this->type], resultTest,
+                    variantTypeName[varTypeUInt]);
+            }
+
+            break;
+        }
+
+        case varTypeInt64:
+        {
+            int64_t resultTest = varInt64(this);
+
+            // If integer is a negative number or too large, throw an error since the resulting conversion would be out of bounds
+            if (resultTest >= 0 && resultTest <= UINT_MAX)
+                result = (unsigned int)resultTest;
+            else
+            {
+                THROW_FMT(
+                    FormatError, "unable to convert %s %" PRId64 " to %s", variantTypeName[this->type], resultTest,
+                    variantTypeName[varTypeUInt]);
+            }
+
+            break;
+        }
+
+        case varTypeUInt:
+        {
+            result = varUInt(this);
+            break;
+        }
+
+        case varTypeUInt64:
+        {
+            uint64_t resultTest = varUInt64(this);
+
+            // If integer is too large, throw an error since the resulting conversion would be out of bounds
+            if (resultTest <= UINT_MAX)
+                result = (unsigned int)resultTest;
+            else
+            {
+                THROW_FMT(
+                    FormatError, "unable to convert %s %" PRIu64 " to %s", variantTypeName[this->type], resultTest,
+                    variantTypeName[varTypeUInt]);
+            }
+
+            break;
+        }
+
         case varTypeString:
         {
-            result = cvtZToInt64(strPtr(varStr(this)));
+            result = cvtZToUInt(strPtr(varStr(this)));
             break;
         }
 
         default:
-            THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[this->type], variantTypeName[varTypeInt64]);
+            THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[this->type], variantTypeName[varTypeUInt]);
     }
 
     FUNCTION_TEST_RETURN(result);
@@ -622,7 +849,13 @@ varNewUInt64(uint64_t data)
         FUNCTION_TEST_PARAM(UINT64, data);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(varNewInternal(varTypeUInt64, (void *)&data, sizeof(data)));
+    // Allocate memory for the variant and set the type and data
+    VariantUInt64 *this = memNew(sizeof(VariantUInt64));
+    this->memContext = memContextCurrent();
+    this->type = varTypeUInt64;
+    this->data = data;
+
+    FUNCTION_TEST_RETURN((Variant *)this);
 }
 
 /***********************************************************************************************************************************
@@ -636,10 +869,9 @@ varUInt64(const Variant *this)
     FUNCTION_TEST_END();
 
     ASSERT(this != NULL);
-
     ASSERT(this->type == varTypeUInt64);
 
-    FUNCTION_TEST_RETURN(*((uint64_t *)varData(this)));
+    FUNCTION_TEST_RETURN(((VariantUInt64 *)this)->data);
 }
 
 /***********************************************************************************************************************************
@@ -698,18 +930,24 @@ varUInt64Force(const Variant *this)
             break;
         }
 
-        case varTypeUInt64:
-        {
-            result = varUInt64(this);
-            break;
-        }
-
         case varTypeString:
         {
             result = cvtZToUInt64(strPtr(varStr(this)));
             break;
         }
 
+        case varTypeUInt:
+        {
+            result = varUInt(this);
+            break;
+        }
+
+        case varTypeUInt64:
+        {
+            result = varUInt64(this);
+            break;
+        }
+
         default:
             THROW_FMT(AssertError, "unable to force %s to %s", variantTypeName[this->type], variantTypeName[varTypeUInt64]);
     }
@@ -719,16 +957,25 @@ varUInt64Force(const Variant *this)
 
 /***********************************************************************************************************************************
 New key/value variant
+
+Note that the kv is not duped because it this a heavy-weight operation.  It is merely moved into the same MemContext as the Variant.
 ***********************************************************************************************************************************/
 Variant *
-varNewKv(void)
+varNewKv(KeyValue *data)
 {
-    FUNCTION_TEST_VOID();
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(KEY_VALUE, data);
+    FUNCTION_TEST_END();
 
-    // Create a new kv for the variant
-    KeyValue *data = kvNew();
+    // Allocate memory for the variant and set the type and data
+    VariantKeyValue *this = memNew(sizeof(VariantKeyValue));
+    this->memContext = memContextCurrent();
+    this->type = varTypeKeyValue;
 
-    FUNCTION_TEST_RETURN(varNewInternal(varTypeKeyValue, (void *)&data, sizeof(data)));
+    if (data != NULL)
+        this->data = kvMove(data, memContextCurrent());
+
+    FUNCTION_TEST_RETURN((Variant *)this);
 }
 
 /***********************************************************************************************************************************
@@ -746,7 +993,7 @@ varKv(const Variant *this)
     if (this != NULL)
     {
         ASSERT(this->type == varTypeKeyValue);
-        result = *((KeyValue **)varData(this));
+        result = ((VariantKeyValue *)this)->data;
     }
 
     FUNCTION_TEST_RETURN(result);
@@ -762,10 +1009,13 @@ varNewStr(const String *data)
         FUNCTION_TEST_PARAM(STRING, data);
     FUNCTION_TEST_END();
 
-    // Create a copy of the string for the variant
-    String *dataCopy = strDup(data);
+    // Allocate memory for the variant and set the type and data
+    VariantString *this = memNew(sizeof(VariantString));
+    this->memContext = memContextCurrent();
+    this->type = varTypeString;
+    this->data = strDup(data);
 
-    FUNCTION_TEST_RETURN(varNewInternal(varTypeString, (void *)&dataCopy, sizeof(dataCopy)));
+    FUNCTION_TEST_RETURN((Variant *)this);
 }
 
 /***********************************************************************************************************************************
@@ -778,19 +1028,13 @@ varNewStrZ(const char *data)
         FUNCTION_TEST_PARAM(STRINGZ, data);
     FUNCTION_TEST_END();
 
-    // Create a string for the variant
-    String *dataCopy = NULL;
-
-    if (data != NULL)
-        dataCopy = strNew(data);
-
-    FUNCTION_TEST_RETURN(varNewInternal(varTypeString, (void *)&dataCopy, sizeof(dataCopy)));
+    FUNCTION_TEST_RETURN(varNewStr(data == NULL ? NULL : strNew(data)));
 }
 
 /***********************************************************************************************************************************
 Return string
 ***********************************************************************************************************************************/
-String *
+const String *
 varStr(const Variant *this)
 {
     FUNCTION_TEST_BEGIN();
@@ -802,7 +1046,7 @@ varStr(const Variant *this)
     if (this != NULL)
     {
         ASSERT(this->type == varTypeString);
-        result = *((String **)varData(this));
+        result = ((VariantString *)this)->data;
     }
 
     FUNCTION_TEST_RETURN(result);
@@ -826,27 +1070,22 @@ varStrForce(const Variant *this)
     {
         case varTypeBool:
         {
-            char working[6];
-
-            cvtBoolToZ(varBool(this), working, sizeof(working));
-            result = strNew(working);
-
+            result = strNew(cvtBoolToConstZ(varBool(this)));
             break;
         }
 
         case varTypeDouble:
         {
-            char working[64];
+            char working[CVT_BASE10_BUFFER_SIZE];
 
             cvtDoubleToZ(varDbl(this), working, sizeof(working));
             result = strNew(working);
-
             break;
         }
 
         case varTypeInt:
         {
-            char working[64];
+            char working[CVT_BASE10_BUFFER_SIZE];
 
             cvtIntToZ(varInt(this), working, sizeof(working));
             result = strNew(working);
@@ -855,27 +1094,38 @@ varStrForce(const Variant *this)
 
         case varTypeInt64:
         {
-            char working[64];
+            char working[CVT_BASE10_BUFFER_SIZE];
 
             cvtInt64ToZ(varInt64(this), working, sizeof(working));
             result = strNew(working);
             break;
         }
 
-        case varTypeUInt64:
-        {
-            result = strNewFmt("%" PRIu64, varUInt64(this));
-            break;
-        }
-
         case varTypeString:
         {
             result = strDup(varStr(this));
             break;
         }
 
-        case varTypeKeyValue:
-        case varTypeVariantList:
+        case varTypeUInt:
+        {
+            char working[CVT_BASE10_BUFFER_SIZE];
+
+            cvtUIntToZ(varUInt(this), working, sizeof(working));
+            result = strNew(working);
+            break;
+        }
+
+        case varTypeUInt64:
+        {
+            char working[CVT_BASE10_BUFFER_SIZE];
+
+            cvtUInt64ToZ(varUInt64(this), working, sizeof(working));
+            result = strNew(working);
+            break;
+        }
+
+        default:
             THROW_FMT(FormatError, "unable to force %s to %s", variantTypeName[this->type], variantTypeName[varTypeString]);
     }
 
@@ -892,13 +1142,15 @@ varNewVarLst(const VariantList *data)
         FUNCTION_TEST_PARAM(VARIANT_LIST, data);
     FUNCTION_TEST_END();
 
-    VariantList *dataCopy = NULL;
+    // Allocate memory for the variant and set the type and data
+    VariantVariantList *this = memNew(sizeof(VariantVariantList));
+    this->memContext = memContextCurrent();
+    this->type = varTypeVariantList;
 
-    // Create a copy of the variant list if it is not null
     if (data != NULL)
-        dataCopy = varLstDup(data);
+        this->data = varLstDup(data);
 
-    FUNCTION_TEST_RETURN(varNewInternal(varTypeVariantList, (void *)&dataCopy, sizeof(dataCopy)));
+    FUNCTION_TEST_RETURN((Variant *)this);
 }
 
 /***********************************************************************************************************************************
@@ -916,7 +1168,7 @@ varVarLst(const Variant *this)
     if (this != NULL)
     {
         ASSERT(this->type == varTypeVariantList);
-        result = *((VariantList **)varData(this));
+        result = ((VariantVariantList *)this)->data;
     }
 
     FUNCTION_TEST_RETURN(result);
@@ -928,40 +1180,46 @@ Convert variant to a zero-terminated string for logging
 String *
 varToLog(const Variant *this)
 {
-    const String *string = NULL;
+    String *result = NULL;
 
-    switch (varType(this))
+    if (this == NULL)
+        result = strDup(NULL_STR);
+    else
     {
-        case varTypeString:
+        switch (varType(this))
         {
-            string = strNewFmt("\"%s\"", strPtr(varStrForce(this)));
-            break;
-        }
+            case varTypeString:
+            {
+                result = strToLog(varStr(this));
+                break;
+            }
 
-        case varTypeKeyValue:
-        {
-            string = STRING_CONST("KeyValue");
-            break;
-        }
+            case varTypeKeyValue:
+            {
+                result = strNew("{KeyValue}");
+                break;
+            }
 
-        case varTypeVariantList:
-        {
-            string = STRING_CONST("VariantList");
-            break;
-        }
+            case varTypeVariantList:
+            {
+                result = strNew("{VariantList}");
+                break;
+            }
 
-        case varTypeBool:
-        case varTypeDouble:
-        case varTypeInt:
-        case varTypeInt64:
-        case varTypeUInt64:
-        {
-            string = varStrForce(this);
-            break;
+            case varTypeBool:
+            case varTypeDouble:
+            case varTypeInt:
+            case varTypeInt64:
+            case varTypeUInt:
+            case varTypeUInt64:
+            {
+                result = strNewFmt("{%s}", strPtr(varStrForce(this)));
+                break;
+            }
         }
     }
 
-    return strNewFmt("{%s}", strPtr(string));
+    return result;
 }
 
 /***********************************************************************************************************************************
@@ -976,40 +1234,77 @@ varFree(Variant *this)
 
     if (this != NULL)
     {
-        MEM_CONTEXT_BEGIN(this->memContext)
+        MemContext *contextOld = memContextCurrent();
+
+        TRY_BEGIN()
         {
-            switch (this->type)
-            {
-                case varTypeKeyValue:
+                switch (this->type)
                 {
-                    kvFree(varKv(this));
-                    break;
-                }
+                    case varTypeBool:
+                    {
+                        memContextSwitch(((VariantBool *)this)->memContext);
+                        break;
+                    }
 
-                case varTypeString:
-                {
-                    strFree(varStr(this));
-                    break;
-                }
+                    case varTypeDouble:
+                    {
+                        memContextSwitch(((VariantDouble *)this)->memContext);
+                        break;
+                    }
 
-                case varTypeVariantList:
-                {
-                    varLstFree(varVarLst(this));
-                    break;
-                }
+                    case varTypeInt:
+                    {
+                        memContextSwitch(((VariantInt *)this)->memContext);
+                        break;
+                    }
 
-                // Nothing additional to free for these types
-                case varTypeBool:
-                case varTypeDouble:
-                case varTypeInt:
-                case varTypeInt64:
-                case varTypeUInt64:
-                    break;
-            }
+                    case varTypeInt64:
+                    {
+                        memContextSwitch(((VariantInt64 *)this)->memContext);
+                        break;
+                    }
+
+                    case varTypeKeyValue:
+                    {
+                        memContextSwitch(((VariantKeyValue *)this)->memContext);
+                        kvFree(((VariantKeyValue *)this)->data);
+                        break;
+                    }
+
+                    case varTypeString:
+                    {
+                        memContextSwitch(((VariantString *)this)->memContext);
+                        strFree(((VariantString *)this)->data);
+                        break;
+                    }
+
+                    case varTypeUInt:
+                    {
+                        memContextSwitch(((VariantUInt *)this)->memContext);
+                        break;
+                    }
+
+                    case varTypeUInt64:
+                    {
+                        memContextSwitch(((VariantUInt64 *)this)->memContext);
+                        break;
+                    }
+
+                    case varTypeVariantList:
+                    {
+                        memContextSwitch(((VariantVariantList *)this)->memContext);
+                        varLstFree(((VariantVariantList *)this)->data);
+                        break;
+                    }
+                }
 
             memFree(this);
         }
-        MEM_CONTEXT_END();
+        FINALLY()
+        {
+            memContextSwitch(contextOld);
+        }
+        TRY_END();
     }
 
     FUNCTION_TEST_RETURN_VOID();
