@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Xml Handler
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <string.h>
 
 #include <libxml/parser.h>
@@ -9,9 +11,15 @@ Xml Handler
 #include "common/debug.h"
 #include "common/log.h"
 #include "common/memContext.h"
+#include "common/object.h"
 #include "common/type/list.h"
 #include "common/type/xml.h"
 
+/***********************************************************************************************************************************
+Encoding constants
+***********************************************************************************************************************************/
+#define XML_ENCODING_TYPE_UTF8                                      "UTF-8"
+
 /***********************************************************************************************************************************
 Node type
 ***********************************************************************************************************************************/
@@ -30,6 +38,8 @@ struct XmlDocument
     XmlNode *root;
 };
 
+OBJECT_DEFINE_FREE(XML_DOCUMENT);
+
 /***********************************************************************************************************************************
 Error handler
 
@@ -38,12 +48,37 @@ messages need to be accumulated and then returned together.
 
 This empty function is required because without it libxml2 will dump errors to stdout.  Really.
 ***********************************************************************************************************************************/
-void xmlErrorHandler(void *ctx, const char *format, ...)
+static void xmlErrorHandler(void *ctx, const char *format, ...)
 {
     (void)ctx;
     (void)format;
 }
 
+/***********************************************************************************************************************************
+Initialize xml
+***********************************************************************************************************************************/
+static void
+xmlInit(void)
+{
+    FUNCTION_TEST_VOID();
+
+    // Initialize xml if it is not already initialized
+    static bool xmlInit = false;
+
+    if (!xmlInit)
+    {
+        LIBXML_TEST_VERSION;
+
+        // It's a pretty weird that we can't just pass a handler function but instead have to assign it to a var...
+        static xmlGenericErrorFunc xmlErrorHandlerFunc = xmlErrorHandler;
+        initGenericErrorDefaultFunc(&xmlErrorHandlerFunc);
+
+        xmlInit = true;
+    }
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
 /***********************************************************************************************************************************
 Create node list
 ***********************************************************************************************************************************/
@@ -118,6 +153,26 @@ xmlNodeNew(xmlNodePtr node)
     FUNCTION_TEST_RETURN(this);
 }
 
+/***********************************************************************************************************************************
+Add a node
+***********************************************************************************************************************************/
+XmlNode *
+xmlNodeAdd(XmlNode *this, const String *name)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(XML_NODE, this);
+        FUNCTION_TEST_PARAM(STRING, name);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+    ASSERT(name != NULL);
+
+    XmlNode *result = xmlNodeNew(xmlNewNode(NULL, BAD_CAST strPtr(name)));
+    xmlAddChild(this->node, result->node);
+
+    FUNCTION_TEST_RETURN(result);
+}
+
 /***********************************************************************************************************************************
 Add a node to a node list
 ***********************************************************************************************************************************/
@@ -146,7 +201,7 @@ xmlNodeLstAdd(XmlNodeList *this, xmlNodePtr node)
 Get node attribute
 ***********************************************************************************************************************************/
 String *
-xmlNodeAttribute(XmlNode *this, const String *name)
+xmlNodeAttribute(const XmlNode *this, const String *name)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(XML_NODE, this);
@@ -172,7 +227,7 @@ xmlNodeAttribute(XmlNode *this, const String *name)
 Get node content
 ***********************************************************************************************************************************/
 String *
-xmlNodeContent(XmlNode *this)
+xmlNodeContent(const XmlNode *this)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(XML_NODE, this);
@@ -190,11 +245,30 @@ xmlNodeContent(XmlNode *this)
     FUNCTION_TEST_RETURN(result);
 }
 
+/***********************************************************************************************************************************
+Set node content
+***********************************************************************************************************************************/
+void
+xmlNodeContentSet(XmlNode *this, const String *content)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(XML_NODE, this);
+        FUNCTION_TEST_PARAM(STRING, content);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+    ASSERT(content != NULL);
+
+    xmlAddChild(this->node, xmlNewText(BAD_CAST strPtr(content)));
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
 /***********************************************************************************************************************************
 Get a list of child nodes
 ***********************************************************************************************************************************/
 XmlNodeList *
-xmlNodeChildList(XmlNode *this, const String *name)
+xmlNodeChildList(const XmlNode *this, const String *name)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(XML_NODE, this);
@@ -219,7 +293,7 @@ xmlNodeChildList(XmlNode *this, const String *name)
 Get a child node
 ***********************************************************************************************************************************/
 XmlNode *
-xmlNodeChildN(XmlNode *this, const String *name, unsigned int index, bool errorOnMissing)
+xmlNodeChildN(const XmlNode *this, const String *name, unsigned int index, bool errorOnMissing)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(XML_NODE, this);
@@ -254,7 +328,7 @@ xmlNodeChildN(XmlNode *this, const String *name, unsigned int index, bool errorO
 }
 
 XmlNode *
-xmlNodeChild(XmlNode *this, const String *name, bool errorOnMissing)
+xmlNodeChild(const XmlNode *this, const String *name, bool errorOnMissing)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(XML_NODE, this);
@@ -271,7 +345,7 @@ xmlNodeChild(XmlNode *this, const String *name, bool errorOnMissing)
 Get child total
 ***********************************************************************************************************************************/
 unsigned int
-xmlNodeChildTotal(XmlNode *this, const String *name)
+xmlNodeChildTotal(const XmlNode *this, const String *name)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(XML_NODE, this);
@@ -306,6 +380,50 @@ xmlNodeFree(XmlNode *this)
     FUNCTION_TEST_RETURN_VOID();
 }
 
+/***********************************************************************************************************************************
+Free document
+***********************************************************************************************************************************/
+OBJECT_DEFINE_FREE_RESOURCE_BEGIN(XML_DOCUMENT, LOG, logLevelTrace)
+{
+    xmlFreeDoc(this->xml);
+}
+OBJECT_DEFINE_FREE_RESOURCE_END(LOG);
+
+/***********************************************************************************************************************************
+Create a new document with the specified root node
+***********************************************************************************************************************************/
+XmlDocument *
+xmlDocumentNew(const String *rootName)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, rootName);
+    FUNCTION_TEST_END();
+
+    ASSERT(rootName != NULL);
+
+    xmlInit();
+
+    // Create object
+    XmlDocument *this = NULL;
+
+    MEM_CONTEXT_NEW_BEGIN("XmlDocument")
+    {
+        this = memNew(sizeof(XmlDocument));
+        this->memContext = MEM_CONTEXT_NEW();
+
+        this->xml = xmlNewDoc(BAD_CAST "1.0");
+
+        // Set callback to ensure xml document is freed
+        memContextCallbackSet(this->memContext, xmlDocumentFreeResource, this);
+
+        this->root = xmlNodeNew(xmlNewNode(NULL, BAD_CAST strPtr(rootName)));
+        xmlDocSetRootElement(this->xml, this->root->node);
+    }
+    MEM_CONTEXT_NEW_END();
+
+    FUNCTION_TEST_RETURN(this);
+}
+
 /***********************************************************************************************************************************
 Create document from C buffer
 ***********************************************************************************************************************************/
@@ -320,19 +438,7 @@ xmlDocumentNewC(const unsigned char *buffer, size_t bufferSize)
     ASSERT(buffer != NULL);
     ASSERT(bufferSize > 0);
 
-    // Initialize xml if it is not already initialized
-    static bool xmlInit = false;
-
-    if (!xmlInit)
-    {
-        LIBXML_TEST_VERSION;
-
-        // It's a pretty weird that we can't just pass a handler function but instead have to assign it to a var...
-        static xmlGenericErrorFunc xmlErrorHandlerFunc = xmlErrorHandler;
-        initGenericErrorDefaultFunc(&xmlErrorHandlerFunc);
-
-        xmlInit = true;
-    }
+    xmlInit();
 
     // Create object
     XmlDocument *this = NULL;
@@ -346,7 +452,7 @@ xmlDocumentNewC(const unsigned char *buffer, size_t bufferSize)
             THROW_FMT(FormatError, "invalid xml");
 
         // Set callback to ensure xml document is freed
-        memContextCallback(this->memContext, (MemContextCallback)xmlDocumentFree, this);
+        memContextCallbackSet(this->memContext, xmlDocumentFreeResource, this);
 
         // Get the root node
         this->root = xmlNodeNew(xmlDocGetRootElement(this->xml));
@@ -388,6 +494,28 @@ xmlDocumentNewZ(const char *string)
     FUNCTION_TEST_RETURN(xmlDocumentNewC((const unsigned char *)string, strlen(string)));
 }
 
+/***********************************************************************************************************************************
+Dump document to a buffer
+***********************************************************************************************************************************/
+Buffer *
+xmlDocumentBuf(const XmlDocument *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(XML_DOCUMENT, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    xmlChar *xml;
+    int xmlSize;
+
+    xmlDocDumpMemoryEnc(this->xml, &xml, &xmlSize, XML_ENCODING_TYPE_UTF8);
+    Buffer *result = bufNewC(xml, (size_t)xmlSize);
+    xmlFree(xml);
+
+    FUNCTION_TEST_RETURN(result);
+}
+
 /***********************************************************************************************************************************
 Get the root node
 ***********************************************************************************************************************************/
@@ -402,24 +530,3 @@ xmlDocumentRoot(const XmlDocument *this)
 
     FUNCTION_TEST_RETURN(this->root);
 }
-
-/***********************************************************************************************************************************
-Free document
-***********************************************************************************************************************************/
-void
-xmlDocumentFree(XmlDocument *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(XML_DOCUMENT, this);
-    FUNCTION_TEST_END();
-
-    if (this != NULL)
-    {
-        xmlFreeDoc(this->xml);
-
-        memContextCallbackClear(this->memContext);
-        memContextFree(this->memContext);
-    }
-
-    FUNCTION_TEST_RETURN_VOID();
-}
