@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Command and Option Configuration
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <string.h>
 
 #include "common/debug.h"
@@ -15,12 +17,16 @@ typedef struct ConfigCommandData
 {
     const char *name;
 
+    bool internal:1;
     bool lockRequired:1;
+    bool lockRemoteRequired:1;
     unsigned int lockType:2;
 
     bool logFile:1;
     unsigned int logLevelDefault:4;
     unsigned int logLevelStdErrMax:4;
+
+    bool parameterAllowed:1;
 } ConfigCommandData;
 
 #define CONFIG_COMMAND_LIST(...)                                                                                                   \
@@ -29,8 +35,12 @@ typedef struct ConfigCommandData
 #define CONFIG_COMMAND(...)                                                                                                        \
     {__VA_ARGS__},
 
+#define CONFIG_COMMAND_INTERNAL(internalParam)                                                                                     \
+    .internal = internalParam,
 #define CONFIG_COMMAND_LOCK_REQUIRED(lockRequiredParam)                                                                            \
     .lockRequired = lockRequiredParam,
+#define CONFIG_COMMAND_LOCK_REMOTE_REQUIRED(lockRemoteRequiredParam)                                                               \
+    .lockRemoteRequired = lockRemoteRequiredParam,
 #define CONFIG_COMMAND_LOCK_TYPE(lockTypeParam)                                                                                    \
     .lockType = lockTypeParam,
 #define CONFIG_COMMAND_LOG_FILE(logFileParam)                                                                                      \
@@ -41,6 +51,8 @@ typedef struct ConfigCommandData
     .logLevelStdErrMax = logLevelStdErrMaxParam,
 #define CONFIG_COMMAND_NAME(nameParam)                                                                                             \
     .name = nameParam,
+#define CONFIG_COMMAND_PARAMETER_ALLOWED(parameterAllowedParam)                                                                    \
+    .parameterAllowed = parameterAllowedParam,
 
 /***********************************************************************************************************************************
 Map options names and indexes to option definitions.
@@ -319,6 +331,21 @@ cfgExeSet(const String *exeParam)
     FUNCTION_TEST_RETURN_VOID();
 }
 
+/***********************************************************************************************************************************
+Is this command internal-only?
+***********************************************************************************************************************************/
+bool
+cfgCommandInternal(ConfigCommand commandId)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(ENUM, commandId);
+    FUNCTION_TEST_END();
+
+    ASSERT(commandId < cfgCmdNone);
+
+    FUNCTION_TEST_RETURN(configCommandData[commandId].internal);
+}
+
 /***********************************************************************************************************************************
 Does this command require an immediate lock?
 ***********************************************************************************************************************************/
@@ -332,6 +359,21 @@ cfgLockRequired(void)
     FUNCTION_TEST_RETURN(configCommandData[cfgCommand()].lockRequired);
 }
 
+/***********************************************************************************************************************************
+Does the command require an immediate lock?
+***********************************************************************************************************************************/
+bool
+cfgLockRemoteRequired(ConfigCommand commandId)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(ENUM, commandId);
+    FUNCTION_TEST_END();
+
+    ASSERT(commandId < cfgCmdNone);
+
+    FUNCTION_TEST_RETURN(configCommandData[commandId].lockRemoteRequired);
+}
+
 /***********************************************************************************************************************************
 Get the lock type required for this command
 ***********************************************************************************************************************************/
@@ -345,6 +387,21 @@ cfgLockType(void)
     FUNCTION_TEST_RETURN((LockType)configCommandData[cfgCommand()].lockType);
 }
 
+/***********************************************************************************************************************************
+Get the remote lock type required for the command
+***********************************************************************************************************************************/
+LockType
+cfgLockRemoteType(ConfigCommand commandId)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(ENUM, commandId);
+    FUNCTION_TEST_END();
+
+    ASSERT(commandId < cfgCmdNone);
+
+    FUNCTION_TEST_RETURN((LockType)configCommandData[commandId].lockType);
+}
+
 /***********************************************************************************************************************************
 Does this command log to a file?
 ***********************************************************************************************************************************/
@@ -384,6 +441,19 @@ cfgLogLevelStdErrMax(void)
     FUNCTION_TEST_RETURN((LogLevel)configCommandData[cfgCommand()].logLevelStdErrMax);
 }
 
+/***********************************************************************************************************************************
+Does this command allow parameters?
+***********************************************************************************************************************************/
+bool
+cfgParameterAllowed(void)
+{
+    FUNCTION_TEST_VOID();
+
+    ASSERT(command != cfgCmdNone);
+
+    FUNCTION_TEST_RETURN(configCommandData[cfgCommand()].parameterAllowed);
+}
+
 /***********************************************************************************************************************************
 Get the option define for this option
 ***********************************************************************************************************************************/
@@ -401,7 +471,55 @@ cfgOptionDefIdFromId(ConfigOption optionId)
 
 /***********************************************************************************************************************************
 Get/set option default
+
+Option defaults are generally not set in advance because the vast majority of them are never used.  It is more efficient to generate
+them when they are requested.
+
+Some defaults are (e.g. the exe path) are set at runtime.
 ***********************************************************************************************************************************/
+static Variant *
+cfgOptionDefaultValue(ConfigDefineOption optionDefId)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(ENUM, optionDefId);
+    FUNCTION_TEST_END();
+
+    Variant *result;
+    Variant *defaultValue = varNewStrZ(cfgDefOptionDefault(cfgCommandDefIdFromId(cfgCommand()), optionDefId));
+
+    switch (cfgDefOptionType(optionDefId))
+    {
+        case cfgDefOptTypeBoolean:
+        {
+            result = varNewBool(varBoolForce(defaultValue));
+            break;
+        }
+
+        case cfgDefOptTypeFloat:
+        {
+            result = varNewDbl(varDblForce(defaultValue));
+            break;
+        }
+
+        case cfgDefOptTypeInteger:
+        case cfgDefOptTypeSize:
+        {
+            result = varNewInt64(varInt64Force(defaultValue));
+            break;
+        }
+
+        case cfgDefOptTypePath:
+        case cfgDefOptTypeString:
+            result = varDup(defaultValue);
+            break;
+
+        default:
+            THROW_FMT(AssertError, "default value not available for option type %d", cfgDefOptionType(optionDefId));
+    }
+
+    FUNCTION_TEST_RETURN(result);
+}
+
 const Variant *
 cfgOptionDefault(ConfigOption optionId)
 {
@@ -417,45 +535,11 @@ cfgOptionDefault(ConfigOption optionId)
 
         if (cfgDefOptionDefault(cfgCommandDefIdFromId(cfgCommand()), optionDefId) != NULL)
         {
-            MEM_CONTEXT_TEMP_BEGIN()
+            MEM_CONTEXT_BEGIN(configMemContext)
             {
-                Variant *defaultValue = varNewStrZ(cfgDefOptionDefault(cfgCommandDefIdFromId(cfgCommand()), optionDefId));
-
-                MEM_CONTEXT_BEGIN(configMemContext)
-                {
-                    switch (cfgDefOptionType(optionDefId))
-                    {
-                        case cfgDefOptTypeBoolean:
-                        {
-                            configOptionValue[optionId].defaultValue = varNewBool(varBoolForce(defaultValue));
-                            break;
-                        }
-
-                        case cfgDefOptTypeFloat:
-                        {
-                            configOptionValue[optionId].defaultValue = varNewDbl(varDblForce(defaultValue));
-                            break;
-                        }
-
-                        case cfgDefOptTypeInteger:
-                        case cfgDefOptTypeSize:
-                        {
-                            configOptionValue[optionId].defaultValue = varNewInt64(varInt64Force(defaultValue));
-                            break;
-                        }
-
-                        case cfgDefOptTypeString:
-                            configOptionValue[optionId].defaultValue = varDup(defaultValue);
-                            break;
-
-                        default:                                    // {uncoverable - other types do not have defaults yet}
-                            THROW_FMT(                              // {+uncoverable}
-                                AssertError, "type for option '%s' does not support defaults", cfgOptionName(optionId));
-                    }
-                }
-                MEM_CONTEXT_END();
+                configOptionValue[optionId].defaultValue = cfgOptionDefaultValue(optionDefId);
             }
-            MEM_CONTEXT_TEMP_END();
+            MEM_CONTEXT_END();
         }
     }
 
@@ -492,6 +576,78 @@ cfgOptionDefaultSet(ConfigOption optionId, const Variant *defaultValue)
     FUNCTION_TEST_RETURN_VOID();
 }
 
+/***********************************************************************************************************************************
+Parse a host option and extract the host and port (if it exists)
+***********************************************************************************************************************************/
+String *
+cfgOptionHostPort(ConfigOption optionId, unsigned int *port)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(ENUM, optionId);
+        FUNCTION_TEST_PARAM_P(UINT, port);
+    FUNCTION_TEST_END();
+
+    ASSERT(optionId < CFG_OPTION_TOTAL);
+    ASSERT(port != NULL);
+
+    String *result = NULL;
+
+    // Proceed if option is valid and has a value
+    if (cfgOptionTest(optionId))
+    {
+        MEM_CONTEXT_TEMP_BEGIN()
+        {
+            const String *host = cfgOptionStr(optionId);
+
+            // If the host contains a colon then it has a port appended
+            if (strChr(host, ':') != -1)
+            {
+                const StringList *hostPart = strLstNewSplitZ(host, ":");
+
+                // More than one colon is invalid
+                if (strLstSize(hostPart) > 2)
+                {
+                    THROW_FMT(
+                        OptionInvalidError,
+                        "'%s' is not valid for option '%s'"
+                            "\nHINT: is more than one port specified?",
+                        strPtr(host), cfgOptionName(optionId));
+                }
+
+                // Set the host
+                memContextSwitch(MEM_CONTEXT_OLD());
+                result = strDup(strLstGet(hostPart, 0));
+                memContextSwitch(MEM_CONTEXT_TEMP());
+
+                // Set the port and error if it is not a positive integer
+                TRY_BEGIN()
+                {
+                    *port = cvtZToUInt(strPtr(strLstGet(hostPart, 1)));
+                }
+                CATCH(FormatError)
+                {
+                    THROW_FMT(
+                        OptionInvalidError,
+                        "'%s' is not valid for option '%s'"
+                            "\nHINT: port is not a positive integer.",
+                        strPtr(host), cfgOptionName(optionId));
+                }
+                TRY_END();
+            }
+            // Else there is no port and just copy the host
+            else
+            {
+                memContextSwitch(MEM_CONTEXT_OLD());
+                result = strDup(host);
+                memContextSwitch(MEM_CONTEXT_TEMP());
+            }
+        }
+        MEM_CONTEXT_TEMP_END();
+    }
+
+    FUNCTION_TEST_RETURN(result);
+}
+
 /***********************************************************************************************************************************
 Get index for option
 ***********************************************************************************************************************************/
@@ -554,19 +710,21 @@ cfgOptionIdFromDefId(ConfigDefineOption optionDefId, unsigned int index)
         FUNCTION_TEST_PARAM(UINT, index);
     FUNCTION_TEST_END();
 
-    ASSERT(optionDefId < cfgDefOptionTotal());
-    ASSERT(index < cfgDefOptionIndexTotal(optionDefId));
-
     // Search for the option
     ConfigOption optionId;
 
-    for (optionId = 0; optionId < CFG_OPTION_TOTAL; optionId++)     // {uncoverable - only a bug in code gen lets this loop end}
+    for (optionId = 0; optionId < CFG_OPTION_TOTAL; optionId++)
+    {
         if (configOptionData[optionId].defineId == optionDefId)
             break;
+    }
 
     // If the mapping is not found then there is a bug in the code generator
     ASSERT(optionId != CFG_OPTION_TOTAL);
 
+    // Make sure the index is valid
+    ASSERT(index < cfgDefOptionIndexTotal(optionDefId));
+
     // Return with original index
     FUNCTION_TEST_RETURN(optionId + index);
 }
@@ -713,6 +871,32 @@ cfgOptionInt64(ConfigOption optionId)
     FUNCTION_LOG_RETURN(INT64, varInt64(configOptionValue[optionId].value));
 }
 
+unsigned int
+cfgOptionUInt(ConfigOption optionId)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(ENUM, optionId);
+    FUNCTION_LOG_END();
+
+    ASSERT(optionId < CFG_OPTION_TOTAL);
+    ASSERT(varType(configOptionValue[optionId].value) == varTypeInt64);
+
+    FUNCTION_LOG_RETURN(UINT, varUIntForce(configOptionValue[optionId].value));
+}
+
+uint64_t
+cfgOptionUInt64(ConfigOption optionId)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(ENUM, optionId);
+    FUNCTION_LOG_END();
+
+    ASSERT(optionId < CFG_OPTION_TOTAL);
+    ASSERT(varType(configOptionValue[optionId].value) == varTypeInt64);
+
+    FUNCTION_LOG_RETURN(UINT64, varUInt64Force(configOptionValue[optionId].value));
+}
+
 const KeyValue *
 cfgOptionKv(ConfigOption optionId)
 {
@@ -841,13 +1025,16 @@ cfgOptionSet(ConfigOption optionId, ConfigSource source, const Variant *value)
                     break;
                 }
 
+                case cfgDefOptTypePath:
                 case cfgDefOptTypeString:
+                {
                     if (varType(value) == varTypeString)
                         configOptionValue[optionId].value = varDup(value);
                     else
                         THROW_FMT(AssertError, "option '%s' must be set with String variant", cfgOptionName(optionId));
 
                     break;
+                }
             }
         }
         else
