@@ -29,15 +29,20 @@ Command Functions
 Access the current command and command parameters.
 ***********************************************************************************************************************************/
 ConfigCommand cfgCommand(void);
+bool cfgCommandInternal(ConfigCommand commandId);
 const char *cfgCommandName(ConfigCommand commandId);
 
 bool cfgLockRequired(void);
+bool cfgLockRemoteRequired(ConfigCommand commandId);
 LockType cfgLockType(void);
+LockType cfgLockRemoteType(ConfigCommand commandId);
 
 bool cfgLogFile(void);
 LogLevel cfgLogLevelDefault(void);
 LogLevel cfgLogLevelStdErrMax(void);
 
+bool cfgParameterAllowed(void);
+
 const StringList *cfgCommandParam(void);
 
 /***********************************************************************************************************************************
@@ -53,6 +58,8 @@ int64_t cfgOptionInt64(ConfigOption optionId);
 const KeyValue *cfgOptionKv(ConfigOption optionId);
 const VariantList *cfgOptionLst(ConfigOption optionId);
 const String *cfgOptionStr(ConfigOption optionId);
+unsigned int cfgOptionUInt(ConfigOption optionId);
+uint64_t cfgOptionUInt64(ConfigOption optionId);
 
 unsigned int cfgOptionIndex(ConfigOption optionId);
 unsigned int cfgOptionIndexTotal(ConfigOption optionDefId);
@@ -96,6 +103,7 @@ void cfgExeSet(const String *exeParam);
 const Variant *cfgOptionDefault(ConfigOption optionId);
 void cfgOptionDefaultSet(ConfigOption optionId, const Variant *defaultValue);
 ConfigDefineOption cfgOptionDefIdFromId(ConfigOption optionId);
+String *cfgOptionHostPort(ConfigOption optionId, unsigned int *port);
 int cfgOptionId(const char *optionName);
 ConfigOption cfgOptionIdFromDefId(ConfigDefineOption optionDefId, unsigned int index);
 bool cfgOptionNegate(ConfigOption optionId);
