@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Exec Configuration
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <string.h>
 
 #include "common/debug.h"
@@ -8,7 +10,7 @@ Exec Configuration
 #include "config/exec.h"
 
 /***********************************************************************************************************************************
-Load log settings
+Generate a list of options required for execution of a new command, replacing options as specified in optionReplace
 ***********************************************************************************************************************************/
 StringList *
 cfgExecParam(ConfigCommand commandId, const KeyValue *optionReplace)
@@ -30,21 +32,37 @@ cfgExecParam(ConfigCommand commandId, const KeyValue *optionReplace)
         {
             ConfigDefineOption optionDefId = cfgOptionDefIdFromId(optionId);
 
-            // Skip the option if it is not valid for the specified command or if is secure
-            if (!cfgDefOptionValid(commandDefId, optionDefId) || cfgDefOptionSecure(optionDefId))
+            // Skip the option if it is not valid for the specified command or if is secure.  Also skip repo1-cipher-type because
+            // there's no point of passing it if the other process doesn't have access to repo1-cipher-pass.  There is probably a
+            // better way to do this...
+            if (!cfgDefOptionValid(commandDefId, optionDefId) || cfgDefOptionSecure(optionDefId) ||
+                optionDefId == cfgDefOptRepoCipherType)
+            {
                 continue;
+            }
 
             // First check for a replacement
+            const Variant *key = VARSTRZ(cfgOptionName(optionId));
             const Variant *value = NULL;
+            bool exists = false;
 
             if (optionReplace != NULL)
-                value = kvGet(optionReplace, varNewStr(strNew(cfgOptionName(optionId))));
+            {
+                exists = kvKeyExists(optionReplace, key);
+
+                if (exists)
+                    value = kvGet(optionReplace, key);
+            }
+
+            // If the key exists but is NULL then skip this option
+            if (exists && value == NULL)
+                continue;
 
             // If no replacement then see if this option is valid for the current command and is not default
             if (value == NULL && cfgOptionValid(optionId))
             {
                 if (cfgOptionNegate(optionId))
-                    value = varNewBool(false);
+                    value = BOOL_FALSE_VAR;
                 else if (cfgOptionSource(optionId) != cfgSourceDefault)
                     value = cfgOption(optionId);
             }
