@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Configuration Load
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <string.h>
 #include <sys/stat.h>
 
@@ -27,6 +29,7 @@ cfgLoadLogSetting(void)
     LogLevel logLevelStdErr = logLevelOff;
     LogLevel logLevelFile = logLevelOff;
     bool logTimestamp = true;
+    unsigned int logProcessMax = 1;
 
     if (cfgOptionValid(cfgOptLogLevelConsole))
         logLevelConsole = logLevelEnum(strPtr(cfgOptionStr(cfgOptLogLevelConsole)));
@@ -46,7 +49,10 @@ cfgLoadLogSetting(void)
     if (cfgOptionValid(cfgOptLogTimestamp))
         logTimestamp = cfgOptionBool(cfgOptLogTimestamp);
 
-    logInit(logLevelConsole, logLevelStdErr, logLevelFile, logTimestamp);
+    if (cfgOptionValid(cfgOptProcessMax))
+        logProcessMax = cfgOptionUInt(cfgOptProcessMax);
+
+    logInit(logLevelConsole, logLevelStdErr, logLevelFile, logTimestamp, logProcessMax);
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -61,7 +67,7 @@ cfgLoadUpdateOption(void)
 
     // Set default for repo-host-cmd
     if (cfgOptionTest(cfgOptRepoHost) && cfgOptionSource(cfgOptRepoHostCmd) == cfgSourceDefault)
-        cfgOptionDefaultSet(cfgOptRepoHostCmd, varNewStr(cfgExe()));
+        cfgOptionDefaultSet(cfgOptRepoHostCmd, VARSTR(cfgExe()));
 
     // Set default for pg-host-cmd
     if (cfgOptionValid(cfgOptPgHostCmd))
@@ -69,7 +75,7 @@ cfgLoadUpdateOption(void)
         for (unsigned int optionIdx = 0; optionIdx < cfgOptionIndexTotal(cfgOptPgHost); optionIdx++)
         {
             if (cfgOptionTest(cfgOptPgHost + optionIdx) && cfgOptionSource(cfgOptPgHostCmd + optionIdx) == cfgSourceDefault)
-                cfgOptionDefaultSet(cfgOptPgHostCmd + optionIdx, varNewStr(cfgExe()));
+                cfgOptionDefaultSet(cfgOptPgHostCmd + optionIdx, VARSTR(cfgExe()));
         }
     }
 
@@ -79,7 +85,7 @@ cfgLoadUpdateOption(void)
     {
         // If protocol-timeout is default then increase it to be greater than db-timeout
         if (cfgOptionSource(cfgOptProtocolTimeout) == cfgSourceDefault)
-            cfgOptionSet(cfgOptProtocolTimeout, cfgSourceDefault, varNewDbl(cfgOptionDbl(cfgOptDbTimeout) + 30));
+            cfgOptionSet(cfgOptProtocolTimeout, cfgSourceDefault, VARDBL(cfgOptionDbl(cfgOptDbTimeout) + 30));
         else if (cfgOptionSource(cfgOptDbTimeout) == cfgSourceDefault)
         {
             double dbTimeout = cfgOptionDbl(cfgOptProtocolTimeout) - 30;
@@ -87,19 +93,20 @@ cfgLoadUpdateOption(void)
             // Normally the protocol time will be greater than 45 seconds so db timeout can be at least 15 seconds
             if (dbTimeout >= 15)
             {
-                cfgOptionSet(cfgOptDbTimeout, cfgSourceDefault, varNewDbl(dbTimeout));
+                cfgOptionSet(cfgOptDbTimeout, cfgSourceDefault, VARDBL(dbTimeout));
             }
             // But in some test cases the protocol timeout will be very small so make db timeout half of protocol timeout
             else
-                cfgOptionSet(cfgOptDbTimeout, cfgSourceDefault, varNewDbl(cfgOptionDbl(cfgOptProtocolTimeout) / 2));
+                cfgOptionSet(cfgOptDbTimeout, cfgSourceDefault, VARDBL(cfgOptionDbl(cfgOptProtocolTimeout) / 2));
         }
         else
         {
-            THROW_FMT(OptionInvalidValueError,
-                "'%s' is not valid for '%s' option\nHINT '%s' option (%s) should be greater than '%s' option (%s).",
-                strPtr(varStrForce(cfgOption(cfgOptProtocolTimeout))), cfgOptionName(cfgOptProtocolTimeout),
-                cfgOptionName(cfgOptProtocolTimeout), strPtr(varStrForce(cfgOption(cfgOptProtocolTimeout))),
-                cfgOptionName(cfgOptDbTimeout), strPtr(varStrForce(cfgOption(cfgOptDbTimeout))));
+            THROW_FMT(
+                OptionInvalidValueError,
+                "'%s' is not valid for '" CFGOPT_PROTOCOL_TIMEOUT "' option\nHINT '" CFGOPT_PROTOCOL_TIMEOUT "' option (%s)"
+                    " should be greater than '" CFGOPT_DB_TIMEOUT "' option (%s).",
+                strPtr(varStrForce(cfgOption(cfgOptProtocolTimeout))),  strPtr(varStrForce(cfgOption(cfgOptProtocolTimeout))),
+                strPtr(varStrForce(cfgOption(cfgOptDbTimeout))));
         }
     }
 
@@ -129,15 +136,16 @@ cfgLoadUpdateOption(void)
     }
 
     // Warn when repo-retention-full is not set on a configured repo
-    if (cfgOptionValid(cfgOptRepoRetentionFull))
+    if (!cfgCommandHelp() && cfgOptionValid(cfgOptRepoRetentionFull))
     {
         for (unsigned int optionIdx = 0; optionIdx < cfgOptionIndexTotal(cfgOptRepoType); optionIdx++)
         {
             // If the repo-type is defined, then see if corresponding retention-full is set
             if (cfgOptionTest(cfgOptRepoType + optionIdx) && !cfgOptionTest(cfgOptRepoRetentionFull + optionIdx))
             {
-                LOG_WARN("option %s is not set, the repository may run out of space\n"
-                    "HINT: to retain full backups indefinitely (without warning), set option '%s' to the maximum.",
+                LOG_WARN(
+                    "option %s is not set, the repository may run out of space"
+                        "\nHINT: to retain full backups indefinitely (without warning), set option '%s' to the maximum.",
                     cfgOptionName(cfgOptRepoRetentionFull + optionIdx),
                     cfgOptionName(cfgOptRepoRetentionFull + optionIdx));
             }
@@ -152,8 +160,9 @@ cfgLoadUpdateOption(void)
         {
             const String *archiveRetentionType = cfgOptionStr(cfgOptRepoRetentionArchiveType + optionIdx);
 
-            const String *msgArchiveOff = strNewFmt("WAL segments will not be expired: option '%s=%s' but",
-                cfgOptionName(cfgOptRepoRetentionArchiveType), strPtr(archiveRetentionType));
+            const String *msgArchiveOff = strNewFmt(
+                "WAL segments will not be expired: option '" CFGOPT_REPO1_RETENTION_ARCHIVE_TYPE "=%s' but",
+                strPtr(archiveRetentionType));
 
             // If the archive retention is not explicitly set then determine what it should be defaulted to
             if (!cfgOptionTest(cfgOptRepoRetentionArchive + optionIdx))
@@ -165,7 +174,7 @@ cfgLoadUpdateOption(void)
                     if (cfgOptionTest(cfgOptRepoRetentionFull + optionIdx))
                     {
                         cfgOptionSet(cfgOptRepoRetentionArchive + optionIdx, cfgSourceDefault,
-                            varNewInt(cfgOptionInt(cfgOptRepoRetentionFull + optionIdx)));
+                            VARUINT(cfgOptionUInt(cfgOptRepoRetentionFull + optionIdx)));
                     }
                 }
                 else if (strEqZ(archiveRetentionType, CFGOPTVAL_TMP_REPO_RETENTION_ARCHIVE_TYPE_DIFF))
@@ -174,7 +183,7 @@ cfgLoadUpdateOption(void)
                     if (cfgOptionTest(cfgOptRepoRetentionDiff + optionIdx))
                     {
                         cfgOptionSet(cfgOptRepoRetentionArchive + optionIdx, cfgSourceDefault,
-                            varNewInt(cfgOptionInt(cfgOptRepoRetentionDiff + optionIdx)));
+                            VARUINT(cfgOptionUInt(cfgOptRepoRetentionDiff + optionIdx)));
                     }
                     else
                     {
@@ -183,8 +192,10 @@ cfgLoadUpdateOption(void)
                             cfgOptionName(cfgOptRepoRetentionDiff + optionIdx));
                     }
                 }
-                else if (strEqZ(archiveRetentionType, CFGOPTVAL_TMP_REPO_RETENTION_ARCHIVE_TYPE_INCR))
+                else
                 {
+                    CHECK(strEqZ(archiveRetentionType, CFGOPTVAL_TMP_REPO_RETENTION_ARCHIVE_TYPE_INCR));
+
                     LOG_WARN("%s option '%s' is not set", strPtr(msgArchiveOff),
                         cfgOptionName(cfgOptRepoRetentionArchive + optionIdx));
                 }
@@ -208,6 +219,19 @@ cfgLoadUpdateOption(void)
         }
     }
 
+    // Error if an S3 bucket name contains dots
+    if (cfgOptionTest(cfgOptRepoS3Bucket) && cfgOptionBool(cfgOptRepoS3VerifyTls) &&
+        strChr(cfgOptionStr(cfgOptRepoS3Bucket), '.') != -1)
+    {
+        THROW_FMT(
+            OptionInvalidValueError,
+            "'%s' is not valid for option '" CFGOPT_REPO1_S3_BUCKET "'"
+                "\nHINT: RFC-2818 forbids dots in wildcard matches"
+                "\nHINT: TLS/SSL verification cannot proceed with this bucket name"
+                "\nHINT: remove dots from the bucket name",
+            strPtr(cfgOptionStr(cfgOptRepoS3Bucket)));
+    }
+
     FUNCTION_LOG_RETURN_VOID();
 }
 
@@ -250,7 +274,7 @@ cfgLoad(unsigned int argListSize, const char *argList[])
         {
             // Set IO buffer size
             if (cfgOptionValid(cfgOptBufferSize))
-                ioBufferSizeSet((size_t)cfgOptionInt(cfgOptBufferSize));
+                ioBufferSizeSet(cfgOptionUInt(cfgOptBufferSize));
 
             // Open the log file if this command logs to a file
             if (cfgLogFile() && !cfgCommandHelp())
@@ -264,8 +288,8 @@ cfgLoad(unsigned int argListSize, const char *argList[])
                 if (cfgCommand() == cfgCmdLocal || cfgCommand() == cfgCmdRemote)
                 {
                     strCatFmt(
-                        logFile, "%s-%s-%03d.log", strPtr(cfgOptionStr(cfgOptCommand)), cfgCommandName(cfgCommand()),
-                        cfgOptionInt(cfgOptProcess));
+                        logFile, "%s-%s-%03u.log", strPtr(cfgOptionStr(cfgOptCommand)), cfgCommandName(cfgCommand()),
+                        cfgOptionUInt(cfgOptProcess));
                 }
                 // Else add command name
                 else
