@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Command and Option Parse
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <getopt.h>
 #include <string.h>
 #include <strings.h>
@@ -37,6 +39,12 @@ Standard config include path name
 ***********************************************************************************************************************************/
 #define PGBACKREST_CONFIG_INCLUDE_PATH                              "conf.d"
 
+/***********************************************************************************************************************************
+Option value constants
+***********************************************************************************************************************************/
+VARIANT_STRDEF_STATIC(OPTION_VALUE_0,                               "0");
+VARIANT_STRDEF_STATIC(OPTION_VALUE_1,                               "1");
+
 /***********************************************************************************************************************************
 Parse option flags
 ***********************************************************************************************************************************/
@@ -97,6 +105,60 @@ optionFind(const String *option)
 /***********************************************************************************************************************************
 Convert the value passed into bytes and update valueDbl for range checking
 ***********************************************************************************************************************************/
+static double
+sizeQualifierToMultiplier(char qualifier)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(CHAR, qualifier);
+    FUNCTION_TEST_END();
+
+    double result;
+
+    switch (qualifier)
+    {
+        case 'b':
+        {
+            result = 1;
+            break;
+        }
+
+        case 'k':
+        {
+            result = 1024;
+            break;
+        }
+
+        case 'm':
+        {
+            result = 1024 * 1024;
+            break;
+        }
+
+        case 'g':
+        {
+            result = 1024 * 1024 * 1024;
+            break;
+        }
+
+        case 't':
+        {
+            result = 1024LL * 1024LL * 1024LL * 1024LL;
+            break;
+        }
+
+        case 'p':
+        {
+            result = 1024LL * 1024LL * 1024LL * 1024LL * 1024LL;
+            break;
+        }
+
+        default:
+            THROW_FMT(AssertError, "'%c' is not a valid size qualifier", qualifier);
+    }
+
+    FUNCTION_TEST_RETURN(result);
+}
+
 void
 convertToByte(String **value, double *valueDbl)
 {
@@ -111,7 +173,7 @@ convertToByte(String **value, double *valueDbl)
     String *result = strLower(strDup(*value));
 
     // Match the value against possible values
-    if (regExpMatchOne(STRING_CONST("^[0-9]+(kb|k|mb|m|gb|g|tb|t|pb|p|b)*$"), result))
+    if (regExpMatchOne(STRDEF("^[0-9]+(kb|k|mb|m|gb|g|tb|t|pb|p|b)*$"), result))
     {
         // Get the character array and size
         const char *strArray = strPtr(result);
@@ -135,46 +197,18 @@ convertToByte(String **value, double *valueDbl)
 
         double multiplier = 1;
 
-        // If a letter was found, then truncate, else do nothing since assumed value is already in bytes
+        // If a letter was found calculate multiplier, else do nothing since assumed value is already in bytes
         if (chrPos != -1)
         {
-            if (strArray[chrPos] != 'b')
-            {
-                switch (strArray[chrPos])
-                {
-                    case 'k':
-                        multiplier = 1024;
-                        break;
-
-                    case 'm':
-                        multiplier = 1024 * 1024;
-                        break;
-
-                    case 'g':
-                        multiplier = 1024 * 1024 * 1024;
-                        break;
-
-                    case 't':
-                        multiplier = 1024LL * 1024LL * 1024LL * 1024LL;
-                        break;
-
-                    case 'p':
-                        multiplier = 1024LL * 1024LL * 1024LL * 1024LL * 1024LL;
-                        break;
-
-                    default:                                        // {uncoverable - regex covers all cases but default required}
-                        THROW_FMT(                                  // {+uncoverable}
-                            AssertError, "character %c is not a valid type", strArray[chrPos]);
-                }
-            }
+            multiplier = sizeQualifierToMultiplier(strArray[chrPos]);
 
             // Remove any letters
             strTrunc(result, chrPos);
         }
 
         // Convert string to bytes
-        double newDbl = varDblForce(varNewStr(result)) * multiplier;
-        result = varStrForce(varNewDbl(newDbl));
+        double newDbl = varDblForce(VARSTR(result)) * multiplier;
+        result = varStrForce(VARDBL(newDbl));
 
         // If nothing has blown up then safe to overwrite the original values
         *valueDbl = newDbl;
@@ -214,6 +248,39 @@ Rules:
 - If --config-path only, the defaults for config and config-include-path will be changed to use that as a base path but the files
   will not be required to exist since this is a default override.
 ***********************************************************************************************************************************/
+static void
+cfgFileLoadPart(String **config, const Buffer *configPart)
+{
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM_P(STRING, config);
+        FUNCTION_LOG_PARAM(BUFFER, configPart);
+    FUNCTION_LOG_END();
+
+    if (configPart != NULL)
+    {
+        String *configPartStr = strNewBuf(configPart);
+
+        // Validate the file by parsing it as an Ini object. If the file is not properly formed, an error will occur.
+        if (strSize(configPartStr) > 0)
+        {
+            Ini *configPartIni = iniNew();
+            iniParse(configPartIni, configPartStr);
+
+            // Create the result config file
+            if (*config == NULL)
+                *config = strNew("");
+            // Else add an LF in case the previous file did not end with one
+            else
+
+            // Add the config part to the result config file
+            strCat(*config, "\n");
+            strCat(*config, strPtr(configPartStr));
+        }
+    }
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
 static String *
 cfgFileLoad(                                                        // NOTE: Passing defaults to enable more complete test coverage
     const ParseOption *optionList,                                  // All options and their current settings
@@ -303,7 +370,7 @@ cfgFileLoad(                                                        // NOTE: Pas
     {
         if (result != NULL)
         {
-            // Validate the file by parsing it as an Ini object. If the file is not properly formed, en error will occur.
+            // Validate the file by parsing it as an Ini object. If the file is not properly formed, an error will occur.
             Ini *ini = iniNew();
             iniParse(ini, result);
         }
@@ -318,7 +385,8 @@ cfgFileLoad(                                                        // NOTE: Pas
 
         // Get a list of conf files from the specified path -error on missing directory if the option was passed on the command line
         StringList *list = storageListP(
-            storageLocal(), configIncludePath, .expression = STRING_CONST(".+\\.conf$"), .errorOnMissing = configIncludeRequired);
+            storageLocal(), configIncludePath, .expression = STRDEF(".+\\.conf$"), .errorOnMissing = configIncludeRequired,
+            .nullOnMissing = !configIncludeRequired);
 
         // If conf files are found, then add them to the config string
         if (list != NULL && strLstSize(list) > 0)
@@ -328,33 +396,12 @@ cfgFileLoad(                                                        // NOTE: Pas
 
             for (unsigned int listIdx = 0; listIdx < strLstSize(list); listIdx++)
             {
-                Buffer *fileBuffer = storageGetNP(
-                    storageNewReadP(
-                        storageLocal(), strNewFmt("%s/%s", strPtr(configIncludePath), strPtr(strLstGet(list, listIdx))),
-                        .ignoreMissing = true));
-
-                if (fileBuffer != NULL) // {uncovered - NULL can only occur if file is missing after file list is retrieved}
-                {
-                    // Convert the contents of the file buffer to a string object
-                    String *configPart = strNewBuf(fileBuffer);
-
-                    // Validate the file by parsing it as an Ini object. If the file is not properly formed, an error will occur.
-                    if (strSize(configPart) > 0)
-                    {
-                        Ini *configPartIni = iniNew();
-                        iniParse(configPartIni, configPart);
-
-                        // Create the result config file
-                        if (result == NULL)
-                            result = strNew("");
-                        // Else add an LF in case the previous file did not end with one
-                        else
-                            strCat(result, "\n");
-
-                        // Add the config part to the result config file
-                        strCat(result, strPtr(configPart));
-                    }
-                }
+                cfgFileLoadPart(
+                    &result,
+                    storageGetNP(
+                        storageNewReadP(
+                            storageLocal(), strNewFmt("%s/%s", strPtr(configIncludePath), strPtr(strLstGet(list, listIdx))),
+                            .ignoreMissing = true)));
             }
         }
     }
@@ -382,7 +429,7 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Set the exe
-        cfgExeSet(strNew(argList[0]));
+        cfgExeSet(STR(argList[0]));
 
         // Phase 1: parse command line parameters
         // -------------------------------------------------------------------------------------------------------------------------
@@ -438,7 +485,7 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                         if (commandParamList == NULL)
                             commandParamList = strLstNew();
 
-                        strLstAdd(commandParamList, strNew(argList[optind - 1]));
+                        strLstAdd(commandParamList, STR(argList[optind - 1]));
                     }
 
                     break;
@@ -484,7 +531,7 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
 
                         // Only set the argument if the option requires one
                         if (optionList[optionListIdx].has_arg == required_argument)
-                            parseOptionList[optionId].valueList = strLstAdd(strLstNew(), strNew(optarg));
+                            parseOptionList[optionId].valueList = strLstAdd(strLstNew(), STR(optarg));
                     }
                     else
                     {
@@ -541,11 +588,16 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
 
         // Set command params
         if (commandParamList != NULL)
+        {
+            if (!cfgCommandHelp() && !cfgParameterAllowed())
+                THROW(ParamInvalidError, "command does not allow parameters");
+
             cfgCommandParamSet(commandParamList);
+        }
 
         // Enable logging (except for local and remote commands) so config file warnings will be output
         if (cfgCommand() != cfgCmdLocal && cfgCommand() != cfgCmdRemote && resetLogLevel)
-            logInit(logLevelWarn, logLevelWarn, logLevelOff, false);
+            logInit(logLevelWarn, logLevelWarn, logLevelOff, false, 1);
 
         // Only continue if command options need to be validated, i.e. a real command is running or we are getting help for a
         // specific command and would like to display actual option values in the help.
@@ -572,10 +624,10 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                     ASSERT(equalPtr != NULL);
 
                     // Get key and value
-                    String *key = strReplaceChr(
+                    const String *key = strReplaceChr(
                         strLower(strNewN(keyValue + PGBACKREST_ENV_SIZE, (size_t)(equalPtr - (keyValue + PGBACKREST_ENV_SIZE)))),
                         '_', '-');
-                    String *value = strNew(equalPtr + 1);
+                    const String *value = STR(equalPtr + 1);
 
                     // Find the option
                     unsigned int optionIdx = optionFind(key);
@@ -642,8 +694,8 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
             // ---------------------------------------------------------------------------------------------------------------------
             // Load the configuration file(s)
             String *configString = cfgFileLoad(parseOptionList,
-                strNew(cfgDefOptionDefault(commandDefId, cfgOptionDefIdFromId(cfgOptConfig))),
-                strNew(cfgDefOptionDefault(commandDefId, cfgOptionDefIdFromId(cfgOptConfigIncludePath))),
+                STR(cfgDefOptionDefault(commandDefId, cfgOptionDefIdFromId(cfgOptConfig))),
+                STR(cfgDefOptionDefault(commandDefId, cfgOptionDefIdFromId(cfgOptConfigIncludePath))),
                 PGBACKREST_CONFIG_ORIG_PATH_FILE_STR);
 
             if (configString != NULL)
@@ -713,7 +765,7 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                         }
 
                         // Make sure this option does not appear in the same section with an alternate name
-                        Variant *optionFoundKey = varNewInt(optionId);
+                        const Variant *optionFoundKey = VARINT(optionId);
                         const Variant *optionFoundName = kvGet(optionFound, optionFoundKey);
 
                         if (optionFoundName != NULL)
@@ -723,7 +775,7 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                                 strPtr(key), strPtr(varStr(optionFoundName)), strPtr(section));
                         }
                         else
-                            kvPut(optionFound, optionFoundKey, varNewStr(key));
+                            kvPut(optionFound, optionFoundKey, VARSTR(key));
 
                         // Continue if the option is not valid for this command
                         if (!cfgDefOptionValid(commandDefId, optionDefId))
@@ -754,39 +806,43 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                         if (parseOptionList[optionId].found)
                             continue;
 
-                        // Get the option value
-                        const Variant *value = iniGetDefault(config, section, key, NULL);
-
-                        if (varType(value) == varTypeString && strSize(varStr(value)) == 0)
-                        {
-                            THROW_FMT(
-                                OptionInvalidValueError, "section '%s', key '%s' must have a value", strPtr(section), strPtr(key));
-                        }
-
                         parseOptionList[optionId].found = true;
                         parseOptionList[optionId].source = cfgSourceConfig;
 
-                        if (varType(value) == varTypeVariantList)
+                        // Process list
+                        if (iniSectionKeyIsList(config, section, key))
                         {
                             // Error if the option cannot be specified multiple times
                             if (!cfgDefOptionMulti(optionDefId))
                                 THROW_FMT(OptionInvalidError, "option '%s' cannot be set multiple times", cfgOptionName(optionId));
 
-                            parseOptionList[optionId].valueList = strLstNewVarLst(varVarLst(value));
+                            parseOptionList[optionId].valueList = iniGetList(config, section, key);
                         }
-                        // Convert boolean
-                        else if (cfgDefOptionType(optionDefId) == cfgDefOptTypeBoolean)
-                        {
-                            if (strcasecmp(strPtr(varStr(value)), "n") == 0)
-                                parseOptionList[optionId].negate = true;
-                            else if (strcasecmp(strPtr(varStr(value)), "y") != 0)
-                                THROW_FMT(OptionInvalidValueError, "boolean option '%s' must be 'y' or 'n'", strPtr(key));
-                        }
-                        // Else add the string value
                         else
                         {
-                            parseOptionList[optionId].valueList = strLstNew();
-                            strLstAdd(parseOptionList[optionId].valueList, varStr(value));
+                            // Get the option value
+                            const String *value = iniGet(config, section, key);
+
+                            if (strSize(value) == 0)
+                            {
+                                THROW_FMT(
+                                    OptionInvalidValueError, "section '%s', key '%s' must have a value", strPtr(section),
+                                    strPtr(key));
+                            }
+
+                            if (cfgDefOptionType(optionDefId) == cfgDefOptTypeBoolean)
+                            {
+                                if (strEqZ(value, "n"))
+                                    parseOptionList[optionId].negate = true;
+                                else if (!strEqZ(value, "y"))
+                                    THROW_FMT(OptionInvalidValueError, "boolean option '%s' must be 'y' or 'n'", strPtr(key));
+                            }
+                            // Else add the string value
+                            else
+                            {
+                                parseOptionList[optionId].valueList = strLstNew();
+                                strLstAdd(parseOptionList[optionId].valueList, value);
+                            }
                         }
                     }
                 }
@@ -849,9 +905,9 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                         if (dependOptionDefType == cfgDefOptTypeBoolean)
                         {
                             if (cfgOptionBool(dependOptionId))
-                                dependValue = varNewStrZ("1");
+                                dependValue = OPTION_VALUE_1;
                             else
-                                dependValue = varNewStrZ("0");
+                                dependValue = OPTION_VALUE_0;
                         }
                     }
 
@@ -881,7 +937,7 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                         if (!dependResolved && optionSet && parseOption->source == cfgSourceParam)
                         {
                             // Get the depend option name
-                            String *dependOptionName = strNew(cfgOptionName(dependOptionId));
+                            const String *dependOptionName = STR(cfgOptionName(dependOptionId));
 
                             // Build the list of possible depend values
                             StringList *dependValueList = strLstNew();
@@ -892,39 +948,21 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                                 const char *dependValue = cfgDefOptionDependValue(commandDefId, optionDefId, listIdx);
 
                                 // Build list based on depend option type
-                                switch (dependOptionDefType)
+                                if (dependOptionDefType == cfgDefOptTypeBoolean)
                                 {
                                     // Boolean outputs depend option name as no-* when false
-                                    case cfgDefOptTypeBoolean:
-                                    {
-                                        if (strcmp(dependValue, "0") == 0)
-                                            dependOptionName = strNewFmt("no-%s", cfgOptionName(dependOptionId));
-
-                                        break;
-                                    }
-
-                                    // String is output with quotes
-                                    case cfgDefOptTypeString:
-                                    {
-                                        strLstAdd(dependValueList, strNewFmt("'%s'", dependValue));
-                                        break;
-                                    }
-
-                                    // Other types are output plain
-                                    case cfgDefOptTypeFloat:                        // {uncovered - no depends of other types}
-                                    case cfgDefOptTypeHash:
-                                    case cfgDefOptTypeInteger:
-                                    case cfgDefOptTypeList:
-                                    case cfgDefOptTypeSize:
-                                    {
-                                        strLstAddZ(dependValueList, dependValue);   // {+uncovered}
-                                        break;                                      // {+uncovered}
-                                    }
+                                    if (strcmp(dependValue, "0") == 0)
+                                        dependOptionName = strNewFmt("no-%s", cfgOptionName(dependOptionId));
+                                }
+                                else
+                                {
+                                    ASSERT(dependOptionDefType == cfgDefOptTypePath || dependOptionDefType == cfgDefOptTypeString);
+                                    strLstAdd(dependValueList, strNewFmt("'%s'", dependValue));
                                 }
                             }
 
                             // Build the error string
-                            String *errorValue = strNew("");
+                            const String *errorValue = EMPTY_STR;
 
                             if (strLstSize(dependValueList) == 1)
                                 errorValue = strNewFmt(" = %s", strPtr(strLstGet(dependValueList, 0)));
@@ -950,11 +988,11 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                     {
                         if (optionDefType == cfgDefOptTypeBoolean)
                         {
-                            cfgOptionSet(optionId, parseOption->source, varNewBool(!parseOption->negate));
+                            cfgOptionSet(optionId, parseOption->source, VARBOOL(!parseOption->negate));
                         }
                         else if (optionDefType == cfgDefOptTypeHash)
                         {
-                            Variant *value = varNewKv();
+                            Variant *value = varNewKv(kvNew());
                             KeyValue *keyValue = varKv(value);
 
                             for (unsigned int listIdx = 0; listIdx < strLstSize(parseOption->valueList); listIdx++)
@@ -969,7 +1007,7 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                                         strPtr(strLstGet(parseOption->valueList, listIdx)), cfgOptionName(optionId));
                                 }
 
-                                kvPut(keyValue, varNewStr(strNewN(pair, (size_t)(equal - pair))), varNewStr(strNew(equal + 1)));
+                                kvPut(keyValue, VARSTR(strNewN(pair, (size_t)(equal - pair))), VARSTRZ(equal + 1));
                             }
 
                             cfgOptionSet(optionId, parseOption->source, value);
@@ -991,21 +1029,16 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                                 // Check that the value can be converted
                                 TRY_BEGIN()
                                 {
-
                                     if (optionDefType == cfgDefOptTypeInteger)
                                     {
-                                        valueDbl = (double)varInt64Force(varNewStr(value));
+                                        valueDbl = (double)varInt64Force(VARSTR(value));
                                     }
                                     else if (optionDefType == cfgDefOptTypeSize)
                                     {
                                         convertToByte(&value, &valueDbl);
                                     }
                                     else
-                                        valueDbl = varDblForce(varNewStr(value));
-                                }
-                                CATCH(AssertError)
-                                {
-                                    RETHROW();                      // {uncovered - asserts can't currently happen here this is JIC}
+                                        valueDbl = varDblForce(VARSTR(value));
                                 }
                                 CATCH_ANY()
                                 {
@@ -1025,6 +1058,37 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                                         cfgOptionName(optionId));
                                 }
                             }
+                            // Else if path make sure it is valid
+                            else if (optionDefType == cfgDefOptTypePath)
+                            {
+                                // Make sure it is long enough to be a path
+                                if (strSize(value) == 0)
+                                {
+                                    THROW_FMT(
+                                        OptionInvalidValueError, "'%s' must be >= 1 character for '%s' option", strPtr(value),
+                                        cfgOptionName(optionId));
+                                }
+
+                                // Make sure it starts with /
+                                if (!strBeginsWithZ(value, "/"))
+                                {
+                                    THROW_FMT(
+                                        OptionInvalidValueError, "'%s' must begin with / for '%s' option", strPtr(value),
+                                        cfgOptionName(optionId));
+                                }
+
+                                // Make sure there are no occurences of //
+                                if (strstr(strPtr(value), "//") != NULL)
+                                {
+                                    THROW_FMT(
+                                        OptionInvalidValueError, "'%s' cannot contain // for '%s' option", strPtr(value),
+                                        cfgOptionName(optionId));
+                                }
+
+                                // If the path ends with a / we'll strip it off (unless the value is just /)
+                                if (strEndsWithZ(value, "/") && strSize(value) != 1)
+                                    strTrunc(value, (int)strSize(value) - 1);
+                            }
 
                             // If the option has an allow list then check it
                             if (cfgDefOptionAllowList(commandDefId, optionDefId) &&
@@ -1035,7 +1099,7 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                                     cfgOptionName(optionId));
                             }
 
-                            cfgOptionSet(optionId, parseOption->source, varNewStr(value));
+                            cfgOptionSet(optionId, parseOption->source, VARSTR(value));
                         }
                     }
                     else if (parseOption->negate)
@@ -1047,7 +1111,7 @@ configParse(unsigned int argListSize, const char *argList[], bool resetLogLevel)
                         const char *value = cfgDefOptionDefault(commandDefId, optionDefId);
 
                         if (value != NULL)
-                            cfgOptionSet(optionId, cfgSourceDefault, varNewStrZ(value));
+                            cfgOptionSet(optionId, cfgSourceDefault, VARSTRZ(value));
                         else if (cfgOptionIndex(optionId) == 0 && cfgDefOptionRequired(commandDefId, optionDefId) &&
                                  !cfgCommandHelp())
                         {
