@@ -1,16 +1,21 @@
 /***********************************************************************************************************************************
 Info Handler
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "common/crypto/cipherBlock.h"
+#include "common/crypto/hash.h"
 #include "common/debug.h"
+#include "common/io/filter/filter.intern.h"
 #include "common/ini.h"
 #include "common/log.h"
 #include "common/memContext.h"
-#include "crypto/cipherBlock.h"
-#include "crypto/hash.h"
+#include "common/object.h"
+#include "common/type/json.h"
 #include "info/info.h"
 #include "storage/helper.h"
 #include "version.h"
@@ -18,8 +23,6 @@ Info Handler
 /***********************************************************************************************************************************
 Internal constants
 ***********************************************************************************************************************************/
-#define INI_COPY_EXT                                                ".copy"
-
 STRING_STATIC(INFO_SECTION_BACKREST_STR,                            "backrest");
 STRING_STATIC(INFO_SECTION_CIPHER_STR,                              "cipher");
 
@@ -33,16 +36,16 @@ Object type
 ***********************************************************************************************************************************/
 struct Info
 {
-    MemContext *memContext;                                         // Context that contains the info
-    String *fileName;                                               // Full path name of the file
-    Ini *ini;                                                       // Parsed file contents
+    MemContext *memContext;                                         // Mem context
     const String *cipherPass;                                       // Cipher passphrase if set
 };
 
+OBJECT_DEFINE_FREE(INFO);
+
 /***********************************************************************************************************************************
-Return a hash of the contents of the info file
+Generate hash for the contents of an ini file
 ***********************************************************************************************************************************/
-static CryptoHash *
+String *
 infoHash(const Ini *ini)
 {
     FUNCTION_TEST_BEGIN();
@@ -51,14 +54,15 @@ infoHash(const Ini *ini)
 
     ASSERT(ini != NULL);
 
-    CryptoHash *result = cryptoHashNew(HASH_TYPE_SHA1_STR);
+    String *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        StringList *sectionList = iniSectionList(ini);
+        IoFilter *hash = cryptoHashNew(HASH_TYPE_SHA1_STR);
+        StringList *sectionList = strLstSort(iniSectionList(ini), sortOrderAsc);
 
         // Initial JSON opening bracket
-        cryptoHashProcessC(result, (const unsigned char *)"{", 1);
+        ioFilterProcessIn(hash, BUFSTRDEF("{"));
 
         // Loop through sections and create hash for checking checksum
         for (unsigned int sectionIdx = 0; sectionIdx < strLstSize(sectionList); sectionIdx++)
@@ -67,14 +71,14 @@ infoHash(const Ini *ini)
 
             // Add a comma before additional sections
             if (sectionIdx != 0)
-                cryptoHashProcessC(result, (const unsigned char *)",", 1);
+                ioFilterProcessIn(hash, BUFSTRDEF(","));
 
             // Create the section header
-            cryptoHashProcessC(result, (const unsigned char *)"\"", 1);
-            cryptoHashProcessStr(result, section);
-            cryptoHashProcessC(result, (const unsigned char *)"\":{", 3);
+            ioFilterProcessIn(hash, BUFSTRDEF("\""));
+            ioFilterProcessIn(hash, BUFSTR(section));
+            ioFilterProcessIn(hash, BUFSTRDEF("\":{"));
 
-            StringList *keyList = iniSectionKeyList(ini, section);
+            StringList *keyList = strLstSort(iniSectionKeyList(ini, section), sortOrderAsc);
             unsigned int keyListSize = strLstSize(keyList);
 
             // Loop through values and build the section
@@ -83,24 +87,30 @@ infoHash(const Ini *ini)
                 String *key = strLstGet(keyList, keyIdx);
 
                 // Skip the backrest checksum in the file
-                if ((strEq(section, INFO_SECTION_BACKREST_STR) && !strEq(key, INFO_KEY_CHECKSUM_STR)) ||
-                    !strEq(section, INFO_SECTION_BACKREST_STR))
+                if (!strEq(section, INFO_SECTION_BACKREST_STR) || !strEq(key, INFO_KEY_CHECKSUM_STR))
                 {
-                    cryptoHashProcessC(result, (const unsigned char *)"\"", 1);
-                    cryptoHashProcessStr(result, key);
-                    cryptoHashProcessC(result, (const unsigned char *)"\":", 2);
-                    cryptoHashProcessStr(result, varStr(iniGet(ini, section, strLstGet(keyList, keyIdx))));
+                    ioFilterProcessIn(hash, BUFSTRDEF("\""));
+                    ioFilterProcessIn(hash, BUFSTR(key));
+                    ioFilterProcessIn(hash, BUFSTRDEF("\":"));
+                    ioFilterProcessIn(hash, BUFSTR(iniGet(ini, section, strLstGet(keyList, keyIdx))));
+
                     if ((keyListSize > 1) && (keyIdx < keyListSize - 1))
-                        cryptoHashProcessC(result, (const unsigned char *)",", 1);
+                        ioFilterProcessIn(hash, BUFSTRDEF(","));
                 }
             }
 
             // Close the key/value list
-            cryptoHashProcessC(result, (const unsigned char *)"}", 1);
+            ioFilterProcessIn(hash, BUFSTRDEF("}"));
         }
 
         // JSON closing bracket
-        cryptoHashProcessC(result, (const unsigned char *)"}", 1);
+        ioFilterProcessIn(hash, BUFSTRDEF("}"));
+
+        Variant *resultVar = ioFilterResult(hash);
+
+        memContextSwitch(MEM_CONTEXT_OLD());
+        result = strDup(varStr(resultVar));
+        memContextSwitch(MEM_CONTEXT_TEMP());
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -110,35 +120,35 @@ infoHash(const Ini *ini)
 /***********************************************************************************************************************************
 Load and validate the info file (or copy)
 ***********************************************************************************************************************************/
-static bool
-infoLoad(Info *this, const Storage *storage, bool copyFile, CipherType cipherType, const String *cipherPass)
+static Ini *
+infoLoad(Info *this, const Storage *storage, const String *fileName, bool copyFile, CipherType cipherType, const String *cipherPass)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace)
         FUNCTION_LOG_PARAM(INFO, this);
         FUNCTION_LOG_PARAM(STORAGE, storage);
+        FUNCTION_LOG_PARAM(STRING, fileName);                     // Full path/filename to load
         FUNCTION_LOG_PARAM(BOOL, copyFile);                       // Is this the copy file?
         FUNCTION_LOG_PARAM(ENUM, cipherType);
-        // cipherPass omitted for security
+        FUNCTION_TEST_PARAM(STRING, cipherPass);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
+    ASSERT(fileName != NULL);
 
-    bool result = false;
+    Ini *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        String *fileName = copyFile ? strCat(strDup(this->fileName), INI_COPY_EXT) : this->fileName;
+        const String *fileNameExt = copyFile ? strNewFmt("%s" INFO_COPY_EXT, strPtr(fileName)) : fileName;
 
         // Attempt to load the file
-        StorageFileRead *infoRead = storageNewReadNP(storage, fileName);
+        StorageRead *infoRead = storageNewReadP(storage, fileNameExt, .compressible = cipherType == cipherTypeNone);
 
         if (cipherType != cipherTypeNone)
         {
-            ioReadFilterGroupSet(
-                storageFileReadIo(infoRead),
-                ioFilterGroupAdd(
-                    ioFilterGroupNew(), cipherBlockFilter(cipherBlockNew(cipherModeDecrypt, cipherType, bufNewStr(cipherPass),
-                    NULL))));
+            ioFilterGroupAdd(
+                ioReadFilterGroup(storageReadIo(infoRead)),
+                cipherBlockNew(cipherModeDecrypt, cipherType, BUFSTR(cipherPass), NULL));
         }
 
         // Load and parse the info file
@@ -151,78 +161,100 @@ infoLoad(Info *this, const Storage *storage, bool copyFile, CipherType cipherTyp
         CATCH(CryptoError)
         {
             THROW_FMT(
-                CryptoError, "'%s' %s\nHINT: Is or was the repo encrypted?", strPtr(storagePathNP(storage, fileName)),
+                CryptoError, "'%s' %s\nHINT: Is or was the repo encrypted?", strPtr(storagePathNP(storage, fileNameExt)),
                 errorMessage());
         }
         TRY_END();
 
-        iniParse(this->ini, strNewBuf(buffer));
+        result = iniNew();
+        iniParse(result, strNewBuf(buffer));
 
         // Make sure the ini is valid by testing the checksum
-        String *infoChecksum = varStr(iniGet(this->ini, INFO_SECTION_BACKREST_STR, INFO_KEY_CHECKSUM_STR));
+        const String *infoChecksumJson = iniGet(result, INFO_SECTION_BACKREST_STR, INFO_KEY_CHECKSUM_STR);
+        const String *checksum = infoHash(result);
 
-        CryptoHash *hash = infoHash(this->ini);
-
-        // ??? Temporary hack until get json parser: add quotes around hash before comparing
-        if (!strEq(infoChecksum, strQuoteZ(bufHex(cryptoHash(hash)), "\"")))
+        if (strSize(infoChecksumJson) == 0)
         {
-            // Is the checksum present?
-            bool checksumMissing = strSize(infoChecksum) < 3;
-
             THROW_FMT(
-                ChecksumError, "invalid checksum in '%s', expected '%s' but %s%s%s", strPtr(storagePathNP(storage, fileName)),
-                strPtr(bufHex(cryptoHash(hash))), checksumMissing ? "no checksum found" : "found '",
-                // ??? Temporary hack until get json parser: remove quotes around hash before displaying in messsage
-                checksumMissing ? "" : strPtr(strSubN(infoChecksum, 1, strSize(infoChecksum) - 2)),
-                checksumMissing ? "" : "'");
+                ChecksumError, "invalid checksum in '%s', expected '%s' but no checksum found",
+                strPtr(storagePathNP(storage, fileNameExt)), strPtr(checksum));
+        }
+        else
+        {
+            const String *infoChecksum = jsonToStr(infoChecksumJson);
+
+            if (!strEq(infoChecksum, checksum))
+            {
+                THROW_FMT(
+                    ChecksumError, "invalid checksum in '%s', expected '%s' but found '%s'",
+                    strPtr(storagePathNP(storage, fileNameExt)), strPtr(checksum), strPtr(infoChecksum));
+            }
         }
 
         // Make sure that the format is current, otherwise error
-        if (varIntForce(iniGet(this->ini, INFO_SECTION_BACKREST_STR, INFO_KEY_FORMAT_STR)) != REPOSITORY_FORMAT)
+        if (jsonToUInt(iniGet(result, INFO_SECTION_BACKREST_STR, INFO_KEY_FORMAT_STR)) != REPOSITORY_FORMAT)
         {
             THROW_FMT(
-                FormatError, "invalid format in '%s', expected %d but found %d", strPtr(fileName), REPOSITORY_FORMAT,
-                varIntForce(iniGet(this->ini, INFO_SECTION_BACKREST_STR, INFO_KEY_FORMAT_STR)));
+                FormatError, "invalid format in '%s', expected %d but found %d", strPtr(fileNameExt), REPOSITORY_FORMAT,
+                varIntForce(VARSTR(iniGet(result, INFO_SECTION_BACKREST_STR, INFO_KEY_FORMAT_STR))));
         }
+
+        iniMove(result, MEM_CONTEXT_OLD());
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(BOOL, result);
+    FUNCTION_LOG_RETURN(INI, result);
 }
 
 /***********************************************************************************************************************************
-Load an Info object
-// ??? Need loadFile parameter to be able to load from a string.
-// ??? Need to handle modified flag, encryption and checksum, initialization, etc.
-// ??? The file MUST exist currently, so this is not actually creating the object - rather it is loading it
+Create new object
 ***********************************************************************************************************************************/
 Info *
-infoNew(const Storage *storage, const String *fileName, CipherType cipherType, const String *cipherPass)
+infoNew(void)
+{
+    FUNCTION_LOG_VOID(logLevelTrace);
+
+    Info *this = NULL;
+
+    MEM_CONTEXT_NEW_BEGIN("Info")
+    {
+        // Create object
+        this = memNew(sizeof(Info));
+        this->memContext = MEM_CONTEXT_NEW();
+    }
+    MEM_CONTEXT_NEW_END();
+
+    FUNCTION_LOG_RETURN(INFO, this);
+}
+
+/***********************************************************************************************************************************
+Create new object and load contents from a file
+***********************************************************************************************************************************/
+Info *
+infoNewLoad(const Storage *storage, const String *fileName, CipherType cipherType, const String *cipherPass, Ini **ini)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE, storage);
         FUNCTION_LOG_PARAM(STRING, fileName);                     // Full path/filename to load
         FUNCTION_LOG_PARAM(ENUM, cipherType);
-        // cipherPass omitted for security
+        FUNCTION_TEST_PARAM(STRING, cipherPass);
+        FUNCTION_LOG_PARAM_P(INI, ini);
     FUNCTION_LOG_END();
 
+    ASSERT(storage != NULL);
     ASSERT(fileName != NULL);
+    ASSERT(cipherType == cipherTypeNone || cipherPass != NULL);
 
-    Info *this = NULL;
+    Info *this = infoNew();
 
-    MEM_CONTEXT_NEW_BEGIN("info")
+    MEM_CONTEXT_BEGIN(this->memContext)
     {
-        // Create object
-        this = memNew(sizeof(Info));
-        this->memContext = MEM_CONTEXT_NEW();
-
-        this->ini = iniNew();
-        this->fileName = strDup(fileName);
+        Ini *iniLocal = NULL;
 
         // Attempt to load the primary file
         TRY_BEGIN()
         {
-            infoLoad(this, storage, false, cipherType, cipherPass);
+            iniLocal = infoLoad(this, storage, fileName, false, cipherType, cipherPass);
         }
         CATCH_ANY()
         {
@@ -233,7 +265,7 @@ infoNew(const Storage *storage, const String *fileName, CipherType cipherType, c
 
             TRY_BEGIN()
             {
-                infoLoad(this, storage, true, cipherType, cipherPass);
+                iniLocal = infoLoad(this, storage, fileName, true, cipherType, cipherPass);
             }
             CATCH_ANY()
             {
@@ -244,8 +276,8 @@ infoNew(const Storage *storage, const String *fileName, CipherType cipherType, c
                     errorType() == primaryErrorType ? errorType() :
                         (errorType() == &FileMissingError ? primaryErrorType :
                         (primaryMissing ? errorType() : &FileOpenError)),
-                    "unable to load info file '%s' or '%s" INI_COPY_EXT "':\n%s\n%s: %s",
-                    strPtr(storagePathNP(storage, this->fileName)), strPtr(storagePathNP(storage, this->fileName)),
+                    "unable to load info file '%s' or '%s" INFO_COPY_EXT "':\n%s\n%s: %s",
+                    strPtr(storagePathNP(storage, fileName)), strPtr(storagePathNP(storage, fileName)),
                     strPtr(primaryError), errorTypeName(errorType()), errorMessage());
             }
             TRY_END();
@@ -253,20 +285,74 @@ infoNew(const Storage *storage, const String *fileName, CipherType cipherType, c
         TRY_END();
 
         // Load the cipher passphrase if it exists
-        String *cipherPass = varStr(iniGetDefault(this->ini, INFO_SECTION_CIPHER_STR, INFO_KEY_CIPHER_PASS_STR, NULL));
+        const String *cipherPass = iniGetDefault(iniLocal, INFO_SECTION_CIPHER_STR, INFO_KEY_CIPHER_PASS_STR, NULL);
 
         if (cipherPass != NULL)
-        {
-            this->cipherPass = strSubN(cipherPass, 1, strSize(cipherPass) - 2);
-            strFree(cipherPass);
-        }
+            this->cipherPass = jsonToStr(cipherPass);
+
+        if (ini != NULL)
+            *ini = iniMove(iniLocal, MEM_CONTEXT_OLD());
     }
-    MEM_CONTEXT_NEW_END();
+    MEM_CONTEXT_END();
 
-    // Return buffer
     FUNCTION_LOG_RETURN(INFO, this);
 }
 
+/***********************************************************************************************************************************
+Save to file
+***********************************************************************************************************************************/
+void
+infoSave(
+    Info *this, Ini *ini, const Storage *storage, const String *fileName, CipherType cipherType, const String *cipherPass)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(INFO, this);
+        FUNCTION_LOG_PARAM(INI, ini);
+        FUNCTION_LOG_PARAM(STORAGE, storage);
+        FUNCTION_LOG_PARAM(STRING, fileName);
+        FUNCTION_LOG_PARAM(ENUM, cipherType);
+        FUNCTION_TEST_PARAM(STRING, cipherPass);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(ini != NULL);
+    ASSERT(storage != NULL);
+    ASSERT(fileName != NULL);
+    ASSERT(cipherType == cipherTypeNone || cipherPass != NULL);
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        // Add version and format
+        iniSet(ini, INFO_SECTION_BACKREST_STR, INFO_KEY_VERSION_STR, jsonFromStr(STRDEF(PROJECT_VERSION)));
+        iniSet(ini, INFO_SECTION_BACKREST_STR, INFO_KEY_FORMAT_STR, jsonFromUInt(REPOSITORY_FORMAT));
+
+        // Add cipher passphrase if defined
+        if (this->cipherPass != NULL)
+            iniSet(ini, INFO_SECTION_CIPHER_STR, INFO_KEY_CIPHER_PASS_STR, jsonFromStr(this->cipherPass));
+
+        // Add checksum (this must be set after all other values or it will not be valid)
+        iniSet(ini, INFO_SECTION_BACKREST_STR, INFO_KEY_CHECKSUM_STR, jsonFromStr(infoHash(ini)));
+
+        // Save info file
+        IoWrite *infoWrite = storageWriteIo(storageNewWriteP(storage, fileName, .compressible = cipherType == cipherTypeNone));
+
+        if (cipherType != cipherTypeNone)
+        {
+            ioFilterGroupAdd(
+                ioWriteFilterGroup(infoWrite), cipherBlockNew(cipherModeEncrypt, cipherType, BUFSTR(cipherPass), NULL));
+        }
+
+        iniSave(ini, infoWrite);
+
+        // Copy to .copy file
+        storageCopyNP(
+            storageNewReadNP(storage, fileName), storageNewWriteNP(storage, strNewFmt("%s" INFO_COPY_EXT, strPtr(fileName))));
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
 /***********************************************************************************************************************************
 Accessor functions
 ***********************************************************************************************************************************/
@@ -281,43 +367,3 @@ infoCipherPass(const Info *this)
 
     FUNCTION_TEST_RETURN(this->cipherPass);
 }
-
-Ini *
-infoIni(const Info *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(INFO, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->ini);
-}
-
-String *
-infoFileName(const Info *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(INFO, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->fileName);
-}
-
-/***********************************************************************************************************************************
-Free the object
-***********************************************************************************************************************************/
-void
-infoFree(Info *this)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(INFO, this);
-    FUNCTION_LOG_END();
-
-    if (this != NULL)
-        memContextFree(this->memContext);
-
-    FUNCTION_LOG_RETURN_VOID();
-}
