@@ -7,31 +7,38 @@ Info Handler
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
+#define INFO_TYPE                                                   Info
+#define INFO_PREFIX                                                 info
+
 typedef struct Info Info;
 
+#include "common/crypto/common.h"
 #include "common/ini.h"
-#include "crypto/crypto.h"
-#include "crypto/hash.h"
 #include "storage/storage.h"
 
 /***********************************************************************************************************************************
 Constants
 ***********************************************************************************************************************************/
+#define INFO_COPY_EXT                                               ".copy"
+
 #define INFO_KEY_FORMAT                                             "backrest-format"
     STRING_DECLARE(INFO_KEY_VERSION_STR);
 #define INFO_KEY_VERSION                                            "backrest-version"
     STRING_DECLARE(INFO_KEY_FORMAT_STR);
 
 /***********************************************************************************************************************************
-Constructor
+Constructors
 ***********************************************************************************************************************************/
-Info *infoNew(const Storage *storage, const String *fileName, CipherType cipherType, const String *cipherPass);
+Info *infoNew(void);
+Info *infoNewLoad(const Storage *storage, const String *fileName, CipherType cipherType, const String *cipherPass, Ini **ini);
+void infoSave(
+    Info *this, Ini *ini, const Storage *storage, const String *fileName, CipherType cipherType, const String *cipherPass);
 
 /***********************************************************************************************************************************
 Getters
 ***********************************************************************************************************************************/
 const String *infoCipherPass(const Info *this);
-String *infoFileName(const Info *this);
+String *infoHash(const Ini *ini);
 Ini *infoIni(const Info *this);
 
 /***********************************************************************************************************************************
