@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Archive Info Handler
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -8,43 +10,52 @@ Archive Info Handler
 
 #include "common/debug.h"
 #include "common/log.h"
-#include "common/memContext.h"
 #include "common/ini.h"
+#include "common/memContext.h"
+#include "common/object.h"
 #include "info/infoArchive.h"
 #include "info/infoPg.h"
 #include "postgres/interface.h"
 #include "storage/helper.h"
 
+/***********************************************************************************************************************************
+Constants
+***********************************************************************************************************************************/
+STRING_EXTERN(INFO_ARCHIVE_PATH_FILE_STR,                           INFO_ARCHIVE_PATH_FILE);
+STRING_EXTERN(INFO_ARCHIVE_PATH_FILE_COPY_STR,                      INFO_ARCHIVE_PATH_FILE_COPY);
+
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
 struct InfoArchive
 {
-    MemContext *memContext;                                         // Context that contains the InfoArchive
+    MemContext *memContext;                                         // Mem context
     InfoPg *infoPg;                                                 // Contents of the DB data
     String *archiveId;                                              // Archive id for the current PG version
 };
 
+OBJECT_DEFINE_FREE(INFO_ARCHIVE);
+
 /***********************************************************************************************************************************
-Create a new InfoArchive object
-// ??? Need loadFile parameter
+Create new object and load contents from a file
 ***********************************************************************************************************************************/
 InfoArchive *
-infoArchiveNew(const Storage *storage, const String *fileName, bool ignoreMissing, CipherType cipherType, const String *cipherPass)
+infoArchiveNewLoad(const Storage *storage, const String *fileName, CipherType cipherType, const String *cipherPass)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE, storage);
         FUNCTION_LOG_PARAM(STRING, fileName);
-        FUNCTION_LOG_PARAM(BOOL, ignoreMissing);
         FUNCTION_LOG_PARAM(ENUM, cipherType);
-        // cipherPass omitted for security
+        FUNCTION_TEST_PARAM(STRING, cipherPass);
     FUNCTION_LOG_END();
 
+    ASSERT(storage != NULL);
     ASSERT(fileName != NULL);
+    ASSERT(cipherType == cipherTypeNone || cipherPass != NULL);
 
     InfoArchive *this = NULL;
 
-    MEM_CONTEXT_NEW_BEGIN("infoArchive")
+    MEM_CONTEXT_NEW_BEGIN("InfoArchive")
     {
         // Create object
         this = memNew(sizeof(InfoArchive));
@@ -53,7 +64,7 @@ infoArchiveNew(const Storage *storage, const String *fileName, bool ignoreMissin
         // Catch file missing error and add archive-specific hints before rethrowing
         TRY_BEGIN()
         {
-            this->infoPg = infoPgNew(storage, fileName, infoPgArchive, cipherType, cipherPass);
+            this->infoPg = infoPgNewLoad(storage, fileName, infoPgArchive, cipherType, cipherPass, NULL);
         }
         CATCH_ANY()
         {
@@ -69,57 +80,13 @@ infoArchiveNew(const Storage *storage, const String *fileName, bool ignoreMissin
         TRY_END();
 
         // Store the archiveId for the current PG db-version db-id
-        this->archiveId = infoPgArchiveId(this->infoPg, 0);
+        this->archiveId = infoPgArchiveId(this->infoPg, infoPgDataCurrentId(this->infoPg));
     }
     MEM_CONTEXT_NEW_END();
 
-    // Return buffer
     FUNCTION_LOG_RETURN(INFO_ARCHIVE, this);
 }
 
-/***********************************************************************************************************************************
-Checks the archive info file's DB section against the PG version and system id passed in
-// ??? Should we still check that the file exists if it is required?
-***********************************************************************************************************************************/
-void
-infoArchiveCheckPg(const InfoArchive *this, unsigned int pgVersion, uint64_t pgSystemId)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(INFO_ARCHIVE, this);
-        FUNCTION_LOG_PARAM(UINT, pgVersion);
-        FUNCTION_LOG_PARAM(UINT64, pgSystemId);
-    FUNCTION_LOG_END();
-
-    ASSERT(this != NULL);
-
-    String *errorMsg = NULL;
-
-    InfoPgData archivePg = infoPgDataCurrent(this->infoPg);
-
-    if (archivePg.version != pgVersion)
-    {
-        errorMsg = strNewFmt(
-            "WAL segment version %s does not match archive version %s", strPtr(pgVersionToStr(pgVersion)),
-            strPtr(pgVersionToStr(archivePg.version)));
-    }
-
-    if (archivePg.systemId != pgSystemId)
-    {
-        errorMsg = errorMsg != NULL ? strCat(errorMsg, "\n") : strNew("");
-        strCatFmt(errorMsg,
-                "WAL segment system-id %" PRIu64 " does not match archive system-id %" PRIu64,
-                pgSystemId, archivePg.systemId);
-    }
-
-    if (errorMsg != NULL)
-    {
-        errorMsg = strCatFmt(errorMsg, "\nHINT: are you archiving to the correct stanza?");
-        THROW(ArchiveMismatchError, strPtr(errorMsg));
-    }
-
-    FUNCTION_LOG_RETURN_VOID();
-}
-
 /***********************************************************************************************************************************
 Given a backrest history id and postgres systemId and version, return the archiveId of the best match
 ***********************************************************************************************************************************/
@@ -176,7 +143,7 @@ infoArchiveIdHistoryMatch(
             strPtr(pgVersionToStr(pgVersion)), pgSystemId);
     }
 
-    FUNCTION_TEST_RETURN(archiveId);
+    FUNCTION_LOG_RETURN(STRING, archiveId);
 }
 
 /***********************************************************************************************************************************
@@ -223,19 +190,3 @@ infoArchivePg(const InfoArchive *this)
 
     FUNCTION_TEST_RETURN(this->infoPg);
 }
-
-/***********************************************************************************************************************************
-Free the info
-***********************************************************************************************************************************/
-void
-infoArchiveFree(InfoArchive *this)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(INFO_ARCHIVE, this);
-    FUNCTION_LOG_END();
-
-    if (this != NULL)
-        memContextFree(this->memContext);
-
-    FUNCTION_LOG_RETURN_VOID();
-}
