@@ -7,12 +7,12 @@ Execute Perl for Legacy Functionality
 /***********************************************************************************************************************************
 Error message for errors thrown from embedded Perl
 ***********************************************************************************************************************************/
-#define PERL_EMBED_ERROR                                            "!!!EMBEDDEDPERLERROR!!!"
+#define PERL_EMBED_ERROR                                            "===PERL-EMBED-ERROR==="
 
 /***********************************************************************************************************************************
 Functions
 ***********************************************************************************************************************************/
-int perlExec();
+int perlExec(void);
 void perlFree(int result);
 
 #endif
