@@ -1,9 +1,12 @@
 /***********************************************************************************************************************************
 Protocol Client
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include "common/debug.h"
 #include "common/log.h"
 #include "common/memContext.h"
+#include "common/object.h"
 #include "common/time.h"
 #include "common/type/json.h"
 #include "common/type/keyValue.h"
@@ -17,16 +20,14 @@ STRING_EXTERN(PROTOCOL_GREETING_NAME_STR,                           PROTOCOL_GRE
 STRING_EXTERN(PROTOCOL_GREETING_SERVICE_STR,                        PROTOCOL_GREETING_SERVICE);
 STRING_EXTERN(PROTOCOL_GREETING_VERSION_STR,                        PROTOCOL_GREETING_VERSION);
 
-STRING_EXTERN(PROTOCOL_COMMAND_STR,                                 PROTOCOL_COMMAND);
 STRING_EXTERN(PROTOCOL_COMMAND_NOOP_STR,                            PROTOCOL_COMMAND_NOOP);
 STRING_EXTERN(PROTOCOL_COMMAND_EXIT_STR,                            PROTOCOL_COMMAND_EXIT);
 
 STRING_EXTERN(PROTOCOL_ERROR_STR,                                   PROTOCOL_ERROR);
+STRING_EXTERN(PROTOCOL_ERROR_STACK_STR,                             PROTOCOL_ERROR_STACK);
 
 STRING_EXTERN(PROTOCOL_OUTPUT_STR,                                  PROTOCOL_OUTPUT);
 
-STRING_EXTERN(PROTOCOL_PARAMETER_STR,                               PROTOCOL_PARAMETER);
-
 /***********************************************************************************************************************************
 Object type
 ***********************************************************************************************************************************/
@@ -40,6 +41,22 @@ struct ProtocolClient
     TimeMSec keepAliveTime;
 };
 
+OBJECT_DEFINE_FREE(PROTOCOL_CLIENT);
+
+/***********************************************************************************************************************************
+Close protocol connection
+***********************************************************************************************************************************/
+OBJECT_DEFINE_FREE_RESOURCE_BEGIN(PROTOCOL_CLIENT, LOG, logLevelTrace)
+{
+    // Send an exit command but don't wait to see if it succeeds
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        protocolClientWriteCommand(this, protocolCommandNew(PROTOCOL_COMMAND_EXIT_STR));
+    }
+    MEM_CONTEXT_TEMP_END();
+}
+OBJECT_DEFINE_FREE_RESOURCE_END(LOG);
+
 /***********************************************************************************************************************************
 Create object
 ***********************************************************************************************************************************/
@@ -74,13 +91,13 @@ protocolClientNew(const String *name, const String *service, IoRead *read, IoWri
         MEM_CONTEXT_TEMP_BEGIN()
         {
             String *greeting = ioReadLine(this->read);
-            KeyValue *greetingKv = varKv(jsonToVar(greeting));
+            KeyValue *greetingKv = jsonToKv(greeting);
 
             const String *expected[] =
             {
-                PROTOCOL_GREETING_NAME_STR, strNew(PROJECT_NAME),
+                PROTOCOL_GREETING_NAME_STR, STRDEF(PROJECT_NAME),
                 PROTOCOL_GREETING_SERVICE_STR, service,
-                PROTOCOL_GREETING_VERSION_STR, strNew(PROJECT_VERSION),
+                PROTOCOL_GREETING_VERSION_STR, STRDEF(PROJECT_VERSION),
             };
 
             for (unsigned int expectedIdx = 0; expectedIdx < sizeof(expected) / sizeof(char *) / 2; expectedIdx++)
@@ -88,7 +105,7 @@ protocolClientNew(const String *name, const String *service, IoRead *read, IoWri
                 const String *expectedKey = expected[expectedIdx * 2];
                 const String *expectedValue = expected[expectedIdx * 2 + 1];
 
-                const Variant *actualValue = kvGet(greetingKv, varNewStr(expectedKey));
+                const Variant *actualValue = kvGet(greetingKv, VARSTR(expectedKey));
 
                 if (actualValue == NULL)
                     THROW_FMT(ProtocolError, "unable to find greeting key '%s'", strPtr(expectedKey));
@@ -110,7 +127,7 @@ protocolClientNew(const String *name, const String *service, IoRead *read, IoWri
         protocolClientNoOp(this);
 
         // Set a callback to shutdown the protocol
-        memContextCallback(this->memContext, (MemContextCallback)protocolClientFree, this);
+        memContextCallbackSet(this->memContext, protocolClientFreeResource, this);
     }
     MEM_CONTEXT_NEW_END();
 
@@ -120,7 +137,7 @@ protocolClientNew(const String *name, const String *service, IoRead *read, IoWri
 /***********************************************************************************************************************************
 Read the command output
 ***********************************************************************************************************************************/
-const VariantList *
+const Variant *
 protocolClientReadOutput(ProtocolClient *this, bool outputRequired)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
@@ -130,7 +147,7 @@ protocolClientReadOutput(ProtocolClient *this, bool outputRequired)
 
     ASSERT(this != NULL);
 
-    const VariantList *result = NULL;
+    const Variant *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
@@ -139,31 +156,39 @@ protocolClientReadOutput(ProtocolClient *this, bool outputRequired)
         KeyValue *responseKv = varKv(jsonToVar(response));
 
         // Process error if any
-        const Variant *error = kvGet(responseKv, varNewStr(PROTOCOL_ERROR_STR));
+        const Variant *error = kvGet(responseKv, VARSTR(PROTOCOL_ERROR_STR));
 
         if (error != NULL)
         {
-            const String *message = varStr(kvGet(responseKv, varNewStr(PROTOCOL_OUTPUT_STR)));
+            const ErrorType *type = errorTypeFromCode(varIntForce(error));
+            const String *message = varStr(kvGet(responseKv, VARSTR(PROTOCOL_OUTPUT_STR)));
 
-            THROWP_FMT(
-                errorTypeFromCode(varIntForce(error)), "%s: %s", strPtr(this->errorPrefix),
-                message == NULL ? "no details available" : strPtr(message));
+            // Required part of the message
+            String *throwMessage = strNewFmt(
+                "%s: %s", strPtr(this->errorPrefix), message == NULL ? "no details available" : strPtr(message));
+
+            // Add stack trace if the error is an assertion or debug-level logging is enabled
+            if (type == &AssertError || logAny(logLevelDebug))
+            {
+                const String *stack = varStr(kvGet(responseKv, VARSTR(PROTOCOL_ERROR_STACK_STR)));
+
+                strCat(throwMessage, "\n");
+                strCat(throwMessage, stack == NULL ? "no stack trace available" : strPtr(stack));
+            }
+
+            THROWP(type, strPtr(throwMessage));
         }
 
         // Get output
-        const Variant *output = kvGet(responseKv, varNewStr(PROTOCOL_OUTPUT_STR));
-
-        ASSERT(output != NULL);
-        ASSERT(varType(output) == varTypeVariantList);
+        result = kvGet(responseKv, VARSTR(PROTOCOL_OUTPUT_STR));
 
         if (outputRequired)
         {
             // Just move the entire response kv since the output is the largest part if it
             kvMove(responseKv, MEM_CONTEXT_OLD());
-            result = varVarLst(output);
         }
         // Else if no output is required then there should not be any
-        else if (varLstSize(varVarLst(output)) != 0)
+        else if (result != NULL)
             THROW(AssertError, "no output required by command");
 
         // Reset the keep alive time
@@ -171,25 +196,25 @@ protocolClientReadOutput(ProtocolClient *this, bool outputRequired)
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN_CONST(VARIANT_LIST, result);
+    FUNCTION_LOG_RETURN_CONST(VARIANT, result);
 }
 
 /***********************************************************************************************************************************
 Write the protocol command
 ***********************************************************************************************************************************/
 void
-protocolClientWriteCommand(ProtocolClient *this, const KeyValue *command)
+protocolClientWriteCommand(ProtocolClient *this, const ProtocolCommand *command)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(PROTOCOL_CLIENT, this);
-        FUNCTION_LOG_PARAM(KEY_VALUE, command);
+        FUNCTION_LOG_PARAM(PROTOCOL_COMMAND, command);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(command != NULL);
 
     // Write out the command
-    ioWriteLine(this->write, kvToJson(command, 0));
+    ioWriteStrLine(this->write, protocolCommandJson(command));
     ioWriteFlush(this->write);
 
     // Reset the keep alive time
@@ -201,12 +226,12 @@ protocolClientWriteCommand(ProtocolClient *this, const KeyValue *command)
 /***********************************************************************************************************************************
 Execute a protocol command and get the output
 ***********************************************************************************************************************************/
-const VariantList *
-protocolClientExecute(ProtocolClient *this, const KeyValue *command, bool outputRequired)
+const Variant *
+protocolClientExecute(ProtocolClient *this, const ProtocolCommand *command, bool outputRequired)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM(PROTOCOL_CLIENT, this);
-        FUNCTION_LOG_PARAM(KEY_VALUE, command);
+        FUNCTION_LOG_PARAM(PROTOCOL_COMMAND, command);
         FUNCTION_LOG_PARAM(BOOL, outputRequired);
     FUNCTION_LOG_END();
 
@@ -215,11 +240,11 @@ protocolClientExecute(ProtocolClient *this, const KeyValue *command, bool output
 
     protocolClientWriteCommand(this, command);
 
-    FUNCTION_LOG_RETURN_CONST(VARIANT_LIST, protocolClientReadOutput(this, outputRequired));
+    FUNCTION_LOG_RETURN_CONST(VARIANT, protocolClientReadOutput(this, outputRequired));
 }
 
 /***********************************************************************************************************************************
-Move the file object to a new context
+Move the protocol client object to a new context
 ***********************************************************************************************************************************/
 ProtocolClient *
 protocolClientMove(ProtocolClient *this, MemContext *parentNew)
@@ -251,7 +276,7 @@ protocolClientNoOp(ProtocolClient *this)
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        protocolClientExecute(this, kvPut(kvNew(), varNewStr(PROTOCOL_COMMAND_STR), varNewStr(PROTOCOL_COMMAND_NOOP_STR)), false);
+        protocolClientExecute(this, protocolCommandNew(PROTOCOL_COMMAND_NOOP_STR), false);
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -296,30 +321,3 @@ protocolClientToLog(const ProtocolClient *this)
 {
     return strNewFmt("{name: %s}", strPtr(this->name));
 }
-
-/***********************************************************************************************************************************
-Free the file
-***********************************************************************************************************************************/
-void
-protocolClientFree(ProtocolClient *this)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(PROTOCOL_CLIENT, this);
-    FUNCTION_LOG_END();
-
-    if (this != NULL)
-    {
-        memContextCallbackClear(this->memContext);
-
-        // Send an exit command but don't wait to see if it succeeds
-        MEM_CONTEXT_TEMP_BEGIN()
-        {
-            protocolClientWriteCommand(this, kvPut(kvNew(), varNewStr(PROTOCOL_COMMAND_STR), varNewStr(PROTOCOL_COMMAND_EXIT_STR)));
-        }
-        MEM_CONTEXT_TEMP_END();
-
-        memContextFree(this->memContext);
-    }
-
-    FUNCTION_LOG_RETURN_VOID();
-}
