@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Storage Helper
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <string.h>
 
 #include "common/debug.h"
@@ -8,9 +10,10 @@ Storage Helper
 #include "common/regExp.h"
 #include "config/config.h"
 #include "protocol/helper.h"
-#include "storage/driver/posix/storage.h"
-#include "storage/driver/remote/storage.h"
-#include "storage/driver/s3/storage.h"
+#include "storage/cifs/storage.h"
+#include "storage/posix/storage.h"
+#include "storage/remote/storage.h"
+#include "storage/s3/storage.h"
 #include "storage/helper.h"
 
 /***********************************************************************************************************************************
@@ -31,7 +34,10 @@ static struct
 
     Storage *storageLocal;                                          // Local read-only storage
     Storage *storageLocalWrite;                                     // Local write storage
+    Storage *storagePg;                                             // PostgreSQL read-only storage
+    Storage *storagePgWrite;                                        // PostgreSQL write storage
     Storage *storageRepo;                                           // Repository read-only storage
+    Storage *storageRepoWrite;                                      // Repository write storage
     Storage *storageSpool;                                          // Spool read-only storage
     Storage *storageSpoolWrite;                                     // Spool write storage
 
@@ -69,7 +75,7 @@ storageHelperStanzaInit(const bool stanzaRequired)
     FUNCTION_TEST_VOID();
 
     // If the stanza is NULL and the storage has not already been initialized then initialize the stanza
-    if (storageHelper.stanza == NULL && !storageHelper.stanzaInit)
+    if (!storageHelper.stanzaInit)
     {
         if (stanzaRequired && cfgOptionStr(cfgOptStanza) == NULL)
             THROW(AssertError, "stanza cannot be NULL for this storage object");
@@ -81,12 +87,6 @@ storageHelperStanzaInit(const bool stanzaRequired)
         }
         MEM_CONTEXT_END();
     }
-    else if ((storageHelper.stanza == NULL && cfgOptionStr(cfgOptStanza) != NULL) ||
-        (cfgOptionStr(cfgOptStanza) != NULL && !strEq(storageHelper.stanza, cfgOptionStr(cfgOptStanza))))
-    {
-        THROW_FMT(
-            AssertError, "stanza has changed from '%s' to '%s'", strPtr(storageHelper.stanza), strPtr(cfgOptionStr(cfgOptStanza)));
-    }
 
     FUNCTION_TEST_RETURN_VOID();
 }
@@ -105,9 +105,8 @@ storageLocal(void)
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
-            storageHelper.storageLocal = storageDriverPosixInterface(
-                storageDriverPosixNew(
-                    FSLASH_STR, STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, false, NULL));
+            storageHelper.storageLocal =  storagePosixNew(
+                FSLASH_STR, STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, false, NULL);
         }
         MEM_CONTEXT_END();
     }
@@ -131,15 +130,109 @@ storageLocalWrite(void)
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
-            storageHelper.storageLocalWrite = storageDriverPosixInterface(
-                storageDriverPosixNew(
-                    FSLASH_STR, STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, true, NULL));
+            storageHelper.storageLocalWrite = storagePosixNew(
+                FSLASH_STR, STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, true, NULL);
         }
         MEM_CONTEXT_END();
     }
 
     FUNCTION_TEST_RETURN(storageHelper.storageLocalWrite);
 }
+/***********************************************************************************************************************************
+Get the pg storage
+***********************************************************************************************************************************/
+static Storage *
+storagePgGet(bool write)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(BOOL, write);
+    FUNCTION_TEST_END();
+
+    Storage *result = NULL;
+
+    // Determine which host to use
+    unsigned int hostId = cfgOptionTest(cfgOptHostId) ? cfgOptionUInt(cfgOptHostId) : 1;
+
+    // Use remote storage
+    if (!pgIsLocal(hostId))
+    {
+        result = storageRemoteNew(
+            STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, NULL,
+            protocolRemoteGet(protocolStorageTypePg, hostId), cfgOptionUInt(cfgOptCompressLevelNetwork));
+    }
+    // Use Posix storage
+    else
+    {
+        result = storagePosixNew(
+            cfgOptionStr(cfgOptPgPath + hostId - 1), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, NULL);
+    }
+
+    FUNCTION_TEST_RETURN(result);
+}
+
+/***********************************************************************************************************************************
+Get ready-only PostgreSQL storage
+***********************************************************************************************************************************/
+const Storage *
+storagePg(void)
+{
+    FUNCTION_TEST_VOID();
+
+    if (storageHelper.storagePg == NULL)
+    {
+        storageHelperInit();
+
+        MEM_CONTEXT_BEGIN(storageHelper.memContext)
+        {
+            storageHelper.storagePg = storagePgGet(false);
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_TEST_RETURN(storageHelper.storagePg);
+}
+
+/***********************************************************************************************************************************
+Get write PostgreSQL storage
+***********************************************************************************************************************************/
+const Storage *
+storagePgWrite(void)
+{
+    FUNCTION_TEST_VOID();
+
+    if (storageHelper.storagePgWrite == NULL)
+    {
+        storageHelperInit();
+
+        MEM_CONTEXT_BEGIN(storageHelper.memContext)
+        {
+            storageHelper.storagePgWrite = storagePgGet(true);
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_TEST_RETURN(storageHelper.storagePgWrite);
+}
+
+/***********************************************************************************************************************************
+Create the WAL regular expression
+***********************************************************************************************************************************/
+static void
+storageHelperRepoInit(void)
+{
+    FUNCTION_TEST_VOID();
+
+    if (storageHelper.walRegExp == NULL)
+    {
+        MEM_CONTEXT_BEGIN(memContextTop())
+        {
+            storageHelper.walRegExp = regExpNew(STRDEF("^[0-F]{24}"));
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_TEST_RETURN_VOID();
+}
 
 /***********************************************************************************************************************************
 Construct a repo path from an expression and path
@@ -206,38 +299,49 @@ storageRepoGet(const String *type, bool write)
     FUNCTION_TEST_END();
 
     ASSERT(type != NULL);
-    ASSERT(write == false);                                         // ??? Need to create cifs driver before storage can be writable
 
     Storage *result = NULL;
 
-    // If the repo is remote then use remote storage
+    // Use remote storage
     if (!repoIsLocal())
     {
-        result = storageDriverRemoteInterface(
-            storageDriverRemoteNew(
-                cfgOptionStr(cfgOptRepoPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write,
-                storageRepoPathExpression, remoteTypeRepo, 1));
+        result = storageRemoteNew(
+            STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, storageRepoPathExpression,
+            protocolRemoteGet(protocolStorageTypeRepo, 1), cfgOptionUInt(cfgOptCompressLevelNetwork));
     }
-    // For now treat posix and cifs drivers as if they are the same.  This won't be true once the repository storage becomes
-    // writable but for now it's OK.  The assertion above should pop if we try to create writable repo storage.
-    else if (strEqZ(type, STORAGE_TYPE_POSIX) || strEqZ(type, STORAGE_TYPE_CIFS))
+    // Use CIFS storage
+    else if (strEqZ(type, STORAGE_TYPE_CIFS))
     {
-        result = storageDriverPosixInterface(
-            storageDriverPosixNew(
-                cfgOptionStr(cfgOptRepoPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write,
-                storageRepoPathExpression));
+        result = storageCifsNew(
+            cfgOptionStr(cfgOptRepoPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, storageRepoPathExpression);
     }
+    // Use Posix storage
+    else if (strEqZ(type, STORAGE_TYPE_POSIX))
+    {
+        result = storagePosixNew(
+            cfgOptionStr(cfgOptRepoPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, write, storageRepoPathExpression);
+    }
+    // Use S3 storage
     else if (strEqZ(type, STORAGE_TYPE_S3))
     {
-        result = storageDriverS3Interface(
-            storageDriverS3New(
-                cfgOptionStr(cfgOptRepoPath), write, storageRepoPathExpression, cfgOptionStr(cfgOptRepoS3Bucket),
-                cfgOptionStr(cfgOptRepoS3Endpoint), cfgOptionStr(cfgOptRepoS3Region), cfgOptionStr(cfgOptRepoS3Key),
-                cfgOptionStr(cfgOptRepoS3KeySecret), cfgOptionTest(cfgOptRepoS3Token) ? cfgOptionStr(cfgOptRepoS3Token) : NULL,
-                cfgOptionTest(cfgOptRepoS3Host) ? cfgOptionStr(cfgOptRepoS3Host) : NULL,
-                STORAGE_DRIVER_S3_PORT_DEFAULT, STORAGE_DRIVER_S3_TIMEOUT_DEFAULT, cfgOptionBool(cfgOptRepoS3VerifySsl),
-                cfgOptionTest(cfgOptRepoS3CaFile) ? cfgOptionStr(cfgOptRepoS3CaFile) : NULL,
-                cfgOptionTest(cfgOptRepoS3CaPath) ? cfgOptionStr(cfgOptRepoS3CaPath) : NULL));
+        // Set the default port
+        unsigned int port = cfgOptionUInt(cfgOptRepoS3Port);
+
+        // Extract port from the endpoint and host if it is present
+        const String *endPoint = cfgOptionHostPort(cfgOptRepoS3Endpoint, &port);
+        const String *host = cfgOptionHostPort(cfgOptRepoS3Host, &port);
+
+        // If the port option was set explicitly then use it in preference to appended ports
+        if (cfgOptionSource(cfgOptRepoS3Port) != cfgSourceDefault)
+            port = cfgOptionUInt(cfgOptRepoS3Port);
+
+        result = storageS3New(
+            cfgOptionStr(cfgOptRepoPath), write, storageRepoPathExpression, cfgOptionStr(cfgOptRepoS3Bucket), endPoint,
+            cfgOptionStr(cfgOptRepoS3Region), cfgOptionStr(cfgOptRepoS3Key), cfgOptionStr(cfgOptRepoS3KeySecret),
+            cfgOptionTest(cfgOptRepoS3Token) ? cfgOptionStr(cfgOptRepoS3Token) : NULL, STORAGE_S3_PARTSIZE_MIN,
+            STORAGE_S3_DELETE_MAX, host, port, STORAGE_S3_TIMEOUT_DEFAULT, cfgOptionBool(cfgOptRepoS3VerifyTls),
+            cfgOptionTest(cfgOptRepoS3CaFile) ? cfgOptionStr(cfgOptRepoS3CaFile) : NULL,
+            cfgOptionTest(cfgOptRepoS3CaPath) ? cfgOptionStr(cfgOptRepoS3CaPath) : NULL);
     }
     else
         THROW_FMT(AssertError, "invalid storage type '%s'", strPtr(type));
@@ -257,10 +361,10 @@ storageRepo(void)
     {
         storageHelperInit();
         storageHelperStanzaInit(false);
+        storageHelperRepoInit();
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
-            storageHelper.walRegExp = regExpNew(STRING_CONST("^[0-F]{24}"));
             storageHelper.storageRepo = storageRepoGet(cfgOptionStr(cfgOptRepoType), false);
         }
         MEM_CONTEXT_END();
@@ -269,6 +373,30 @@ storageRepo(void)
     FUNCTION_TEST_RETURN(storageHelper.storageRepo);
 }
 
+/***********************************************************************************************************************************
+Get a writable repository storage object
+***********************************************************************************************************************************/
+const Storage *
+storageRepoWrite(void)
+{
+    FUNCTION_TEST_VOID();
+
+    if (storageHelper.storageRepoWrite == NULL)
+    {
+        storageHelperInit();
+        storageHelperStanzaInit(false);
+        storageHelperRepoInit();
+
+        MEM_CONTEXT_BEGIN(storageHelper.memContext)
+        {
+            storageHelper.storageRepoWrite = storageRepoGet(cfgOptionStr(cfgOptRepoType), true);
+        }
+        MEM_CONTEXT_END();
+    }
+
+    FUNCTION_TEST_RETURN(storageHelper.storageRepoWrite);
+}
+
 /***********************************************************************************************************************************
 Get a spool storage object
 ***********************************************************************************************************************************/
@@ -320,10 +448,9 @@ storageSpool(void)
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
-            storageHelper.storageSpool = storageDriverPosixInterface(
-                storageDriverPosixNew(
-                    cfgOptionStr(cfgOptSpoolPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, false,
-                    storageSpoolPathExpression));
+            storageHelper.storageSpool = storagePosixNew(
+                cfgOptionStr(cfgOptSpoolPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, false,
+                storageSpoolPathExpression);
         }
         MEM_CONTEXT_END();
     }
@@ -346,13 +473,31 @@ storageSpoolWrite(void)
 
         MEM_CONTEXT_BEGIN(storageHelper.memContext)
         {
-            storageHelper.storageSpoolWrite = storageDriverPosixInterface(
-                storageDriverPosixNew(
-                    cfgOptionStr(cfgOptSpoolPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, true,
-                    storageSpoolPathExpression));
+            storageHelper.storageSpoolWrite = storagePosixNew(
+                cfgOptionStr(cfgOptSpoolPath), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, true,
+                storageSpoolPathExpression);
         }
         MEM_CONTEXT_END();
     }
 
     FUNCTION_TEST_RETURN(storageHelper.storageSpoolWrite);
 }
+
+/***********************************************************************************************************************************
+Free all storage helper objects.
+
+This should be done on any config load to ensure that stanza changes are honored.  Currently this is only done in testing, but in
+the future it will likely be done in production as well.
+***********************************************************************************************************************************/
+void
+storageHelperFree(void)
+{
+    FUNCTION_TEST_VOID();
+
+    if (storageHelper.memContext != NULL)
+        memContextFree(storageHelper.memContext);
+
+    memset(&storageHelper, 0, sizeof(storageHelper));
+
+    FUNCTION_TEST_RETURN_VOID();
+}
