@@ -34,8 +34,13 @@ Functions
 ***********************************************************************************************************************************/
 const Storage *storageLocal(void);
 const Storage *storageLocalWrite(void);
+const Storage *storagePg(void);
+const Storage *storagePgWrite(void);
 const Storage *storageRepo(void);
+const Storage *storageRepoWrite(void);
 const Storage *storageSpool(void);
 const Storage *storageSpoolWrite(void);
 
+void storageHelperFree(void);
+
 #endif
