@@ -21,10 +21,15 @@ Object type
 ***********************************************************************************************************************************/
 typedef struct StorageInfo
 {
-    bool exists;                                                    // Does the path/file/link exist?
+    const String *name;                                             // Name of path/file/link
+    const String *linkDestination;                                  // Destination if this is a link
     StorageType type;                                               // Type file/path/link)
-    size_t size;                                                    // Size (path/link is 0)
+    bool exists;                                                    // Does the path/file/link exist?
+    const String *user;                                             // User that owns the file
+    const String *group;                                            // Group that owns the file
     mode_t mode;                                                    // Mode of path/file/link
+    time_t timeModified;                                            // Time file was last modified
+    uint64_t size;                                                  // Size (path/link is 0)
 } StorageInfo;
 
 /***********************************************************************************************************************************
