@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Storage Interface
 ***********************************************************************************************************************************/
+#include "build.auto.h"
+
 #include <stdio.h>
 #include <string.h>
 
@@ -8,6 +10,7 @@ Storage Interface
 #include "common/io/io.h"
 #include "common/log.h"
 #include "common/memContext.h"
+#include "common/object.h"
 #include "common/wait.h"
 #include "storage/storage.intern.h"
 
@@ -25,16 +28,18 @@ struct Storage
     mode_t modeFile;
     mode_t modePath;
     bool write;
-    bool pathResolve;
+    bool pathEnforce;
     StoragePathExpressionCallback pathExpressionFunction;
 };
 
+OBJECT_DEFINE_FREE(STORAGE);
+
 /***********************************************************************************************************************************
 New storage object
 ***********************************************************************************************************************************/
 Storage *
 storageNew(
-    const String *type, const String *path, mode_t modeFile, mode_t modePath, bool write, bool pathResolve,
+    const String *type, const String *path, mode_t modeFile, mode_t modePath, bool write,
     StoragePathExpressionCallback pathExpressionFunction, void *driver, StorageInterface interface)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
@@ -43,23 +48,19 @@ storageNew(
         FUNCTION_LOG_PARAM(MODE, modeFile);
         FUNCTION_LOG_PARAM(MODE, modePath);
         FUNCTION_LOG_PARAM(BOOL, write);
-        FUNCTION_LOG_PARAM(BOOL, pathResolve);
         FUNCTION_LOG_PARAM(FUNCTIONP, pathExpressionFunction);
         FUNCTION_LOG_PARAM_P(VOID, driver);
         FUNCTION_LOG_PARAM(STORAGE_INTERFACE, interface);
     FUNCTION_LOG_END();
 
     ASSERT(type != NULL);
-    ASSERT(path != NULL && strSize(path) >= 1 && strPtr(path)[0] == '/');
+    ASSERT(path == NULL || (strSize(path) >= 1 && strPtr(path)[0] == '/'));
     ASSERT(driver != NULL);
     ASSERT(interface.exists != NULL);
-    ASSERT(interface.info != NULL);
     ASSERT(interface.list != NULL);
     ASSERT(interface.newRead != NULL);
     ASSERT(interface.newWrite != NULL);
-    ASSERT(interface.pathCreate != NULL);
     ASSERT(interface.pathRemove != NULL);
-    ASSERT(interface.pathSync != NULL);
     ASSERT(interface.remove != NULL);
 
     Storage *this = NULL;
@@ -72,8 +73,8 @@ storageNew(
     this->path = strDup(path);
     this->modeFile = modeFile;
     this->modePath = modePath;
+    this->pathEnforce = true;
     this->write = write;
-    this->pathResolve = pathResolve;
     this->pathExpressionFunction = pathExpressionFunction;
 
     FUNCTION_LOG_RETURN(STORAGE, this);
@@ -83,11 +84,11 @@ storageNew(
 Copy a file
 ***********************************************************************************************************************************/
 bool
-storageCopy(StorageFileRead *source, StorageFileWrite *destination)
+storageCopy(StorageRead *source, StorageWrite *destination)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(STORAGE_FILE_READ, source);
-        FUNCTION_LOG_PARAM(STORAGE_FILE_WRITE, destination);
+        FUNCTION_LOG_PARAM(STORAGE_READ, source);
+        FUNCTION_LOG_PARAM(STORAGE_WRITE, destination);
     FUNCTION_LOG_END();
 
     ASSERT(source != NULL);
@@ -98,25 +99,25 @@ storageCopy(StorageFileRead *source, StorageFileWrite *destination)
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Open source file
-        if (ioReadOpen(storageFileReadIo(source)))
+        if (ioReadOpen(storageReadIo(source)))
         {
             // Open the destination file now that we know the source file exists and is readable
-            ioWriteOpen(storageFileWriteIo(destination));
+            ioWriteOpen(storageWriteIo(destination));
 
             // Copy data from source to destination
             Buffer *read = bufNew(ioBufferSize());
 
             do
             {
-                ioRead(storageFileReadIo(source), read);
-                ioWrite(storageFileWriteIo(destination), read);
+                ioRead(storageReadIo(source), read);
+                ioWrite(storageWriteIo(destination), read);
                 bufUsedZero(read);
             }
-            while (!ioReadEof(storageFileReadIo(source)));
+            while (!ioReadEof(storageReadIo(source)));
 
             // Close the source and destination files
-            ioReadClose(storageFileReadIo(source));
-            ioWriteClose(storageFileWriteIo(destination));
+            ioReadClose(storageReadIo(source));
+            ioWriteClose(storageWriteIo(destination));
 
             // Set result to indicate that the file was copied
             result = true;
@@ -128,7 +129,7 @@ storageCopy(StorageFileRead *source, StorageFileWrite *destination)
 }
 
 /***********************************************************************************************************************************
-Does a file exist? This function is only for files, not directories.
+Does a file exist? This function is only for files, not paths.
 ***********************************************************************************************************************************/
 bool
 storageExists(const Storage *this, const String *pathExp, StorageExistsParam param)
@@ -168,10 +169,10 @@ storageExists(const Storage *this, const String *pathExp, StorageExistsParam par
 Read from storage into a buffer
 ***********************************************************************************************************************************/
 Buffer *
-storageGet(StorageFileRead *file, StorageGetParam param)
+storageGet(StorageRead *file, StorageGetParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(STORAGE_FILE_READ, file);
+        FUNCTION_LOG_PARAM(STORAGE_READ, file);
         FUNCTION_LOG_PARAM(SIZE, param.exactSize);
     FUNCTION_LOG_END();
 
@@ -180,7 +181,7 @@ storageGet(StorageFileRead *file, StorageGetParam param)
     Buffer *result = NULL;
 
     // If the file exists
-    if (ioReadOpen(storageFileReadIo(file)))
+    if (ioReadOpen(storageReadIo(file)))
     {
         MEM_CONTEXT_TEMP_BEGIN()
         {
@@ -188,13 +189,13 @@ storageGet(StorageFileRead *file, StorageGetParam param)
             if (param.exactSize > 0)
             {
                 result = bufNew(param.exactSize);
-                ioRead(storageFileReadIo(file), result);
+                ioRead(storageReadIo(file), result);
 
                 // If an exact read make sure the size is as expected
                 if (bufUsed(result) != param.exactSize)
                 {
                     THROW_FMT(
-                        FileReadError, "unable to read %zu byte(s) from '%s'", param.exactSize, strPtr(storageFileReadName(file)));
+                        FileReadError, "unable to read %zu byte(s) from '%s'", param.exactSize, strPtr(storageReadName(file)));
                 }
             }
             // Else read entire file
@@ -206,13 +207,13 @@ storageGet(StorageFileRead *file, StorageGetParam param)
                 do
                 {
                     // Read data
-                    ioRead(storageFileReadIo(file), read);
+                    ioRead(storageReadIo(file), read);
 
                     // Add to result and free read buffer
                     bufCat(result, read);
                     bufUsedZero(read);
                 }
-                while (!ioReadEof(storageFileReadIo(file)));
+                while (!ioReadEof(storageReadIo(file)));
             }
 
             // Move buffer to parent context on success
@@ -220,7 +221,7 @@ storageGet(StorageFileRead *file, StorageGetParam param)
         }
         MEM_CONTEXT_TEMP_END();
 
-        ioReadClose(storageFileReadIo(file));
+        ioReadClose(storageReadIo(file));
     }
 
     FUNCTION_LOG_RETURN(BUFFER, result);
@@ -236,9 +237,11 @@ storageInfo(const Storage *this, const String *fileExp, StorageInfoParam param)
         FUNCTION_LOG_PARAM(STORAGE, this);
         FUNCTION_LOG_PARAM(STRING, fileExp);
         FUNCTION_LOG_PARAM(BOOL, param.ignoreMissing);
+        FUNCTION_LOG_PARAM(BOOL, param.followLink);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
+    ASSERT(this->interface.info != NULL);
 
     StorageInfo result = {0};
 
@@ -248,13 +251,62 @@ storageInfo(const Storage *this, const String *fileExp, StorageInfoParam param)
         String *file = storagePathNP(this, fileExp);
 
         // Call driver function
-        result = this->interface.info(this->driver, file, param.ignoreMissing);
+        result = this->interface.info(this->driver, file, param.followLink);
+
+        // Error if the file missing and not ignoring
+        if (!result.exists && !param.ignoreMissing)
+            THROW_SYS_ERROR_FMT(FileOpenError, STORAGE_ERROR_INFO_MISSING, strPtr(file));
+
+        // Dup the strings into the calling context
+        memContextSwitch(MEM_CONTEXT_OLD());
+        result.linkDestination = strDup(result.linkDestination);
+        result.user = strDup(result.user);
+        result.group = strDup(result.group);
+        memContextSwitch(MEM_CONTEXT_TEMP());
     }
     MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN(STORAGE_INFO, result);
 }
 
+/***********************************************************************************************************************************
+Info for all files/paths in a path
+***********************************************************************************************************************************/
+bool
+storageInfoList(
+    const Storage *this, const String *pathExp, StorageInfoListCallback callback, void *callbackData, StorageInfoListParam param)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(STORAGE, this);
+        FUNCTION_LOG_PARAM(STRING, pathExp);
+        FUNCTION_LOG_PARAM(FUNCTIONP, callback);
+        FUNCTION_LOG_PARAM_P(VOID, callbackData);
+        FUNCTION_LOG_PARAM(BOOL, param.errorOnMissing);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(callback != NULL);
+    ASSERT(this->interface.infoList != NULL);
+    ASSERT(!param.errorOnMissing || storageFeature(this, storageFeaturePath));
+
+    bool result = false;
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        // Build the path
+        String *path = storagePathNP(this, pathExp);
+
+        // Call driver function
+        result = this->interface.infoList(this->driver, path, callback, callbackData);
+
+        if (!result && param.errorOnMissing)
+            THROW_FMT(PathMissingError, STORAGE_ERROR_LIST_INFO_MISSING, strPtr(path));
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN(BOOL, result);
+}
+
 /***********************************************************************************************************************************
 Get a list of files from a directory
 ***********************************************************************************************************************************/
@@ -265,10 +317,13 @@ storageList(const Storage *this, const String *pathExp, StorageListParam param)
         FUNCTION_LOG_PARAM(STORAGE, this);
         FUNCTION_LOG_PARAM(STRING, pathExp);
         FUNCTION_LOG_PARAM(BOOL, param.errorOnMissing);
+        FUNCTION_LOG_PARAM(BOOL, param.nullOnMissing);
         FUNCTION_LOG_PARAM(STRING, param.expression);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
+    ASSERT(!param.errorOnMissing || !param.nullOnMissing);
+    ASSERT(!param.errorOnMissing || storageFeature(this, storageFeaturePath));
 
     StringList *result = NULL;
 
@@ -277,8 +332,24 @@ storageList(const Storage *this, const String *pathExp, StorageListParam param)
         // Build the path
         String *path = storagePathNP(this, pathExp);
 
+        // Get the list
+        result = this->interface.list(this->driver, path, param.expression);
+
+        // If the path does not exist
+        if (result == NULL)
+        {
+            // Error if requested
+            if (param.errorOnMissing)
+                THROW_FMT(PathMissingError, STORAGE_ERROR_LIST_MISSING, strPtr(path));
+
+            // Build an empty list if the directory does not exist by default.  This makes the logic in calling functions simpler
+            // when they don't care if the path is missing.
+            if (!param.nullOnMissing)
+                result = strLstNew();
+        }
+
         // Move list up to the old context
-        result = strLstMove(this->interface.list(this->driver, path, param.errorOnMissing, param.expression), MEM_CONTEXT_OLD());
+        result = strLstMove(result, MEM_CONTEXT_OLD());
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -289,36 +360,37 @@ storageList(const Storage *this, const String *pathExp, StorageListParam param)
 Move a file
 ***********************************************************************************************************************************/
 void
-storageMove(const Storage *this, StorageFileRead *source, StorageFileWrite *destination)
+storageMove(const Storage *this, StorageRead *source, StorageWrite *destination)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(STORAGE_FILE_READ, source);
-        FUNCTION_LOG_PARAM(STORAGE_FILE_WRITE, destination);
+        FUNCTION_LOG_PARAM(STORAGE_READ, source);
+        FUNCTION_LOG_PARAM(STORAGE_WRITE, destination);
     FUNCTION_LOG_END();
 
+    ASSERT(this != NULL);
     ASSERT(this->interface.move != NULL);
     ASSERT(source != NULL);
     ASSERT(destination != NULL);
-    ASSERT(!storageFileReadIgnoreMissing(source));
-    ASSERT(strEq(this->type, storageFileReadType(source)));
-    ASSERT(strEq(storageFileReadType(source), storageFileWriteType(destination)));
+    ASSERT(!storageReadIgnoreMissing(source));
+    ASSERT(strEq(this->type, storageReadType(source)));
+    ASSERT(strEq(storageReadType(source), storageWriteType(destination)));
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // If the file can't be moved it will need to be copied
-        if (!this->interface.move(this->driver, storageFileReadDriver(source), storageFileWriteFileDriver(destination)))
+        if (!this->interface.move(this->driver, source, destination))
         {
             // Perform the copy
             storageCopyNP(source, destination);
 
             // Remove the source file
-            this->interface.remove(this->driver, storageFileReadName(source), false);
+            this->interface.remove(this->driver, storageReadName(source), false);
 
             // Sync source path if the destination path was synced.  We know the source and destination paths are different because
             // the move did not succeed.  This will need updating when drivers other than Posix/CIFS are implemented becaue there's
             // no way to get coverage on it now.
-            if (storageFileWriteSyncPath(destination))
-                this->interface.pathSync(this->driver, strPath(storageFileReadName(source)), false);
+            if (storageWriteSyncPath(destination))
+                this->interface.pathSync(this->driver, strPath(storageReadName(source)));
         }
     }
     MEM_CONTEXT_TEMP_END();
@@ -329,38 +401,35 @@ storageMove(const Storage *this, StorageFileRead *source, StorageFileWrite *dest
 /***********************************************************************************************************************************
 Open a file for reading
 ***********************************************************************************************************************************/
-StorageFileRead *
+StorageRead *
 storageNewRead(const Storage *this, const String *fileExp, StorageNewReadParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE, this);
         FUNCTION_LOG_PARAM(STRING, fileExp);
         FUNCTION_LOG_PARAM(BOOL, param.ignoreMissing);
-        FUNCTION_LOG_PARAM(IO_FILTER_GROUP, param.filterGroup);
+        FUNCTION_LOG_PARAM(BOOL, param.compressible);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
 
-    StorageFileRead *result = NULL;
+    StorageRead *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        result = this->interface.newRead(this->driver, storagePathNP(this, fileExp), param.ignoreMissing);
-
-        if (param.filterGroup != NULL)
-            ioReadFilterGroupSet(storageFileReadIo(result), param.filterGroup);
-
-        storageFileReadMove(result, MEM_CONTEXT_OLD());
+        result = storageReadMove(
+            this->interface.newRead(this->driver, storagePathNP(this, fileExp), param.ignoreMissing, param.compressible),
+            MEM_CONTEXT_OLD());
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(STORAGE_FILE_READ, result);
+    FUNCTION_LOG_RETURN(STORAGE_READ, result);
 }
 
 /***********************************************************************************************************************************
 Open a file for writing
 ***********************************************************************************************************************************/
-StorageFileWrite *
+StorageWrite *
 storageNewWrite(const Storage *this, const String *fileExp, StorageNewWriteParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -368,33 +437,33 @@ storageNewWrite(const Storage *this, const String *fileExp, StorageNewWriteParam
         FUNCTION_LOG_PARAM(STRING, fileExp);
         FUNCTION_LOG_PARAM(MODE, param.modeFile);
         FUNCTION_LOG_PARAM(MODE, param.modePath);
+        FUNCTION_LOG_PARAM(STRING, param.user);
+        FUNCTION_LOG_PARAM(STRING, param.group);
+        FUNCTION_LOG_PARAM(INT64, param.timeModified);
         FUNCTION_LOG_PARAM(BOOL, param.noCreatePath);
         FUNCTION_LOG_PARAM(BOOL, param.noSyncFile);
         FUNCTION_LOG_PARAM(BOOL, param.noSyncPath);
         FUNCTION_LOG_PARAM(BOOL, param.noAtomic);
-        FUNCTION_LOG_PARAM(IO_FILTER_GROUP, param.filterGroup);
+        FUNCTION_LOG_PARAM(BOOL, param.compressible);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(this->write);
 
-    StorageFileWrite *result = NULL;
+    StorageWrite *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        result = this->interface.newWrite(
-            this->driver, storagePathNP(this, fileExp), param.modeFile != 0 ? param.modeFile : this->modeFile,
-            param.modePath != 0 ? param.modePath : this->modePath, !param.noCreatePath, !param.noSyncFile, !param.noSyncPath,
-            !param.noAtomic);
-
-        if (param.filterGroup != NULL)
-            ioWriteFilterGroupSet(storageFileWriteIo(result), param.filterGroup);
-
-        storageFileWriteMove(result, MEM_CONTEXT_OLD());
+        result = storageWriteMove(
+            this->interface.newWrite(
+                this->driver, storagePathNP(this, fileExp), param.modeFile != 0 ? param.modeFile : this->modeFile,
+                param.modePath != 0 ? param.modePath : this->modePath, param.user, param.group, param.timeModified,
+                !param.noCreatePath, !param.noSyncFile, !param.noSyncPath, !param.noAtomic, param.compressible),
+            MEM_CONTEXT_OLD());
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(STORAGE_FILE_WRITE, result);
+    FUNCTION_LOG_RETURN(STORAGE_WRITE, result);
 }
 
 /***********************************************************************************************************************************
@@ -412,13 +481,8 @@ storagePath(const Storage *this, const String *pathExp)
 
     String *result = NULL;
 
-    // If the path will not be resolved here then just make a copy
-    if (!this->pathResolve)
-    {
-        result = strDup(pathExp);
-    }
     // If there is no path expression then return the base storage path
-    else if (pathExp == NULL)
+    if (pathExp == NULL)
     {
         result = strDup(this->path);
     }
@@ -428,10 +492,10 @@ storagePath(const Storage *this, const String *pathExp)
         if ((strPtr(pathExp))[0] == '/')
         {
             // Make sure the base storage path is contained within the path expression
-            if (!strEqZ(this->path, "/"))
+            if (this->path != NULL && !strEqZ(this->path, "/"))
             {
-                if (!strBeginsWith(pathExp, this->path) ||
-                    !(strSize(pathExp) == strSize(this->path) || *(strPtr(pathExp) + strSize(this->path)) == '/'))
+                if (this->pathEnforce && (!strBeginsWith(pathExp, this->path) ||
+                    !(strSize(pathExp) == strSize(this->path) || *(strPtr(pathExp) + strSize(this->path)) == '/')))
                 {
                     THROW_FMT(AssertError, "absolute path '%s' is not in base path '%s'", strPtr(pathExp), strPtr(this->path));
                 }
@@ -492,7 +556,9 @@ storagePath(const Storage *this, const String *pathExp)
                 strFree(path);
             }
 
-            if (strEqZ(this->path, "/"))
+            if (this->path == NULL)
+                result = strDup(pathExp);
+            else if (strEqZ(this->path, "/"))
                 result = strNewFmt("/%s", strPtr(pathExp));
             else
                 result = strNewFmt("%s/%s", strPtr(this->path), strPtr(pathExp));
@@ -519,12 +585,9 @@ storagePathCreate(const Storage *this, const String *pathExp, StoragePathCreateP
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
+    ASSERT(this->interface.pathCreate != NULL && storageFeature(this, storageFeaturePath));
     ASSERT(this->write);
 
-    // It doesn't make sense to combine these parameters because if we are creating missing parent paths why error when they exist?
-    // If this somehow wasn't caught in testing, the worst case is that the path would not be created and an error would be thrown.
-    ASSERT(!(param.noParentCreate && param.errorOnExists));
-
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Build the path
@@ -539,6 +602,31 @@ storagePathCreate(const Storage *this, const String *pathExp, StoragePathCreateP
     FUNCTION_LOG_RETURN_VOID();
 }
 
+/***********************************************************************************************************************************
+Does a path exist?
+***********************************************************************************************************************************/
+bool
+storagePathExists(const Storage *this, const String *pathExp)
+{
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(STORAGE, this);
+        FUNCTION_LOG_PARAM(STRING, pathExp);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(this->interface.pathExists != NULL);
+
+    bool result = false;
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        result = this->interface.pathExists(this->driver, storagePathNP(this, pathExp));
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN(BOOL, result);
+}
+
 /***********************************************************************************************************************************
 Remove a path
 ***********************************************************************************************************************************/
@@ -554,6 +642,8 @@ storagePathRemove(const Storage *this, const String *pathExp, StoragePathRemoveP
 
     ASSERT(this != NULL);
     ASSERT(this->write);
+    ASSERT(!param.errorOnMissing || storageFeature(this, storageFeaturePath));
+    ASSERT(param.recurse || storageFeature(this, storageFeaturePath));
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
@@ -561,7 +651,8 @@ storagePathRemove(const Storage *this, const String *pathExp, StoragePathRemoveP
         String *path = storagePathNP(this, pathExp);
 
         // Call driver function
-        this->interface.pathRemove(this->driver, path, param.errorOnMissing, param.recurse);
+        if (!this->interface.pathRemove(this->driver, path, param.recurse) && param.errorOnMissing)
+            THROW_FMT(PathRemoveError, STORAGE_ERROR_PATH_REMOVE_MISSING, strPtr(path));
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -571,26 +662,25 @@ storagePathRemove(const Storage *this, const String *pathExp, StoragePathRemoveP
 /***********************************************************************************************************************************
 Sync a path
 ***********************************************************************************************************************************/
-void storagePathSync(const Storage *this, const String *pathExp, StoragePathSyncParam param)
+void storagePathSync(const Storage *this, const String *pathExp)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE, this);
         FUNCTION_LOG_PARAM(STRING, pathExp);
-        FUNCTION_LOG_PARAM(BOOL, param.ignoreMissing);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(this->write);
 
-    MEM_CONTEXT_TEMP_BEGIN()
+    // Not all storage requires path sync so just do nothing if the function is not implemented
+    if (this->interface.pathSync != NULL)
     {
-        // Build the path
-        String *path = storagePathNP(this, pathExp);
-
-        // Call driver function
-        this->interface.pathSync(this->driver, path, param.ignoreMissing);
+        MEM_CONTEXT_TEMP_BEGIN()
+        {
+            this->interface.pathSync(this->driver, storagePathNP(this, pathExp));
+        }
+        MEM_CONTEXT_TEMP_END();
     }
-    MEM_CONTEXT_TEMP_END();
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -599,18 +689,18 @@ void storagePathSync(const Storage *this, const String *pathExp, StoragePathSync
 Write a buffer to storage
 ***********************************************************************************************************************************/
 void
-storagePut(StorageFileWrite *file, const Buffer *buffer)
+storagePut(StorageWrite *file, const Buffer *buffer)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(STORAGE_FILE_WRITE, file);
+        FUNCTION_LOG_PARAM(STORAGE_WRITE, file);
         FUNCTION_LOG_PARAM(BUFFER, buffer);
     FUNCTION_LOG_END();
 
     ASSERT(file != NULL);
 
-    ioWriteOpen(storageFileWriteIo(file));
-    ioWrite(storageFileWriteIo(file), buffer);
-    ioWriteClose(storageFileWriteIo(file));
+    ioWriteOpen(storageWriteIo(file));
+    ioWrite(storageWriteIo(file), buffer);
+    ioWriteClose(storageWriteIo(file));
 
     FUNCTION_LOG_RETURN_VOID();
 }
@@ -643,6 +733,83 @@ storageRemove(const Storage *this, const String *fileExp, StorageRemoveParam par
     FUNCTION_LOG_RETURN_VOID();
 }
 
+/***********************************************************************************************************************************
+Get the storage driver
+***********************************************************************************************************************************/
+void *
+storageDriver(const Storage *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STORAGE, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    FUNCTION_TEST_RETURN(this->driver);
+}
+
+/***********************************************************************************************************************************
+Is the feature supported by this storage?
+***********************************************************************************************************************************/
+bool
+storageFeature(const Storage *this, StorageFeature feature)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STORAGE, this);
+        FUNCTION_TEST_PARAM(ENUM, feature);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    FUNCTION_TEST_RETURN(this->interface.feature >> feature & 1);
+}
+
+/***********************************************************************************************************************************
+Get the storage interface
+***********************************************************************************************************************************/
+StorageInterface
+storageInterface(const Storage *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STORAGE, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    FUNCTION_TEST_RETURN(this->interface);
+}
+
+/***********************************************************************************************************************************
+Set whether absolute paths are required to be in the base path
+***********************************************************************************************************************************/
+void
+storagePathEnforceSet(Storage *this, bool enforce)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STORAGE, this);
+        FUNCTION_TEST_PARAM(BOOL, enforce);
+    FUNCTION_TEST_END();
+
+    this->pathEnforce = enforce;
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
+/***********************************************************************************************************************************
+Get the storage type (posix, cifs, etc.)
+***********************************************************************************************************************************/
+const String *
+storageType(const Storage *this)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STORAGE, this);
+    FUNCTION_TEST_END();
+
+    ASSERT(this != NULL);
+
+    FUNCTION_TEST_RETURN(this->type);
+}
+
 /***********************************************************************************************************************************
 Render as string for logging
 ***********************************************************************************************************************************/
@@ -652,19 +819,3 @@ storageToLog(const Storage *this)
     return strNewFmt(
         "{type: %s, path: %s, write: %s}", strPtr(this->type), strPtr(strToLog(this->path)), cvtBoolToConstZ(this->write));
 }
-
-/***********************************************************************************************************************************
-Free storage
-***********************************************************************************************************************************/
-void
-storageFree(const Storage *this)
-{
-    FUNCTION_LOG_BEGIN(logLevelDebug);
-        FUNCTION_LOG_PARAM(STORAGE, this);
-    FUNCTION_LOG_END();
-
-    if (this != NULL)
-        memContextFree(this->memContext);
-
-    FUNCTION_LOG_RETURN_VOID();
-}
