@@ -48,6 +48,7 @@ sub codeCountScan
                  $strFile eq 'LICENSE' ||
                  $strFile =~ '^doc/example/' ||
                  $strFile =~ '^doc/output/' ||
+                 $strFile =~ '^doc/resource/fake\-cert' ||
                  $strFile =~ '\.png$' ||
                  $strFile =~ '\.eps$' ||
                  $strFile =~ '\.cache$' ||
@@ -79,7 +80,8 @@ sub codeCountScan
         {
             $strClass = 'doc/core';
         }
-        elsif ($strFile =~ '^build/' || $strFile =~ '^libc/build/' || $strFile eq 'libc/Makefile.PL' || $strFile eq 'src/Makefile')
+        elsif ($strFile =~ '^build/' || $strFile =~ '^libc/build/' || $strFile eq 'libc/Makefile.PL' ||
+               $strFile eq 'src/Makefile.in' || $strFile eq 'src/configure' || $strFile eq 'src/configure.ac')
         {
             $strClass = 'build';
         }
@@ -109,7 +111,7 @@ sub codeCountScan
             $strType = 'xs';
             $strForceLang = 'XS';
         }
-        elsif ($strFile =~ '\.h$' || $strFile =~ '\.xsh$')
+        elsif ($strFile =~ '\.h$' || $strFile =~ '\.h\.in$' || $strFile =~ '\.xsh$')
         {
             $strType = 'c/h';
             $strForceLang = 'C/C++ Header';
@@ -129,7 +131,7 @@ sub codeCountScan
             $strType = 'yaml';
             $strForceLang = 'YAML';
         }
-        elsif ($strFile =~ 'Makefile$')
+        elsif ($strFile =~ 'Makefile\.in$' || $strFile =~ '^src\/configure')
         {
             $strType = 'make';
             $strForceLang = 'make';
