@@ -69,11 +69,6 @@ use constant CERT_FAKE_SERVER_KEY                                   => CERT_FAKE
 ####################################################################################################################################
 use constant CONTAINER_DEBUG                                        => false;
 
-####################################################################################################################################
-# Container Debug - speeds container debugging by splitting each section into a separate intermediate container
-####################################################################################################################################
-use constant CONTAINER_S3_SERVER_TAG                                => 's3-server-20180612A';
-
 ####################################################################################################################################
 # Store cache container checksums
 ####################################################################################################################################
@@ -303,51 +298,6 @@ sub certSetup
     return $strScript;
 }
 
-####################################################################################################################################
-# S3 server setup
-####################################################################################################################################
-sub s3ServerSetup
-{
-    my $strOS = shift;
-
-    # Install node.js
-    my $strScript = sectionHeader() .
-        "# Install node.js\n";
-
-    if ($strOS eq VM_CO7)
-    {
-        $strScript .=
-            "    wget -O /root/nodejs.sh https://rpm.nodesource.com/setup_6.x && \\\n" .
-            "    bash /root/nodejs.sh && \\\n" .
-            "    yum install -y nodejs";
-    }
-    else
-    {
-        $strScript .=
-            "    wget -O /root/nodejs.sh https://deb.nodesource.com/setup_6.x && \\\n" .
-            "    bash /root/nodejs.sh && \\\n" .
-            "    wget -qO- https://deb.nodesource.com/setup_8.x | bash - && \\\n" .
-            "    apt-get install -y nodejs";
-    }
-
-    # Install Scality S3
-    $strScript .= sectionHeader() .
-        "# Install Scality S3\n";
-
-    $strScript .=
-        "    wget -O /root/scalitys3.tar.gz https://github.com/scality/S3/archive/GA6.4.2.1.tar.gz && \\\n" .
-        "    mkdir /root/scalitys3 && \\\n" .
-        "    tar -C /root/scalitys3 --strip-components 1 -xvf /root/scalitys3.tar.gz && \\\n" .
-        "    cd /root/scalitys3 && \\\n" .
-        "    npm install && \\\n" .
-        '    sed -i "0,/,/s//,\n    \"certFilePaths\":{\"key\":\"\/etc\/fake\-cert\/server.key\",\"cert\":' .
-            '\"\/etc\/fake\-cert\/server.crt\",\"ca\":\"\/etc\/fake\-cert\/ca.crt\"},/"' . " \\\n" .
-        '        ./config.json' . " && \\\n" .
-        '    sed -i "s/ort\"\: 8000/ort\"\: 443/" ./config.json';
-
-    return $strScript;
-}
-
 ####################################################################################################################################
 # Entry point setup
 ####################################################################################################################################
@@ -426,17 +376,22 @@ sub containerBuild
 
         #---------------------------------------------------------------------------------------------------------------------------
         my $strScript = sectionHeader() .
-            "# Install base packages\n";
+            "# Install packages\n";
 
         if ($$oVm{$strOS}{&VM_OS_BASE} eq VM_OS_BASE_RHEL)
         {
+            if ($strOS eq VM_CO6 || $strOS eq VM_CO7)
+            {
+                $strScript .=
+                    "    yum -y install epel-release && \\\n";
+            }
+
             $strScript .=
-                "    yum -y install epel-release && \\\n" .
                 "    yum -y update && \\\n" .
-                "    yum -y install openssh-server openssh-clients wget sudo python-pip build-essential valgrind git \\\n" .
-                "        perl perl-Digest-SHA perl-DBD-Pg perl-XML-LibXML perl-IO-Socket-SSL perl-YAML-LibYAML \\\n" .
+                "    yum -y install openssh-server openssh-clients wget sudo valgrind git \\\n" .
+                "        perl perl-Digest-SHA perl-DBD-Pg perl-YAML-LibYAML openssl \\\n" .
                 "        gcc make perl-ExtUtils-MakeMaker perl-Test-Simple openssl-devel perl-ExtUtils-Embed rpm-build \\\n" .
-                "        zlib-devel libxml2-devel";
+                "        zlib-devel libxml2-devel lz4-devel lcov";
 
             if ($strOS eq VM_CO6)
             {
@@ -446,38 +401,24 @@ sub containerBuild
             {
                 $strScript .= ' perl-JSON-PP';
             }
-
-            if (vmCoverageC($strOS))
-            {
-                $strScript .= ' lcov';
-            }
         }
         else
         {
             $strScript .=
                 "    export DEBCONF_NONINTERACTIVE_SEEN=true DEBIAN_FRONTEND=noninteractive && \\\n" .
                 "    apt-get update && \\\n" .
-                "    apt-get -y install wget python && \\\n" .
-                "    wget --no-check-certificate -O /root/get-pip.py https://bootstrap.pypa.io/get-pip.py && \\\n" .
-                "    python /root/get-pip.py && \\\n" .
-                "    apt-get -y install openssh-server wget sudo python-pip build-essential valgrind git \\\n" .
-                "        libdbd-pg-perl libhtml-parser-perl libio-socket-ssl-perl libxml-libxml-perl libssl-dev libperl-dev \\\n" .
+                "    apt-get -y install openssh-server wget sudo gcc make valgrind git \\\n" .
+                "        libdbd-pg-perl libhtml-parser-perl libssl-dev libperl-dev \\\n" .
                 "        libyaml-libyaml-perl tzdata devscripts lintian libxml-checker-perl txt2man debhelper \\\n" .
-                "        libppi-html-perl libtemplate-perl libtest-differences-perl zlib1g-dev libxml2-dev";
+                "        libppi-html-perl libtemplate-perl libtest-differences-perl zlib1g-dev libxml2-dev lcov";
 
             if ($strOS eq VM_U12)
             {
                 $strScript .= ' libperl5.14';
             }
-
-            if (vmLintC($strOS))
+            else
             {
-                $strScript .= ' clang-6.0 clang-tools-6.0';
-            }
-
-            if (vmCoverageC($strOS))
-            {
-                $strScript .= ' lcov';
+                $strScript .= ' libjson-pp-perl liblz4-dev';
             }
         }
 
@@ -532,30 +473,28 @@ sub containerBuild
 
             if ($$oVm{$strOS}{&VM_OS_BASE} eq VM_OS_BASE_RHEL)
             {
+                $strScript .=
+                    "    rpm --import http://yum.postgresql.org/RPM-GPG-KEY-PGDG && \\\n";
+
                 if ($strOS eq VM_CO6)
                 {
                     $strScript .=
                         "    rpm -ivh \\\n" .
-                        "        http://yum.postgresql.org/9.0/redhat/rhel-6-x86_64/pgdg-centos90-9.0-5.noarch.rpm \\\n" .
                         "        http://yum.postgresql.org/9.1/redhat/rhel-6-x86_64/pgdg-centos91-9.1-6.noarch.rpm \\\n" .
                         "        http://yum.postgresql.org/9.2/redhat/rhel-6-x86_64/pgdg-centos92-9.2-8.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/9.3/redhat/rhel-6-x86_64/pgdg-centos93-9.3-3.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/9.4/redhat/rhel-6-x86_64/pgdg-centos94-9.4-3.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/9.5/redhat/rhel-6-x86_64/pgdg-centos95-9.5-3.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/9.6/redhat/rhel-6-x86_64/pgdg-centos96-9.6-3.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/10/redhat/rhel-6-x86_64/pgdg-centos10-10-2.noarch.rpm";
+                        "        https://download.postgresql.org/pub/repos/yum/11/redhat/rhel-6-x86_64/" .
+                            "pgdg-redhat-repo-latest.noarch.rpm && \\\n";
                 }
                 elsif ($strOS eq VM_CO7)
                 {
                     $strScript .=
                         "    rpm -ivh \\\n" .
                         "        http://yum.postgresql.org/9.2/redhat/rhel-7-x86_64/pgdg-centos92-9.2-3.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/9.3/redhat/rhel-7-x86_64/pgdg-centos93-9.3-3.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/9.4/redhat/rhel-7-x86_64/pgdg-centos94-9.4-3.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/9.5/redhat/rhel-7-x86_64/pgdg-centos95-9.5-3.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/9.6/redhat/rhel-7-x86_64/pgdg-centos96-9.6-3.noarch.rpm \\\n" .
-                        "        http://yum.postgresql.org/10/redhat/rhel-7-x86_64/pgdg-centos10-10-2.noarch.rpm";
+                        "        https://download.postgresql.org/pub/repos/yum/11/redhat/rhel-7-x86_64/" .
+                            "pgdg-redhat-repo-latest.noarch.rpm && \\\n";
                 }
+
+                $strScript .= "    yum -y install postgresql-devel";
             }
             else
             {
@@ -565,7 +504,7 @@ sub containerBuild
                         "' >> /etc/apt/sources.list.d/pgdg.list && \\\n" .
                     "    wget --quiet -O - https://www.postgresql.org/media/keys/ACCC4CF8.asc | apt-key add - && \\\n" .
                     "    apt-get update && \\\n" .
-                    "    apt-get install -y postgresql-common && \\\n" .
+                    "    apt-get install -y postgresql-common libpq-dev && \\\n" .
                     "    sed -i 's/^\\#create\\_main\\_cluster.*\$/create\\_main\\_cluster \\= false/' " .
                         "/etc/postgresql-common/createcluster.conf";
             }
@@ -602,19 +541,14 @@ sub containerBuild
             }
         }
 
-        #---------------------------------------------------------------------------------------------------------------------------
-        if (!$bDeprecated)
-        {
-            $strScript .= sectionHeader() .
-                "# Install AWS CLI\n" .
-                "    pip install --upgrade --no-cache-dir pip==9.0.3 && \\\n" .
-                "    pip install --upgrade awscli";
-        }
 
         #---------------------------------------------------------------------------------------------------------------------------
-        if (!$bDeprecated && $strOS ne VM_CO6 && $strOS ne VM_U12)
+        if ($$oVm{$strOS}{&VM_OS_BASE} eq VM_OS_BASE_DEBIAN)
         {
-            $strScript .= s3ServerSetup($strOS);
+        $strScript .=  sectionHeader() .
+            "# Cleanup\n";
+
+            $strScript .= "    apt-get clean";
         }
 
         containerWrite(
@@ -627,7 +561,6 @@ sub containerBuild
         $strCopy = undef;
 
         my $strPkgDevelCover = packageDevelCover($oVm->{$strOS}{&VM_ARCH});
-        my $bPkgDevelCoverBuild = vmCoveragePerl($strOS) && !$oStorageDocker->exists("test/package/${strOS}-${strPkgDevelCover}");
 
         $strScript = sectionHeader() .
             "# Create test user\n" .
@@ -640,18 +573,6 @@ sub containerBuild
             $strScript .=  sectionHeader() .
                 "# Install pgBackRest package source\n" .
                 "    git clone https://salsa.debian.org/postgresql/pgbackrest.git /root/package-src";
-
-            # Build only when a new version has been specified
-            if ($bPkgDevelCoverBuild)
-            {
-                $strScript .=  sectionHeader() .
-                    "# Install Devel::Cover package source & build\n" .
-                    "    git clone https://salsa.debian.org/perl-team/modules/packages/libdevel-cover-perl.git" .
-                        " /root/libdevel-cover-perl && \\\n" .
-                    "    cd /root/libdevel-cover-perl && \\\n" .
-                    "    git checkout debian/" . LIB_COVER_VERSION . " && \\\n" .
-                    "    debuild -i -us -uc -b";
-            }
         }
         else
         {
@@ -670,45 +591,6 @@ sub containerBuild
 
         containerWrite($oStorageDocker, $strTempPath, $strOS, 'Build', $strImageParent, $strImage, $strCopy, $strScript, $bVmForce);
 
-        # Copy Devel::Cover to host so it can be installed in other containers (if it doesn't already exist)
-        if ($bPkgDevelCoverBuild)
-        {
-            executeTest('docker rm -f test-build', {bSuppressError => true});
-            executeTest(
-                "docker run -itd -h test-build --name=test-build" .
-                " -v ${strTempPath}:${strTempPath} " . containerRepo() . ":${strOS}-build",
-                {bSuppressStdErr => true});
-            executeTest(
-                "docker exec -i test-build " .
-                "bash -c 'cp /root/${strPkgDevelCover} ${strTempPath}/${strOS}-${strPkgDevelCover}'");
-            executeTest('docker rm -f test-build');
-
-            $oStorageDocker->move(
-                "test/.vagrant/docker/${strOS}-${strPkgDevelCover}", "test/package/${strOS}-${strPkgDevelCover}");
-        }
-
-        # S3 image
-        ###########################################################################################################################
-        if (!$bDeprecated)
-        {
-            $strImage = "${strOS}-s3-server";
-            $strScript = '';
-            $strCopy = undef;
-
-            if ($strOS ne VM_CO6 && $strOS ne VM_U12)
-            {
-                $strImageParent = containerRepo() . ":${strOS}-base";
-                $strScript = "\n\nENTRYPOINT npm start --prefix /root/scalitys3";
-            }
-            else
-            {
-                $strImageParent = containerRepo() . ':' . CONTAINER_S3_SERVER_TAG;
-            }
-
-            containerWrite(
-                $oStorageDocker, $strTempPath, $strOS, 'S3 Server', $strImageParent, $strImage, $strCopy, $strScript, $bVmForce);
-        }
-
         # Test image
         ########################################################################################################################
         if (!$bDeprecated)
@@ -716,28 +598,8 @@ sub containerBuild
             $strImageParent = containerRepo() . ":${strOS}-base";
             $strImage = "${strOS}-test";
 
-            if (vmCoveragePerl($strOS))
-            {
-                $oStorageDocker->copy(
-                    "test/package/${strOS}-${strPkgDevelCover}", "test/.vagrant/docker/${strOS}-${strPkgDevelCover}");
-
-                $strCopy =
-                    "# Copy Devel::Cover\n" .
-                    "COPY ${strOS}-${strPkgDevelCover} /tmp/${strPkgDevelCover}";
-
-                $strScript = sectionHeader() .
-                    "# Install packages\n" .
-                    "    apt-get install -y libjson-maybexs-perl";
-
-                $strScript .= sectionHeader() .
-                    "# Install Devel::Cover\n" .
-                    "    dpkg -i /tmp/${strPkgDevelCover}";
-            }
-            else
-            {
-                $strCopy = undef;
-                $strScript = '';
-            }
+            $strCopy = undef;
+            $strScript = '';
 
             #---------------------------------------------------------------------------------------------------------------------------
             $strScript .= sectionHeader() .
@@ -774,15 +636,6 @@ sub containerBuild
             $strScript .=
                 sshSetup($strOS, TEST_USER, TEST_GROUP, $$oVm{$strOS}{&VM_CONTROL_MASTER});
 
-            if (!$bDeprecated)
-            {
-                $strScript .= sectionHeader() .
-                    "# Config AWS CLI\n" .
-                    '    sudo -i -u ' . TEST_USER . " aws configure set region us-east-1 && \\\n" .
-                    '    sudo -i -u ' . TEST_USER . " aws configure set aws_access_key_id accessKey1 && \\\n" .
-                    '    sudo -i -u ' . TEST_USER . " aws configure set aws_secret_access_key verySecretKey1";
-            }
-
             $strScript .= sectionHeader() .
                 "# Create pgbackrest user\n" .
                 '    ' . userCreate($strOS, BACKREST_USER, BACKREST_USER_ID, TEST_GROUP);
