@@ -26,6 +26,7 @@ use pgBackRest::Version;
 
 use pgBackRestTest::Common::BuildTest;
 use pgBackRestTest::Common::ContainerTest;
+use pgBackRestTest::Common::CoverageTest;
 use pgBackRestTest::Common::DefineTest;
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::ListTest;
@@ -70,10 +71,13 @@ sub new
         $self->{iRetry},
         $self->{bValgrindUnit},
         $self->{bCoverageUnit},
+        $self->{bCoverageSummary},
         $self->{bOptimize},
         $self->{bBackTrace},
         $self->{bProfile},
         $self->{bDebug},
+        $self->{bDebugTestTrace},
+        $self->{iBuildMax},
     ) =
         logDebugParam
         (
@@ -97,10 +101,13 @@ sub new
             {name => 'iRetry'},
             {name => 'bValgrindUnit'},
             {name => 'bCoverageUnit'},
+            {name => 'bCoverageSummary'},
             {name => 'bOptimize'},
             {name => 'bBackTrace'},
             {name => 'bProfile'},
             {name => 'bDebug'},
+            {name => 'bDebugTestTrace'},
+            {name => 'iBuildMax'},
         );
 
     # Set try to 0
@@ -207,7 +214,9 @@ sub run
                     {
                         executeTest(
                             'rsync -rt --delete --exclude=*.o --exclude=test.c --exclude=test.gcno --exclude=LibC.h --exclude=xs' .
-                                " --exclude=test --exclude=buildflags --exclude=testflags --exclude=harnessflags" .
+                                ' --exclude=test --exclude=buildflags --exclude=testflags --exclude=harnessflags' .
+                                ' --exclude=build.auto.h --exclude=build.auto.h.in --exclude=Makefile --exclude=Makefile.in' .
+                                ' --exclude=configure --exclude=configure.ac' .
                                 " $self->{strBackRestBase}/src/ $self->{strGCovPath} && " .
                             "rsync -t $self->{strBackRestBase}/libc/LibC.h $self->{strGCovPath} && " .
                             "rsync -rt --delete $self->{strBackRestBase}/libc/xs/ $self->{strGCovPath}/xs && " .
@@ -244,7 +253,7 @@ sub run
             $strCommand =
                 'docker exec -i -u ' . TEST_USER . " ${strImage} bash -l -c '" .
                 "cd $self->{strGCovPath} && " .
-                "make -s 2>&1 &&" .
+                "make -j $self->{iBuildMax} -s 2>&1 &&" .
                 ($self->{oTest}->{&TEST_VM} ne VM_CO6 && $self->{bValgrindUnit}?
                     " valgrind -q --gen-suppressions=all --suppressions=$self->{strGCovPath}/test/valgrind.suppress" .
                     " --leak-check=full --leak-resolution=high --error-exitcode=25" : '') .
@@ -254,9 +263,7 @@ sub run
         {
             $strCommand =
                 ($self->{oTest}->{&TEST_CONTAINER} ? 'docker exec -i -u ' . TEST_USER . " ${strImage} " : '') .
-                testRunExe(
-                    vmCoverageC($self->{oTest}->{&TEST_VM}), undef, abs_path($0), dirname($self->{strCoveragePath}),
-                    $self->{strBackRestBase}, $self->{oTest}->{&TEST_MODULE}, $self->{oTest}->{&TEST_NAME}) .
+                abs_path($0) .
                 " --test-path=${strVmTestPath}" .
                 " --vm=$self->{oTest}->{&TEST_VM}" .
                 " --vm-id=$self->{iVmIdx}" .
@@ -284,6 +291,8 @@ sub run
 
                 foreach my $strFile (sort(keys(%{$self->{oStorageTest}->manifest($self->{strGCovPath})})))
                 {
+                    my $strFileNoExt = substr($strFile, 0, length($strFile) - 2);
+
                     # Skip all files except .c files (including .auto.c)
                     next if $strFile !~ /(?<!\.auto)\.c$/;
 
@@ -293,7 +302,7 @@ sub run
                     # Skip test.c -- it will be added manually at the end
                     next if $strFile =~ /test\.c$/;
 
-                    if (!defined($hTestCoverage->{substr($strFile, 0, length($strFile) - 2)}) &&
+                    if (!defined($hTestCoverage->{$strFileNoExt}) && !grep(/^$strFileNoExt$/, @{$hTest->{&TESTDEF_INCLUDE}}) &&
                         $strFile !~ /^test\/module\/[^\/]*\/.*Test\.c$/)
                     {
                         push(@stryCFile, "${strFile}");
@@ -306,7 +315,7 @@ sub run
                     "test/module/$self->{oTest}->{&TEST_MODULE}/" . testRunName($self->{oTest}->{&TEST_NAME}, false) . 'Test.c';
                 my $strCInclude;
 
-                foreach my $strFile (sort(keys(%{$hTestCoverage})))
+                foreach my $strFile (sort(keys(%{$hTestCoverage}), @{$hTest->{&TESTDEF_INCLUDE}}))
                 {
                     # Don't include the test file as it is already included below
                     next if $strFile =~ /Test$/;
@@ -342,7 +351,7 @@ sub run
                 # Build dependencies for the test file
                 my $rhDependencyTree = {};
                 buildDependencyTreeSub(
-                    $self->{oStorageTest}, $rhDependencyTree, $strTestFile, $self->{strGCovPath}, ['', 'test']);
+                    $self->{oStorageTest}, $rhDependencyTree, $strTestFile, false, $self->{strGCovPath}, ['', 'test']);
 
                 foreach my $strDepend (@{$rhDependencyTree->{$strTestFile}{include}})
                 {
@@ -381,26 +390,15 @@ sub run
                 $strTestC =~ s/\{\[C\_TEST\_LIST\]\}/$strTestInit/g;
                 buildPutDiffers($self->{oStorageTest}, "$self->{strGCovPath}/test.c", $strTestC);
 
-                # Flags that are common to all builds
-                my $strCommonFlags =
-                    '-I. -Itest -std=c99 -fPIC -g -Wno-clobbered -D_POSIX_C_SOURCE=200112L' .
-                        ' `perl -MExtUtils::Embed -e ccopts` -DWITH_PERL' .
-                        ' `xml2-config --cflags`' . ($self->{bProfile} ? " -pg" : '') .
-                    ($self->{oTest}->{&TEST_DEBUG_UNIT_SUPPRESS} ? '' : " -DDEBUG_UNIT") .
-                    (vmWithBackTrace($self->{oTest}->{&TEST_VM}) && $self->{bBackTrace} ? ' -DWITH_BACKTRACE' : '') .
-                    ($self->{oTest}->{&TEST_CDEF} ? " $self->{oTest}->{&TEST_CDEF}" : '') .
-                    ($self->{bDebug} ? '' : " -DNDEBUG");
+                # Create build.auto.h
+                my $strBuildAutoH =
+                    "#define HAVE_LIBPERL\n";
 
-                # Flags used to buid harness files
-                my $strHarnessFlags =
-                    '-O0' . ($self->{oTest}->{&TEST_VM} ne VM_U12 ? ' -ftree-coalesce-vars' : '') .
-                    ($self->{oTest}->{&TEST_CTESTDEF} ? " $self->{oTest}->{&TEST_CTESTDEF}" : '');
+                buildPutDiffers($self->{oStorageTest}, "$self->{strGCovPath}/" . BUILD_AUTO_H, $strBuildAutoH);
 
-                buildPutDiffers($self->{oStorageTest}, "$self->{strGCovPath}/harnessflags", "${strCommonFlags} ${strHarnessFlags}");
-
-                # Flags used to buid test.c
-                my $strTestFlags =
-                    '-Werror -Wfatal-errors -Wall -Wextra -Wwrite-strings -Wswitch-enum -Wconversion -Wformat=2' .
+                # Determine which warnings are available
+                my $strWarningFlags =
+                    '-Werror -Wfatal-errors -Wall -Wextra -Wwrite-strings -Wconversion -Wformat=2' .
                     ' -Wformat-nonliteral -Wstrict-prototypes -Wpointer-arith -Wvla' .
                     ($self->{oTest}->{&TEST_VM} eq VM_U16 || $self->{oTest}->{&TEST_VM} eq VM_U18 ?
                         ' -Wformat-signedness' : '') .
@@ -408,25 +406,53 @@ sub run
                         ' -Wduplicated-branches -Wduplicated-cond' : '') .
                     # This warning appears to be broken on U12/CO6 even though the functionality is fine
                     ($self->{oTest}->{&TEST_VM} eq VM_U12 || $self->{oTest}->{&TEST_VM} eq VM_CO6 ?
-                        ' -Wno-missing-field-initializers' : '') .
-                    ' -O0' . ($self->{oTest}->{&TEST_VM} ne VM_U12 ? ' -ftree-coalesce-vars' : '') .
+                        ' -Wno-missing-field-initializers' : '');
+
+                # Flags that are common to all builds
+                my $strCommonFlags =
+                    '-I. -Itest -std=c99 -fPIC -g -Wno-clobbered -D_POSIX_C_SOURCE=200112L' .
+                        ' `perl -MExtUtils::Embed -e ccopts`' .
+                        ' `xml2-config --cflags`' . ($self->{bProfile} ? " -pg" : '') .
+                        ' -I`pg_config --includedir`' .
+                    ($self->{oTest}->{&TEST_DEBUG_UNIT_SUPPRESS} ? '' : " -DDEBUG_UNIT") .
+                    (vmWithBackTrace($self->{oTest}->{&TEST_VM}) && $self->{bBackTrace} ? ' -DWITH_BACKTRACE' : '') .
+                    ($self->{oTest}->{&TEST_CDEF} ? " $self->{oTest}->{&TEST_CDEF}" : '') .
+                    (vmCoverageC($self->{oTest}->{&TEST_VM}) && $self->{bCoverageUnit} ? ' -DDEBUG_COVERAGE' : '') .
+                    ($self->{bDebug} ? '' : ' -DNDEBUG') . ($self->{bDebugTestTrace} ? ' -DDEBUG_TEST_TRACE' : '');
+
+                # Flags used to buid harness files
+                my $strHarnessFlags =
+                    '-O2' . ($self->{oTest}->{&TEST_VM} ne VM_U12 ? ' -ftree-coalesce-vars' : '') .
+                    ($self->{oTest}->{&TEST_CTESTDEF} ? " $self->{oTest}->{&TEST_CTESTDEF}" : '');
+
+                buildPutDiffers(
+                    $self->{oStorageTest}, "$self->{strGCovPath}/harnessflags",
+                    "${strCommonFlags} ${strWarningFlags} ${strHarnessFlags}");
+
+                # Flags used to buid test.c
+                my $strTestFlags =
+                    '-DDEBUG_TEST_TRACE -O0' . ($self->{oTest}->{&TEST_VM} ne VM_U12 ? ' -ftree-coalesce-vars' : '') .
                     (vmCoverageC($self->{oTest}->{&TEST_VM}) && $self->{bCoverageUnit} ?
                         ' -fprofile-arcs -ftest-coverage' : '') .
                     ($self->{oTest}->{&TEST_CTESTDEF} ? " $self->{oTest}->{&TEST_CTESTDEF}" : '');
 
                 buildPutDiffers(
-                    $self->{oStorageTest}, "$self->{strGCovPath}/testflags", "${strCommonFlags} ${strTestFlags}");
+                    $self->{oStorageTest}, "$self->{strGCovPath}/testflags",
+                    "${strCommonFlags} ${strWarningFlags} ${strTestFlags}");
 
                 # Flags used to buid all other files
                 my $strBuildFlags =
                     ($self->{bOptimize} ? '-O2' : '-O0' . ($self->{oTest}->{&TEST_VM} ne VM_U12 ? ' -ftree-coalesce-vars' : ''));
 
-                buildPutDiffers($self->{oStorageTest}, "$self->{strGCovPath}/buildflags", "${strCommonFlags} ${strBuildFlags}");
+                buildPutDiffers(
+                    $self->{oStorageTest}, "$self->{strGCovPath}/buildflags",
+                    "${strCommonFlags} ${strWarningFlags} ${strBuildFlags}");
 
                 # Build the Makefile
                 my $strMakefile =
                     "CC=gcc\n" .
                     "COMMONFLAGS=${strCommonFlags}\n" .
+                    "WARNINGFLAGS=${strWarningFlags}\n" .
                     "BUILDFLAGS=${strBuildFlags}\n" .
                     "HARNESSFLAGS=${strHarnessFlags}\n" .
                     "TESTFLAGS=${strTestFlags}\n" .
@@ -442,33 +468,36 @@ sub run
                     "\t\$(CC) -o test.bin \$(OBJS) test.o"  . ($self->{bProfile} ? " -pg" : '') . " \$(LDFLAGS)\n" .
                     "\n" .
                     "test.o: testflags test.c${strTestDepend}\n" .
-	                "\t\$(CC) \$(COMMONFLAGS) \$(TESTFLAGS) -c test.c\n";
+	                "\t\$(CC) \$(COMMONFLAGS) \$(WARNINGFLAGS) \$(TESTFLAGS) -c test.c\n";
 
                 # Build C file dependencies
                 foreach my $strCFile (@stryCFile)
                 {
+                    my $bHarnessFile = $strCFile =~ /^test\// ? true : false;
+
                     buildDependencyTreeSub(
-                        $self->{oStorageTest}, $rhDependencyTree, $strCFile, $self->{strGCovPath}, ['', 'test']);
+                        $self->{oStorageTest}, $rhDependencyTree, $strCFile, !$bHarnessFile, $self->{strGCovPath}, ['', 'test']);
 
                     $strMakefile .=
                         "\n" . substr($strCFile, 0, length($strCFile) - 2) . ".o:" .
-                        ($strCFile =~ /^test\// ? " harnessflags" : " buildflags") . " $strCFile";
+                        ($bHarnessFile ? " harnessflags" : " buildflags") . " $strCFile";
 
                     foreach my $strDepend (@{$rhDependencyTree->{$strCFile}{include}})
                     {
                         $strMakefile .=
-                            ' ' . ($rhDependencyTree->{$strDepend}{path} ne '' ? $rhDependencyTree->{$strDepend}{path} . '/' : '') .
+                            ' ' .
+                            ($rhDependencyTree->{$strDepend}{path} ne '' ? $rhDependencyTree->{$strDepend}{path} . '/' : '') .
                             $strDepend;
                     }
 
                     $strMakefile .=
                         "\n" .
-                        "\t\$(CC) \$(COMMONFLAGS)" .
+                        "\t\$(CC) \$(COMMONFLAGS) \$(WARNINGFLAGS)" .
                             ($strCFile =~ /^test\// ? " \$(HARNESSFLAGS)" : " \$(BUILDFLAGS)") .
                             " -c $strCFile -o " . substr($strCFile, 0, length($strCFile) - 2) . ".o\n";
                 }
 
-                $self->{oStorageTest}->put($self->{strGCovPath} . "/Makefile", $strMakefile);
+                buildPutDiffers($self->{oStorageTest}, $self->{strGCovPath} . "/Makefile", $strMakefile);
             }
 
             my $oExec = new pgBackRestTest::Common::ExecuteTest(
@@ -555,13 +584,16 @@ sub end
                 "module/$self->{oTest}->{&TEST_MODULE}/" . testRunName($self->{oTest}->{&TEST_NAME}, false) . 'Test');
 
             # Generate coverage reports for the modules
-            my $strLCovExe = "lcov --config-file=$self->{strGCovPath}/test/lcov.conf";
+            my $strLCovConf = $self->{strBackRestBase} . '/test/.vagrant/code/lcov.conf';
+            coverageLCovConfigGenerate($self->{oStorageTest}, $strLCovConf, $self->{bCoverageSummary});
+
+            my $strLCovExeBase = "lcov --config-file=${strLCovConf}";
             my $strLCovOut = $self->{strGCovPath} . '/test.lcov';
             my $strLCovOutTmp = $self->{strGCovPath} . '/test.tmp.lcov';
 
             executeTest(
                 'docker exec -i -u ' . TEST_USER . " ${strImage} " .
-                "${strLCovExe} --capture --directory=$self->{strGCovPath} --o=${strLCovOut}");
+                "${strLCovExeBase} --capture --directory=$self->{strGCovPath} --o=${strLCovOut}");
 
             # Generate coverage report for each module
             foreach my $strModule (@stryCoveredModule)
@@ -576,6 +608,14 @@ sub end
                     $bTest = true;
                 }
 
+                # Disable branch coverage for test files
+                my $strLCovExe = $strLCovExeBase;
+
+                if ($bTest)
+                {
+                    $strLCovExe .= ' --rc lcov_branch_coverage=0';
+                }
+
                 # Generate lcov reports
                 my $strModulePath = $self->{strBackRestBase} . "/test/.vagrant/code/${strModuleOutName}";
                 my $strLCovFile = "${strModulePath}.lcov";
