@@ -16,10 +16,9 @@ use File::Basename qw(basename dirname);
 use Fcntl qw(:mode);
 use File::stat qw{lstat};
 
-use pgBackRestDoc::Common::Exception;
-use pgBackRestDoc::Common::Log;
-
-use pgBackRestTest::Common::StorageBase;
+use pgBackRest::Common::Exception;
+use pgBackRest::Common::Log;
+use pgBackRest::Storage::Base;
 use pgBackRestTest::Common::StoragePosixRead;
 use pgBackRestTest::Common::StoragePosixWrite;
 
@@ -287,7 +286,7 @@ sub list
     # Attempt to open the path
     if (opendir($hPath, $strPath))
     {
-        @stryFileList = grep(!/^(\.|\.\.)$/m, readdir($hPath));
+        @stryFileList = grep(!/^(\.)|(\.\.)$/i, readdir($hPath));
         close($hPath);
     }
     # Else process errors
@@ -697,7 +696,7 @@ sub owner
         my $iUserId;
         my $iGroupId;
 
-        # If the user or group is not defined then get it by stat'ing the file. This is because the chown function requires that
+        # If the user or group is not defined then get it by stat'ing the file.  This is because the chown function requires that
         # both user and group be set.
         my $oStat = $self->info($strFilePath);
 
@@ -979,6 +978,5 @@ sub remove
 sub className {STORAGE_POSIX_DRIVER}
 sub tempExtension {shift->{strTempExtension}}
 sub tempExtensionSet {my $self = shift; $self->{strTempExtension} = shift}
-sub type {STORAGE_POSIX}
 
 1;
