@@ -31,14 +31,10 @@ use constant VM_CONTROL_MASTER                                      => 'control-
     push @EXPORT, qw(VM_CONTROL_MASTER);
 # Will coverage testing be run for C?
 use constant VMDEF_COVERAGE_C                                       => 'coverage-c';
-# Will coverage testing be run for Perl?
-use constant VMDEF_COVERAGE_PERL                                    => 'coverage-perl';
 use constant VM_DEPRECATED                                          => 'deprecated';
     push @EXPORT, qw(VM_DEPRECATED);
 use constant VM_IMAGE                                               => 'image';
     push @EXPORT, qw(VM_IMAGE);
-# Will static code analysis be run for C?
-use constant VMDEF_LINT_C                                           => 'lint-c';
 use constant VM_OS                                                  => 'os';
     push @EXPORT, qw(VM_OS);
 use constant VM_OS_BASE                                             => 'os-base';
@@ -109,8 +105,18 @@ use constant VM_EXPECT                                              => VM_CO7;
 use constant VM_HOST_DEFAULT                                        => VM_U18;
     push @EXPORT, qw(VM_HOST_DEFAULT);
 
-# Lists valid VMs
-use constant VM_LIST                                                => (VM_U18, VM_CO6, VM_CO7, VM_U12);
+# VM aliases for run matrices (numbered oldest to newest)
+use constant VM1                                                    => VM_CO6;
+    push @EXPORT, qw(VM1);
+use constant VM2                                                    => VM_U12;
+    push @EXPORT, qw(VM2);
+use constant VM3                                                    => VM_CO7;
+    push @EXPORT, qw(VM3);
+use constant VM4                                                    => VM_U18;
+    push @EXPORT, qw(VM4);
+
+# List of default test VMs (in this order: newest, oldest, next newest, next oldest)
+use constant VM_LIST                                                => (VM4, VM1, VM3, VM2);
     push @EXPORT, qw(VM_LIST);
 
 my $oyVm =
@@ -127,10 +133,8 @@ my $oyVm =
 
         &VM_DB =>
         [
-            PG_VERSION_90,
             PG_VERSION_91,
             PG_VERSION_92,
-            PG_VERSION_93,
             PG_VERSION_94,
             PG_VERSION_95,
             PG_VERSION_96,
@@ -139,8 +143,8 @@ my $oyVm =
 
         &VM_DB_TEST =>
         [
-            PG_VERSION_90,
             PG_VERSION_91,
+            PG_VERSION_92,
             PG_VERSION_94,
             PG_VERSION_95,
         ],
@@ -161,7 +165,6 @@ my $oyVm =
         &VM_DB =>
         [
             PG_VERSION_92,
-            PG_VERSION_93,
             PG_VERSION_94,
             PG_VERSION_95,
             PG_VERSION_96,
@@ -252,16 +255,13 @@ my $oyVm =
             PG_VERSION_91,
             PG_VERSION_92,
             PG_VERSION_93,
-            PG_VERSION_94,
-            PG_VERSION_95,
-            PG_VERSION_96,
         ],
 
         &VM_DB_TEST =>
         [
             PG_VERSION_83,
             PG_VERSION_84,
-            PG_VERSION_92,
+            PG_VERSION_90,
             PG_VERSION_93,
         ],
     },
@@ -338,8 +338,6 @@ my $oyVm =
         &VM_IMAGE => 'ubuntu:18.04',
         &VM_ARCH => VM_ARCH_AMD64,
         &VMDEF_COVERAGE_C => true,
-        &VMDEF_COVERAGE_PERL => true,
-        &VMDEF_LINT_C => true,
         &VMDEF_PGSQL_BIN => '/usr/lib/postgresql/{[version]}/bin',
         &VMDEF_PERL_ARCH_PATH => '/usr/local/lib/x86_64-linux-gnu/perl/5.26.1',
 
@@ -380,7 +378,6 @@ foreach my $strVm (sort(keys(%{$oyVm})))
 foreach my $strPgVersion (versionSupport())
 {
     my $strVmPgVersionRun;
-    my $bVmCoveragePerl = false;
     my $bVmCoverageC = false;
 
     foreach my $strVm (VM_LIST)
@@ -390,11 +387,6 @@ foreach my $strPgVersion (versionSupport())
             $bVmCoverageC = true;
         }
 
-        if (vmCoveragePerl($strVm))
-        {
-            $bVmCoveragePerl = true;
-        }
-
         foreach my $strVmPgVersion (@{$oyVm->{$strVm}{&VM_DB_TEST}})
         {
             if ($strPgVersion eq $strVmPgVersion)
@@ -416,11 +408,6 @@ foreach my $strPgVersion (versionSupport())
         confess &log(ASSERT, "C coverage ${strErrorSuffix}");
     }
 
-    if (!$bVmCoveragePerl)
-    {
-        confess &log(ASSERT, "Perl coverage ${strErrorSuffix}");
-    }
-
     if (!defined($strVmPgVersionRun))
     {
         confess &log(ASSERT, "PostgreSQL ${strPgVersion} ${strErrorSuffix}");
@@ -462,30 +449,6 @@ sub vmCoverageC
 
 push @EXPORT, qw(vmCoverageC);
 
-####################################################################################################################################
-# vmCoveragePerl
-####################################################################################################################################
-sub vmCoveragePerl
-{
-    my $strVm = shift;
-
-    return $oyVm->{$strVm}{&VMDEF_COVERAGE_PERL} ? true : false;
-}
-
-push @EXPORT, qw(vmCoveragePerl);
-
-####################################################################################################################################
-# vmLintC
-####################################################################################################################################
-sub vmLintC
-{
-    my $strVm = shift;
-
-    return $oyVm->{$strVm}{&VMDEF_LINT_C} ? true : false;
-}
-
-push @EXPORT, qw(vmLintC);
-
 ####################################################################################################################################
 # Get vm architecture bits
 ####################################################################################################################################
