@@ -32,6 +32,7 @@ use pgBackRestTest::Common::ContainerTest;
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::FileTest;
 use pgBackRestTest::Common::RunTest;
+use pgBackRestTest::Common::VmTest;
 use pgBackRestTest::Env::Host::HostBackupTest;
 use pgBackRestTest::Env::Host::HostS3Test;
 use pgBackRestTest::Env::HostEnvTest;
@@ -59,22 +60,33 @@ sub run
 {
     my $self = shift;
 
-    foreach my $bS3 (false, true)
+    foreach my $rhRun
+    (
+        {vm => VM1, remote => false, s3 =>  true, encrypt => false, delta =>  true},
+        {vm => VM1, remote =>  true, s3 => false, encrypt =>  true, delta => false},
+        {vm => VM2, remote => false, s3 => false, encrypt =>  true, delta =>  true},
+        {vm => VM2, remote =>  true, s3 =>  true, encrypt => false, delta => false},
+        {vm => VM3, remote => false, s3 => false, encrypt => false, delta =>  true},
+        {vm => VM3, remote =>  true, s3 =>  true, encrypt =>  true, delta => false},
+        {vm => VM4, remote => false, s3 =>  true, encrypt =>  true, delta => false},
+        {vm => VM4, remote =>  true, s3 => false, encrypt => false, delta =>  true},
+    )
     {
-    foreach my $bRemote ($bS3 ? (true) : (false, true))
-    {
-        my $bRepoEncrypt = $bS3 ? true : false;
-        my $bDeltaBackup = !$bRemote || ($bRepoEncrypt && $bRemote) ? true : false;
+        # Only run tests for this vm
+        next if ($rhRun->{vm} ne $self->vm());
 
         # Increment the run, log, and decide whether this unit test should be run
-        if (!$self->begin("rmt ${bRemote}, s3 ${bS3}, enc ${bRepoEncrypt}")) {next}
+        my $bRemote = $rhRun->{remote};
+        my $bS3 = $rhRun->{s3};
+        my $bEncrypt = $rhRun->{encrypt};
+        my $bDeltaBackup = $rhRun->{delta};
+
+        # Increment the run, log, and decide whether this unit test should be run
+        if (!$self->begin("rmt ${bRemote}, s3 ${bS3}, enc ${bEncrypt}, delta ${bDeltaBackup}")) {next}
 
         # Create hosts, file object, and config
         my ($oHostDbMaster, $oHostDbStandby, $oHostBackup, $oHostS3) = $self->setup(
-            true, $self->expect(), {bHostBackup => $bRemote, bCompress => false, bS3 => $bS3, bRepoEncrypt => $bRepoEncrypt});
-
-        # Reduce log level for many tests
-        my $strLogReduced = '--' . cfgOptionName(CFGOPT_LOG_LEVEL_CONSOLE) . '=' . lc(DETAIL);
+            true, $self->expect(), {bHostBackup => $bRemote, bCompress => false, bS3 => $bS3, bRepoEncrypt => $bEncrypt});
 
         # If S3 set process max to 2.  This seems like the best place for parallel testing since it will help speed S3 processing
         # without slowing down the other tests too much.
@@ -83,10 +95,9 @@ sub run
             $oHostBackup->configUpdate({&CFGDEF_SECTION_GLOBAL => {cfgOptionName(CFGOPT_PROCESS_MAX) => 2}});
             $oHostDbMaster->configUpdate({&CFGDEF_SECTION_GLOBAL => {cfgOptionName(CFGOPT_PROCESS_MAX) => 2}});
 
-            # Reduce log level to detail because parallel tests do not create deterministic logs
+            # Reduce log level to warn because parallel tests do not create deterministic logs
             $oHostBackup->configUpdate({&CFGDEF_SECTION_GLOBAL => {cfgOptionName(CFGOPT_LOG_LEVEL_CONSOLE) => lc(WARN)}});
             $oHostDbMaster->configUpdate({&CFGDEF_SECTION_GLOBAL => {cfgOptionName(CFGOPT_LOG_LEVEL_CONSOLE) => lc(WARN)}});
-            $strLogReduced = '--' . cfgOptionName(CFGOPT_LOG_LEVEL_CONSOLE) . '=' . lc(WARN);
         }
 
         # Get base time
@@ -100,13 +111,17 @@ sub run
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_ARCHIVE_CHECK} = JSON::PP::true;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_ARCHIVE_COPY} = JSON::PP::true;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_BACKUP_STANDBY} = JSON::PP::false;
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_BUFFER_SIZE} = 16384;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_CHECKSUM_PAGE} = JSON::PP::true;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_COMPRESS} = JSON::PP::false;
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_COMPRESS_LEVEL} = 3;
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_COMPRESS_LEVEL_NETWORK} = $bRemote ? 1 : 3;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_HARDLINK} = JSON::PP::false;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_ONLINE} = JSON::PP::false;
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = $bS3 ? 2 : 1;
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_DELTA} = JSON::PP::false;
 
-        if ($bRepoEncrypt)
+        if ($bEncrypt)
         {
             $oManifest{&INI_SECTION_CIPHER}{&INI_KEY_CIPHER_PASS} = 'REPLACEME';
         }
@@ -169,10 +184,10 @@ sub run
             pageBuild($tBasePage, 0) .
             pageBuild($tBasePage, 1) .
             pageBuild($tBasePage, 2) .
-            pageBuild($tBasePage, 0, 0xFFFF, 0xFFFF);
+            pageBuild($tBasePage, 0, 0xFFFFFFFF, 0xFFFFFFFF);
 
         $oHostDbMaster->manifestFileCreate(
-            \%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/33000', $tPageValid, '4a383e4fb8b5cd2a4e8fab91ef63dce48e532a2f',
+            \%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/33000', $tPageValid, '7a16d165e4775f7c92e8cdf60c0af57313f0bf90',
             $lTime);
 
         my $iBlockOffset = 32767 * 131072;
@@ -181,11 +196,11 @@ sub run
             pageBuild($tBasePage, $iBlockOffset + 0) .
             pageBuild($tBasePage, $iBlockOffset + 1) .
             ("\0" x 8192) .
-            pageBuild($tBasePage, 0, 0xFFFF, 0xFFFF);
+            pageBuild($tBasePage, 0, 0xFFFFFFFF, 0xFFFFFFFF);
 
         $oHostDbMaster->manifestFileCreate(
             \%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/33000.32767', $tPageValidSeg32767,
-            '21e2c7c1a326682c07053b7d6a5a40dbd49c2ec5', $lTime);
+            '6e99b589e550e68e934fd235ccba59fe5b592a9e', $lTime);
 
         my $tPageInvalid33001 =
             pageBuild($tBasePage, 1) .
@@ -267,7 +282,7 @@ sub run
 
             # Unlog and temp files to ignore (unlog _init will NOT be ignored)
             $oHostDbMaster->manifestFileCreate(\%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/44000_init', $tPageValid,
-                '4a383e4fb8b5cd2a4e8fab91ef63dce48e532a2f', $lTime);
+                '7a16d165e4775f7c92e8cdf60c0af57313f0bf90', $lTime);
             $oHostDbMaster->dbFileCreate(\%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/44000', 'IGNORE');
             $oHostDbMaster->dbFileCreate(\%oManifest, MANIFEST_TARGET_PGDATA, 'base/32768/t333_44000', 'IGNORE');
 
@@ -340,8 +355,7 @@ sub run
 
         my $strFullBackup = $oHostBackup->backup(
             $strType, 'error on identical link destinations',
-            {oExpectedManifest => \%oManifest, strOptionalParam => $strLogReduced,
-                iExpectedExitStatus => ERROR_LINK_DESTINATION});
+            {oExpectedManifest => \%oManifest,  iExpectedExitStatus => ERROR_LINK_DESTINATION});
 
         # Remove failing link
         $oHostDbMaster->manifestLinkRemove(\%oManifest, MANIFEST_TARGET_PGDATA, 'pg_config_bad');
@@ -353,8 +367,7 @@ sub run
         # Fail bacause two links point to the same place
         $strFullBackup = $oHostBackup->backup(
             $strType, 'error on link to a link',
-            {oExpectedManifest => \%oManifest, strOptionalParam => $strLogReduced,
-                iExpectedExitStatus => ERROR_LINK_DESTINATION});
+            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_LINK_DESTINATION});
 
         # Remove failing link
         $oHostDbMaster->manifestLinkRemove(\%oManifest, MANIFEST_TARGET_PGDATA, 'postgresql.conf.bad');
@@ -368,7 +381,9 @@ sub run
 
         # Create file with special characters
         $oHostDbMaster->manifestFileCreate(
-            \%oManifest, MANIFEST_TARGET_PGDATA, 'special-@!#$^&*()-_+~`{}[]\|:;"<>\',.?%', undef, undef, $lTime, undef, true);
+            \%oManifest, MANIFEST_TARGET_PGDATA, 'special-!_.*\'()&!@;:+,?', undef, undef, $lTime, undef, true);
+
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = 1;
 
         $strFullBackup = $oHostBackup->backup(
             $strType, 'create pg_stat link, pg_clog dir',
@@ -379,11 +394,14 @@ sub run
                     # Pass bogus ssh port to make sure it is passed through the protocol layer (it won't be used)
                     ($bRemote ? ' --' . cfgOptionName(CFGOPT_PG_PORT) . '=9999' : '') .
                     # Pass bogus socket path to make sure it is passed through the protocol layer (it won't be used)
-                    ($bRemote ? ' --' . cfgOptionName(CFGOPT_PG_SOCKET_PATH) . ' =/test_socket_path' : '') .
+                    ($bRemote ? ' --' . cfgOptionName(CFGOPT_PG_SOCKET_PATH) . '=/test_socket_path' : '') .
                     ' --' . cfgOptionName(CFGOPT_BUFFER_SIZE) . '=16384 --' . cfgOptionName(CFGOPT_CHECKSUM_PAGE) .
                     ' --' . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1',
                 strRepoType => $bS3 ? undef : CFGOPTVAL_REPO_TYPE_CIFS, strTest => $strTestPoint, fTestDelay => 0});
 
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = $bS3 ? 2 : 1;
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_BUFFER_SIZE} = 4194304;
+
         # Error on backup option to check logging
         #---------------------------------------------------------------------------------------------------------------------------
         if (!$bRemote)
@@ -548,8 +566,8 @@ sub run
         #---------------------------------------------------------------------------------------------------------------------------
         $oHostBackup->backup(
             $strType, 'invalid repo',
-            {oExpectedManifest => \%oManifest, strOptionalParam => '--' . cfgOptionName(CFGOPT_REPO_PATH) . '=/bogus_path' .
-             "  ${strLogReduced}", iExpectedExitStatus => $bS3 ? ERROR_FILE_MISSING : ERROR_PATH_MISSING});
+            {oExpectedManifest => \%oManifest, strOptionalParam => '--' . cfgOptionName(CFGOPT_REPO_PATH) . '=/bogus_path',
+                iExpectedExitStatus => $bS3 ? ERROR_FILE_MISSING : ERROR_PATH_MISSING});
 
         # Restore - tests various mode, extra files/paths, missing files/paths
         #---------------------------------------------------------------------------------------------------------------------------
@@ -627,7 +645,7 @@ sub run
         $oHostDbMaster->restore(
             'fix broken symlink', $strFullBackup,
             {rhExpectedManifest => \%oManifest, bDelta => true,
-                strOptionalParam => " --link-all ${strLogReduced}" . ($bRemote ? ' --compress-level-network=0' : '')});
+                strOptionalParam => ' --link-all' . ($bRemote ? ' --compress-level-network=0' : '')});
 
         # Test operations not running on their usual host
         if ($bRemote && !$bS3)
@@ -643,7 +661,7 @@ sub run
             $oHostBackup->restore(
                 'on backup host', $strFullBackup,
                 {rhExpectedManifest => \%oManifest, strUser => TEST_USER,
-                    strOptionalParam => "${strLogReduced} --reset-pg1-host --pg1-path=${strBackupHostDbPath}"});
+                    strOptionalParam => "--reset-pg1-host --pg1-path=${strBackupHostDbPath}"});
 
             $oHostDbMaster->backup(
                 $strType, 'backup errors on db host',
@@ -659,7 +677,7 @@ sub run
                 'restore all links by mapping', $strFullBackup,
                 {rhExpectedManifest => \%oManifest, bDelta => true,
                     strOptionalParam =>
-                        $strLogReduced . ' --link-map=pg_stat=../pg_stat --link-map=postgresql.conf=../pg_config/postgresql.conf' .
+                        '--link-map=pg_stat=../pg_stat --link-map=postgresql.conf=../pg_config/postgresql.conf' .
                             ' --link-map=pg_hba.conf=../pg_config/pg_hba.conf'});
 
             # Error when links overlap
@@ -704,7 +722,7 @@ sub run
             $oHostDbMaster->restore(
                 'restore all links --link-all and mapping', $strFullBackup,
                 {rhExpectedManifest => \%oManifest, bDelta => true,
-                    strOptionalParam => "${strLogReduced} --link-map=pg_stat=../pg_stat --link-all"});
+                    strOptionalParam => '--link-map=pg_stat=../pg_stat --link-all'});
         }
 
         # Restore - test errors when $PGDATA cannot be verified
@@ -715,8 +733,7 @@ sub run
         # Attempt the restore
         $oHostDbMaster->restore(
             'fail on missing ' . DB_FILE_PGVERSION, $strFullBackup,
-            {rhExpectedManifest => \%oManifest, bDelta => true, bForce => true, iExpectedExitStatus => ERROR_PATH_NOT_EMPTY,
-                strOptionalParam => $strLogReduced});
+            {rhExpectedManifest => \%oManifest, bDelta => true, bForce => true, iExpectedExitStatus => ERROR_PATH_NOT_EMPTY});
 
         # Write a backup.manifest file to make $PGDATA valid
         testFileCreate($oHostDbMaster->dbBasePath() . '/backup.manifest', 'BOGUS');
@@ -735,8 +752,7 @@ sub run
 
         $oHostDbMaster->restore(
             'restore succeeds with backup.manifest file', $strFullBackup,
-            {rhExpectedManifest => \%oManifest, bDelta => true, bForce => true,
-                strOptionalParam => $strLogReduced});
+            {rhExpectedManifest => \%oManifest, bDelta => true, bForce => true});
 
         # No longer need pg_hba.conf since it is no longer a link and doesn't provide additional coverage
         $oHostDbMaster->manifestFileRemove(\%oManifest, MANIFEST_TARGET_PGDATA, 'pg_hba.conf');
@@ -753,7 +769,7 @@ sub run
 
         $oHostBackup->backup(
             $strType, 'invalid database version',
-            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_BACKUP_MISMATCH, strOptionalParam => $strLogReduced});
+            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_BACKUP_MISMATCH});
 
         # Break the database system id
         $oHostBackup->infoMunge(
@@ -762,7 +778,7 @@ sub run
 
         $oHostBackup->backup(
             $strType, 'invalid system id',
-            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_BACKUP_MISMATCH, strOptionalParam => $strLogReduced});
+            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_BACKUP_MISMATCH});
 
         # Break the control version
         $oHostBackup->infoMunge(
@@ -771,7 +787,7 @@ sub run
 
         $oHostBackup->backup(
             $strType, 'invalid control version',
-            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_BACKUP_MISMATCH, strOptionalParam => $strLogReduced});
+            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_BACKUP_MISMATCH});
 
         # Break the catalog version
         $oHostBackup->infoMunge(
@@ -780,7 +796,7 @@ sub run
 
         $oHostBackup->backup(
             $strType, 'invalid catalog version',
-            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_BACKUP_MISMATCH, strOptionalParam => $strLogReduced});
+            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_BACKUP_MISMATCH});
 
         # Restore the file to its original condition
         $oHostBackup->infoRestore(storageRepo()->pathGet(STORAGE_REPO_BACKUP . qw{/} . FILE_BACKUP_INFO));
@@ -795,7 +811,7 @@ sub run
 
         $oHostBackup->backup(
             $strType, 'invalid path in ' . DB_PATH_PGTBLSPC,
-            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_LINK_EXPECTED, strOptionalParam => $strLogReduced});
+            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_LINK_EXPECTED});
 
         testPathRemove("${strTblSpcPath}/path");
 
@@ -806,8 +822,7 @@ sub run
 
             $oHostBackup->backup(
                 $strType, 'invalid relative tablespace is ../',
-                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA,
-                    strOptionalParam => $strLogReduced});
+                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA});
 
             testFileRemove("${strTblSpcPath}/99999");
 
@@ -815,8 +830,7 @@ sub run
 
             $oHostBackup->backup(
                 $strType, 'invalid relative tablespace is ..',
-                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA,
-                    strOptionalParam => $strLogReduced});
+                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA});
 
             testFileRemove("${strTblSpcPath}/99999");
 
@@ -824,8 +838,7 @@ sub run
 
             $oHostBackup->backup(
                 $strType, 'invalid relative tablespace is ../../$PGDATA',
-                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA,
-                    strOptionalParam => $strLogReduced});
+                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA});
 
             testFileRemove("${strTblSpcPath}/99999");
 
@@ -833,8 +846,7 @@ sub run
 
             $oHostBackup->backup(
                 $strType, 'invalid relative tablespace is ../../$PGDATA',
-                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA,
-                    strOptionalParam => $strLogReduced});
+                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA});
 
             testFileRemove("${strTblSpcPath}/99999");
 
@@ -844,8 +856,7 @@ sub run
 
             $oHostBackup->backup(
                 $strType, 'tablespace link references a link',
-                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_LINK_DESTINATION,
-                    strOptionalParam => $strLogReduced});
+                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_LINK_DESTINATION});
 
             testFileRemove($oHostDbMaster->dbPath() . "/intermediate_link");
             testFileRemove("${strTblSpcPath}/99999");
@@ -856,8 +867,7 @@ sub run
 
         $oHostBackup->backup(
             $strType, 'invalid relative tablespace in $PGDATA',
-            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA,
-                strOptionalParam => $strLogReduced});
+            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA});
 
         testFileRemove("${strTblSpcPath}/99999");
 
@@ -868,8 +878,7 @@ sub run
 
             $oHostBackup->backup(
                 $strType, '$PGDATA is a substring of valid tblspc excluding / (file missing err expected)',
-                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_FILE_MISSING,
-                    strOptionalParam => $strLogReduced});
+                {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_PATH_MISSING});
 
             testFileRemove("${strTblSpcPath}/99999");
         }
@@ -879,8 +888,7 @@ sub run
 
         $oHostBackup->backup(
             $strType, 'invalid tablespace in $PGDATA',
-            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA,
-                strOptionalParam => $strLogReduced});
+            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_TABLESPACE_IN_PGDATA});
 
         testFileRemove("${strTblSpcPath}/99999");
 
@@ -927,7 +935,7 @@ sub run
         forceStorageMove(storageRepo(), 'backup/' . $self->stanza() . "/${strBackup}", $strResumePath);
 
         # Munge manifest so the resumed file in the repo appears to be bad
-        if ($bRemote)
+        if ($bEncrypt || $bRemote)
         {
             $oHostBackup->manifestMunge(
                 basename($strResumePath),
@@ -985,11 +993,15 @@ sub run
         delete($oManifest{&MANIFEST_SECTION_TARGET_FILE}{'pg_data/changecontent.txt'}{&MANIFEST_SUBKEY_REFERENCE});
         $oManifest{&MANIFEST_SECTION_TARGET_FILE}{'pg_data/changetime.txt'}{&MANIFEST_SUBKEY_TIMESTAMP} = $lTime - 100;
 
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = 1;
+
         $strBackup = $oHostBackup->backup(
             $strType, 'resume and add tablespace 2',
             {oExpectedManifest => \%oManifest, strTest => TEST_BACKUP_RESUME,
                 strOptionalParam => '--' . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1' . ($bDeltaBackup ? ' --delta' : '')});
 
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = $bS3 ? 2 : 1;
+
         if (!$bRemote)
         {
             # Remove the size-changed test file to avoid expect log churn
@@ -1010,12 +1022,16 @@ sub run
         forceStorageRemove(storageRepo(), "${strResumePath}/" . FILE_MANIFEST);
 
         # The aborted backup is of a different type so is not resumable and is removed. A differential is created.
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = 1;
+
         $strBackup = $oHostBackup->backup(
             $strType, 'cannot resume - new diff',
             {oExpectedManifest => \%oManifest, strTest => TEST_BACKUP_NORESUME,
-                strOptionalParam => "$strLogReduced --" . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1' .
+                strOptionalParam => '--' . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1' .
                 ($bDeltaBackup ? ' --delta' : '')});
 
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = $bS3 ? 2 : 1;
+
         # Resume Diff Backup
         #---------------------------------------------------------------------------------------------------------------------------
         $strType = CFGOPTVAL_BACKUP_TYPE_DIFF;
@@ -1028,19 +1044,22 @@ sub run
 
         # The aborted backup is of the same type so it is resumable but passing --no-resume. Pass --delta same as before to avoid
         # expect log churn and to test restore with a --delta manifest.
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = 1;
+
         $strBackup = $oHostBackup->backup(
             $strType, 'cannot resume - disabled / no repo link',
             {oExpectedManifest => \%oManifest, strTest => TEST_BACKUP_NORESUME,
-                strOptionalParam => "--no-resume ${strLogReduced} --" . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1' .
+                strOptionalParam => '--no-resume --' . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1' .
                 ($bDeltaBackup ? ' --delta' : '')});
 
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = $bS3 ? 2 : 1;
+
         # Restore
         #---------------------------------------------------------------------------------------------------------------------------
         # Fail on used path
         $oHostDbMaster->restore(
             'fail on used path', $strBackup,
-            {rhExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_PATH_NOT_EMPTY,
-                strOptionalParam => $strLogReduced});
+            {rhExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_PATH_NOT_EMPTY});
 
         # Remap the base and tablespace paths
         my %oRemapHash;
@@ -1056,15 +1075,13 @@ sub run
         }
 
         $oHostDbMaster->restore(
-            'remap all paths', $strBackup,
-            {rhExpectedManifest => \%oManifest, rhRemapHash => \%oRemapHash, strOptionalParam => ($bRemote ? $strLogReduced : '')});
+            'remap all paths', $strBackup, {rhExpectedManifest => \%oManifest, rhRemapHash => \%oRemapHash});
 
         # Restore (make sure file in root tablespace path is not deleted by --delta)
         #---------------------------------------------------------------------------------------------------------------------------
         $oHostDbMaster->restore(
             'ensure file in tblspc root remains after --delta', $strBackup,
-            {rhExpectedManifest => \%oManifest, rhRemapHash => \%oRemapHash, bDelta => true,
-                strOptionalParam => $strLogReduced});
+            {rhExpectedManifest => \%oManifest, rhRemapHash => \%oRemapHash, bDelta => true});
 
         if (!-e $strDoNotDeleteFile)
         {
@@ -1089,9 +1106,13 @@ sub run
         # Munge the version to make sure it gets corrected on the next run
         $oHostBackup->manifestMunge($strBackup, {&INI_SECTION_BACKREST => {&INI_KEY_VERSION => '0.00'}}, false);
 
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = 1;
+
         $strBackup = $oHostBackup->backup(
             $strType, 'add files and remove tablespace 2',
-            {oExpectedManifest => \%oManifest, strOptionalParam => "$strLogReduced --" . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1'});
+            {oExpectedManifest => \%oManifest, strOptionalParam => '--' . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1'});
+
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = $bS3 ? 2 : 1;
 
         # Incr Backup
         #---------------------------------------------------------------------------------------------------------------------------
@@ -1105,7 +1126,7 @@ sub run
         my $strBackupInfoOldFile = "${strBackupInfoFile}.old";
         my $strBackupInfoCopyOldFile = "${strBackupInfoCopyFile}.old";
 
-        if ($bRepoEncrypt)
+        if ($bEncrypt)
         {
             forceStorageMove(storageRepo(), $strBackupInfoFile, $strBackupInfoOldFile, {bRecurse => false});
             forceStorageMove(storageRepo(), $strBackupInfoCopyFile, $strBackupInfoCopyOldFile, {bRecurse => false});
@@ -1124,8 +1145,7 @@ sub run
         {
             $strBackup =$oHostBackup->backup(
             $strType, 'update files - fail on missing backup.info',
-            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_FILE_MISSING,
-             strOptionalParam => $strLogReduced});
+            {oExpectedManifest => \%oManifest, iExpectedExitStatus => ERROR_FILE_MISSING});
 
             # Fail on attempt to create the stanza data since force was not used
             $oHostBackup->stanzaCreate('fail on backup directory missing backup.info',
@@ -1135,28 +1155,30 @@ sub run
         # Use force to create the stanza (this is expected to fail for encrypted repos)
         $oHostBackup->stanzaCreate('create required data for stanza',
             {strOptionalParam => '--no-' . cfgOptionName(CFGOPT_ONLINE) . ' --' . cfgOptionName(CFGOPT_FORCE),
-                iExpectedExitStatus => $bRepoEncrypt ? ERROR_FILE_MISSING : undef});
+                iExpectedExitStatus => $bEncrypt ? ERROR_FILE_MISSING : undef});
 
         # Copy encrypted backup info files back so testing can proceed
-        if ($bRepoEncrypt)
+        if ($bEncrypt)
         {
             forceStorageMove(storageRepo(), $strBackupInfoOldFile, $strBackupInfoFile, {bRecurse => false});
             forceStorageMove(storageRepo(), $strBackupInfoCopyOldFile, $strBackupInfoCopyFile, {bRecurse => false});
         }
 
         # Perform the backup
-        $strBackup =$oHostBackup->backup(
-            $strType, 'update files', {oExpectedManifest => \%oManifest, strOptionalParam => $strLogReduced});
+        $strBackup =$oHostBackup->backup($strType, 'update files', {oExpectedManifest => \%oManifest});
 
         # Diff Backup
         #---------------------------------------------------------------------------------------------------------------------------
         $strType = CFGOPTVAL_BACKUP_TYPE_DIFF;
         $oHostDbMaster->manifestReference(\%oManifest, $strFullBackup, true);
 
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = 1;
+
         $strBackup = $oHostBackup->backup(
             $strType, 'updates since last full', {oExpectedManifest => \%oManifest,
-                strOptionalParam => "$strLogReduced --" . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1' .
-                ($bDeltaBackup ? ' --delta' : '')});
+                strOptionalParam => '--' . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1' . ($bDeltaBackup ? ' --delta' : '')});
+
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = $bS3 ? 2 : 1;
 
         # Incr Backup
         #
@@ -1172,8 +1194,7 @@ sub run
 
         my $oBackupExecute = $oHostBackup->backupBegin(
             $strType, 'remove files - but won\'t affect manifest',
-            {oExpectedManifest => \%oManifest, strTest => TEST_MANIFEST_BUILD, fTestDelay => 1,
-                strOptionalParam => $strLogReduced});
+            {oExpectedManifest => \%oManifest, strTest => TEST_MANIFEST_BUILD, fTestDelay => 1});
 
         $oHostDbMaster->dbFileRemove(\%oManifest, MANIFEST_TARGET_PGDATA, 'base/16384/17000');
 
@@ -1201,11 +1222,12 @@ sub run
             $oHostBackup->configUpdate({&CFGDEF_SECTION_GLOBAL => {cfgOptionName(CFGOPT_REPO_HARDLINK) => 'y'}});
         }
 
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = 1;
+
         $oBackupExecute = $oHostBackup->backupBegin(
             $strType, 'remove files during backup',
             {oExpectedManifest => \%oManifest, strTest => TEST_MANIFEST_BUILD, fTestDelay => 1,
-                strOptionalParam => "$strLogReduced --" . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1' .
-                ($bDeltaBackup ? ' --delta' : '')});
+                strOptionalParam => '--' . cfgOptionName(CFGOPT_PROCESS_MAX) . '=1' . ($bDeltaBackup ? ' --delta' : '')});
 
         $oHostDbMaster->manifestFileCreate(
             \%oManifest, MANIFEST_TARGET_PGTBLSPC . '/2', '32768/tablespace2c.txt', 'TBLSPCBIGGER',
@@ -1215,6 +1237,8 @@ sub run
 
         $strBackup = $oHostBackup->backupEnd($strType, $oBackupExecute, {oExpectedManifest => \%oManifest});
 
+        $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_PROCESS_MAX} = $bS3 ? 2 : 1;
+
         # Full Backup
         #---------------------------------------------------------------------------------------------------------------------------
         $strType = CFGOPTVAL_BACKUP_TYPE_FULL;
@@ -1237,7 +1261,7 @@ sub run
         $oManifest{&MANIFEST_SECTION_BACKUP_OPTION}{&MANIFEST_KEY_CHECKSUM_PAGE} = JSON::PP::false;
 
         $strFullBackup = $oHostBackup->backup(
-            $strType, 'update file', {oExpectedManifest => \%oManifest, strOptionalParam => $strLogReduced});
+            $strType, 'update file', {oExpectedManifest => \%oManifest});
 
         # Backup Info
         #---------------------------------------------------------------------------------------------------------------------------
@@ -1264,8 +1288,8 @@ sub run
             $strFullBackup, {&MANIFEST_SECTION_BACKUP_OPTION => {&MANIFEST_KEY_CHECKSUM_PAGE => undef}}, false);
 
         $strBackup = $oHostBackup->backup(
-            $strType, 'add file', {oExpectedManifest => \%oManifest,
-                strOptionalParam => "${strLogReduced} --" . cfgOptionName(CFGOPT_CHECKSUM_PAGE)});
+            $strType, 'add file',
+            {oExpectedManifest => \%oManifest, strOptionalParam => '--' . cfgOptionName(CFGOPT_CHECKSUM_PAGE)});
 
         # Selective Restore
         #---------------------------------------------------------------------------------------------------------------------------
@@ -1283,11 +1307,11 @@ sub run
         $oHostDbMaster->restore(
             'selective restore 16384', cfgDefOptionDefault(CFGCMD_RESTORE, CFGOPT_SET),
             {rhExpectedManifest => \%oManifest, rhRemapHash => \%oRemapHash, bDelta => true,
-                strOptionalParam => "${strLogReduced} --db-include=16384"});
+                strOptionalParam => '--db-include=16384'});
 
         # Restore checksum values for next test
         $oManifest{&MANIFEST_SECTION_TARGET_FILE}{'pg_data/base/32768/33000'}{&MANIFEST_SUBKEY_CHECKSUM} =
-            '4a383e4fb8b5cd2a4e8fab91ef63dce48e532a2f';
+            '7a16d165e4775f7c92e8cdf60c0af57313f0bf90';
         $oManifest{&MANIFEST_SECTION_TARGET_FILE}{'pg_data/base/32768/33001'}{&MANIFEST_SUBKEY_CHECKSUM} =
             '6bf316f11d28c28914ea9be92c00de9bea6d9a6b';
         $oManifest{&MANIFEST_SECTION_TARGET_FILE}{'pg_tblspc/2/PG_9.4_201409291/32768/tablespace2.txt'}
@@ -1301,7 +1325,7 @@ sub run
         $oHostDbMaster->restore(
             'selective restore 32768', cfgDefOptionDefault(CFGCMD_RESTORE, CFGOPT_SET),
             {rhExpectedManifest => \%oManifest, rhRemapHash => \%oRemapHash, bDelta => true,
-                strOptionalParam => "${strLogReduced} --db-include=32768"});
+                strOptionalParam => '--db-include=32768'});
 
         $oManifest{&MANIFEST_SECTION_TARGET_FILE}{'pg_data/base/16384/17000'}{&MANIFEST_SUBKEY_CHECKSUM} =
             '7579ada0808d7f98087a0a586d0df9de009cdc33';
@@ -1329,14 +1353,14 @@ sub run
         $oHostDbMaster->restore(
             'no tablespace remap - error when tablespace dir does not exist', cfgDefOptionDefault(CFGCMD_RESTORE, CFGOPT_SET),
             {rhExpectedManifest => \%oManifest, rhRemapHash => \%oRemapHash, iExpectedExitStatus => ERROR_PATH_MISSING,
-                bTablespace => false, strOptionalParam => "${strLogReduced} --tablespace-map-all=../../tablespace"});
+                bTablespace => false, strOptionalParam => '--tablespace-map-all=../../tablespace'});
 
         storageTest()->pathCreate($oHostDbMaster->dbBasePath(2) . '/tablespace', {strMode => '0700'});
 
         $oHostDbMaster->restore(
             'no tablespace remap', cfgDefOptionDefault(CFGCMD_RESTORE, CFGOPT_SET),
             {rhExpectedManifest => \%oManifest, bTablespace => false,
-                strOptionalParam => "--tablespace-map-all=../../tablespace ${strLogReduced}"});
+                strOptionalParam => '--tablespace-map-all=../../tablespace'});
 
         $oManifest{&MANIFEST_SECTION_BACKUP_TARGET}{'pg_tblspc/2'}{&MANIFEST_SUBKEY_PATH} = '../../tablespace/ts2';
         $oManifest{&MANIFEST_SECTION_TARGET_LINK}{'pg_data/pg_tblspc/2'}{&MANIFEST_SUBKEY_DESTINATION} = '../../tablespace/ts2';
@@ -1400,7 +1424,6 @@ sub run
                     strOptionalParam => '--log-level-console=info --' . cfgOptionName(CFGOPT_BACKUP_STANDBY)});
         }
     }
-    }
 }
 
 1;
