@@ -28,6 +28,7 @@ use pgBackRest::Storage::Helper;
 use pgBackRestTest::Env::HostEnvTest;
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::RunTest;
+use pgBackRestTest::Common::VmTest;
 
 ####################################################################################################################################
 # run
@@ -41,33 +42,43 @@ sub run
     my $strWalHash = $self->walGenerateContentChecksum(PG_VERSION_94);
     storageTest()->put($strWalTestFile, $self->walGenerateContent(PG_VERSION_94));
 
-    foreach my $bS3 (false, true)
+    foreach my $rhRun
+    (
+        {vm => VM1, remote => false, s3 => false, encrypt => false, compress =>  true, error => 0},
+        {vm => VM1, remote =>  true, s3 =>  true, encrypt =>  true, compress => false, error => 1},
+        {vm => VM2, remote => false, s3 =>  true, encrypt => false, compress => false, error => 0},
+        {vm => VM2, remote =>  true, s3 => false, encrypt =>  true, compress =>  true, error => 0},
+        {vm => VM3, remote => false, s3 => false, encrypt =>  true, compress => false, error => 0},
+        {vm => VM3, remote =>  true, s3 =>  true, encrypt => false, compress =>  true, error => 1},
+        {vm => VM4, remote => false, s3 =>  true, encrypt =>  true, compress =>  true, error => 0},
+        {vm => VM4, remote =>  true, s3 => false, encrypt => false, compress => false, error => 0},
+    )
     {
-    foreach my $bRemote ($bS3 ? (true) : (false, true))
-    {
-    foreach my $bCompress ($bS3 ? (false) : (false, true))
-    {
-    foreach my $iError ($bS3 ? (1) : ($bRemote ? (0, 1) : (0)))
-    {
-        my $bRepoEncrypt = ($bCompress && !$bS3) ? true : false;
+        # Only run tests for this vm
+        next if ($rhRun->{vm} ne $self->vm());
+
+        # Increment the run, log, and decide whether this unit test should be run
+        my $bRemote = $rhRun->{remote};
+        my $bS3 = $rhRun->{s3};
+        my $bEncrypt = $rhRun->{encrypt};
+        my $bCompress = $rhRun->{compress};
+        my $iError = $rhRun->{error};
 
         # Increment the run, log, and decide whether this unit test should be run
         if (!$self->begin("rmt ${bRemote}, cmp ${bCompress}, error " . ($iError ? 'connect' : 'version') . ", s3 ${bS3}, " .
-            "enc ${bRepoEncrypt}")) {next}
+            "enc ${bEncrypt}")) {next}
 
         # Create hosts, file object, and config
         my ($oHostDbMaster, $oHostDbStandby, $oHostBackup, $oHostS3) = $self->setup(
             true, $self->expect(), {bHostBackup => $bRemote, bCompress => $bCompress, bArchiveAsync => true, bS3 => $bS3,
-            bRepoEncrypt => $bRepoEncrypt});
-
-        my $oStorage = storageRepo();
+            bRepoEncrypt => $bEncrypt});
 
         # Create compression extension
         my $strCompressExt = $bCompress ? qw{.} . COMPRESS_EXT : '';
 
         # Create the wal path
         my $strWalPath = $oHostDbMaster->dbBasePath() . '/pg_xlog';
-        $oStorage->pathCreate($strWalPath, {bCreateParent => true});
+        storageTest()->pathCreate($strWalPath, {bCreateParent => true});
 
         # Create the test path for pg_control and generate pg_control for stanza-create
         storageTest()->pathCreate($oHostDbMaster->dbBasePath() . '/' . DB_PATH_GLOBAL, {bCreateParent => true});
@@ -77,35 +88,43 @@ sub run
         $oHostBackup->stanzaCreate('create required data for stanza', {strOptionalParam => '--no-' . cfgOptionName(CFGOPT_ONLINE)});
 
         # Push a WAL segment
+        &log(INFO, '    push first WAL');
         $oHostDbMaster->archivePush($strWalPath, $strWalTestFile, 1);
 
         # Break the database version of the archive info file
         if ($iError == 0)
         {
             $oHostBackup->infoMunge(
-                $oStorage->pathGet(STORAGE_REPO_ARCHIVE . qw{/} . ARCHIVE_INFO_FILE),
-                {&INFO_ARCHIVE_SECTION_DB => {&INFO_ARCHIVE_KEY_DB_VERSION => '8.0'}});
+                storageRepo()->pathGet(STORAGE_REPO_ARCHIVE . qw{/} . ARCHIVE_INFO_FILE),
+                {&INFO_ARCHIVE_SECTION_DB => {&INFO_ARCHIVE_KEY_DB_VERSION => '8.0'},
+                 &INFO_ARCHIVE_SECTION_DB_HISTORY => {1 => {&INFO_ARCHIVE_KEY_DB_VERSION => '8.0'}}});
         }
 
         # Push two more segments with errors to exceed archive-push-queue-max
-        $oHostDbMaster->archivePush(
-            $strWalPath, $strWalTestFile, 2, $iError ? ERROR_FILE_READ : ERROR_ARCHIVE_MISMATCH);
+        &log(INFO, '    push second WAL');
 
         $oHostDbMaster->archivePush(
-            $strWalPath, $strWalTestFile, 3, $iError ? ERROR_FILE_READ : ERROR_ARCHIVE_MISMATCH);
+            $strWalPath, $strWalTestFile, 2, $iError ? ERROR_UNKNOWN : ERROR_ARCHIVE_MISMATCH);
+
+        &log(INFO, '    push third WAL');
+
+        $oHostDbMaster->archivePush(
+            $strWalPath, $strWalTestFile, 3, $iError ? ERROR_UNKNOWN : ERROR_ARCHIVE_MISMATCH);
 
         # Now this segment will get dropped
+        &log(INFO, '    push fourth WAL');
+
         $oHostDbMaster->archivePush($strWalPath, $strWalTestFile, 4, undef, undef, '--repo1-host=bogus');
 
         # Fix the database version
         if ($iError == 0)
         {
-            $oHostBackup->infoRestore($oStorage->pathGet(STORAGE_REPO_ARCHIVE . qw{/} . ARCHIVE_INFO_FILE));
+            $oHostBackup->infoRestore(storageRepo()->pathGet(STORAGE_REPO_ARCHIVE . qw{/} . ARCHIVE_INFO_FILE));
         }
 
         #---------------------------------------------------------------------------------------------------------------------------
         $self->testResult(
-            sub {$oStorage->list(
+            sub {storageRepo()->list(
                 STORAGE_REPO_ARCHIVE . qw{/} . PG_VERSION_94 . '-1/0000000100000001')},
             "000000010000000100000001-${strWalHash}${strCompressExt}",
             'segment 2-4 not pushed', {iWaitSeconds => 5});
@@ -114,15 +133,12 @@ sub run
         $oHostDbMaster->archivePush($strWalPath, $strWalTestFile, 5);
 
         $self->testResult(
-            sub {$oStorage->list(
+            sub {storageRepo()->list(
                 STORAGE_REPO_ARCHIVE . qw{/} . PG_VERSION_94 . '-1/0000000100000001')},
             "(000000010000000100000001-${strWalHash}${strCompressExt}, " .
                 "000000010000000100000005-${strWalHash}${strCompressExt})",
             'segment 5 is pushed', {iWaitSeconds => 5});
     }
-    }
-    }
-    }
 }
 
 1;
