@@ -28,6 +28,7 @@ use pgBackRest::Storage::Helper;
 use pgBackRestTest::Env::HostEnvTest;
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::RunTest;
+use pgBackRestTest::Common::VmTest;
 
 ####################################################################################################################################
 # archiveCheck
@@ -79,34 +80,34 @@ sub run
 
     my $strArchiveChecksum = $self->walGenerateContentChecksum(PG_VERSION_94, {iSourceNo => 2});
 
-    foreach my $bS3 (false, true)
+    foreach my $rhRun
+    (
+        {vm => VM1, remote => false, s3 => false, encrypt => false},
+        {vm => VM1, remote =>  true, s3 =>  true, encrypt =>  true},
+        {vm => VM2, remote => false, s3 =>  true, encrypt => false},
+        {vm => VM2, remote =>  true, s3 => false, encrypt =>  true},
+        {vm => VM3, remote => false, s3 => false, encrypt =>  true},
+        {vm => VM3, remote =>  true, s3 =>  true, encrypt => false},
+        {vm => VM4, remote => false, s3 =>  true, encrypt =>  true},
+        {vm => VM4, remote =>  true, s3 => false, encrypt => false},
+    )
     {
-    foreach my $bRemote ($bS3 ? (false) : (false, true))
-    {
-        my $bRepoEncrypt = !$bRemote && !$bS3 ? true : false;
+        # Only run tests for this vm
+        next if ($rhRun->{vm} ne $self->vm());
 
         # Increment the run, log, and decide whether this unit test should be run
-        if (!$self->begin("rmt ${bRemote}, s3 ${bS3}, enc ${bRepoEncrypt}")) {next}
+        my $bRemote = $rhRun->{remote};
+        my $bS3 = $rhRun->{s3};
+        my $bEncrypt = $rhRun->{encrypt};
+
+        if (!$self->begin("rmt ${bRemote}, s3 ${bS3}, enc ${bEncrypt}")) {next}
 
         # Create hosts, file object, and config
         my ($oHostDbMaster, $oHostDbStandby, $oHostBackup) = $self->setup(
-            true, $self->expect(), {bHostBackup => $bRemote, bCompress => false, bS3 => $bS3, bRepoEncrypt => $bRepoEncrypt});
+            true, $self->expect(), {bHostBackup => $bRemote, bCompress => false, bS3 => $bS3, bRepoEncrypt => $bEncrypt});
 
         # Reduce console logging to detail
         $oHostDbMaster->configUpdate({&CFGDEF_SECTION_GLOBAL => {cfgOptionName(CFGOPT_LOG_LEVEL_CONSOLE) => lc(DETAIL)}});
-        my $strLogDebug = '--' . cfgOptionName(CFGOPT_LOG_LEVEL_CONSOLE) . qw{=} . lc(DEBUG);
-
-        # If S3 set process max to 2.  This seems like the best place for parallel testing since it will help speed S3 processing
-        # without slowing down the other tests too much.
-        if ($bS3)
-        {
-            $oHostBackup->configUpdate({&CFGDEF_SECTION_GLOBAL => {cfgOptionName(CFGOPT_PROCESS_MAX) => 2}});
-            $oHostDbMaster->configUpdate({&CFGDEF_SECTION_GLOBAL => {cfgOptionName(CFGOPT_PROCESS_MAX) => 2}});
-
-            # Reduce console logging to warn (even for debug exceptions)
-            $oHostDbMaster->configUpdate({&CFGDEF_SECTION_GLOBAL => {cfgOptionName(CFGOPT_LOG_LEVEL_CONSOLE) => lc(WARN)}});
-            $strLogDebug = '--' . cfgOptionName(CFGOPT_LOG_LEVEL_CONSOLE) . qw{=} . lc(WARN);
-        }
 
         # Create the wal path
         my $strWalPath = $oHostDbMaster->dbBasePath() . '/pg_xlog';
@@ -152,7 +153,7 @@ sub run
         my $strArchiveFile = $self->walGenerate($strWalPath, PG_VERSION_94, 2, $strSourceFile);
 
         $oHostDbMaster->executeSimple(
-            $strCommandPush . ($bRemote ? ' --cmd-ssh=/usr/bin/ssh' : '') . " --compress ${strLogDebug} ${strWalPath}/${strSourceFile}",
+            $strCommandPush . ($bRemote ? ' --cmd-ssh=/usr/bin/ssh' : '') . " --compress ${strWalPath}/${strSourceFile}",
             {oLogTest => $self->expect()});
         push(@stryExpectedWAL, "${strSourceFile}-${strArchiveChecksum}.gz");
 
@@ -169,15 +170,14 @@ sub run
         &log(INFO, '    get missing WAL');
 
         $oHostDbMaster->executeSimple(
-            $strCommandGet . " ${strLogDebug} 700000007000000070000000 ${strWalPath}/RECOVERYXLOG",
+            $strCommandGet . " 700000007000000070000000 ${strWalPath}/RECOVERYXLOG",
             {iExpectedExitStatus => 1, oLogTest => $self->expect()});
 
         #---------------------------------------------------------------------------------------------------------------------------
         &log(INFO, '    get first WAL');
 
         $oHostDbMaster->executeSimple(
-            $strCommandGet . " ${strLogDebug} ${strSourceFile} ${strWalPath}/RECOVERYXLOG",
-            {oLogTest => $self->expect()});
+            $strCommandGet . " ${strSourceFile} ${strWalPath}/RECOVERYXLOG", {oLogTest => $self->expect()});
 
         # Check that the destination file exists
         if (storageDb()->exists("${strWalPath}/RECOVERYXLOG"))
@@ -273,39 +273,6 @@ sub run
 
         storageTest()->remove("${strWalPath}/archive_status/00000002.history.ready");
 
-        #---------------------------------------------------------------------------------------------------------------------------
-        &log(INFO, '    db version mismatch in db section only - archive-push errors but archive-get succeeds');
-
-        $oHostBackup->infoMunge(
-            storageRepo()->pathGet(STORAGE_REPO_ARCHIVE . qw{/} . ARCHIVE_INFO_FILE),
-            {&INFO_ARCHIVE_SECTION_DB => {&INFO_ARCHIVE_KEY_DB_VERSION => '8.0'}});
-
-        $oHostDbMaster->executeSimple(
-            $strCommandPush . " ${strWalPath}/${strSourceFile}",
-            {iExpectedExitStatus => ERROR_ARCHIVE_MISMATCH, oLogTest => $self->expect()});
-
-        # Remove RECOVERYXLOG so it can be recovered
-        storageTest()->remove("${strWalPath}/RECOVERYXLOG", {bIgnoreMissing => false});
-
-        $oHostDbMaster->executeSimple(
-            $strCommandGet . " ${strSourceFile1} ${strWalPath}/RECOVERYXLOG",
-            {oLogTest => $self->expect()});
-
-        # Check that the destination file exists
-        if (storageDb()->exists("${strWalPath}/RECOVERYXLOG"))
-        {
-            my ($strActualChecksum) = storageDb()->hashSize("${strWalPath}/RECOVERYXLOG");
-
-            if ($strActualChecksum ne $strArchiveChecksum)
-            {
-                confess "recovered file hash '${strActualChecksum}' does not match expected '${strArchiveChecksum}'";
-            }
-        }
-        else
-        {
-            confess "archive file '${strWalPath}/RECOVERYXLOG' is not in destination";
-        }
-
         #---------------------------------------------------------------------------------------------------------------------------
         &log(INFO, '    db version mismatch error - archive-get unable to retrieve archiveId');
 
@@ -446,7 +413,6 @@ sub run
             '(' . join(', ', @stryExpectedWAL) . ')',
             'all WAL in archive', {iWaitSeconds => 5});
     }
-    }
 }
 
 1;
