@@ -22,12 +22,12 @@ use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::Wait;
 use pgBackRest::Config::Config;
-use pgBackRest::Expire;
 use pgBackRest::Manifest;
 use pgBackRest::Protocol::Storage::Helper;
 
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::RunTest;
+use pgBackRestTest::Common::VmTest;
 use pgBackRestTest::Env::ExpireEnvTest;
 use pgBackRestTest::Env::Host::HostS3Test;
 use pgBackRestTest::Env::HostEnvTest;
@@ -61,7 +61,7 @@ sub initStanzaOption
         $self->optionTestSet(CFGOPT_REPO_S3_ENDPOINT, HOST_S3_ENDPOINT);
         $self->optionTestSet(CFGOPT_REPO_S3_REGION, HOST_S3_REGION);
         $self->optionTestSet(CFGOPT_REPO_S3_HOST, $oHostS3->ipGet());
-        $self->optionTestSetBool(CFGOPT_REPO_S3_VERIFY_SSL, false);
+        $self->optionTestSetBool(CFGOPT_REPO_S3_VERIFY_TLS, false);
     }
 }
 
@@ -76,17 +76,32 @@ sub run
     my $lBaseTime = time() - (SECONDS_PER_DAY * 56);
     my $strDescription;
 
-    my $bS3 = false;
-
-    foreach my $bRepoEncrypt (false, true)
+    foreach my $rhRun
+    (
+        {vm => VM1, s3 =>  true, encrypt => false},
+        {vm => VM2, s3 => false, encrypt =>  true},
+        {vm => VM3, s3 => false, encrypt => false},
+        {vm => VM4, s3 =>  true, encrypt =>  true},
+    )
     {
+        # Only run tests for this vm
+        next if ($rhRun->{vm} ne $self->vm());
+
+        # Increment the run, log, and decide whether this unit test should be run
+        my $bS3 = $rhRun->{s3};
+        my $bEncrypt = $rhRun->{encrypt};
+
+        if ($bS3 && ($self->vm() eq VM3))
+        {
+            confess &log("cannot configure s3 for expect log tests");
+        }
+
         ############################################################################################################################
-        # Pass !$bRepoEncrypt so expect logs are not generated for encryption tests
-        if ($self->begin("local, enc ${bRepoEncrypt}", !$bRepoEncrypt))
+        if ($self->begin("simple, enc ${bEncrypt}, s3 ${bS3}"))
         {
             # Create hosts, file object, and config
             my ($oHostDbMaster, $oHostDbStandby, $oHostBackup, $oHostS3) = $self->setup(
-                true, $self->expect(), {bS3 => $bS3, bRepoEncrypt => $bRepoEncrypt});
+                true, $self->expect(), {bS3 => $bS3, bRepoEncrypt => $bEncrypt});
 
             $self->initStanzaOption($oHostDbMaster->dbBasePath(), $oHostBackup->{strRepoPath}, $oHostS3);
             $self->configTestLoad(CFGCMD_STANZA_CREATE);
@@ -181,12 +196,11 @@ sub run
         }
 
         ############################################################################################################################
-        # Pass !$bRepoEncrypt so expect logs are not generated for encryption tests
-        if ($self->begin("Expire::stanzaUpgrade, enc ${bRepoEncrypt}", !$bRepoEncrypt))
+        if ($self->begin("stanzaUpgrade, enc ${bEncrypt}, s3 ${bS3}"))
         {
             # Create hosts, file object, and config
             my ($oHostDbMaster, $oHostDbStandby, $oHostBackup, $oHostS3) = $self->setup(
-                true, $self->expect(), {bS3 => $bS3, bRepoEncrypt => $bRepoEncrypt});
+                true, $self->expect(), {bS3 => $bS3, bRepoEncrypt => $bEncrypt});
 
             $self->initStanzaOption($oHostDbMaster->dbBasePath(), $oHostBackup->{strRepoPath}, $oHostS3);
             $self->configTestLoad(CFGCMD_STANZA_CREATE);
@@ -246,7 +260,6 @@ sub run
             $self->configTestLoad(CFGCMD_EXPIRE);
 
             $strDescription = 'Expiration cannot occur due to info file db mismatch';
-            my $oExpire = new pgBackRest::Expire();
 
             # Mismatched version
             $oHostBackup->infoMunge(storageRepo()->pathGet(STORAGE_REPO_ARCHIVE . qw{/} . ARCHIVE_INFO_FILE),
@@ -258,11 +271,6 @@ sub run
                         {&INFO_ARCHIVE_KEY_DB_VERSION => PG_VERSION_93,
                             &INFO_ARCHIVE_KEY_DB_ID => $self->dbSysId(PG_VERSION_95)}}});
 
-            $self->testException(sub {$oExpire->process()},
-                ERROR_FILE_INVALID,
-                "archive and backup database versions do not match\n" .
-                "HINT: has a stanza-upgrade been performed?");
-
             # Restore the info file
             $oHostBackup->infoRestore(storageRepo()->pathGet(STORAGE_REPO_ARCHIVE . qw{/} . ARCHIVE_INFO_FILE));
 
@@ -274,11 +282,6 @@ sub run
                     {'3' =>
                         {&INFO_ARCHIVE_KEY_DB_VERSION => PG_VERSION_95, &INFO_ARCHIVE_KEY_DB_ID => 6999999999999999999}}});
 
-            $self->testException(sub {$oExpire->process()},
-                ERROR_FILE_INVALID,
-                "archive and backup database versions do not match\n" .
-                "HINT: has a stanza-upgrade been performed?");
-
             # Restore the info file
             $oHostBackup->infoRestore(storageRepo()->pathGet(STORAGE_REPO_ARCHIVE . qw{/} . ARCHIVE_INFO_FILE));
         }
