@@ -25,13 +25,13 @@ use pgBackRest::InfoCommon;
 use pgBackRest::Manifest;
 use pgBackRest::Protocol::Storage::Helper;
 use pgBackRest::Storage::Base;
-use pgBackRest::Storage::Filter::Gzip;
 use pgBackRest::Storage::Helper;
 
 use pgBackRestTest::Env::HostEnvTest;
 use pgBackRestTest::Common::ExecuteTest;
 use pgBackRestTest::Common::FileTest;
 use pgBackRestTest::Common::RunTest;
+use pgBackRestTest::Common::VmTest;
 
 ####################################################################################################################################
 # run
@@ -51,18 +51,32 @@ sub run
     my $strBackupInfoOldFile = "${strBackupInfoFile}.old";
     my $strBackupInfoCopyOldFile = "${strBackupInfoCopyFile}.old";
 
-    foreach my $bS3 (false, true)
+    foreach my $rhRun
+    (
+        {vm => VM1, remote => false, s3 => false, encrypt =>  true},
+        {vm => VM1, remote =>  true, s3 =>  true, encrypt => false},
+        {vm => VM2, remote => false, s3 =>  true, encrypt =>  true},
+        {vm => VM2, remote =>  true, s3 => false, encrypt => false},
+        {vm => VM3, remote => false, s3 => false, encrypt => false},
+        {vm => VM3, remote =>  true, s3 =>  true, encrypt =>  true},
+        {vm => VM4, remote => false, s3 =>  true, encrypt => false},
+        {vm => VM4, remote =>  true, s3 => false, encrypt =>  true},
+    )
     {
-    foreach my $bRemote ($bS3 ? (true) : (false, true))
-    {
-        my $bRepoEncrypt = $bRemote && !$bS3 ? true : false;
+        # Only run tests for this vm
+        next if ($rhRun->{vm} ne $self->vm());
 
         # Increment the run, log, and decide whether this unit test should be run
-        if (!$self->begin("remote $bRemote, s3 $bS3, enc ${bRepoEncrypt}")) {next}
+        my $bRemote = $rhRun->{remote};
+        my $bS3 = $rhRun->{s3};
+        my $bEncrypt = $rhRun->{encrypt};
+
+        # Increment the run, log, and decide whether this unit test should be run
+        if (!$self->begin("remote ${bRemote}, s3 ${bS3}, enc ${bEncrypt}")) {next}
 
         # Create hosts, file object, and config
         my ($oHostDbMaster, $oHostDbStandby, $oHostBackup, $oHostS3) = $self->setup(
-            true, $self->expect(), {bHostBackup => $bRemote, bS3 => $bS3, bRepoEncrypt => $bRepoEncrypt});
+            true, $self->expect(), {bHostBackup => $bRemote, bS3 => $bS3, bRepoEncrypt => $bEncrypt});
 
         # Create the stanza
         $oHostBackup->stanzaCreate('fail on missing control file', {iExpectedExitStatus => ERROR_FILE_OPEN,
@@ -117,7 +131,7 @@ sub run
         $oHostDbMaster->executeSimple($strCommand . " ${strSourceFile}", {oLogTest => $self->expect()});
 
         # With data existing in the archive dir, remove the info files and confirm failure
-        if ($bRepoEncrypt)
+        if ($bEncrypt)
         {
             forceStorageMove(storageRepo(), $strArchiveInfoFile, $strArchiveInfoOldFile, {bRecurse => false});
             forceStorageMove(storageRepo(), $strArchiveInfoCopyFile, $strArchiveInfoCopyOldFile, {bRecurse => false});
@@ -128,7 +142,7 @@ sub run
             forceStorageRemove(storageRepo(), $strArchiveInfoCopyFile);
         }
 
-        if (!$bRepoEncrypt)
+        if (!$bEncrypt)
         {
             $oHostBackup->stanzaCreate('fail on archive info file missing from non-empty dir',
                 {iExpectedExitStatus => ERROR_FILE_MISSING, strOptionalParam => '--no-' . cfgOptionName(CFGOPT_ONLINE)});
@@ -162,9 +176,9 @@ sub run
         # Force creation of archive info from the gz file
         $oHostBackup->stanzaCreate('force create archive.info from gz file',
             {strOptionalParam => '--no-' . cfgOptionName(CFGOPT_ONLINE) . ' --' . cfgOptionName(CFGOPT_FORCE),
-                iExpectedExitStatus => $bRepoEncrypt ? ERROR_FILE_MISSING : undef});
+                iExpectedExitStatus => $bEncrypt ? ERROR_FILE_MISSING : undef});
 
-        if (!$bRepoEncrypt)
+        if (!$bEncrypt)
         {
             $self->testResult(sub {storageRepo()->exists($strArchiveInfoFile)}, true, "    archive.info file was created");
         }
@@ -174,7 +188,7 @@ sub run
         # Unzip the archive file and recreate the archive.info file from it
         my $strArchiveTest = PG_VERSION_93 . "-1/${strArchiveFile}-" . $self->walGenerateContentChecksum(PG_VERSION_93);
 
-        if (!$bRepoEncrypt)
+        if (!$bEncrypt)
         {
             forceStorageMode(
                 storageRepo(), dirname(storageRepo()->pathGet(STORAGE_REPO_ARCHIVE . "/${strArchiveTest}.gz")), 'g+w',
@@ -183,7 +197,7 @@ sub run
             storageRepo()->copy(
                 storageRepo()->openRead(
                     STORAGE_REPO_ARCHIVE . "/${strArchiveTest}.gz",
-                    {rhyFilter => [{strClass => STORAGE_FILTER_GZIP, rxyParam => [{strCompressType => STORAGE_DECOMPRESS}]}]}),
+                    {rhyFilter => [{strClass => STORAGE_FILTER_GZIP, rxyParam => [STORAGE_DECOMPRESS, false]}]}),
                 STORAGE_REPO_ARCHIVE . "/${strArchiveTest}");
 
             $oHostBackup->stanzaCreate('force create archive.info from uncompressed file',
@@ -193,7 +207,7 @@ sub run
         # Stanza Create succeeds when using force - missing archive file
         #--------------------------------------------------------------------------------------------------------------------------
         # Remove the uncompressed WAL archive file and archive.info
-        if (!$bRepoEncrypt)
+        if (!$bEncrypt)
         {
             forceStorageRemove(storageRepo(), STORAGE_REPO_ARCHIVE . "/${strArchiveTest}");
             forceStorageRemove(storageRepo(), $strArchiveInfoFile);
@@ -206,7 +220,7 @@ sub run
         # Stanza Create succeeds when using force - missing archive directory
         #--------------------------------------------------------------------------------------------------------------------------
         # Remove the WAL archive directory
-        if (!$bRepoEncrypt)
+        if (!$bEncrypt)
         {
             forceStorageRemove(
                 storageRepo(),
@@ -219,7 +233,7 @@ sub run
         }
 
         # Encrypted info files could not be reconstructed above so just copy them back
-        if ($bRepoEncrypt)
+        if ($bEncrypt)
         {
             forceStorageMove(storageRepo(), $strArchiveInfoOldFile, $strArchiveInfoFile, {bRecurse => false});
             forceStorageMove(storageRepo(), $strArchiveInfoCopyOldFile, $strArchiveInfoCopyFile, {bRecurse => false});
@@ -275,8 +289,10 @@ sub run
         # Create a DB history mismatch between the info files
         #--------------------------------------------------------------------------------------------------------------------------
         # Remove the archive info file and force reconstruction
-        if (!$bRepoEncrypt)
+        if (!$bEncrypt)
         {
+            forceStorageMove(storageRepo(), $strArchiveInfoFile, $strArchiveInfoOldFile, {bRecurse => false});
+            forceStorageMove(storageRepo(), $strArchiveInfoCopyFile, $strArchiveInfoCopyOldFile, {bRecurse => false});
             forceStorageRemove(storageRepo(), $strArchiveInfoFile);
             forceStorageRemove(storageRepo(), $strArchiveInfoCopyFile);
 
@@ -286,8 +302,10 @@ sub run
 
         # Create a DB-ID mismatch between the info files
         #--------------------------------------------------------------------------------------------------------------------------
-        if (!$bRepoEncrypt)
+        if (!$bEncrypt)
         {
+            forceStorageMove(storageRepo(), $strBackupInfoFile, $strBackupInfoOldFile, {bRecurse => false});
+            forceStorageMove(storageRepo(), $strBackupInfoCopyFile, $strBackupInfoCopyOldFile, {bRecurse => false});
             forceStorageRemove(storageRepo(), $strBackupInfoFile);
             forceStorageRemove(storageRepo(), $strBackupInfoCopyFile);
 
@@ -295,18 +313,19 @@ sub run
                 {strOptionalParam => '--no-' . cfgOptionName(CFGOPT_ONLINE) . ' --' . cfgOptionName(CFGOPT_FORCE)});
         }
 
-        # Confirm successful backup at db-1 although archive at db-2
+        # Confirm successful backup at db-1 although archive at db-2 and format error thrown by expire (if not encrypted) since
+        # archive.info history and backup.info history are mismatched
         #--------------------------------------------------------------------------------------------------------------------------
         # Create the tablespace directory and perform a backup
         storageTest()->pathCreate($oHostDbMaster->dbBasePath() . '/' . DB_PATH_PGTBLSPC);
         $oHostBackup->backup(
             'full', 'create first full backup ',
-            {strOptionalParam => '--repo1-retention-full=2 --no-' . cfgOptionName(CFGOPT_ONLINE) . ' --log-level-console=detail'},
-            false);
+            {iExpectedExitStatus => $bEncrypt ? undef : ERROR_FORMAT,
+            strOptionalParam => '--repo1-retention-full=2 --no-' . cfgOptionName(CFGOPT_ONLINE)}, false);
 
         # Stanza Create fails when not using force - no backup.info but backup exists
         #--------------------------------------------------------------------------------------------------------------------------
-        if ($bRepoEncrypt)
+        if ($bEncrypt)
         {
             forceStorageMove(storageRepo(), $strBackupInfoFile, $strBackupInfoOldFile, {bRecurse => false});
             forceStorageMove(storageRepo(), $strBackupInfoCopyFile, $strBackupInfoCopyOldFile, {bRecurse => false});
@@ -324,33 +343,47 @@ sub run
         #--------------------------------------------------------------------------------------------------------------------------
         $oHostBackup->stanzaCreate('use force to recreate the stanza from backups',
             {strOptionalParam => '--no-' . cfgOptionName(CFGOPT_ONLINE) . ' --' . cfgOptionName(CFGOPT_FORCE),
-                iExpectedExitStatus => $bRepoEncrypt ? ERROR_FILE_MISSING : undef});
+                iExpectedExitStatus => $bEncrypt ? ERROR_FILE_MISSING : undef});
 
-        # Encrypted info files could not be reconstructed above so just copy them back
-        if ($bRepoEncrypt)
-        {
-            forceStorageMove(storageRepo(), $strBackupInfoOldFile, $strBackupInfoFile, {bRecurse => false});
-            forceStorageMove(storageRepo(), $strBackupInfoCopyOldFile, $strBackupInfoCopyFile, {bRecurse => false});
-        }
-
-        # Test archive dir version XX.Y-Z ensuring sort order of db ids is reconstructed correctly from the directory db-id value
-        #--------------------------------------------------------------------------------------------------------------------------
-        # Create the 10.0-3 directory and copy a WAL file to it (something that has a different system id)
-        forceStorageMode(storageRepo(), STORAGE_REPO_ARCHIVE, '770');
-        storageRepo()->pathCreate(STORAGE_REPO_ARCHIVE . '/10.0-3/0000000100000001', {bCreateParent => true});
-        storageRepo()->put(
-            storageRepo()->openWrite(
-                STORAGE_REPO_ARCHIVE . '/10.0-3/0000000100000001/000000010000000100000001',
-                {strCipherPass => $oHostBackup->cipherPassArchive()}),
-            $self->walGenerateContent(PG_VERSION_94));
-        forceStorageOwner(storageRepo(), STORAGE_REPO_ARCHIVE . '/10.0-3', $oHostBackup->userGet(), {bRecurse => true});
+        # Copy old backup.info files back to avoid history mismatch
+        forceStorageMove(storageRepo(), $strBackupInfoOldFile, $strBackupInfoFile, {bRecurse => false});
+        forceStorageMove(storageRepo(), $strBackupInfoCopyOldFile, $strBackupInfoCopyFile, {bRecurse => false});
 
         # Copy pg_control for 9.5
         $self->controlGenerate($oHostDbMaster->dbBasePath(), PG_VERSION_95);
         forceStorageMode(storageDb(), $oHostDbMaster->dbBasePath() . '/' . DB_FILE_PGCONTROL, '600');
 
+        # Test archive dir version XX.Y-Z ensuring sort order of db ids is reconstructed correctly from the directory db-id value.
+        # Not testing with encryption since unnecessary and copying files around in this case is burdensome.
+        #--------------------------------------------------------------------------------------------------------------------------
+        if (!$bEncrypt)
+        {
+            # Create the 10-3 directory and copy a WAL file to it (with a different system id than what it will be upgraded to)
+            forceStorageMode(storageRepo(), STORAGE_REPO_ARCHIVE, '770');
+            storageRepo()->pathCreate(STORAGE_REPO_ARCHIVE . '/10-3/0000000100000001', {bCreateParent => true});
+            storageRepo()->put(
+                storageRepo()->openWrite(
+                    STORAGE_REPO_ARCHIVE . '/10-3/0000000100000001/000000010000000100000001',
+                    {strCipherPass => $oHostBackup->cipherPassArchive()}),
+                $self->walGenerateContent(PG_VERSION_10));
+            forceStorageOwner(storageRepo(), STORAGE_REPO_ARCHIVE . '/10-3', $oHostBackup->userGet(), {bRecurse => true});
+
+            # Make sure the archive.info has the history in the db-id order such that 10 is before 9.5.
+            $oHostBackup->stanzaUpgrade(
+                'successfully upgrade with XX.Y-Z',
+                {strOptionalParam => '--no-' . cfgOptionName(CFGOPT_ONLINE)});
+
+            # Remove the 10-3 directory and copy old archive.info file back. Recreate so archive.info and backup.info files match.
+            forceStorageRemove(storageRepo(), STORAGE_REPO_ARCHIVE . '/10-3', {bRecurse => true});
+            forceStorageRemove(storageRepo(), $strArchiveInfoFile);
+            forceStorageRemove(storageRepo(), $strArchiveInfoCopyFile);
+            forceStorageMove(storageRepo(), $strArchiveInfoOldFile, $strArchiveInfoFile, {bRecurse => false});
+            forceStorageMove(storageRepo(), $strArchiveInfoCopyOldFile, $strArchiveInfoCopyFile, {bRecurse => false});
+        }
+
         $oHostBackup->stanzaUpgrade(
-            'successfully upgrade with XX.Y-Z', {strOptionalParam => '--no-' . cfgOptionName(CFGOPT_ONLINE)});
+            'successfully upgrade - no info file mismatch',
+            {strOptionalParam => '--no-' . cfgOptionName(CFGOPT_ONLINE)});
 
         # Push a WAL and create a backup in the new DB to confirm diff changed to full and info command displays the JSON correctly
         #--------------------------------------------------------------------------------------------------------------------------
@@ -359,7 +392,7 @@ sub run
 
         # Test backup is changed from type=DIFF to FULL (WARN message displayed)
         my $oExecuteBackup = $oHostBackup->backupBegin('diff', 'diff changed to full backup',
-            {strOptionalParam => '--repo1-retention-full=2 --no-' . cfgOptionName(CFGOPT_ONLINE) . ' --log-level-console=detail'});
+            {strOptionalParam => '--repo1-retention-full=2 --no-' . cfgOptionName(CFGOPT_ONLINE)});
         $oHostBackup->backupEnd('full', $oExecuteBackup, undef, false);
 
         # Confirm info command displays the JSON correctly
@@ -372,7 +405,6 @@ sub run
         $oHostBackup->stop({strStanza => $self->stanza()});
         $oHostBackup->stanzaDelete('successfully delete the stanza');
     }
-    }
 }
 
 1;
