@@ -98,8 +98,7 @@ sub run
         $self->optionTestSetBool(CFGOPT_ONLINE, true);
         $self->configTestLoad(CFGCMD_STANZA_CREATE);
 
-        $self->testException(sub {(new pgBackRest::Stanza())}, ERROR_DB_CONNECT,
-            "could not connect to server: No such file or directory\n");
+        $self->testException(sub {(new pgBackRest::Stanza())}, ERROR_DB_CONNECT, "unable to connect to.*");
 
         $self->optionTestSetBool(CFGOPT_ONLINE, false);
     }
@@ -280,11 +279,11 @@ sub run
         # Change the permissions on the archived file so reconstruction fails
         executeTest('sudo chmod 220 ' . $strArchivedFile);
         $self->testException(sub {(new pgBackRest::Stanza())->stanzaCreate()}, ERROR_FILE_OPEN,
-            "unable to open '" . $strArchivedFile . "': Permission denied");
+            "unable to open file '${strArchivedFile}' for read");
         executeTest('sudo chmod 644 ' . $strArchivedFile);
 
         # Clear the cached repo settings and change repo settings to encrypted
-        storageRepoCacheClear($self->stanza());
+        storageRepoCacheClear();
         $self->optionTestSet(CFGOPT_REPO_CIPHER_TYPE, CFGOPTVAL_REPO_CIPHER_TYPE_AES_256_CBC);
         $self->optionTestSet(CFGOPT_REPO_CIPHER_PASS, 'x');
         $self->configTestLoad(CFGCMD_STANZA_CREATE);
@@ -312,7 +311,10 @@ sub run
         #---------------------------------------------------------------------------------------------------------------------------
         storageRepo()->pathCreate(STORAGE_REPO_ARCHIVE . "/" . PG_VERSION_94 . "-1");
         storageRepo()->pathCreate(STORAGE_REPO_ARCHIVE . "/" . PG_VERSION_94 . "-1/0000000100000001");
-        storageRepo()->put($strArchivedFile, $tUnencryptedArchiveContent);
+        storageRepo()->put(
+            storageRepo()->openWrite(
+                $strArchivedFile, {strCipherPass => new pgBackRest::Archive::Info($self->{strArchivePath})->cipherPassSub()}),
+            $tUnencryptedArchiveContent);
         storageRepo()->pathCreate($strBackupPath); # Empty backup path - no backup in progress
 
         # Confirm encrypted and create the stanza with force
@@ -402,7 +404,7 @@ sub run
         # Change repo encryption settings to unencrypted - stanza create is not allowed even with force
         #---------------------------------------------------------------------------------------------------------------------------
         # Clear the cached repo settings and change repo settings to unencrypted
-        storageRepoCacheClear($self->stanza());
+        storageRepoCacheClear();
         $self->optionTestClear(CFGOPT_REPO_CIPHER_TYPE);
         $self->optionTestClear(CFGOPT_REPO_CIPHER_PASS);
         $self->configTestLoad(CFGCMD_STANZA_CREATE);
@@ -477,7 +479,7 @@ sub run
         #---------------------------------------------------------------------------------------------------------------------------
         executeTest('sudo chmod 220 ' . $self->{strArchivePath});
         $self->testException(sub {$oStanza->infoObject(STORAGE_REPO_ARCHIVE, $self->{strArchivePath})}, ERROR_FILE_OPEN,
-            "unable to open '" . $self->{strArchivePath} . "/archive.info': Permission denied");
+            "unable to open file '" . $self->{strArchivePath} . "/archive.info' for read");
         executeTest('sudo chmod 640 ' . $self->{strArchivePath});
 
         # Reset force option --------
@@ -492,8 +494,8 @@ sub run
         forceStorageRemove(storageRepo(), storageRepo()->pathGet(STORAGE_REPO_BACKUP . qw{/} . FILE_BACKUP_INFO . INI_COPY_EXT));
         executeTest('sudo chmod 220 ' . storageRepo()->pathGet(STORAGE_REPO_BACKUP . qw{/} . FILE_BACKUP_INFO));
         $self->testException(sub {$oStanza->infoObject(STORAGE_REPO_BACKUP, $self->{strBackupPath})}, ERROR_FILE_OPEN,
-            "unable to open '" . storageRepo()->pathGet(STORAGE_REPO_BACKUP . qw{/} . FILE_BACKUP_INFO) .
-            "': Permission denied");
+            "unable to open file '" . storageRepo()->pathGet(STORAGE_REPO_BACKUP . qw{/} . FILE_BACKUP_INFO) .
+            "' for read");
         executeTest('sudo chmod 640 ' . storageRepo()->pathGet(STORAGE_REPO_BACKUP . qw{/} . FILE_BACKUP_INFO));
     }
 
@@ -520,7 +522,7 @@ sub run
         # Attempt to change the encryption settings
         #---------------------------------------------------------------------------------------------------------------------------
         # Clear the cached repo settings and change repo settings to encrypted
-        storageRepoCacheClear($self->stanza());
+        storageRepoCacheClear();
         $self->optionTestSet(CFGOPT_REPO_CIPHER_TYPE, CFGOPTVAL_REPO_CIPHER_TYPE_AES_256_CBC);
         $self->optionTestSet(CFGOPT_REPO_CIPHER_PASS, 'x');
         $self->configTestLoad(CFGCMD_STANZA_UPGRADE);
@@ -543,7 +545,7 @@ sub run
         $oBackupInfo->create(PG_VERSION_93, $self->dbSysId(PG_VERSION_93), '937', '201306121', true);
 
          # Attempt to upgrade with a different passphrase
-        storageRepoCacheClear($self->stanza());
+        storageRepoCacheClear();
         $self->optionTestSet(CFGOPT_REPO_CIPHER_TYPE, CFGOPTVAL_REPO_CIPHER_TYPE_AES_256_CBC);
         $self->optionTestSet(CFGOPT_REPO_CIPHER_PASS, 'y');
         $self->configTestLoad(CFGCMD_STANZA_UPGRADE);
@@ -552,7 +554,7 @@ sub run
             "unable to parse '" . $self->{strArchivePath} . "/archive.info'" .
             "\nHINT: Is or was the repo encrypted?");
 
-        storageRepoCacheClear($self->stanza());
+        storageRepoCacheClear();
         $self->optionTestSet(CFGOPT_REPO_CIPHER_TYPE, CFGOPTVAL_REPO_CIPHER_TYPE_AES_256_CBC);
         $self->optionTestSet(CFGOPT_REPO_CIPHER_PASS, 'x');
         $self->configTestLoad(CFGCMD_STANZA_UPGRADE);
@@ -592,7 +594,7 @@ sub run
             '    encrypted archive and backup info files upgraded');
 
         # Clear configuration
-        storageRepoCacheClear($self->stanza());
+        storageRepoCacheClear();
         $self->optionTestClear(CFGOPT_REPO_CIPHER_TYPE);
         $self->optionTestClear(CFGOPT_REPO_CIPHER_PASS);
     }
@@ -762,15 +764,15 @@ sub run
         executeTest("sudo chown 7777 " . $self->{strArchivePath});
 
         lockStop();
-        $self->testException(sub {$oStanza->stanzaDelete()}, ERROR_FILE_OPEN,
-            "unable to remove file '" . $self->{strArchivePath} . "/" . ARCHIVE_INFO_FILE . "': Permission denied");
+        $self->testException(sub {$oStanza->stanzaDelete()}, ERROR_FILE_REMOVE,
+            "unable to remove '" . $self->{strArchivePath} . "/" . ARCHIVE_INFO_FILE . "'");
 
         # Remove the repo
         executeTest("sudo rm -rf " . $self->{strArchivePath});
 
         # Clear the cached repo settings and change repo settings to encrypted
         #---------------------------------------------------------------------------------------------------------------------------
-        storageRepoCacheClear($self->stanza());
+        storageRepoCacheClear();
         $self->optionTestSet(CFGOPT_REPO_CIPHER_TYPE, CFGOPTVAL_REPO_CIPHER_TYPE_AES_256_CBC);
         $self->optionTestSet(CFGOPT_REPO_CIPHER_PASS, 'x');
         $self->configTestLoad(CFGCMD_STANZA_CREATE);
