@@ -6,6 +6,7 @@ Harness for Loading Test Configurations
 
 #include "config/load.h"
 #include "config/parse.h"
+#include "storage/helper.h"
 
 /***********************************************************************************************************************************
 Load a test configuration without any side effects
@@ -20,6 +21,9 @@ harnessCfgLoad(unsigned int argListSize, const char *argList[])
         FUNCTION_HARNESS_PARAM(CHARPY, argList);
     FUNCTION_HARNESS_END();
 
+    // Free objects in storage helper
+    storageHelperFree();
+
     configParse(argListSize, argList, false);
     cfgLoadUpdateOption();
 
