@@ -20,6 +20,7 @@ Expose log internal data for unit testing/debugging
 extern LogLevel logLevelFile;
 extern int logHandleFile;
 extern bool logFileBanner;
+extern void logAnySet(void);
 
 /***********************************************************************************************************************************
 Default log level for testing
@@ -66,11 +67,12 @@ harnessLogInit(void)
 {
     FUNCTION_HARNESS_VOID();
 
-    logInit(logLevelTestDefault, logLevelOff, logLevelInfo, false);
+    logInit(logLevelTestDefault, logLevelOff, logLevelInfo, false, 99);
     logFileBanner = true;
 
     snprintf(logFile, sizeof(logFile), "%s/expect.log", testExpectPath());
     logHandleFile = harnessLogOpen(logFile, O_WRONLY | O_CREAT | O_TRUNC, 0640);
+    logAnySet();
 
     FUNCTION_HARNESS_RESULT_VOID();
 }
@@ -83,7 +85,7 @@ This is info by default but it can sometimes be useful to set the log level to s
 void
 harnessLogLevelSet(LogLevel logLevel)
 {
-    logInit(logLevelTestDefault, logLevelOff, logLevel, false);
+    logInit(logLevelTestDefault, logLevelOff, logLevel, false, 99);
 }
 
 /***********************************************************************************************************************************
@@ -94,7 +96,7 @@ Set back to info
 void
 harnessLogLevelReset(void)
 {
-    logInit(logLevelTestDefault, logLevelOff, logLevelInfo, false);
+    logInit(logLevelTestDefault, logLevelOff, logLevelInfo, false, 99);
 }
 
 /***********************************************************************************************************************************
