@@ -1,9 +1,10 @@
 /***********************************************************************************************************************************
 Test Info Command
 ***********************************************************************************************************************************/
-#include "storage/driver/posix/storage.h"
+#include "storage/posix/storage.h"
 
 #include "common/harnessConfig.h"
+#include "common/harnessInfo.h"
 
 /***********************************************************************************************************************************
 Test Run
@@ -25,7 +26,7 @@ testRun(void)
     {
         StringList *argList = strLstNew();
         strLstAddZ(argList, "pgbackrest");
-        strLstAdd(argList, strNewFmt("--repo-path=%s", strPtr(repoPath)));
+        strLstAdd(argList, strNewFmt("--repo-path=%s/", strPtr(repoPath)));
         strLstAddZ(argList, "info");
         StringList *argListText = strLstDup(argList);
 
@@ -71,13 +72,8 @@ testRun(void)
         //--------------------------------------------------------------------------------------------------------------------------
         String *content = strNew
         (
-            "[backrest]\n"
-            "backrest-checksum=\"80798255547bafa3eee805ab0fccaa1da03cab0e\"\n"
-            "backrest-format=5\n"
-            "backrest-version=\"2.04\"\n"
-            "\n"
             "[cipher]\n"
-            "cipher-pass=12345\n"
+            "cipher-pass=\"12345\"\n"
             "\n"
             "[db]\n"
             "db-catalog-version=201409291\n"
@@ -95,28 +91,25 @@ testRun(void)
 
         TEST_RESULT_VOID(
             storagePutNP(storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/backup.info", strPtr(backupStanza1Path))),
-                bufNewStr(content)), "put backup info to file");
+                harnessInfoChecksum(content)), "put backup info to file");
 
         TEST_ERROR_FMT(infoRender(), FileMissingError,
             "unable to load info file '%s/archive.info' or '%s/archive.info.copy':\n"
-            "FileMissingError: unable to open '%s/archive.info' for read: [2] No such file or directory\n"
-            "FileMissingError: unable to open '%s/archive.info.copy' for read: [2] No such file or directory\n"
+            "FileMissingError: " STORAGE_ERROR_READ_MISSING "\n"
+            "FileMissingError: " STORAGE_ERROR_READ_MISSING "\n"
             "HINT: archive.info cannot be opened but is required to push/get WAL segments.\n"
             "HINT: is archive_command configured correctly in postgresql.conf?\n"
             "HINT: has a stanza-create been performed?\n"
             "HINT: use --no-archive-check to disable archive checks during backup if you have an alternate archiving scheme.",
-            strPtr(archiveStanza1Path), strPtr(archiveStanza1Path), strPtr(archiveStanza1Path), strPtr(archiveStanza1Path));
+            strPtr(archiveStanza1Path), strPtr(archiveStanza1Path),
+            strPtr(strNewFmt("%s/archive.info", strPtr(archiveStanza1Path))),
+            strPtr(strNewFmt("%s/archive.info.copy", strPtr(archiveStanza1Path))));
 
         // backup.info/archive.info files exist, mismatched db ids, no backup:current section so no valid backups
         // Only the current db information from the db:history will be processed.
         //--------------------------------------------------------------------------------------------------------------------------
         content = strNew
         (
-            "[backrest]\n"
-            "backrest-checksum=\"0da11608456bae64c42cc1dc8df4ae79b953d597\"\n"
-            "backrest-format=5\n"
-            "backrest-version=\"2.04\"\n"
-            "\n"
             "[db]\n"
             "db-id=1\n"
             "db-system-id=6569239123849665679\n"
@@ -130,7 +123,7 @@ testRun(void)
 
         TEST_RESULT_VOID(
             storagePutNP(storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/archive.info", strPtr(archiveStanza1Path))),
-                bufNewStr(content)), "put archive info to file");
+                harnessInfoChecksum(content)), "put archive info to file");
 
         // archive section will cross reference backup db-id 2 to archive db-id 3 but db section will only use the db-ids from
         // backup.info
@@ -179,6 +172,30 @@ testRun(void)
             "        wal archive min/max (9.4-3): none present\n",
             "text - single stanza, no valid backups");
 
+        // Add WAL segments
+        //--------------------------------------------------------------------------------------------------------------------------
+        String *archiveDb3 = strNewFmt("%s/9.4-3/0000000100000000", strPtr(archiveStanza1Path));
+        TEST_RESULT_VOID(storagePathCreateNP(storageLocalWrite(), archiveDb3), "create db3 archive WAL1 directory");
+
+        String *archiveDb3Wal = strNewFmt(
+            "%s/000000010000000000000004-47dff2b7552a9d66e4bae1a762488a6885e7082c.gz", strPtr(archiveDb3));
+        TEST_RESULT_VOID(storagePutNP(storageNewWriteNP(storageLocalWrite(), archiveDb3Wal), bufNew(0)), "touch WAL3 file");
+
+        StringList *argList2 = strLstDup(argListText);
+        strLstAddZ(argList2, "--stanza=stanza1");
+        harnessCfgLoad(strLstSize(argList2), strLstPtr(argList2));
+
+        TEST_RESULT_STR(strPtr(infoRender()),
+            "stanza: stanza1\n"
+            "    status: error (no valid backups)\n"
+            "    cipher: aes-256-cbc\n"
+            "\n"
+            "    db (current)\n"
+            "        wal archive min/max (9.4-3): 000000010000000000000004/000000010000000000000004\n",
+            "text - single stanza, one wal segment");
+
+        TEST_RESULT_VOID(storageRemoveP(storageLocalWrite(), archiveDb3Wal, .errorOnMissing = true), "remove WAL file");
+
         // Coverage for stanzaStatus branches
         //--------------------------------------------------------------------------------------------------------------------------
         String *archiveDb1_1 = strNewFmt("%s/9.4-1/0000000100000000", strPtr(archiveStanza1Path));
@@ -199,17 +216,9 @@ testRun(void)
         String *archiveDb1_3 = strNewFmt("%s/9.4-1/0000000300000000", strPtr(archiveStanza1Path));
         TEST_RESULT_VOID(storagePathCreateNP(storageLocalWrite(), archiveDb1_3), "create db1 archive WAL3 directory");
 
-        String *archiveDb3 = strNewFmt("%s/9.4-3/0000000100000000", strPtr(archiveStanza1Path));
-        TEST_RESULT_VOID(storagePathCreateNP(storageLocalWrite(), archiveDb3), "create db3 archive WAL1 directory");
-
         harnessCfgLoad(strLstSize(argList), strLstPtr(argList));
         content = strNew
         (
-            "[backrest]\n"
-            "backrest-checksum=\"2edbac400200cc0bd559951f1ee166de5c6f5f49\"\n"
-            "backrest-format=5\n"
-            "backrest-version=\"2.04\"\n"
-            "\n"
             "[db]\n"
             "db-catalog-version=201409291\n"
             "db-control-version=942\n"
@@ -237,7 +246,7 @@ testRun(void)
 
         TEST_RESULT_VOID(
             storagePutNP(storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/backup.info", strPtr(backupStanza1Path))),
-                bufNewStr(content)), "put backup info to file");
+                harnessInfoChecksum(content)), "put backup info to file");
 
         TEST_RESULT_STR(strPtr(infoRender()),
             "[\n"
@@ -340,11 +349,6 @@ testRun(void)
         //--------------------------------------------------------------------------------------------------------------------------
         content = strNew
         (
-            "[backrest]\n"
-            "backrest-checksum=\"075a202d42c3b6a0257da5f73a68fa77b342f777\"\n"
-            "backrest-format=5\n"
-            "backrest-version=\"2.08dev\"\n"
-            "\n"
             "[db]\n"
             "db-id=2\n"
             "db-system-id=6626363367545678089\n"
@@ -357,15 +361,10 @@ testRun(void)
 
         TEST_RESULT_VOID(
             storagePutNP(storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/archive.info", strPtr(archiveStanza1Path))),
-                bufNewStr(content)), "put archive info to file - stanza1");
+                harnessInfoChecksum(content)), "put archive info to file - stanza1");
 
         content = strNew
         (
-            "[backrest]\n"
-            "backrest-checksum=\"b50db7cf8f659ac15a0c7a2f45a0813f46a68c6b\"\n"
-            "backrest-format=5\n"
-            "backrest-version=\"2.08dev\"\n"
-            "\n"
             "[backup:current]\n"
             "20181119-152138F={"
             "\"backrest-format\":5,\"backrest-version\":\"2.08dev\","
@@ -409,7 +408,7 @@ testRun(void)
 
         TEST_RESULT_VOID(
             storagePutNP(storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/backup.info", strPtr(backupStanza1Path))),
-                bufNewStr(content)), "put backup info to file - stanza1");
+                harnessInfoChecksum(content)), "put backup info to file - stanza1");
 
         String *archiveStanza2Path = strNewFmt("%s/stanza2", strPtr(archivePath));
         String *backupStanza2Path = strNewFmt("%s/stanza2", strPtr(backupPath));
@@ -418,11 +417,6 @@ testRun(void)
 
         content = strNew
         (
-            "[backrest]\n"
-            "backrest-checksum=\"6779d476833114925a73e058ef9ff04e5a8c7bd2\"\n"
-            "backrest-format=5\n"
-            "backrest-version=\"2.08dev\"\n"
-            "\n"
             "[db]\n"
             "db-id=1\n"
             "db-system-id=6625633699176220261\n"
@@ -434,15 +428,10 @@ testRun(void)
 
         TEST_RESULT_VOID(
             storagePutNP(storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/archive.info", strPtr(archiveStanza2Path))),
-                bufNewStr(content)), "put archive info to file - stanza2");
+                harnessInfoChecksum(content)), "put archive info to file - stanza2");
 
         content = strNew
         (
-            "[backrest]\n"
-            "backrest-checksum=\"2393c52cb48aff2d6c6e87e21a34a3e28200f42e\"\n"
-            "backrest-format=5\n"
-            "backrest-version=\"2.08dev\"\n"
-            "\n"
             "[db]\n"
             "db-catalog-version=201409291\n"
             "db-control-version=942\n"
@@ -457,7 +446,7 @@ testRun(void)
 
         TEST_RESULT_VOID(
             storagePutNP(storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/backup.info", strPtr(backupStanza2Path))),
-                bufNewStr(content)), "put backup info to file - stanza2");
+                harnessInfoChecksum(content)), "put backup info to file - stanza2");
 
         harnessCfgLoad(strLstSize(argList), strLstPtr(argList));
         TEST_RESULT_STR(strPtr(infoRender()),
@@ -664,7 +653,7 @@ testRun(void)
 
         // Stanza not found
         //--------------------------------------------------------------------------------------------------------------------------
-        StringList *argList2 = strLstDup(argList);
+        argList2 = strLstDup(argList);
         strLstAddZ(argList2, "--stanza=silly");
         harnessCfgLoad(strLstSize(argList2), strLstPtr(argList2));
         TEST_RESULT_STR(strPtr(infoRender()),
@@ -744,7 +733,7 @@ testRun(void)
 
         TEST_RESULT_VOID(
             storagePutNP(storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/pgbackrest.conf", testPath())),
-                bufNewStr(content)), "put pgbackrest.conf file");
+                BUFSTR(content)), "put pgbackrest.conf file");
         strLstAddZ(argListText, "--repo-cipher-type=aes-256-cbc");
         strLstAdd(argListText, strNewFmt("--config=%s/pgbackrest.conf", testPath()));
         harnessCfgLoad(strLstSize(argListText), strLstPtr(argListText));
@@ -753,11 +742,12 @@ testRun(void)
             "unable to load info file '%s/backup.info' or '%s/backup.info.copy':\n"
             "CryptoError: '%s/backup.info' cipher header invalid\n"
             "HINT: Is or was the repo encrypted?\n"
-            "FileMissingError: unable to open '%s/backup.info.copy' for read: [2] No such file or directory\n"
+            "FileMissingError: " STORAGE_ERROR_READ_MISSING "\n"
             "HINT: backup.info cannot be opened and is required to perform a backup.\n"
             "HINT: has a stanza-create been performed?\n"
-            "HINT: use option --stanza if encryption settings are different for the stanza than the global settings"
-            ,strPtr(backupStanza2Path), strPtr(backupStanza2Path), strPtr(backupStanza2Path), strPtr(backupStanza2Path));
+            "HINT: use option --stanza if encryption settings are different for the stanza than the global settings",
+            strPtr(backupStanza2Path), strPtr(backupStanza2Path), strPtr(backupStanza2Path),
+            strPtr(strNewFmt("%s/backup.info.copy", strPtr(backupStanza2Path))));
     }
 
     //******************************************************************************************************************************
@@ -766,38 +756,38 @@ testRun(void)
         // These tests cover branches not covered in other tests
         KeyValue *stanzaInfo = kvNew();
         VariantList *dbSection = varLstNew();
-        Variant *pgInfo = varNewKv();
-        kvPut(varKv(pgInfo), varNewStr(DB_KEY_ID_STR), varNewUInt64(1));
-        kvPut(varKv(pgInfo), varNewStr(DB_KEY_SYSTEM_ID_STR), varNewUInt64(6625633699176220261));
-        kvPut(varKv(pgInfo), varNewStr(DB_KEY_VERSION_STR), varNewStr(pgVersionToStr(90500)));
+        Variant *pgInfo = varNewKv(kvNew());
+        kvPut(varKv(pgInfo), DB_KEY_ID_VAR, varNewUInt(1));
+        kvPut(varKv(pgInfo), DB_KEY_SYSTEM_ID_VAR, varNewUInt64(6625633699176220261));
+        kvPut(varKv(pgInfo), DB_KEY_VERSION_VAR, VARSTR(pgVersionToStr(90500)));
 
         varLstAdd(dbSection, pgInfo);
 
         // Add the database history, backup and archive sections to the stanza info
-        kvPut(stanzaInfo, varNewStr(STANZA_KEY_DB_STR), varNewVarLst(dbSection));
+        kvPut(stanzaInfo, STANZA_KEY_DB_VAR, varNewVarLst(dbSection));
 
         VariantList *backupSection = varLstNew();
-        Variant *backupInfo = varNewKv();
+        Variant *backupInfo = varNewKv(kvNew());
 
-        kvPut(varKv(backupInfo), varNewStr(BACKUP_KEY_LABEL_STR), varNewStr(strNew("20181119-152138F")));
-        kvPut(varKv(backupInfo), varNewStr(BACKUP_KEY_TYPE_STR), varNewStr(strNew("full")));
-        kvPutKv(varKv(backupInfo), varNewStr(KEY_ARCHIVE_STR));
-        KeyValue *infoInfo = kvPutKv(varKv(backupInfo), varNewStr(BACKUP_KEY_INFO_STR));
-        kvPut(infoInfo, varNewStr(KEY_SIZE_STR), varNewUInt64(0));
-        kvPut(infoInfo, varNewStr(KEY_DELTA_STR), varNewUInt64(0));
-        KeyValue *repoInfo = kvPutKv(infoInfo, varNewStr(INFO_KEY_REPOSITORY_STR));
-        kvAdd(repoInfo, varNewStr(KEY_SIZE_STR), varNewUInt64(0));
-        kvAdd(repoInfo, varNewStr(KEY_DELTA_STR), varNewUInt64(0));
-        KeyValue *databaseInfo = kvPutKv(varKv(backupInfo), varNewStr(KEY_DATABASE_STR));
-        kvAdd(databaseInfo, varNewStr(DB_KEY_ID_STR), varNewUInt64(1));
-        KeyValue *timeInfo = kvPutKv(varKv(backupInfo), varNewStr(BACKUP_KEY_TIMESTAMP_STR));
-        kvAdd(timeInfo, varNewStr(KEY_START_STR), varNewUInt64(1542383276));
-        kvAdd(timeInfo, varNewStr(KEY_STOP_STR), varNewUInt64(1542383289));
+        kvPut(varKv(backupInfo), BACKUP_KEY_LABEL_VAR, VARSTRDEF("20181119-152138F"));
+        kvPut(varKv(backupInfo), BACKUP_KEY_TYPE_VAR, VARSTRDEF("full"));
+        kvPutKv(varKv(backupInfo), KEY_ARCHIVE_VAR);
+        KeyValue *infoInfo = kvPutKv(varKv(backupInfo), BACKUP_KEY_INFO_VAR);
+        kvPut(infoInfo, KEY_SIZE_VAR, varNewUInt64(0));
+        kvPut(infoInfo, KEY_DELTA_VAR, varNewUInt64(0));
+        KeyValue *repoInfo = kvPutKv(infoInfo, INFO_KEY_REPOSITORY_VAR);
+        kvAdd(repoInfo, KEY_SIZE_VAR, varNewUInt64(0));
+        kvAdd(repoInfo, KEY_DELTA_VAR, varNewUInt64(0));
+        KeyValue *databaseInfo = kvPutKv(varKv(backupInfo), KEY_DATABASE_VAR);
+        kvAdd(databaseInfo, DB_KEY_ID_VAR, varNewUInt(1));
+        KeyValue *timeInfo = kvPutKv(varKv(backupInfo), BACKUP_KEY_TIMESTAMP_VAR);
+        kvAdd(timeInfo, KEY_START_VAR, varNewUInt64(1542383276));
+        kvAdd(timeInfo, KEY_STOP_VAR, varNewUInt64(1542383289));
 
         varLstAdd(backupSection, backupInfo);
 
-        kvPut(stanzaInfo, varNewStr(STANZA_KEY_BACKUP_STR), varNewVarLst(backupSection));
-        kvPut(stanzaInfo, varNewStr(KEY_ARCHIVE_STR), varNewVarLst(varLstNew()));
+        kvPut(stanzaInfo, STANZA_KEY_BACKUP_VAR, varNewVarLst(backupSection));
+        kvPut(stanzaInfo, KEY_ARCHIVE_VAR, varNewVarLst(varLstNew()));
 
         String *result = strNew("");
         formatTextDb(stanzaInfo, result);
@@ -829,8 +819,7 @@ testRun(void)
         int stdoutSave = dup(STDOUT_FILENO);
         String *stdoutFile = strNewFmt("%s/stdout.info", testPath());
 
-        if (freopen(strPtr(stdoutFile), "w", stdout) == NULL)                                       // {uncoverable - does not fail}
-            THROW_SYS_ERROR(FileWriteError, "unable to reopen stdout");                             // {uncoverable+}
+        THROW_ON_SYS_ERROR(freopen(strPtr(stdoutFile), "w", stdout) == NULL, FileWriteError, "unable to reopen stdout");
 
         // Not in a test wrapper to avoid writing to stdout
         cmdInfo();
@@ -838,8 +827,8 @@ testRun(void)
         // Restore normal stdout
         dup2(stdoutSave, STDOUT_FILENO);
 
-        Storage *storage = storageDriverPosixInterface(
-            storageDriverPosixNew(strNew(testPath()), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, false, NULL));
+        Storage *storage = storagePosixNew(
+            strNew(testPath()), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, false, NULL);
         TEST_RESULT_STR(
             strPtr(strNewBuf(storageGetNP(storageNewReadNP(storage, stdoutFile)))), "No stanzas exist in the repository.\n",
             "    check text");
