@@ -1,6 +1,7 @@
 /***********************************************************************************************************************************
 Test Debug Macros and Routines when Disabled
 ***********************************************************************************************************************************/
+#include "common/macro.h"
 
 /***********************************************************************************************************************************
 Test Run
@@ -25,7 +26,7 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("DEBUG_UNIT_EXTERN"))
     {
-        const char *debugUnitExtern = MACRO_TO_STR(DEBUG_UNIT_EXTERN);
+        const char *debugUnitExtern = STRINGIFY(DEBUG_UNIT_EXTERN);
         TEST_RESULT_STR(debugUnitExtern, "static", "DEBUG_UNIT_EXTERN is static");
     }
 
