@@ -12,13 +12,13 @@ testRun(void)
     FUNCTION_HARNESS_VOID();
 
     // *****************************************************************************************************************************
-    if (testBegin("forkAndDetach()"))
+    if (testBegin("forkSafe() and forkAndDetach()"))
     {
         int sessionId = getsid(0);
 
         HARNESS_FORK_BEGIN()
         {
-            HARNESS_FORK_CHILD()
+            HARNESS_FORK_CHILD_BEGIN(0, false)
             {
                 char buffer[1024];
 
@@ -30,6 +30,7 @@ testRun(void)
                 TEST_RESULT_INT(write(STDOUT_FILENO, buffer, strlen(buffer)), -1, "write to stdout fails");
                 TEST_RESULT_INT(write(STDERR_FILENO, buffer, strlen(buffer)), -1, "write to stderr fails");
             }
+            HARNESS_FORK_CHILD_END();
         }
         HARNESS_FORK_END();
     }
