@@ -142,6 +142,26 @@ testHttpServer(void)
             "Connection:ack\r\n"
             "\r\n");
 
+        // Head request with content-length but no content
+        harnessTlsServerExpect(
+            "HEAD / HTTP/1.1\r\n"
+            "\r\n");
+
+        harnessTlsServerReply(
+            "HTTP/1.1 200 OK\r\n"
+            "content-length:380\r\n"
+            "\r\n");
+
+        // Head request with transfer encoding but no content
+        harnessTlsServerExpect(
+            "HEAD / HTTP/1.1\r\n"
+            "\r\n");
+
+        harnessTlsServerReply(
+            "HTTP/1.1 200 OK\r\n"
+            "Transfer-Encoding: chunked\r\n"
+            "\r\n");
+
         // Error with content length 0 (with a few slow down errors)
         harnessTlsServerExpect(
             "GET / HTTP/1.1\r\n"
@@ -196,7 +216,9 @@ testHttpServer(void)
 
         harnessTlsServerExpect(
             "GET /path/file%201.txt HTTP/1.1\r\n"
-            "\r\n");
+            "content-length:30\r\n"
+            "\r\n"
+            "012345678901234567890123456789");
 
         harnessTlsServerReply(
             "HTTP/1.1 200 OK\r\n"
@@ -317,7 +339,6 @@ testRun(void)
         TEST_RESULT_STR(strPtr(httpHeaderToLog(header)), "{key1: 'value1', key2: 'value2a'}", "log output");
 
         TEST_RESULT_VOID(httpHeaderFree(header), "free header");
-        TEST_RESULT_VOID(httpHeaderFree(NULL), "free null header");
 
         // Redacted headers
         // -------------------------------------------------------------------------------------------------------------------------
@@ -326,6 +347,16 @@ testRun(void)
         httpHeaderAdd(header, strNew("public"), strNew("public-value"));
 
         TEST_RESULT_STR(strPtr(httpHeaderToLog(header)), "{public: 'public-value', secret: <redacted>}", "log output");
+
+        // Duplicate
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_RESULT_STR(
+            strPtr(httpHeaderToLog(httpHeaderDup(header, NULL))),
+            "{public: 'public-value', secret: <redacted>}", "dup and keep redactions");
+        TEST_RESULT_STR(
+            strPtr(httpHeaderToLog(httpHeaderDup(header, strLstAddZ(strLstNew(), "public")))),
+            "{public: <redacted>, secret: 'secret-value'}", "dup and change redactions");
+        TEST_RESULT_PTR(httpHeaderDup(NULL, NULL), NULL, "dup null http header");
     }
 
     // *****************************************************************************************************************************
@@ -361,7 +392,6 @@ testRun(void)
         TEST_RESULT_STR(strPtr(httpQueryToLog(query)), "{key1: 'value 1?', key2: 'value2a'}", "log output");
 
         TEST_RESULT_VOID(httpQueryFree(query), "free query");
-        TEST_RESULT_VOID(httpQueryFree(NULL), "free null query");
     }
 
     // *****************************************************************************************************************************
@@ -370,10 +400,14 @@ testRun(void)
         HttpClient *client = NULL;
         ioBufferSizeSet(35);
 
+        // Reset statistics
+        httpClientStatLocal = (HttpClientStat){0};
+        TEST_RESULT_STR(httpClientStatStr(), NULL, "no stats yet");
+
         TEST_ASSIGN(client, httpClientNew(strNew("localhost"), TLS_TEST_PORT, 500, true, NULL, NULL), "new client");
 
         TEST_ERROR(
-            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), HostConnectError,
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), HostConnectError,
             "unable to connect to 'localhost:9443': [111] Connection refused");
 
         // Start http test server
@@ -384,41 +418,41 @@ testRun(void)
         client->timeout = 0;
 
         TEST_ERROR(
-            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), FileReadError,
-            "unable to read data from '" TLS_TEST_HOST ":9443' after 500ms");
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), FileReadError,
+            "timeout after 500ms waiting for read from '" TLS_TEST_HOST ":9443'");
 
         // Test invalid http version
         TEST_ERROR(
-            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), FormatError,
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), FormatError,
             "http version of response 'HTTP/1.0 200 OK' must be HTTP/1.1");
 
         // Test no space in status
         TEST_ERROR(
-            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), FormatError,
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), FormatError,
             "response status '200OK' must have a space");
 
         // Test unexpected end of headers
         TEST_ERROR(
-            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), FileReadError,
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), FileReadError,
             "unexpected eof while reading line");
 
         // Test missing colon in header
         TEST_ERROR(
-            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), FormatError,
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), FormatError,
             "header 'header-value' missing colon");
 
         // Test invalid transfer encoding
         TEST_ERROR(
-            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), FormatError,
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), FormatError,
             "only 'chunked' is supported for 'transfer-encoding' header");
 
         // Test content length and transfer encoding both set
         TEST_ERROR(
-            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), FormatError,
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), FormatError,
             "'transfer-encoding' and 'content-length' headers are both set");
 
         // Test 5xx error with no retry
-        TEST_ERROR(httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), ServiceError, "[503] Slow Down");
+        TEST_ERROR(httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), ServiceError, "[503] Slow Down");
 
         // Request with no content
         client->timeout = 500;
@@ -431,15 +465,40 @@ testRun(void)
         httpQueryAdd(query, strNew("type"), strNew("test"));
 
         TEST_RESULT_VOID(
-            httpClientRequest(client, strNew("GET"), strNew("/"), query, headerRequest, false), "request with no content");
+            httpClientRequest(client, strNew("GET"), strNew("/"), query, headerRequest, NULL, false), "request with no content");
         TEST_RESULT_UINT(httpClientResponseCode(client), 200, "    check response code");
         TEST_RESULT_STR(strPtr(httpClientResponseMessage(client)), "OK", "    check response message");
+        TEST_RESULT_UINT(httpClientEof(client), true, "    io is eof");
         TEST_RESULT_STR(
             strPtr(httpHeaderToLog(httpClientReponseHeader(client))),  "{connection: 'ack', key1: '0', key2: 'value2'}",
             "    check response headers");
 
+        // Head request with content-length but no content
+        TEST_RESULT_VOID(
+            httpClientRequest(client, strNew("HEAD"), strNew("/"), NULL, httpHeaderNew(NULL), NULL, true),
+            "head request with content-length");
+        TEST_RESULT_UINT(httpClientResponseCode(client), 200, "    check response code");
+        TEST_RESULT_STR(strPtr(httpClientResponseMessage(client)), "OK", "    check response message");
+        TEST_RESULT_BOOL(httpClientEof(client), true, "    io is eof");
+        TEST_RESULT_BOOL(httpClientBusy(client), false, "    client is not busy");
+        TEST_RESULT_STR(
+            strPtr(httpHeaderToLog(httpClientReponseHeader(client))),  "{content-length: '380'}", "    check response headers");
+
+        // Head request with transfer encoding but no content
+        TEST_RESULT_VOID(
+            httpClientRequest(client, strNew("HEAD"), strNew("/"), NULL, httpHeaderNew(NULL), NULL, true),
+            "head request with transfer encoding");
+        TEST_RESULT_UINT(httpClientResponseCode(client), 200, "    check response code");
+        TEST_RESULT_STR(strPtr(httpClientResponseMessage(client)), "OK", "    check response message");
+        TEST_RESULT_BOOL(httpClientEof(client), true, "    io is eof");
+        TEST_RESULT_BOOL(httpClientBusy(client), false, "    client is not busy");
+        TEST_RESULT_STR(
+            strPtr(httpHeaderToLog(httpClientReponseHeader(client))),  "{transfer-encoding: 'chunked'}",
+            "    check response headers");
+
         // Error with content length 0
-        TEST_RESULT_VOID(httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), "error with content length 0");
+        TEST_RESULT_VOID(
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), "error with content length 0");
         TEST_RESULT_UINT(httpClientResponseCode(client), 404, "    check response code");
         TEST_RESULT_STR(strPtr(httpClientResponseMessage(client)), "Not Found", "    check response message");
         TEST_RESULT_STR(
@@ -448,7 +507,8 @@ testRun(void)
         // Error with content
         Buffer *buffer = NULL;
 
-        TEST_ASSIGN(buffer, httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), "error with content length");
+        TEST_ASSIGN(
+            buffer, httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), "error with content length");
         TEST_RESULT_UINT(httpClientResponseCode(client), 403, "    check response code");
         TEST_RESULT_STR(strPtr(httpClientResponseMessage(client)), "Auth Error", "    check response message");
         TEST_RESULT_STR(
@@ -460,7 +520,11 @@ testRun(void)
 
         TEST_ASSIGN(
             buffer,
-            httpClientRequest(client, strNew("GET"), strNew("/path/file 1.txt"), NULL, NULL, true), "request with content length");
+            httpClientRequest(
+                client, strNew("GET"), strNew("/path/file 1.txt"), NULL,
+                httpHeaderAdd(httpHeaderNew(NULL), strNew("content-length"), strNew("30")),
+                BUFSTRDEF("012345678901234567890123456789"), true),
+            "request with content length");
         TEST_RESULT_STR(
             strPtr(httpHeaderToLog(httpClientReponseHeader(client))),  "{connection: 'close', content-length: '32'}",
             "    check response headers");
@@ -469,7 +533,7 @@ testRun(void)
 
         // Request with eof before content complete with retry
         TEST_ASSIGN(
-            buffer, httpClientRequest(client, strNew("GET"), strNew("/path/file 1.txt"), NULL, NULL, true),
+            buffer, httpClientRequest(client, strNew("GET"), strNew("/path/file 1.txt"), NULL, NULL, NULL, true),
             "request with content length retry");
         TEST_RESULT_STR(strPtr(strNewBuf(buffer)),  "01234567890123456789012345678901", "    check response");
         TEST_RESULT_UINT(httpClientRead(client, bufNew(1), true), 0, "    call internal read to check eof");
@@ -477,13 +541,15 @@ testRun(void)
         // Request with eof before content and error
         buffer = bufNew(32);
         TEST_RESULT_VOID(
-            httpClientRequest(client, strNew("GET"), strNew("/path/file 1.txt"), NULL, NULL, false),
+            httpClientRequest(client, strNew("GET"), strNew("/path/file 1.txt"), NULL, NULL, NULL, false),
             "request with content length error");
+        TEST_RESULT_BOOL(httpClientBusy(client), true, "    client is busy");
         TEST_ERROR(
             ioRead(httpClientIoRead(client), buffer), FileReadError, "unexpected EOF reading HTTP content");
 
         // Request with content using chunked encoding
-        TEST_RESULT_VOID(httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, false), "request with chunked encoding");
+        TEST_RESULT_VOID(
+            httpClientRequest(client, strNew("GET"), strNew("/"), NULL, NULL, NULL, false), "request with chunked encoding");
         TEST_RESULT_STR(
             strPtr(httpHeaderToLog(httpClientReponseHeader(client))),  "{transfer-encoding: 'chunked'}",
             "    check response headers");
@@ -492,8 +558,34 @@ testRun(void)
         TEST_RESULT_VOID(ioRead(httpClientIoRead(client), buffer),  "    read response");
         TEST_RESULT_STR(strPtr(strNewBuf(buffer)),  "01234567890123456789012345678901012", "    check response");
 
+        TEST_RESULT_BOOL(httpClientStatStr() != NULL, true, "check statistics exist");
+
         TEST_RESULT_VOID(httpClientFree(client), "free client");
-        TEST_RESULT_VOID(httpClientFree(NULL), "free null client");
+    }
+
+    // *****************************************************************************************************************************
+    if (testBegin("HttpClientCache"))
+    {
+        HttpClientCache *cache = NULL;
+        HttpClient *client1 = NULL;
+        HttpClient *client2 = NULL;
+
+        TEST_ASSIGN(cache, httpClientCacheNew(strNew("localhost"), TLS_TEST_PORT, 500, true, NULL, NULL), "new http client cache");
+        TEST_ASSIGN(client1, httpClientCacheGet(cache), "get http client");
+        TEST_RESULT_PTR(client1, *(HttpClient **)lstGet(cache->clientList, 0), "    check http client");
+        TEST_RESULT_PTR(httpClientCacheGet(cache), *(HttpClient **)lstGet(cache->clientList, 0), "    get same http client");
+
+        // Make client 1 look like it is busy
+        client1->ioRead = (IoRead *)1;
+
+        TEST_ASSIGN(client2, httpClientCacheGet(cache), "get http client");
+        TEST_RESULT_PTR(client2, *(HttpClient **)lstGet(cache->clientList, 1), "    check http client");
+        TEST_RESULT_BOOL(client1 != client2, true, "clients are not the same");
+
+        // Set back to NULL so bad things don't happen during free
+        client1->ioRead = NULL;
+
+        TEST_RESULT_VOID(httpClientCacheFree(cache), "free http client cache");
     }
 
     FUNCTION_HARNESS_RESULT_VOID();
