@@ -2,7 +2,7 @@
 Test Lock Handler
 ***********************************************************************************************************************************/
 #include "common/time.h"
-#include "storage/driver/posix/storage.h"
+#include "storage/posix/storage.h"
 
 #include "common/harnessFork.h"
 
@@ -15,8 +15,8 @@ testRun(void)
     FUNCTION_HARNESS_VOID();
 
     // Create default storage object for testing
-    Storage *storageTest = storageDriverPosixInterface(
-        storageDriverPosixNew(strNew(testPath()), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, true, NULL));
+    Storage *storageTest = storagePosixNew(
+        strNew(testPath()), STORAGE_MODE_FILE_DEFAULT, STORAGE_MODE_PATH_DEFAULT, true, NULL);
 
     // *****************************************************************************************************************************
     if (testBegin("lockAcquireFile() and lockReleaseFile()"))
@@ -82,13 +82,14 @@ testRun(void)
 
         HARNESS_FORK_BEGIN()
         {
-            HARNESS_FORK_CHILD()
+            HARNESS_FORK_CHILD_BEGIN(0, false)
             {
                 TEST_RESULT_BOOL(lockAcquireFile(backupLock, 0, true), true, "lock on fork");
                 sleepMSec(500);
             }
+            HARNESS_FORK_CHILD_END();
 
-            HARNESS_FORK_PARENT()
+            HARNESS_FORK_PARENT_BEGIN()
             {
                 sleepMSec(250);
                 TEST_ERROR(
@@ -99,6 +100,7 @@ testRun(void)
                             "unable to acquire lock on file '%s': Resource temporarily unavailable\n"
                             "HINT: is another pgBackRest process running?", strPtr(backupLock))));
             }
+            HARNESS_FORK_PARENT_END();
         }
         HARNESS_FORK_END();
     }
