@@ -6,22 +6,26 @@ Test Memory Contexts
 testFree - test callback function
 ***********************************************************************************************************************************/
 MemContext *memContextCallbackArgument = NULL;
+bool testFreeThrow = false;
 
 void
-testFree(MemContext *this)
+testFree(void *thisVoid)
 {
+    MemContext *this = thisVoid;
+
     TEST_RESULT_INT(this->state, memContextStateFreeing, "state should be freeing before memContextFree() in callback");
     memContextFree(this);
     TEST_RESULT_INT(this->state, memContextStateFreeing, "state should still be freeing after memContextFree() in callback");
 
-    TEST_ERROR(
-        memContextCallback(this, (MemContextCallback)testFree, this),
-        AssertError, "cannot assign callback to inactive context");
+    TEST_ERROR(memContextCallbackSet(this, testFree, this), AssertError, "cannot assign callback to inactive context");
 
     TEST_ERROR(memContextSwitch(this), AssertError, "cannot switch to inactive context");
     TEST_ERROR(memContextName(this), AssertError, "cannot get name for inactive context");
 
     memContextCallbackArgument = this;
+
+    if (testFreeThrow)
+        THROW(AssertError, "error in callback");
 }
 
 /***********************************************************************************************************************************
@@ -37,7 +41,7 @@ testRun(void)
     {
         // Test too large allocation -- only test this on 64-bit systems since 32-bit systems tend to work with any value that
         // valgrind will accept
-        if (sizeof(size_t) == 8)
+        if (TEST_64BIT())
         {
             TEST_ERROR(memAllocInternal((size_t)5629499534213120, false), MemoryError, "unable to allocate 5629499534213120 bytes");
             TEST_ERROR(memFreeInternal(NULL), AssertError, "assertion 'buffer != NULL' failed");
@@ -97,10 +101,7 @@ testRun(void)
         TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "top context == current context");
 
         // Context name length errors
-        TEST_ERROR(memContextNew(""), AssertError, "context name length must be > 0 and <= 64");
-        TEST_ERROR(
-            memContextNew("12345678901234567890123456789012345678901234567890123456789012345"),
-            AssertError, "context name length must be > 0 and <= 64");
+        TEST_ERROR(memContextNew(""), AssertError, "assertion 'name[0] != '\\0'' failed");
 
         MemContext *memContext = memContextNew("test1");
         TEST_RESULT_STR(memContextName(memContext), "test1", "test1 context name");
@@ -123,12 +124,14 @@ testRun(void)
         // This forces the child context array to grow
         memContextNew("test5");
         TEST_RESULT_INT(memContextTop()->contextChildListSize, MEM_CONTEXT_INITIAL_SIZE * 2, "increased child context list size");
+        TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, MEM_CONTEXT_INITIAL_SIZE + 1, "check context free idx");
 
         // Free a context
         memContextFree(memContextTop()->contextChildList[1]);
         TEST_RESULT_BOOL(
             memContextTop()->contextChildList[1]->state == memContextStateActive, false, "child context inactive after free");
         TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "current context is top");
+        TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, 1, "check context free idx");
 
         // Create a new context and it should end up in the same spot
         memContextNew("test-reuse");
@@ -136,6 +139,11 @@ testRun(void)
             memContextTop()->contextChildList[1]->state == memContextStateActive,
             true, "new context in same index as freed context is active");
         TEST_RESULT_STR(memContextName(memContextTop()->contextChildList[1]), "test-reuse", "new context name");
+        TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, 2, "check context free idx");
+
+        // Next context will be at the end
+        memContextNew("test-at-end");
+        TEST_RESULT_UINT(memContextTop()->contextChildFreeIdx, MEM_CONTEXT_INITIAL_SIZE + 2, "check context free idx");
 
         // Create a child context to test recursive free
         memContextSwitch(memContextTop()->contextChildList[MEM_CONTEXT_INITIAL_SIZE]);
@@ -206,10 +214,22 @@ testRun(void)
         TEST_RESULT_INT(expectedTotal, sizeof(size_t), "all bytes are 0xFE in original portion");
 
         // Free memory
-        TEST_RESULT_VOID(memFree(memContextCurrent()->allocList[MEM_CONTEXT_ALLOC_INITIAL_SIZE].buffer), "free allocation");
-        TEST_ERROR(
-            memFree(memContextCurrent()->allocList[MEM_CONTEXT_ALLOC_INITIAL_SIZE].buffer), AssertError,
-            "unable to find allocation");
+        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, MEM_CONTEXT_ALLOC_INITIAL_SIZE + 2, "check alloc free idx");
+        TEST_RESULT_VOID(memFree(memContextCurrent()->allocList[0].buffer), "free allocation");
+        TEST_ERROR(memFree(memContextCurrent()->allocList[0].buffer), AssertError, "unable to find allocation");
+        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 0, "check alloc free idx");
+
+        TEST_RESULT_VOID(memFree(memContextCurrent()->allocList[1].buffer), "free allocation");
+        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 0, "check alloc free idx");
+
+        TEST_RESULT_VOID(memNew(3), "new allocation");
+        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 1, "check alloc free idx");
+
+        TEST_RESULT_VOID(memNew(3), "new allocation");
+        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, 2, "check alloc free idx");
+
+        TEST_RESULT_VOID(memNew(3), "new allocation");
+        TEST_RESULT_UINT(memContextCurrent()->allocFreeIdx, MEM_CONTEXT_ALLOC_INITIAL_SIZE + 3, "check alloc free idx");
 
         TEST_ERROR(memFree(NULL), AssertError, "assertion 'buffer != NULL' failed");
         TEST_ERROR(memFree((void *)0x01), AssertError, "unable to find allocation");
@@ -220,24 +240,31 @@ testRun(void)
     }
 
     // *****************************************************************************************************************************
-    if (testBegin("memContextCallback()"))
+    if (testBegin("memContextCallbackSet()"))
     {
         TEST_ERROR(
-            memContextCallback(memContextTop(), (MemContextCallback)testFree, NULL), AssertError,
-            "top context may not have a callback");
+            memContextCallbackSet(memContextTop(), testFree, NULL), AssertError, "top context may not have a callback");
 
         MemContext *memContext = memContextNew("test-callback");
-        memContextCallback(memContext, (MemContextCallback)testFree, memContext);
+        memContextCallbackSet(memContext, testFree, memContext);
         TEST_ERROR(
-            memContextCallback(memContext, (MemContextCallback)testFree, memContext),
-            AssertError, "callback is already set for context 'test-callback'");
+            memContextCallbackSet(memContext, testFree, memContext), AssertError,
+            "callback is already set for context 'test-callback'");
 
         // Clear and reset it
         memContextCallbackClear(memContext);
-        memContextCallback(memContext, (MemContextCallback)testFree, memContext);
+        memContextCallbackSet(memContext, testFree, memContext);
 
         memContextFree(memContext);
         TEST_RESULT_PTR(memContextCallbackArgument, memContext, "callback argument is context");
+
+        // Now test with an error
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_ASSIGN(memContext, memContextNew("test-callback-error"), "new mem context");
+        testFreeThrow = true;
+        TEST_RESULT_VOID(memContextCallbackSet(memContext, testFree, memContext), "    set callback");
+        TEST_ERROR(memContextFree(memContext), AssertError, "error in callback");
+        TEST_RESULT_UINT(memContext->state, memContextStateFree, "    check that mem context was completely freed");
     }
 
     // *****************************************************************************************************************************
@@ -311,7 +338,7 @@ testRun(void)
 
         TEST_RESULT_BOOL(bCatch, true, "new context error was caught");
         TEST_RESULT_PTR(memContextCurrent(), memContextTop(), "context is now 'TOP'");
-        TEST_RESULT_BOOL(memContext->state == memContextStateFree, true, "new mem context is not active");
+        TEST_RESULT_UINT(memContext->state == memContextStateFree, true, "new mem context is not active");
     }
 
     // *****************************************************************************************************************************
@@ -321,7 +348,9 @@ testRun(void)
 
         // -------------------------------------------------------------------------------------------------------------------------
         MemContext *memContext = NULL;
+        MemContext *memContext2 = NULL;
         void *mem = NULL;
+        void *mem2 = NULL;
 
         MEM_CONTEXT_NEW_BEGIN("outer")
         {
@@ -347,11 +376,24 @@ testRun(void)
                 memContextCurrent()->contextChildList[1] = memContext;
                 TEST_RESULT_VOID(memContextMove(memContext, MEM_CONTEXT_OLD()), "move context");
                 TEST_RESULT_VOID(memContextMove(memContext, MEM_CONTEXT_OLD()), "move context again");
+
+                // Move another context
+                MEM_CONTEXT_NEW_BEGIN("inner2")
+                {
+                    memContext2 = MEM_CONTEXT_NEW();
+                    mem2 = memNew(sizeof(int));
+                }
+                MEM_CONTEXT_NEW_END();
+
+                TEST_RESULT_VOID(memContextMove(memContext2, MEM_CONTEXT_OLD()), "move context");
             }
             MEM_CONTEXT_TEMP_END();
 
             TEST_RESULT_PTR(memContext->allocList[0].buffer, mem, "check memory allocation");
             TEST_RESULT_PTR(memContextCurrent()->contextChildList[1], memContext, "check memory context");
+
+            TEST_RESULT_PTR(memContext2->allocList[0].buffer, mem2, "check memory allocation 2");
+            TEST_RESULT_PTR(memContextCurrent()->contextChildList[2], memContext2, "check memory context 2");
         }
         MEM_CONTEXT_NEW_END();
     }
