@@ -21,11 +21,10 @@ testRun(void)
         TEST_RESULT_INT(lstSize(store->list), 0, "list empty");
 
         TEST_RESULT_VOID(kvFree(store), "free kv");
-        TEST_RESULT_VOID(kvFree(NULL), "free null kv");
     }
 
     // -----------------------------------------------------------------------------------------------------------------------------
-    if (testBegin("kvPut(), kvAdd(), kvKeyList(), kvGet(), kvGetList(), and kvDup()"))
+    if (testBegin("kvPut(), kvAdd(), kvKeyExists(), kvKeyList(), kvGet(), kvGetList(), and kvDup()"))
     {
         KeyValue *store = NULL;
 
@@ -53,6 +52,11 @@ testRun(void)
         TEST_RESULT_PTR(kvGet(store, varNewInt(78)), NULL, "get int/null");
         TEST_RESULT_PTR(kvGet(store, varNewInt(777)), NULL, "get missing key");
 
+        // Check key exists
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_RESULT_BOOL(kvKeyExists(store, varNewStr(strNew("str-key"))), true, "key exists");
+        TEST_RESULT_BOOL(kvKeyExists(store, varNewStr(strNew(BOGUS_STR))), false, "key does not exist");
+
         // Check that a null value can be changed to non-null
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_RESULT_PTR(kvPut(store, varNewInt(78), varNewInt(66)), store, "update int/null to int/int");
