@@ -29,7 +29,7 @@ testRun(void)
     FUNCTION_HARNESS_VOID();
 
     // *****************************************************************************************************************************
-    if (testBegin("lstNew(), lstMemContext(), and lstFree()"))
+    if (testBegin("lstNew(), lstMemContext(), lstToLog(), and lstFree()"))
     {
         List *list = lstNew(sizeof(void *));
 
@@ -37,9 +37,14 @@ testRun(void)
         TEST_RESULT_INT(list->listSize, 0, "list size");
         TEST_RESULT_INT(list->listSizeMax, 0, "list size max");
         TEST_RESULT_PTR(lstMemContext(list), list->memContext, "list mem context");
+        TEST_RESULT_VOID(lstClear(list), "clear list");
 
         void *ptr = NULL;
         TEST_RESULT_PTR(lstAdd(list, &ptr), list, "add item");
+        TEST_RESULT_STR(strPtr(lstToLog(list)), "{size: 1}", "check log");
+
+        TEST_RESULT_VOID(lstClear(list), "clear list");
+        TEST_RESULT_STR(strPtr(lstToLog(list)), "{size: 0}", "check log after clear");
 
         TEST_RESULT_VOID(lstFree(list), "free list");
         TEST_RESULT_VOID(lstFree(lstNew(1)), "free empty list");
@@ -77,7 +82,27 @@ testRun(void)
             TEST_RESULT_INT(*item, listIdx, "check item %u", listIdx);
         }
 
-        TEST_ERROR(lstGet(list, lstSize(list)), AssertError, "cannot get index 9 from list with 9 value(s)");
+        // Remove first item
+        TEST_RESULT_VOID(lstRemove(list, 0), "remove first item");
+
+        // Read them back and check values
+        for (unsigned int listIdx = 0; listIdx < lstSize(list); listIdx++)
+        {
+            int *item = lstGet(list, listIdx);
+            TEST_RESULT_INT(*item, listIdx + 1, "check item %u", listIdx);
+        }
+
+        // Remove last item
+        TEST_RESULT_VOID(lstRemove(list, lstSize(list) - 1), "remove last item");
+
+        // Read them back and check values
+        for (unsigned int listIdx = 0; listIdx < lstSize(list); listIdx++)
+        {
+            int *item = lstGet(list, listIdx);
+            TEST_RESULT_INT(*item, listIdx + 1, "check item %u", listIdx);
+        }
+
+        TEST_ERROR(lstGet(list, lstSize(list)), AssertError, "cannot get index 7 from list with 7 value(s)");
         TEST_RESULT_VOID(lstMove(NULL, memContextTop()), "move null list");
     }
 
