@@ -1,6 +1,8 @@
 /***********************************************************************************************************************************
 Test Configuration Parse
 ***********************************************************************************************************************************/
+#include "storage/storage.intern.h"
+
 #define TEST_BACKREST_EXE                                           "pgbackrest"
 
 #define TEST_COMMAND_ARCHIVE_GET                                    "archive-get"
@@ -14,7 +16,6 @@ Expose log internal data for unit testing/debugging
 ***********************************************************************************************************************************/
 extern LogLevel logLevelStdOut;
 extern LogLevel logLevelStdErr;
-extern LogLevel logLevelFile;
 
 /***********************************************************************************************************************************
 Option find test -- this is done a lot in the deprecated tests
@@ -66,14 +67,14 @@ testRun(void)
 
         storagePut(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[global]\n"
                 "compress-level=3\n"
                 "spool-path=/path/to/spool\n"));
 
         storagePut(
             storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/global-backup.conf", strPtr(configIncludePath))),
-            bufNewZ(
+            BUFSTRDEF(
                 "[global:backup]\n"
                 "repo1-hardlink=y\n"
                 "bogus=bogus\n"
@@ -87,14 +88,14 @@ testRun(void)
 
         storagePut(
             storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/db-backup.conf", strPtr(configIncludePath))),
-            bufNewZ(
+            BUFSTRDEF(
                 "[db:backup]\n"
                 "compress=n\n"
                 "recovery-option=a=b\n"));
 
         storagePut(
             storageNewWriteNP(storageLocalWrite(), strNewFmt("%s/stanza.db.conf", strPtr(configIncludePath))),
-            bufNewZ(
+            BUFSTRDEF(
                 "[db]\n"
                 "pg1-host=db\n"
                 "pg1-path=/path/to/db\n"
@@ -154,6 +155,8 @@ testRun(void)
         parseOptionList[cfgOptConfigIncludePath].source = cfgSourceParam;
         parseOptionList[cfgOptConfigIncludePath].valueList = strLstAdd(strLstNew(), configIncludePath);
 
+        TEST_RESULT_VOID(cfgFileLoadPart(NULL, NULL), "check null part");
+
         TEST_RESULT_STR(strPtr(cfgFileLoad(parseOptionList, backupCmdDefConfigValue,
             backupCmdDefConfigInclPathValue, oldConfigDefault)),
             "[global]\n"
@@ -172,17 +175,16 @@ testRun(void)
         parseOptionList[cfgOptConfigIncludePath].valueList = strLstAddZ(strLstNew(), BOGUS_STR);
         TEST_ERROR(
             cfgFileLoad(parseOptionList, backupCmdDefConfigValue,
-                backupCmdDefConfigInclPathValue, oldConfigDefault), PathOpenError,
-                "unable to open path '/BOGUS' for read: [2] No such file or directory");
+                backupCmdDefConfigInclPathValue, oldConfigDefault), PathMissingError,
+                "unable to list files for missing path '/BOGUS'");
 
         // --config-include-path valid, --config invalid (does not exist)
         parseOptionList[cfgOptConfigIncludePath].valueList = strLstAdd(strLstNew(), configIncludePath);
         parseOptionList[cfgOptConfig].valueList = strLstAdd(strLstNew(), strNewFmt("%s/%s", testPath(), BOGUS_STR));
 
-        TEST_ERROR(
-            cfgFileLoad(parseOptionList, backupCmdDefConfigValue,
-                backupCmdDefConfigInclPathValue, oldConfigDefault), FileMissingError,
-                strPtr(strNewFmt("unable to open '%s/%s' for read: [2] No such file or directory", testPath(), BOGUS_STR)));
+        TEST_ERROR_FMT(
+            cfgFileLoad(parseOptionList, backupCmdDefConfigValue, backupCmdDefConfigInclPathValue, oldConfigDefault),
+            FileMissingError, STORAGE_ERROR_READ_MISSING, strPtr(strNewFmt("%s/BOGUS", testPath())));
 
         strLstFree(parseOptionList[cfgOptConfig].valueList);
         strLstFree(parseOptionList[cfgOptConfigIncludePath].valueList);
@@ -226,7 +228,7 @@ testRun(void)
         mkdir(strPtr(strPath(oldConfigDefault)), 0750);
         storagePut(
             storageNewWriteNP(storageLocalWrite(), oldConfigDefault),
-            bufNewZ(
+            BUFSTRDEF(
                 "[global:backup]\n"
                 "buffer-size=65536\n"));
 
@@ -468,6 +470,7 @@ testRun(void)
         double valueDbl = 0;
         String *value = strNew("10.0");
 
+        TEST_ERROR(sizeQualifierToMultiplier('w'), AssertError, "'w' is not a valid size qualifier");
         TEST_ERROR(convertToByte(&value, &valueDbl), FormatError, "value '10.0' is not valid");
         strTrunc(value, strChr(value, '.'));
         strCat(value, "K2");
@@ -542,7 +545,6 @@ testRun(void)
         strCat(value, "99999999999999999999p");
         convertToByte(&value, &valueDbl);
         TEST_RESULT_STR(strPtr(value), "225179981368524800000000000000000000", "value really large  to bytes");
-
     }
 
     // *****************************************************************************************************************************
@@ -575,6 +577,22 @@ testRun(void)
             configParse(strLstSize(argList), strLstPtr(argList), false), OptionInvalidError,
             "option '--pg1-host' requires argument");
 
+        // -------------------------------------------------------------------------------------------------------------------------
+        argList = strLstNew();
+        strLstAdd(argList, strNew(TEST_BACKREST_EXE));
+        strLstAdd(argList, strNew("backup"));
+        strLstAdd(argList, strNew("param1"));
+        TEST_ERROR(
+            configParse(strLstSize(argList), strLstPtr(argList), false), ParamInvalidError, "command does not allow parameters");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        argList = strLstNew();
+        strLstAdd(argList, strNew(TEST_BACKREST_EXE));
+        strLstAdd(argList, strNew("help"));
+        strLstAdd(argList, strNew("backup"));
+        strLstAdd(argList, strNew("param1"));
+        TEST_RESULT_VOID(configParse(strLstSize(argList), strLstPtr(argList), false), "ignore params when help command");
+
         // -------------------------------------------------------------------------------------------------------------------------
         argList = strLstNew();
         strLstAdd(argList, strNew(TEST_BACKREST_EXE));
@@ -673,6 +691,42 @@ testRun(void)
         TEST_ERROR(configParse(strLstSize(argList), strLstPtr(argList), false), OptionInvalidValueError,
             "'225179981368524800000000000000000000' is out of range for 'manifest-save-threshold' option");
 
+        // -------------------------------------------------------------------------------------------------------------------------
+        argList = strLstNew();
+        strLstAdd(argList, strNew(TEST_BACKREST_EXE));
+        strLstAdd(argList, strNew(TEST_COMMAND_BACKUP));
+        strLstAdd(argList, strNew("--stanza=db"));
+        strLstAdd(argList, strNew("--pg1-path="));
+        TEST_ERROR(configParse(strLstSize(argList), strLstPtr(argList), false), OptionInvalidValueError,
+            "'' must be >= 1 character for 'pg1-path' option");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        argList = strLstNew();
+        strLstAdd(argList, strNew(TEST_BACKREST_EXE));
+        strLstAdd(argList, strNew(TEST_COMMAND_BACKUP));
+        strLstAdd(argList, strNew("--stanza=db"));
+        strLstAdd(argList, strNew("--pg1-path=bogus"));
+        TEST_ERROR(configParse(strLstSize(argList), strLstPtr(argList), false), OptionInvalidValueError,
+            "'bogus' must begin with / for 'pg1-path' option");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        argList = strLstNew();
+        strLstAdd(argList, strNew(TEST_BACKREST_EXE));
+        strLstAdd(argList, strNew(TEST_COMMAND_BACKUP));
+        strLstAdd(argList, strNew("--stanza=db"));
+        strLstAdd(argList, strNew("--pg1-path=/path1//path2"));
+        TEST_ERROR(configParse(strLstSize(argList), strLstPtr(argList), false), OptionInvalidValueError,
+            "'/path1//path2' cannot contain // for 'pg1-path' option");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        argList = strLstNew();
+        strLstAdd(argList, strNew(TEST_BACKREST_EXE));
+        strLstAdd(argList, strNew(TEST_COMMAND_BACKUP));
+        strLstAdd(argList, strNew("--stanza=db"));
+        strLstAdd(argList, strNew("--pg1-path=/path1/path2//"));
+        TEST_ERROR(configParse(strLstSize(argList), strLstPtr(argList), false), OptionInvalidValueError,
+            "'/path1/path2//' cannot contain // for 'pg1-path' option");
+
         // Local and remove commands should not modify log levels during parsing
         // -------------------------------------------------------------------------------------------------------------------------
         argList = strLstNew();
@@ -908,7 +962,7 @@ testRun(void)
 
         storagePutNP(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[global]\n"
                 "compress=bogus\n"));
 
@@ -925,7 +979,7 @@ testRun(void)
 
         storagePutNP(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[global]\n"
                 "compress=\n"));
 
@@ -942,7 +996,7 @@ testRun(void)
 
         storagePutNP(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[db]\n"
                 "pg1-path=/path/to/db\n"
                 "db-path=/also/path/to/db\n"));
@@ -960,7 +1014,7 @@ testRun(void)
 
         storagePutNP(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[db]\n"
                 "pg1-path=/path/to/db\n"
                 "pg1-path=/also/path/to/db\n"));
@@ -972,7 +1026,7 @@ testRun(void)
         // Also test with a boolean option since this gets converted immediately and will blow up if it is multi
         storagePutNP(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[db]\n"
                 "start-fast=y\n"
                 "start-fast=n\n"));
@@ -1043,7 +1097,7 @@ testRun(void)
         argList = strLstNew();
         strLstAdd(argList, strNew(TEST_BACKREST_EXE));
         strLstAdd(argList, strNew("--stanza=db"));
-        strLstAdd(argList, strNew("--pg1-path=/path/to/db"));
+        strLstAdd(argList, strNew("--pg1-path=/path/to/db/"));
         strLstAdd(argList, strNew("--no-online"));
         strLstAdd(argList, strNew("--no-config"));
         strLstAdd(argList, strNew("--repo1-type=s3"));
@@ -1094,15 +1148,17 @@ testRun(void)
         setenv("PGBACKREST_RESET_REPO1_HOST", "", true);
         setenv("PGBACKREST_TARGET", "xxx", true);
         setenv("PGBACKREST_ONLINE", "y", true);
+        setenv("PGBACKREST_DELTA", "y", true);
         setenv("PGBACKREST_START_FAST", "n", true);
         setenv("PGBACKREST_PG1_SOCKET_PATH", "/path/to/socket", true);
 
         storagePutNP(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[global]\n"
                 "compress-level=3\n"
                 "spool-path=/path/to/spool\n"
+                "lock-path=/\n"
                 "\n"
                 "[global:backup]\n"
                 "repo1-hardlink=y\n"
@@ -1142,6 +1198,8 @@ testRun(void)
         TEST_RESULT_BOOL(cfgOptionTest(cfgOptPgHost), false, "    pg1-host is not set (command line reset override)");
         TEST_RESULT_STR(strPtr(cfgOptionStr(cfgOptPgPath)), "/path/to/db", "    pg1-path is set");
         TEST_RESULT_INT(cfgOptionSource(cfgOptPgPath), cfgSourceConfig, "    pg1-path is source config");
+        TEST_RESULT_STR(strPtr(cfgOptionStr(cfgOptLockPath)), "/", "    lock-path is set");
+        TEST_RESULT_INT(cfgOptionSource(cfgOptLockPath), cfgSourceConfig, "    lock-path is source config");
         TEST_RESULT_STR(strPtr(cfgOptionStr(cfgOptPgSocketPath)), "/path/to/socket", "    pg1-socket-path is set");
         TEST_RESULT_INT(cfgOptionSource(cfgOptPgSocketPath), cfgSourceConfig, "    pg1-socket-path is config param");
         TEST_RESULT_BOOL(cfgOptionBool(cfgOptOnline), false, "    online not is set");
@@ -1158,6 +1216,8 @@ testRun(void)
         TEST_RESULT_INT(cfgOptionSource(cfgOptCompressLevel), cfgSourceConfig, "    compress-level is source config");
         TEST_RESULT_BOOL(cfgOptionBool(cfgOptBackupStandby), false, "    backup-standby not is set");
         TEST_RESULT_INT(cfgOptionSource(cfgOptBackupStandby), cfgSourceDefault, "    backup-standby is source default");
+        TEST_RESULT_BOOL(cfgOptionBool(cfgOptDelta), true, "    delta is set");
+        TEST_RESULT_INT(cfgOptionSource(cfgOptDelta), cfgSourceConfig, "    delta is source config");
         TEST_RESULT_BOOL(cfgOptionInt64(cfgOptBufferSize), 65536, "    buffer-size is set");
         TEST_RESULT_INT(cfgOptionSource(cfgOptBufferSize), cfgSourceConfig, "    backup-standby is source config");
 
@@ -1180,7 +1240,7 @@ testRun(void)
 
         storagePutNP(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[global]\n"
                 "spool-path=/path/to/spool\n"));
 
@@ -1254,7 +1314,7 @@ testRun(void)
 
         storagePutNP(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[global:restore]\n"
                 "recovery-option=f=g\n"
                 "recovery-option=hijk=l\n"
@@ -1300,7 +1360,7 @@ testRun(void)
 
         storagePutNP(
             storageNewWriteNP(storageLocalWrite(), configFile),
-            bufNewZ(
+            BUFSTRDEF(
                 "[global]\n"
                 "repo1-path=/path/to/repo\n"
                 "\n"
@@ -1341,8 +1401,9 @@ testRun(void)
         testOptionFind("repo-s3-key", PARSE_DEPRECATE_FLAG | cfgOptRepoS3Key);
         testOptionFind("repo-s3-key-secret", PARSE_DEPRECATE_FLAG | cfgOptRepoS3KeySecret);
         testOptionFind("repo-s3-region", PARSE_DEPRECATE_FLAG | cfgOptRepoS3Region);
-        testOptionFind("repo-s3-verify-ssl", PARSE_DEPRECATE_FLAG | cfgOptRepoS3VerifySsl);
-        testOptionFind("no-repo-s3-verify-ssl", PARSE_DEPRECATE_FLAG | PARSE_NEGATE_FLAG | cfgOptRepoS3VerifySsl);
+        testOptionFind("repo-s3-verify-ssl", PARSE_DEPRECATE_FLAG | cfgOptRepoS3VerifyTls);
+        testOptionFind("repo1-s3-verify-ssl", PARSE_DEPRECATE_FLAG | cfgOptRepoS3VerifyTls);
+        testOptionFind("no-repo-s3-verify-ssl", PARSE_DEPRECATE_FLAG | PARSE_NEGATE_FLAG | cfgOptRepoS3VerifyTls);
 
         // PostreSQL options
         // -------------------------------------------------------------------------------------------------------------------------
