@@ -31,8 +31,6 @@ use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Common::Wait;
-use pgBackRest::Storage::Posix::Driver;
-use pgBackRest::Storage::Local;
 use pgBackRest::Version;
 
 use pgBackRestBuild::Build;
@@ -55,6 +53,8 @@ use pgBackRestTest::Common::HostGroupTest;
 use pgBackRestTest::Common::JobTest;
 use pgBackRestTest::Common::ListTest;
 use pgBackRestTest::Common::RunTest;
+use pgBackRestTest::Common::Storage;
+use pgBackRestTest::Common::StoragePosix;
 use pgBackRestTest::Common::VmTest;
 
 ####################################################################################################################################
@@ -77,8 +77,8 @@ test.pl [options]
    --no-cleanup         don't cleaup after the last test is complete - useful for debugging
    --pg-version         version of postgres to test (all, defaults to minimal)
    --log-force          force overwrite of current test log files
-   --no-lint            disable static source code analysis
    --build-only         compile the test library / packages and run tests only
+   --build-max          max processes to use for builds (default 4)
    --coverage-only      only run coverage tests (as a subset of selected tests)
    --c-only             only run C tests
    --gen-only           only run auto-generation
@@ -87,15 +87,19 @@ test.pl [options]
    --smart              perform libc/package builds only when source timestamps have changed
    --no-package         do not build packages
    --no-ci-config       don't overwrite the current continuous integration config
-   --dev                --no-lint --smart --no-package --no-optimize
-   --dev-test           --no-lint --no-package
-   --expect             --no-lint --no-package --vm=co7 --db=9.6 --log-force
+   --dev                --smart --no-package --no-optimize
+   --dev-test           --no-package
+   --expect             --no-package --vm=co7 --db=9.6 --log-force
    --no-valgrind        don't run valgrind on C unit tests (saves time)
    --no-coverage        don't run coverage on C unit tests (saves time)
    --no-optimize        don't do compile optimization for C (saves compile time)
    --backtrace          enable backtrace when available (adds stack trace line numbers -- very slow)
    --profile            generate profile info
    --no-debug           don't generate a debug build
+   --debug-test-trace   test stack trace for low-level functions (slow, esp w/valgrind, may cause timeouts)
+
+ Report Options:
+   --coverage-summary   generate a coverage summary report for the documentation
 
  Configuration Options:
    --psql-bin           path to the psql executables (e.g. /usr/lib/postgresql/9.3/bin/)
@@ -140,9 +144,10 @@ my $strVm;
 my $strVmHost = VM_HOST_DEFAULT;
 my $bVmBuild = false;
 my $bVmForce = false;
-my $bNoLint = false;
 my $bBuildOnly = false;
+my $iBuildMax = 4;
 my $bCoverageOnly = false;
+my $bCoverageSummary = false;
 my $bNoCoverage = false;
 my $bCOnly = false;
 my $bGenOnly = false;
@@ -159,8 +164,11 @@ my $bExpect = false;
 my $bNoValgrind = false;
 my $bNoOptimize = false;
 my $bNoDebug = false;
+my $bDebugTestTrace = false;
 my $iRetry = 0;
 
+my @cmdOptions = @ARGV;
+
 GetOptions ('q|quiet' => \$bQuiet,
             'version' => \$bVersion,
             'help' => \$bHelp,
@@ -182,11 +190,12 @@ GetOptions ('q|quiet' => \$bQuiet,
             'no-cleanup' => \$bNoCleanup,
             'pg-version=s' => \$strPgVersion,
             'log-force' => \$bLogForce,
-            'no-lint' => \$bNoLint,
             'build-only' => \$bBuildOnly,
+            'build-max=s' => \$iBuildMax,
             'no-package' => \$bNoPackage,
             'no-ci-config' => \$bNoCiConfig,
             'coverage-only' => \$bCoverageOnly,
+            'coverage-summary' => \$bCoverageSummary,
             'no-coverage' => \$bNoCoverage,
             'c-only' => \$bCOnly,
             'gen-only' => \$bGenOnly,
@@ -201,6 +210,7 @@ GetOptions ('q|quiet' => \$bQuiet,
             'no-valgrind' => \$bNoValgrind,
             'no-optimize' => \$bNoOptimize,
             'no-debug', => \$bNoDebug,
+            'debug-test-trace', => \$bDebugTestTrace,
             'retry=s' => \$iRetry)
     or pod2usage(2);
 
@@ -232,6 +242,15 @@ eval
         pod2usage();
     }
 
+    ################################################################################################################################
+    # Update options for --coverage-summary
+    ################################################################################################################################
+    if ($bCoverageSummary)
+    {
+        $bCoverageOnly = true;
+        $bCOnly = true;
+    }
+
     ################################################################################################################################
     # Update options for --dev and --dev-fast and --dev-test
     ################################################################################################################################
@@ -242,7 +261,6 @@ eval
 
     if ($bDev)
     {
-        $bNoLint = true;
         $bSmart = true;
         $bNoPackage = true;
         $bNoOptimize = true;
@@ -251,7 +269,6 @@ eval
     if ($bDevTest)
     {
         $bNoPackage = true;
-        $bNoLint = true;
     }
 
     ################################################################################################################################
@@ -268,7 +285,6 @@ eval
     ################################################################################################################################
     if ($bExpect)
     {
-        $bNoLint = true;
         $bNoPackage = true;
         $strVm = VM_EXPECT;
         $strPgVersion = '9.6';
@@ -306,8 +322,8 @@ eval
         $strTestPath = cwd() . '/test';
     }
 
-    my $oStorageTest = new pgBackRest::Storage::Local(
-        $strTestPath, new pgBackRest::Storage::Posix::Driver({bFileSync => false, bPathSync => false}));
+    my $oStorageTest = new pgBackRestTest::Common::Storage(
+        $strTestPath, new pgBackRestTest::Common::StoragePosix({bFileSync => false, bPathSync => false}));
 
     if ($bCoverageOnly)
     {
@@ -320,10 +336,6 @@ eval
         {
             confess &log(ERROR, "select a single Debian-based VM for coverage testing");
         }
-        elsif (!vmCoveragePerl($strVm))
-        {
-            confess &log(ERROR, "only Debian-based VMs can be used for coverage testing");
-        }
     }
 
     # If VM is not defined then set it to all
@@ -335,8 +347,8 @@ eval
     # Get the base backrest path
     my $strBackRestBase = dirname(dirname(abs_path($0)));
 
-    my $oStorageBackRest = new pgBackRest::Storage::Local(
-        $strBackRestBase, new pgBackRest::Storage::Posix::Driver({bFileSync => false, bPathSync => false}));
+    my $oStorageBackRest = new pgBackRestTest::Common::Storage(
+        $strBackRestBase, new pgBackRestTest::Common::StoragePosix({bFileSync => false, bPathSync => false}));
 
     ################################################################################################################################
     # Build Docker containers
@@ -417,6 +429,61 @@ eval
             my @stryBuiltAll;
             &log(INFO, "check code autogenerate");
 
+            # Auto-generate version for configure.ac script
+            #-----------------------------------------------------------------------------------------------------------------------
+            if (!$bSmart || grep(/^lib\/pgBackRest\/Version\.pm/, @stryModifiedList))
+            {
+                my $strConfigureAcOld = ${$oStorageTest->get("${strBackRestBase}/src/configure.ac")};
+                my $strConfigureAcNew;
+
+                foreach my $strLine (split("\n", $strConfigureAcOld))
+                {
+                    if ($strLine =~ /^AC_INIT\(/)
+                    {
+                        $strLine = 'AC_INIT([' . PROJECT_NAME . '], [' . PROJECT_VERSION . '])';
+                    }
+
+                    $strConfigureAcNew .= "${strLine}\n";
+                }
+
+                # Save into the src dir
+                my @stryBuilt;
+                my $strBuilt = 'src/configure.ac';
+
+                if (buildPutDiffers($oStorageBackRest, "${strBackRestBase}/${strBuilt}", $strConfigureAcNew))
+                {
+                    push(@stryBuilt, $strBuilt);
+                    push(@stryBuiltAll, @stryBuilt);
+                    push(@stryModifiedList, @stryBuilt);
+                }
+
+                &log(INFO,
+                    "    autogenerated version in configure.ac script: " . (@stryBuilt ? join(', ', @stryBuilt) : 'no changes'));
+            }
+
+            # Auto-generate configure script
+            #-----------------------------------------------------------------------------------------------------------------------
+            if (!$bSmart || grep(/^src\/configure\.ac/, @stryModifiedList))
+            {
+                my $strConfigure = executeTest("autoconf ${strBackRestBase}/src/configure.ac");
+
+                # Trim off any trailing LFs
+                $strConfigure = trim($strConfigure) . "\n";
+
+                # Save into the src dir
+                my @stryBuilt;
+                my $strBuilt = 'src/configure';
+
+                if (buildPutDiffers($oStorageBackRest, "${strBackRestBase}/${strBuilt}", $strConfigure))
+                {
+                    push(@stryBuilt, $strBuilt);
+                    push(@stryBuiltAll, @stryBuilt);
+                    push(@stryModifiedList, @stryBuilt);
+                }
+
+                &log(INFO, "    autogenerated configure script: " . (@stryBuilt ? join(', ', @stryBuilt) : 'no changes'));
+            }
+
             # Auto-generate C files
             #-----------------------------------------------------------------------------------------------------------------------
             if (!$bSmart || grep(/^build\//, @stryModifiedList))
@@ -463,7 +530,7 @@ eval
             # Auto-generate Perl code
             #-----------------------------------------------------------------------------------------------------------------------
             use lib dirname(dirname($0)) . '/libc/build/lib';
-            use pgBackRestLibC::Build;                                      ## no critic (Modules::ProhibitConditionalUseStatements)
+            use pgBackRestLibC::Build;
 
             if (!$bSmart || grep(/^(build|libc\/build)\//, @stryModifiedList))
             {
@@ -537,15 +604,15 @@ eval
             if (!$bSmart || grep(/^(src|libc)\//, @stryModifiedList))
             {
                 my @stryBuilt;
-                my $strBuilt = 'src/Makefile';
+                my $strBuilt = 'src/Makefile.in';
 
                 if (buildPutDiffers(
                     $oStorageBackRest,
                     $strBuilt,
                     buildMakefile(
                         $oStorageBackRest,
-                        ${$oStorageBackRest->get("src/Makefile")},
-                        {rhOption => {'postgres/pageChecksum.o' => '-funroll-loops -ftree-vectorize'}})))
+                        ${$oStorageBackRest->get("src/Makefile.in")},
+                        {rhOption => {'postgres/pageChecksum.o' => '@COPTIMIZE_PAGE_CHECKSUM@'}})))
                 {
                     push(@stryBuilt, $strBuilt);
                     push(@stryBuiltAll, @stryBuilt);
@@ -660,7 +727,7 @@ eval
             $oStorageTest->pathCreate($strCoveragePath, {strMode => '0770', bIgnoreExists => true, bCreateParent => true});
 
             # Remove old coverage dirs -- do it this way so the dirs stay open in finder/explorer, etc.
-            executeTest("rm -rf ${strBackRestBase}/test/coverage/c/* ${strBackRestBase}/test/coverage/perl/*");
+            executeTest("rm -rf ${strBackRestBase}/test/coverage/c/*");
 
             # Overwrite the C coverage report so it will load but not show old coverage
             $oStorageTest->pathCreate("${strBackRestBase}/test/coverage", {strMode => '0770', bIgnoreExists => true});
@@ -769,18 +836,30 @@ eval
                     my $bRebuild = !$bSmart;
                     $rhBinBuild->{$strBuildVM} = true;
 
+                    # Build configure/compile options and see if they have changed from the previous build
+                    my $strCFlags =
+                        "-Wfatal-errors -g -fPIC -D_FILE_OFFSET_BITS=64" .
+                        (vmWithBackTrace($strBuildVM) && $bBackTrace ? ' -DWITH_BACKTRACE' : '') .
+                        ($bDebugTestTrace ? ' -DDEBUG_TEST_TRACE' : '');
+                    my $strLdFlags = vmWithBackTrace($strBuildVM) && $bBackTrace  ? '-lbacktrace' : '';
+                    my $strConfigOptions = (vmDebugIntegration($strBuildVM) ? ' --enable-test' : '');
+                    my $strBuildFlags = "CFLAGS=${strCFlags}\nLDFLAGS=${strLdFlags}\nCONFIGURE=${strConfigOptions}";
+                    my $strBuildFlagFile = "${strBinPath}/${strBuildVM}/build.flags";
+
+                    my $bBuildOptionsDiffer = buildPutDiffers($oStorageBackRest, $strBuildFlagFile, $strBuildFlags);
+                    $bBuildOptionsDiffer |= grep(/^src\/configure|src\/Makefile.in|src\/build\.auto\.h$/, @stryModifiedList);
+
                     # Rebuild if the modification time of the smart file does equal the last changes in source paths
-                    if ($bSmart)
+                    my $strBinSmart = "${strBuildPath}/pgbackrest";
+
+                    if ($bBuildOptionsDiffer ||
+                        ($bSmart &&
+                         (!$oStorageBackRest->exists($strBinSmart) ||
+                          $oStorageBackRest->info($strBinSmart)->mtime < $lTimestampLast)))
                     {
-                        my $strBinSmart = "${strBuildPath}/pgbackrest";
+                        &log(INFO, "    bin dependencies have changed for ${strBuildVM}, rebuilding...");
 
-                        if (!$oStorageBackRest->exists($strBinSmart) ||
-                            $oStorageBackRest->info($strBinSmart)->mtime < $lTimestampLast)
-                        {
-                            &log(INFO, "    bin dependencies have changed for ${strBuildVM}, rebuilding...");
-
-                            $bRebuild = true;
-                        }
+                        $bRebuild = true;
                     }
 
                     if ($bRebuild)
@@ -799,25 +878,22 @@ eval
                         }
 
                         executeTest(
-                            "rsync -rt" . (!$bSmart ? " --delete-excluded" : '') .
+                            "rsync -rt" . (!$bSmart || $bBuildOptionsDiffer ? " --delete-excluded" : '') .
                             " --include=" . join('/*** --include=', @stryBinSrcPath) . '/*** --exclude=*' .
                             " ${strBackRestBase}/ ${strBinPath}/${strBuildVM}");
+                        buildPutDiffers($oStorageBackRest, $strBuildFlagFile, $strBuildFlags);
 
-                        if (vmLintC($strVm) && !$bNoLint)
+                        if ($bBuildOptionsDiffer || !$oStorageBackRest->exists("${strBuildPath}/Makefile"))
                         {
-                            &log(INFO, "    clang static analyzer ${strBuildVM} (${strBuildPath})");
+                            executeTest(
+                                "docker exec -i test-build bash -c 'cd ${strBuildPath} && ./configure${strConfigOptions}'",
+                                {bShowOutputAsync => $bLogDetail});
                         }
 
-                        my $strCExtra =
-                            "'-g -fPIC -D_FILE_OFFSET_BITS=64" .
-                            (vmWithBackTrace($strBuildVM) && $bNoLint && $bBackTrace ? ' -DWITH_BACKTRACE' : '') . "'";
-                        my $strLdExtra = vmWithBackTrace($strBuildVM) && $bNoLint && $bBackTrace  ? '-lbacktrace' : '';
-                        my $strCDebug = vmDebugIntegration($strBuildVM) ? 'CDEBUG=' : '';
-
                         executeTest(
                             'docker exec -i test-build' .
-                            (vmLintC($strVm) && !$bNoLint ? ' scan-build-6.0' : '') .
-                            " make --silent --directory ${strBuildPath} CEXTRA=${strCExtra} LDEXTRA=${strLdExtra} ${strCDebug}",
+                            " make -j ${iBuildMax}" . ($bLogDetail ? '' : ' --silent') .
+                                " --directory ${strBuildPath} CFLAGS='${strCFlags}' LDFLAGS='${strLdFlags}'",
                             {bShowOutputAsync => $bLogDetail});
 
                         executeTest("docker rm -f test-build");
@@ -848,6 +924,7 @@ eval
                 {
                     my $strBuildPath = "${strLibCPath}/${strBuildVM}/libc";
                     my $bContainerExists = $strBuildVM ne $strVmHost;
+                    my $strConfigOptions = (vmDebugIntegration($strBuildVM) ? ' --enable-test' : '');
 
                     my $strLibCSmart = "${strBuildPath}/blib/arch/auto/pgBackRest/LibC/LibC.so";
                     my $bRebuild = !$bSmart;
@@ -917,9 +994,17 @@ eval
                                 {bSuppressStdErr => true, bShowOutputAsync => $bLogDetail});
                         }
 
+                        if (!$oStorageBackRest->exists("${strLibCPath}/${strBuildVM}/src/Makefile"))
+                        {
+                            executeTest(
+                                ($bContainerExists ? 'docker exec -i test-build ' : '') .
+                                    "bash -c 'cd ${strLibCPath}/${strBuildVM}/src && ./configure${strConfigOptions}'",
+                                {bShowOutputAsync => $bLogDetail});
+                        }
+
                         executeTest(
                             ($bContainerExists ? 'docker exec -i test-build ' : '') .
-                                "make --silent --directory ${strBuildPath}",
+                                "make -j ${iBuildMax} --silent --directory ${strBuildPath}",
                             {bShowOutputAsync => $bLogDetail});
 
                         if ($bContainerExists)
@@ -1128,21 +1213,6 @@ eval
         #---------------------------------------------------------------------------------------------------------------------------
         if (!$bDryRun)
         {
-            # Run Perl critic
-            if (!$bNoLint && !$bBuildOnly)
-            {
-                my $strBasePath = dirname(dirname(abs_path($0)));
-
-                &log(INFO, "Performing static code analysis using perlcritic");
-
-                executeTest('perlcritic --quiet --verbose=8 --brutal --top=10' .
-                            ' --verbose "[%p] %f: %m at line %l, column %c.  %e.  (Severity: %s)\n"' .
-                            " \"--profile=${strBasePath}/test/lint/perlcritic.policy\"" .
-                            " ${strBasePath}/lib/*" .
-                            " ${strBasePath}/test/test.pl ${strBasePath}/test/lib/*" .
-                            " ${strBasePath}/doc/doc.pl ${strBasePath}/doc/lib/*");
-            }
-
             logFileSet($oStorageTest, cwd() . "/test");
         }
 
@@ -1225,8 +1295,9 @@ eval
                 {
                     my $oJob = new pgBackRestTest::Common::JobTest(
                         $oStorageTest, $strBackRestBase, $strTestPath, $strCoveragePath, $$oyTestRun[$iTestIdx], $bDryRun, $bVmOut,
-                        $iVmIdx, $iVmMax, $iTestIdx, $iTestMax, $strLogLevel, $strLogLevelTest, $bLogForce, $bShowOutputAsync, $bNoCleanup, $iRetry,
-                        !$bNoValgrind, !$bNoCoverage, !$bNoOptimize, $bBackTrace, $bProfile, !$bNoDebug);
+                        $iVmIdx, $iVmMax, $iTestIdx, $iTestMax, $strLogLevel, $strLogLevelTest, $bLogForce, $bShowOutputAsync,
+                        $bNoCleanup, $iRetry, !$bNoValgrind, !$bNoCoverage, $bCoverageSummary, !$bNoOptimize, $bBackTrace,
+                        $bProfile, !$bNoDebug, $bDebugTestTrace, $iBuildMax / $iVmMax < 1 ? 1 : int($iBuildMax / $iVmMax));
                     $iTestIdx++;
 
                     if ($oJob->run())
@@ -1244,7 +1315,7 @@ eval
         #---------------------------------------------------------------------------------------------------------------------------
         my $iUncoveredCodeModuleTotal = 0;
 
-        if ((vmCoverageC($strVm) || vmCoveragePerl($strVm)) && !$bNoCoverage && !$bDryRun && $iTestFail == 0)
+        if (vmCoverageC($strVm) && !$bNoCoverage && !$bDryRun && $iTestFail == 0)
         {
             # Determine which modules were covered (only check coverage if all tests were successful)
             #-----------------------------------------------------------------------------------------------------------------------
@@ -1301,104 +1372,6 @@ eval
                 &log(INFO, 'no code modules had all tests run required for coverage');
             }
 
-            # Generate Perl coverage report
-            #-----------------------------------------------------------------------------------------------------------------------
-            if (vmCoveragePerl($strVm))
-            {
-                &log(INFO, 'writing Perl coverage report');
-
-                executeTest("cp -rp ${strCoveragePath} ${strCoveragePath}_temp");
-                executeTest(
-                    "cd ${strCoveragePath}_temp && " .
-                    LIB_COVER_EXE . " -report json -outputdir ${strBackRestBase}/test/coverage/perl ${strCoveragePath}_temp",
-                    {bSuppressStdErr => true});
-                executeTest("sudo rm -rf ${strCoveragePath}_temp");
-                executeTest("sudo cp -rp ${strCoveragePath} ${strCoveragePath}_temp");
-                executeTest(
-                    "cd ${strCoveragePath}_temp && " .
-                    LIB_COVER_EXE . " -outputdir ${strBackRestBase}/test/coverage/perl ${strCoveragePath}_temp",
-                    {bSuppressStdErr => true});
-                executeTest("sudo rm -rf ${strCoveragePath}_temp");
-
-                # Load the results of coverage testing from JSON
-                my $oJSON = JSON::PP->new()->allow_nonref();
-                my $hCoverageResult = $oJSON->decode(${$oStorageBackRest->get('test/coverage/perl/cover.json')});
-
-                foreach my $strCodeModule (sort(keys(%{$hCoverageActual})))
-                {
-                    # If the first char of the module is lower case then it's a c module
-                    if (substr($strCodeModule, 0, 1) eq lc(substr($strCodeModule, 0, 1)))
-                    {
-                        next;
-                    }
-
-                    # Create code module path -- where the file is located on disk
-                    my $strCodeModulePath = "${strBackRestBase}/lib/" . PROJECT_NAME . "/${strCodeModule}.pm";
-
-                    # Get summary results
-                    my $hCoverageResultAll = $hCoverageResult->{'summary'}{$strCodeModulePath}{total};
-
-                    # Try an extra / if the module is not found
-                    if (!defined($hCoverageResultAll))
-                    {
-                        $strCodeModulePath = "/${strCodeModulePath}";
-                        $hCoverageResultAll = $hCoverageResult->{'summary'}{$strCodeModulePath}{total};
-                    }
-
-                    # If module is marked as having no code
-                    if ($hCoverageActual->{$strCodeModule} eq TESTDEF_COVERAGE_NOCODE)
-                    {
-                        # Error if it really does have coverage
-                        if ($hCoverageResultAll)
-                        {
-                            confess &log(ERROR, "perl module ${strCodeModule} is marked 'no code' but has code");
-                        }
-
-                        # Skip to next module
-                        next;
-                    }
-
-                    if (!defined($hCoverageResultAll))
-                    {
-                        confess &log(ERROR, "unable to find coverage results for ${strCodeModule}");
-                    }
-
-                    # Check that all code has been covered
-                    my $iCoverageTotal = $hCoverageResultAll->{total};
-                    my $iCoverageUncoverable = coalesce($hCoverageResultAll->{uncoverable}, 0);
-                    my $iCoverageCovered = coalesce($hCoverageResultAll->{covered}, 0);
-
-                    if ($hCoverageActual->{$strCodeModule} eq TESTDEF_COVERAGE_FULL)
-                    {
-                        my $iUncoveredLines = $iCoverageTotal - $iCoverageCovered - $iCoverageUncoverable;
-
-                        if ($iUncoveredLines != 0)
-                        {
-                            &log(ERROR, "perl module ${strCodeModule} is not fully covered");
-                            $iUncoveredCodeModuleTotal++;
-
-                            &log(ERROR, ('-' x 80));
-                            executeTest(
-                                "/usr/bin/cover -report text ${strCoveragePath} --select ${strBackRestBase}/lib/" .
-                                PROJECT_NAME . "/${strCodeModule}.pm",
-                                {bShowOutputAsync => true});
-                            &log(ERROR, ('-' x 80));
-                        }
-                    }
-                    # Else test how much partial coverage there was
-                    elsif ($hCoverageActual->{$strCodeModule} eq TESTDEF_COVERAGE_PARTIAL)
-                    {
-                        my $iCoveragePercent = int(($iCoverageCovered + $iCoverageUncoverable) * 100 / $iCoverageTotal);
-
-                        if ($iCoveragePercent == 100)
-                        {
-                            &log(ERROR, "perl module ${strCodeModule} has 100% coverage but is not marked fully covered");
-                            $iUncoveredCodeModuleTotal++;
-                        }
-                    }
-                }
-            }
-
             # Generate C coverage report
             #---------------------------------------------------------------------------------------------------------------------------
             if (vmCoverageC($strVm))
@@ -1410,7 +1383,7 @@ eval
                 if ($oStorageBackRest->exists($strLCovFile))
                 {
                     executeTest(
-                        "genhtml ${strLCovFile} --config-file=${strBackRestBase}/test/src/lcov.conf" .
+                        "genhtml ${strLCovFile} --config-file=${strBackRestBase}/test/.vagrant/code/lcov.conf" .
                             " --prefix=${strBackRestBase}/test/.vagrant/code" .
                             " --output-directory=${strBackRestBase}/test/coverage/c");
 
@@ -1467,9 +1440,19 @@ eval
                         }
                     }
 
-                $oStorageBackRest->remove("${strBackRestBase}/test/.vagrant/code/all.lcov", {bIgnoreMissing => true});
-                coverageGenerate(
-                    $oStorageBackRest, "${strBackRestBase}/test/.vagrant/code", "${strBackRestBase}/test/coverage/c-coverage.html");
+                    $oStorageBackRest->remove("${strBackRestBase}/test/.vagrant/code/all.lcov", {bIgnoreMissing => true});
+                    coverageGenerate(
+                        $oStorageBackRest, "${strBackRestBase}/test/.vagrant/code",
+                        "${strBackRestBase}/test/coverage/c-coverage.html");
+
+                    if ($bCoverageSummary)
+                    {
+                        &log(INFO, 'writing C coverage summary report');
+
+                        coverageDocSummaryGenerate(
+                            $oStorageBackRest, "${strBackRestBase}/test/.vagrant/code",
+                            "${strBackRestBase}/doc/xml/auto/metric-coverage-report.auto.xml");
+                    }
                 }
                 else
                 {
@@ -1486,7 +1469,7 @@ eval
             " WITH ${iTestFail} FAILURE(S)") . ($iTestRetry == 0 ? '' : ", ${iTestRetry} RETRY(IES)") .
                 ' (' . (time() - $lStartTime) . 's)');
 
-        exit 1 if ($iTestFail > 0 || $iUncoveredCodeModuleTotal > 0);
+        exit 1 if ($iTestFail > 0 || ($iUncoveredCodeModuleTotal > 0 && !$bCoverageSummary));
 
         exit 0;
     }
